/*  The FORTRAN SSP library subroutine DMFSD translated into C
//
// The original code available at http://pdp-10.trailing-edge.com/decuslib20-02/index.html
C
C     ..................................................................
C
C        SUBROUTINE DMFSD
C
C        PURPOSE
C           FACTOR A GIVEN SYMMETRIC POSITIVE DEFINITE MATRIX
C
C        USAGE
C           CALL DMFSD(A,N,EPS,IER)
C
C        DESCRIPTION OF PARAMETERS
C           A      - DOUBLE PRECISION UPPER TRIANGULAR PART OF GIVEN
C                    SYMMETRIC POSITIVE DEFINITE N BY N COEFFICIENT
C                    MATRIX.
C                    ON RETURN A CONTAINS THE RESULTANT UPPER
C                    TRIANGULAR MATRIX IN DOUBLE PRECISION.
C           N      - THE NUMBER OF ROWS (COLUMNS) IN GIVEN MATRIX.
C           EPS    - SINGLE PRECISION INPUT CONSTANT WHICH IS USED
C                    AS RELATIVE TOLERANCE FOR TEST ON LOSS OF
C                    SIGNIFICANCE.
C           IER    - RESULTING ERROR PARAMETER CODED AS FOLLOWS
C                    IER=0  - NO ERROR
C                    IER=-1 - NO RESULT BECAUSE OF WRONG INPUT PARAME-
C                             TER N OR BECAUSE SOME RADICAND IS NON-
C                             POSITIVE (MATRIX A IS NOT POSITIVE
C                             DEFINITE, POSSIBLY DUE TO LOSS OF SIGNI-
C                             FICANCE)
C                    IER=K  - WARNING WHICH INDICATES LOSS OF SIGNIFI-
C                             CANCE. THE RADICAND FORMED AT FACTORIZA-
C                             TION STEP K+1 WAS STILL POSITIVE BUT NO
C                             LONGER GREATER THAN ABS(EPS*A(K+1,K+1)).
C
C        REMARKS
C           THE UPPER TRIANGULAR PART OF GIVEN MATRIX IS ASSUMED TO BE
C           STORED COLUMNWISE IN N*(N+1)/2 SUCCESSIVE STORAGE LOCATIONS.
C           IN THE SAME STORAGE LOCATIONS THE RESULTING UPPER TRIANGU-
C           LAR MATRIX IS STORED COLUMNWISE TOO.
C           THE PROCEDURE GIVES RESULTS IF N IS GREATER THAN 0 AND ALL
C           CALCULATED RADICANDS ARE POSITIVE.
C           THE PRODUCT OF RETURNED DIAGONAL TERMS IS EQUAL TO THE
C           SQUARE-ROOT OF THE DETERMINANT OF THE GIVEN MATRIX.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           SOLUTION IS DONE USING THE SQUARE-ROOT METHOD OF CHOLESKY.
C           THE GIVEN MATRIX IS REPRESENTED AS PRODUCT OF TWO TRIANGULAR
C           MATRICES, WHERE THE LEFT HAND FACTOR IS THE TRANSPOSE OF
C           THE RETURNED RIGHT HAND FACTOR.
C
C     ..................................................................
*/
#include <math.h>

void DMFSD(double *A,long N,float EPS,long &IER)
{
double DPIV, DSUM;
long I, K, L, KPIV, IND, LEND, LANF, LIND;
float TOL;
//
//        TEST ON WRONG INPUT PARAMETER N
if (N-1 < 0) {
	IER=-1;
	return;
}

IER=0;
//
//        INITIALIZE DIAGONAL-LOOP
KPIV=0;
for (K = 1; K <= N; K++) {
	KPIV=KPIV+K;
	IND=KPIV;
	LEND=K-1;
//
//        CALCULATE TOLERANCE
	TOL=(float)fabs(EPS*(float)A[KPIV-1]);
//
//        START FACTORIZATION-LOOP OVER K-TH ROW
	for (I = K; I <= N; I++) {
		DSUM=0.;
      if(LEND) {
//
//        START INNER LOOP
			for (L = 1; L <= LEND; L++) {
				LANF=KPIV-L;
				LIND=IND-L;
				DSUM=DSUM+A[LANF-1]*A[LIND-1];
			}
//        END OF INNER LOOP
		}
//
//        TRANSFORM ELEMENT A[IND-1]
		DSUM=A[IND-1]-DSUM;
      if(I-K == 0) {
//
//        TEST FOR NEGATIVE PIVOT ELEMENT AND FOR LOSS OF SIGNIFICANCE
			if((float)DSUM-TOL <= 0.) {
				if(DSUM<=0.) {
					IER=-1;
					return;
				}
				if(IER<=0.) {
					IER=K-1;
				}
			}
//
//        COMPUTE PIVOT ELEMENT
			DPIV=sqrt(DSUM);
			A[KPIV-1]=DPIV;
			DPIV=1./DPIV;
		}
		else {
//
//        CALCULATE TERMS IN ROW
			A[IND-1]=DSUM*DPIV;
		}
		IND=IND+I;
	}
}
//        END OF DIAGONAL-LOOP
//
return;
}
