/*  The FORTRAN SSP library subroutine DSINV translated into C
//
// The original code available at http://pdp-10.trailing-edge.com/decuslib20-02/index.html
C
C     ..................................................................
C
C        SUBROUTINE DSINV
C
C        PURPOSE
C           INVERT A GIVEN SYMMETRIC POSITIVE DEFINITE MATRIX
C
C        USAGE
C           CALL DSINV(A,N,EPS,IER)
C
C        DESCRIPTION OF PARAMETERS
C           A      - DOUBLE PRECISION UPPER TRIANGULAR PART OF GIVEN
C                    SYMMETRIC POSITIVE DEFINITE N BY N COEFFICIENT
C                    MATRIX.
C                    ON RETURN A CONTAINS THE RESULTANT UPPER
C                    TRIANGULAR MATRIX IN DOUBLE PRECISION.
C           N      - THE NUMBER OF ROWS (COLUMNS) IN GIVEN MATRIX.
C           EPS    - SINGLE PRECISION INPUT CONSTANT WHICH IS USED
C                    AS RELATIVE TOLERANCE FOR TEST ON LOSS OF
C                    SIGNIFICANCE.
C           IER    - RESULTING ERROR PARAMETER CODED AS FOLLOWS
C                    IER=0  - NO ERROR
C                    IER=-1 - NO RESULT BECAUSE OF WRONG INPUT PARAME-
C                             TER N OR BECAUSE SOME RADICAND IS NON-
C                             POSITIVE (MATRIX A IS NOT POSITIVE
C                             DEFINITE, POSSIBLY DUE TO LOSS OF SIGNI-
C                             FICANCE)
C                    IER=K  - WARNING WHICH INDICATES LOSS OF SIGNIFI-
C                             CANCE. THE RADICAND FORMED AT FACTORIZA-
C                             TION STEP K+1 WAS STILL POSITIVE BUT NO
C                             LONGER GREATER THAN ABS(EPS*A(K+1,K+1)).
C
C        REMARKS
C           THE UPPER TRIANGULAR PART OF GIVEN MATRIX IS ASSUMED TO BE
C           STORED COLUMNWISE IN N*(N+1)/2 SUCCESSIVE STORAGE LOCATIONS.
C           IN THE SAME STORAGE LOCATIONS THE RESULTING UPPER TRIANGU-
C           LAR MATRIX IS STORED COLUMNWISE TOO.
C           THE PROCEDURE GIVES RESULTS IF N IS GREATER THAN 0 AND ALL
C           CALCULATED RADICANDS ARE POSITIVE.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           DMFSD
C
C        METHOD
C           SOLUTION IS DONE USING FACTORIZATION BY SUBROUTINE DMFSD.
C
C     ..................................................................
*/
extern void DMFSD(double *A,long N,float EPS,long &IER);

void DSINV(double *A,long N,float EPS,long &IER)
{
//
double DIN,WORK;
long I, J, K, L, IPIV, IND, MIN, KEND, LANF, LHOR, LVER;
//
//        FACTORIZE GIVEN MATRIX BY MEANS OF SUBROUTINE DMFSD
//        A = TRANSPOSE(T) * T
DMFSD(A,N,EPS,IER);
if (IER < 0) return;
//
//        INVERT UPPER TRIANGULAR MATRIX T
//        PREPARE INVERSION-LOOP
IPIV=N*(N+1)/2;
IND=IPIV;
//
//        INITIALIZE INVERSION-LOOP
for (I=1; I<=N; I++) {
	DIN=1./A[IPIV-1];
	A[IPIV-1]=DIN;
	MIN=N;
	KEND=I-1;
	LANF=N-KEND;
	if(KEND>0) {
		J=IND;
//
//        INITIALIZE ROW-LOOP
		for (K=1; K<=KEND; K++) {
			WORK=0.;
		   MIN=MIN-1;
			LHOR=IPIV;
			LVER=J;
//
//        START INNER LOOP
			for (L=LANF; L<=MIN; L++) {
				LVER=LVER+1;
				LHOR=LHOR+L;
				WORK=WORK+A[LVER-1]*A[LHOR-1];
			}
//        END OF INNER LOOP
//
			A[J-1]=-WORK*DIN;
			J=J-MIN;
		}
//        END OF ROW-LOOP
//
	}
	IPIV=IPIV-MIN;
	IND=IND-1;
}
//        END OF INVERSION-LOOP
//
//        CALCULATE INVERSE(A) BY MEANS OF INVERSE(T)
//        INVERSE(A) = INVERSE(T) * TRANSPOSE(INVERSE(T))
//        INITIALIZE MULTIPLICATION-LOOP
for (I=1; I<=N; I++) {
	IPIV=IPIV+I;
	J=IPIV;
//
//        INITIALIZE ROW-LOOP
	for (K=I; K<=N; K++) {
		WORK=0.;
		LHOR=J;
//
//        START INNER LOOP
		for (L=K; L<=N; L++) {
			LVER=LHOR+K-I;
			WORK=WORK+A[LHOR-1]*A[LVER-1];
			LHOR=LHOR+L;
		}
//        END OF INNER LOOP
//
		A[J-1]=WORK;
		J=J+K;
	}
}
//        END OF ROW- AND MULTIPLICATION-LOOP
//
return;
}
