/****************************************************************************

Copyright (C) 2016 - 2017 Andrius Poskus (e-mail:  andrius.poskus@ff.vu.lt  or  andrius.poskus@live.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

This program makes extensive use of the MFC Grid Control v2.26 written by Chris Maunder.
The code of the grid control is in the subfolder "GridCtrl".

****************************************************************************/

#include "Compiler/Compiler.h"
#include "MCNScriptDlg.h"
#include "MCNScript.h"
#include "TabFilteringByTransportParameters.h"
#include "TabFilteringByParticleType.h"
#include <conio.h>
#include <afxdialogex.h>
#include <io.h>
#include <psapi.h>

#include "WindowHandleFromProcessID.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

VECTOR operator-(VECTOR a) {
	VECTOR out;
	out.x = -a.x;
	out.y = -a.y;
	out.z = -a.z;
	return out;
}

VECTOR operator-(VECTOR a, VECTOR b) {
	VECTOR out;
	out.x = a.x - b.x;
	out.y = a.y - b.y;
	out.z = a.z - b.z;
	return out;
}

VECTOR operator-=(VECTOR &a, VECTOR b) {
	a = a - b;
	return a;
}

VECTOR operator+(VECTOR a, VECTOR b) {
	VECTOR out;
	out.x = a.x + b.x;
	out.y = a.y + b.y;
	out.z = a.z + b.z;
	return out;
}

VECTOR operator+=(VECTOR &a, VECTOR b) {
	a = a + b;
	return a;
}

double operator*(VECTOR a, VECTOR b) { // scalar product
	return a.x * b.x + a.y * b.y + a.z * b.z;
}

VECTOR operator*(double a, VECTOR b) {
	VECTOR out;
	out.x = a * b.x;
	out.y = a * b.y;
	out.z = a * b.z;
	return out;
}

VECTOR operator*(VECTOR a, double b) {
	return b * a;
}

VECTOR operator*=(VECTOR &a, double b) {
	a = a * b;
	return a;
}

VECTOR operator/(VECTOR a,  double b) {
	VECTOR out;
	out.x = a.x / b;
	out.y = a.y / b;
	out.z = a.z / b;
	return out;
}

VECTOR operator/=(VECTOR &a,  double b) {
	a = a / b;
	return a;
}

VECTOR operator&(VECTOR a, VECTOR b) { // vector product
	VECTOR out;
	out.x = a.y * b.z - a.z * b.y;
	out.y = a.z * b.x - a.x * b.z;
	out.z = a.x * b.y - a.y * b.x;
	return out;
}

VECTOR Init(double x, double y, double z) {
	VECTOR out;
	out.x = x;
	out.y = y;
	out.z = z;
	return out;
}

double Norm(VECTOR a)
{
	return sqrt(a.x * a.x + a.y * a.y + a.z * a.z);
}

#define NROOTS_MAX 10
double anglePrecision = 1e-12; // threshold angle for testing for parallel lines or planes (rad)
double distancePrecision = 1e-12; // threshold distance for testing for coinciding or collinear points (cm)
double maxDistance_ratio = 1e20;
bool roots_improvedPrecision = true;
double roots_minRelChange = 1e-6;
int roots_maxIterationsWithoutChange = 10;

double pi = 3.1415926535897932384626433832795;
VECTOR zero, xDir, yDir, zDir;

int nSmax_cell; // maximum number of surfaces in a cell
VECTOR *edgeBuffer;

char currentFolder[1001];

void Projection(double azimuth, double elevation, double twist, double factor, double coord[3][3])
{
	double angleFactor = pi / 180.;
	azimuth *= angleFactor;
	elevation *= angleFactor;
	twist *= angleFactor;
//	Azimuth:
//	coord[0][0] = -sin(azimuth); coord[0][1] = 0.; coord[0][2] = cos(azimuth);
//	coord[1][0] = cos(azimuth); coord[1][1] = 0.; coord[1][2] = sin(azimuth);
//	coord[2][0] = 0.; coord[2][1] = 1.; coord[2][2] = 0.;
//	Azimuth + elevation:
//	coord[0][0] = -sin(azimuth); coord[0][1] = -cos(azimuth) * sin(elevation); coord[0][2] = cos(azimuth) * cos(elevation);
//	coord[1][0] = cos(azimuth); coord[1][1] = -sin(azimuth) * sin(elevation); coord[1][2] = sin(azimuth) * cos(elevation);
//	coord[2][0] = 0.; coord[2][1] = cos(elevation); coord[2][2] = sin(elevation);
//	Azimuth + elevation + twist:
	double sina = sin(azimuth);
	double cosa = cos(azimuth);
	double sine = sin(elevation);
	double cose = cos(elevation);
	double sint = sin(twist);
	double cost = cos(twist);
	coord[0][0] = factor * (-sina * cost - cosa * sine * sint);
	coord[0][1] = factor * (sina * sint - cosa * sine * cost);
	coord[0][2] = factor * cosa * cose;
	coord[1][0] = factor * (cosa * cost - sina * sine * sint);
	coord[1][1] = factor * (-cosa * sint - sina * sine * cost);
	coord[1][2] = factor * sina * cose;
	coord[2][0] = factor * cose * sint;
	coord[2][1] = factor * cose * cost;
	coord[2][2] = factor * sine;
}

bool UpdateTrackStats(TRACKSTATS *stats, int iHist, int iTrack, int nVertices, long gen, unsigned long originType, VECTOR *coord, double *energy, char type, int &iStart, int &iEnd, int &add1, VECTOR &initialPoint, int &add2, VECTOR &finalPoint, TRACKFILTERDATA *pTrackFilter = NULL, TRACKSEGMENTDATA *pTrackSegmentSelection = NULL, BOOL trackFilteringBeforeSegmentSelection = TRUE)
{
	int i;
	bool truncated = (type < 0);
	bool escape = (type > 0);
	int nVertices1 = escape ? nVertices - 1 : nVertices;
	double Eloss, relEloss, displ;
	VECTOR displVector;
	double path = 0., peakDispl = 0.;
	if (pTrackFilter) {
		if (trackFilteringBeforeSegmentSelection) {
			if (gen && (gen < pTrackFilter->gen[0] || pTrackFilter->gen[1] >= 0 && gen > pTrackFilter->gen[1]) || !(originType & pTrackFilter->originTypeMask)) return false;
			if (nVertices1 < pTrackFilter->nVert[0] || pTrackFilter->nVert[1] > 0 && nVertices1 > pTrackFilter->nVert[1]) return false;
			if (energy[0] < 0.999999999999 * pTrackFilter->Estart[0] || pTrackFilter->Estart[1] >= 0. && energy[0] > 1.000000000001 * pTrackFilter->Estart[1]) return false;
			if (energy[nVertices1-1] < 0.999999999999 * pTrackFilter->Efinal[0] || pTrackFilter->Efinal[1] >= 0. && energy[nVertices1-1] > 1.000000000001 * pTrackFilter->Efinal[1]) return false;
			Eloss = energy[0] - energy[nVertices1-1];
			if (Eloss < 0.999999999999 * pTrackFilter->Eloss[0] || pTrackFilter->Eloss[1] >= 0. && Eloss > 1.000000000001 * pTrackFilter->Eloss[1]) return false;
			relEloss = energy[0] ? 1. - energy[nVertices1-1] / energy[0] : 0.;
			if (relEloss < 0.999999999999 * pTrackFilter->relEloss[0] || relEloss > 1.000000000001 * pTrackFilter->relEloss[1]) return false;
			if (nVertices1 > 1) {
				displVector = coord[nVertices1-1] - coord[0];
				displ = Norm(displVector);
				if (displ < 0.999999999999 * pTrackFilter->displ[0] || pTrackFilter->displ[1] >= 0. && displ > 1.000000000001 * pTrackFilter->displ[1]) return false;
				if (pTrackFilter->path[0] > 0. || pTrackFilter->path[1] >= 0.) {
					for (i = 1; i < nVertices1; i++) path += Norm(coord[i] - coord[i-1]);
					if (path < 0.999999999999 * pTrackFilter->path[0] || pTrackFilter->path[1] >= 0. && path > 1.000000000001 * pTrackFilter->path[1]) return false;
				}
				if (pTrackFilter->peakDispl[0] > 0. || pTrackFilter->peakDispl[1] >= 0.) {
					for (i = 1; i < nVertices1; i++) {
						double displ = Norm(coord[i] - coord[0]);
						if (displ > peakDispl) peakDispl = displ;
					}
					if (peakDispl < 0.999999999999 * pTrackFilter->peakDispl[0] || pTrackFilter->peakDispl[1] >= 0. && peakDispl > 1.000000000001 * pTrackFilter->peakDispl[1]) return false;
				}
			}
		}
	}

	int iVert_start;
	if (!pTrackSegmentSelection) iVert_start = 0;
	else iVert_start = pTrackSegmentSelection->iVert[0];
	int iVert_end;
	if (!pTrackSegmentSelection || pTrackSegmentSelection->iVert[1] < 0) iVert_end = nVertices1 - 1;
	else iVert_end = pTrackSegmentSelection->iVert[1];
	if (iVert_end >= nVertices1) iVert_end = nVertices1 - 1;
	if (iVert_end < iVert_start) return false;

	bool pathSegmentDefined = pTrackSegmentSelection && pTrackSegmentSelection->path[0] > 0.;

	double Emin;
	if (!pTrackSegmentSelection) Emin = energy[iVert_end];
	else Emin = pTrackSegmentSelection->E[0];
	double Emax;
	if (!pTrackSegmentSelection || pTrackSegmentSelection->E[1] < 0.) Emax = energy[iVert_start];
	else Emax = pTrackSegmentSelection->E[1];
	if (Emin > energy[iVert_end] || Emax < energy[iVert_start]) {
		if (Emin > Emax) return false;
		iStart = -1;
		for (i = iVert_start; i <= iVert_end; i++) {
			double testEnergy = i && pathSegmentDefined ? energy[i-1] : energy[i];
			if (testEnergy < 0.999999999999 * Emin) break;
			if (energy[i] > 1.000000000001 * Emax) continue;
			if (iStart < 0) {
				iStart = i;
				if (Emin <= energy[iVert_end]) i = iVert_end;
			}
		}
		if (iStart < 0) return false;
		iVert_start = iStart;
		iVert_end = i - 1;
	}

	double delta_E = energy[iVert_start] - energy[iVert_end];
	double Eloss_min;
	if (!pTrackSegmentSelection) Eloss_min = 0.;
	else Eloss_min = pTrackSegmentSelection->Eloss[0];
	double Eloss_max;
	if (!pTrackSegmentSelection || pTrackSegmentSelection->Eloss[1] < 0.) Eloss_max = delta_E;
	else Eloss_max = pTrackSegmentSelection->Eloss[1];
	if (Eloss_min > 0. || Eloss_max < delta_E) {
		if (Eloss_min > Eloss_max) return false;
		iEnd = -1;
		double baseEnergy = iVert_start && pathSegmentDefined ? energy[iVert_start-1] : energy[iVert_start];
		for (i = iVert_end; i >= iVert_start; i--) {
			double Eloss = baseEnergy - energy[i];
			if (Eloss < 0.999999999999 * Eloss_min) break;
			if (Eloss > 1.000000000001 * Eloss_max) continue;
			if (iEnd < 0) {
				iEnd = i;
				if (!Eloss_min) i = iVert_start;
			}
		}
		if (iEnd < 0) return false;
		iVert_start = i + 1;
		iVert_end = iEnd;
	}

	double delta_E_rel = 1. - energy[iVert_end] / energy[iVert_start];
	double relEloss_min;
	if (!pTrackSegmentSelection) relEloss_min = 0.;
	else relEloss_min = pTrackSegmentSelection->relEloss[0];
	double relEloss_max;
	if (!pTrackSegmentSelection || pTrackSegmentSelection->relEloss[1] == 1.) relEloss_max = delta_E_rel;
	else relEloss_max = pTrackSegmentSelection->relEloss[1];
	if (relEloss_min > 0. || relEloss_max < delta_E_rel) {
		if (relEloss_min > relEloss_max) return false;
		iEnd = -1;
		double baseEnergy = iVert_start && pathSegmentDefined ? energy[iVert_start-1] : energy[iVert_start];
		for (i = iVert_end; i >= iVert_start; i--) {
			double relEloss = baseEnergy ? 1. - energy[i] / baseEnergy : 0.;
			if (relEloss < 0.999999999999 * relEloss_min) break;
			if (relEloss > 1.000000000001 * relEloss_max) continue;
			if (iEnd < 0) {
				iEnd = i;
				if (!relEloss_min) i = iVert_start;
			}
		}
		if (iEnd < 0) return false;
		iVert_start = i + 1;
		iVert_end = iEnd;
	}

	double pathMin, pathMax;
	if (!pTrackSegmentSelection) pathMin = 0.;
	else pathMin = pTrackSegmentSelection->path[0];
	if (!pTrackSegmentSelection || pTrackSegmentSelection->path[1] < 0.) pathMax = -1.;
	else pathMax = pTrackSegmentSelection->path[1];
	if (pathMax >= 0. && pathMin > pathMax) return false;
	iStart = -1;
	double totalPath0, totalPath_prev, totalPath = 0.;
	for (i = iVert_start; i <= iVert_end; i++) {
		totalPath_prev = totalPath;
		if (i > iVert_start) totalPath += Norm(coord[i] - coord[i-1]);
		if (iStart < 0) {
			if (totalPath < 0.999999999999 * pathMin) continue;
			totalPath0 = totalPath;
			iStart = i;
		}
		if (pathMax >= 0. && totalPath > pathMax) {
			totalPath = totalPath_prev;
			break;
		}
	}
	if (iStart < 0) return false;
	path = totalPath - totalPath0;
	iEnd = i - 1;

	int nVert1 = iEnd - iStart + 1;
	displVector = coord[iEnd] - coord[iStart];
	double lastDist1, additionalPath1, initialEnergy;
	if (totalPath0 > pathMin) {
		add1 = 1;
		nVert1++;
		initialEnergy =  energy[iStart-1];
		additionalPath1 = totalPath0 - pathMin;
		lastDist1 = Norm(coord[iStart] - coord[iStart-1]);
		VECTOR dir = (coord[iStart] - coord[iStart-1]) / lastDist1;
		initialPoint = coord[iStart] - dir * additionalPath1;
		path += additionalPath1;
		displVector += dir * additionalPath1;
	}
	else {
		add1 = 0;
		initialEnergy =  energy[iStart];
		additionalPath1 = 0.;
		initialPoint = coord[iStart];
	}
	double lastDist2, additionalPath2;
	if (pathMax >= 0. && path < pathMax && iEnd < iVert_end) {
		add2 = 1;
		nVert1++;
		additionalPath2 = pathMax - totalPath;
		lastDist2 = Norm(coord[iEnd+1] - coord[iEnd]);
		VECTOR dir = (coord[iEnd+1] - coord[iEnd]) / lastDist2;
		finalPoint = coord[iEnd] + dir * additionalPath2;
		path += additionalPath2;
		displVector += dir * additionalPath2;
	}
	else {
		add2 = 0;
		additionalPath2 = 0.;
		finalPoint = coord[iEnd];
	}
	displ = Norm(displVector);
	Eloss = initialEnergy - energy[iEnd];
	relEloss = initialEnergy ? 1. - energy[iEnd] / initialEnergy : 0.;
	int nVert2 = nVert1 - add1 - add2;
	int nVert = nVert2 + add1;

	if (pTrackFilter && !trackFilteringBeforeSegmentSelection) {
		if (gen && (gen < pTrackFilter->gen[0] || pTrackFilter->gen[1] >= 0 && gen > pTrackFilter->gen[1]) || !(originType & pTrackFilter->originTypeMask)) return false;
		if (nVert < pTrackFilter->nVert[0] || pTrackFilter->nVert[1] > 0 && nVert > pTrackFilter->nVert[1]) return false;
		if (initialEnergy < 0.999999999999 * pTrackFilter->Estart[0] || pTrackFilter->Estart[1] >= 0. && initialEnergy > 1.000000000001 * pTrackFilter->Estart[1]) return false;
		if (energy[iEnd] < 0.999999999999 * pTrackFilter->Efinal[0] || pTrackFilter->Efinal[1] >= 0. && energy[iEnd] > 1.000000000001 * pTrackFilter->Efinal[1]) return false;
		if (Eloss < 0.999999999999 * pTrackFilter->Eloss[0] || pTrackFilter->Eloss[1] >= 0. && Eloss > 1.000000000001 * pTrackFilter->Eloss[1]) return false;
		if (relEloss < 0.999999999999 * pTrackFilter->relEloss[0] || relEloss > 1.000000000001 * pTrackFilter->relEloss[1]) return false;
		if (nVert1 > 1) {
			if (displ < 0.999999999999 * pTrackFilter->displ[0] || pTrackFilter->displ[1] >= 0. && displ > 1.000000000001 * pTrackFilter->displ[1]) return false;
			if (path < 0.999999999999 * pTrackFilter->path[0] || pTrackFilter->path[1] >= 0. && path > 1.000000000001 * pTrackFilter->path[1]) return false;
		}
	}

	peakDispl = Norm(finalPoint - initialPoint);
	for (i = iStart + 1; i <= iEnd; i++) {
		double displ = Norm(coord[i] - initialPoint);
		if (displ > peakDispl) peakDispl = displ;
	}

	if (pTrackFilter && !trackFilteringBeforeSegmentSelection) {
		if (nVert1 > 1 && (peakDispl < 0.999999999999 * pTrackFilter->peakDispl[0] || pTrackFilter->peakDispl[1] >= 0. && peakDispl > 1.000000000001 * pTrackFilter->peakDispl[1])) return false;
	}

	stats->nTracks++;
	if (iHist != stats->lastHistory) {
		stats->nHistories++;
		stats->lastHistory = iHist;
	}
	if (escape && iEnd == nVertices1 - 1) stats->nTracksEscape++;
	else if (energy[iEnd] > 0.) stats->nTracksTruncated++;
	int nColl0 = iStart - add1;
	stats->nColl0_sum += nColl0; stats->nColl0_sum2 += (double)nColl0 * nColl0;
	int nColl = nVert - 1;
	stats->nColl_sum += nColl; stats->nColl_sum2 += (double)nColl * nColl;
	stats->startE_sum += initialEnergy; stats->startE_sum2 += initialEnergy * initialEnergy;
	stats->finalE_sum += energy[iEnd]; stats->finalE_sum2 += energy[iEnd] * energy[iEnd];
	double Eloss0 = energy[0] - initialEnergy;
	stats->Eloss0_sum += Eloss0; stats->Eloss0_sum2 += Eloss0 * Eloss0;
	stats->Eloss_sum += Eloss; stats->Eloss_sum2 += Eloss * Eloss;
	double relEloss0 = 1. - initialEnergy / energy[0];
	stats->relEloss0_sum += relEloss0; stats->relEloss0_sum2 += relEloss0 * relEloss0;
	stats->relEloss_sum += relEloss; stats->relEloss_sum2 += relEloss * relEloss;
	double displ0 = Norm(initialPoint - coord[0]);
	stats->displ0_sum += displ0; stats->displ0_sum2 += displ0 * displ0;
	double path0 = 0.;
	for (i = 1; i <= iStart - add1; i++) path0 += Norm(coord[i] - coord[i-1]);
	if (add1) path0 += Norm(initialPoint - coord[iStart-1]);
	stats->path0_sum += path0; stats->path0_sum2 += path0 * path0;
	if (stats->nTracks == 1) {
		stats->iHist_min = iHist; stats->iHist_max = iHist;
		stats->iTrack_min = iTrack; stats->iTrack_max = iTrack;
		stats->nColl0_min = nColl0; stats->nColl0_max = nColl0;
		stats->nColl_min = nColl; stats->nColl_max = nColl;
		stats->startE_min = initialEnergy; stats->startE_max = initialEnergy;
		stats->finalE_min = energy[iEnd]; stats->finalE_max = energy[iEnd];
		stats->Eloss0_min = Eloss0; stats->Eloss0_max = Eloss0;
		stats->Eloss_min = Eloss; stats->Eloss_max = Eloss;
		stats->relEloss0_min = relEloss0; stats->relEloss0_max = relEloss0;
		stats->relEloss_min = relEloss; stats->relEloss_max = relEloss;
		stats->displ0_min = displ0; stats->displ0_max = displ0;
		stats->path0_min = path0; stats->path0_max = path0;
	}
	else {
		if (iHist < stats->iHist_min) stats->iHist_min = iHist;
		else if (iHist > stats->iHist_max) stats->iHist_max = iHist;
		if (iTrack < stats->iTrack_min) stats->iTrack_min = iTrack;
		else if (iTrack > stats->iTrack_max) stats->iTrack_max = iTrack;
		if (nColl0 < stats->nColl0_min) stats->nColl0_min = nColl0;
		else if (nColl0 > stats->nColl0_max) stats->nColl0_max = nColl0;
		if (nColl < stats->nColl_min) stats->nColl_min = nColl;
		else if (nColl > stats->nColl_max) stats->nColl_max = nColl;
		if (initialEnergy < stats->startE_min) stats->startE_min = initialEnergy;
		else if (initialEnergy > stats->startE_max) stats->startE_max = initialEnergy;
		if (energy[iEnd] < stats->finalE_min) stats->finalE_min = energy[iEnd];
		else if (energy[iEnd] > stats->finalE_max) stats->finalE_max = energy[iEnd];
		if (Eloss0 < stats->Eloss0_min) stats->Eloss0_min = Eloss0;
		else if (Eloss0 > stats->Eloss0_max) stats->Eloss0_max = Eloss0;
		if (Eloss < stats->Eloss_min) stats->Eloss_min = Eloss;
		else if (Eloss > stats->Eloss_max) stats->Eloss_max = Eloss;
		if (relEloss0 < stats->relEloss0_min) stats->relEloss0_min = relEloss0;
		else if (relEloss0 > stats->relEloss0_max) stats->relEloss0_max = relEloss0;
		if (relEloss < stats->relEloss_min) stats->relEloss_min = relEloss;
		else if (relEloss > stats->relEloss_max) stats->relEloss_max = relEloss;
		if (displ0 < stats->displ0_min) stats->displ0_min = displ0;
		else if (displ0 > stats->displ0_max) stats->displ0_max = displ0;
		if (path0 < stats->path0_min) stats->path0_min = path0;
		else if (path0 > stats->path0_max) stats->path0_max = path0;
	}

	if (nVert1 == 1) return true;

	stats->nTracksWithTwoOrMoreVertices++;
	stats->displ_sum += displ; stats->displ_sum2 += displ * displ;
	stats->peakDispl_sum += peakDispl; stats->peakDispl_sum2 += peakDispl * peakDispl;
	stats->path_sum += path; stats->path_sum2 += path * path;
	if (stats->nTracksWithTwoOrMoreVertices == 1) {
		stats->displ_min = displ; stats->displ_max = displ;
		stats->peakDispl_min = peakDispl; stats->peakDispl_max = peakDispl;
		stats->path_min = path; stats->path_max = path;
	}
	else {
		if (displ < stats->displ_min) stats->displ_min = displ;
		else if (displ > stats->displ_max) stats->displ_max = displ;
		if (peakDispl < stats->peakDispl_min) stats->peakDispl_min = peakDispl;
		else if (peakDispl > stats->peakDispl_max) stats->peakDispl_max = peakDispl;
		if (path < stats->path_min) stats->path_min = path;
		else if (path > stats->path_max) stats->path_max = path;
	}

	if (nColl < 1) return true;

	stats->nTracksWithCollisions++;
	double path2 = path;
	if (add1) path2 += Norm(initialPoint - coord[iStart-1]);
	if (add2) path2 -= Norm(finalPoint - coord[iEnd]);
	double meanFreePath = path2 / nColl;
	double meanElossPerCollision = Eloss / nColl;
	stats->meanFreePath_sum += meanFreePath; stats->meanFreePath_sum2 += meanFreePath * meanFreePath;
	stats->meanElossPerCollision_sum += meanElossPerCollision; stats->meanElossPerCollision_sum2 += meanElossPerCollision * meanElossPerCollision;
	if (stats->nTracksWithCollisions == 1) {
		stats->meanFreePath_min = meanFreePath; stats->meanFreePath_max = meanFreePath;
		stats->meanElossPerCollision_min = meanElossPerCollision; stats->meanElossPerCollision_max = meanElossPerCollision;
	}
	else {
		if (meanFreePath < stats->meanFreePath_min) stats->meanFreePath_min = meanFreePath;
		else if (meanFreePath > stats->meanFreePath_max) stats->meanFreePath_max = meanFreePath;
		if (meanElossPerCollision < stats->meanElossPerCollision_min) stats->meanElossPerCollision_min = meanElossPerCollision;
		else if (meanElossPerCollision > stats->meanElossPerCollision_max) stats->meanElossPerCollision_max = meanElossPerCollision;
	}
	return true;
}

// CMCNScriptDlg dialog
IMPLEMENT_DYNCREATE(CMCNScriptDlg, CDialogEx)

CMCNScriptDlg::CMCNScriptDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CMCNScriptDlg::IDD, pParent)
{
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());
	app->pDlg = this;
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	GetModuleFileName(NULL, exeName, 999);
	char drive[1000], dir[1000], fname[1000], ext[1000];
	_splitpath(exeName, drive, dir, fname, ext);
	sprintf(exePath, "%s%s", drive, dir);
	parFileSpecified = (app->parFileName[0] != '\0');
	if (!parFileSpecified) sprintf(app->parFileName, "%sMCNScript.mcnprm", exePath);
//	if (!parFileSpecified) sprintf(app->parFileName, "%s%s.mcnprm", exePath, fname);

	int i = 0;
	strcpy(stdColorName[i], "White"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 255; stdColor[i].b = 255; i++;
	strcpy(stdColorName[i], "Red"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 0; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Blue"); stdColor[i].index = i; stdColor[i].r = 0; stdColor[i].g = 0; stdColor[i].b = 255; i++;
	strcpy(stdColorName[i], "Green"); stdColor[i].index = i; stdColor[i].r = 0; stdColor[i].g = 255; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Amber"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 191; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Pink"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 192; stdColor[i].b = 203; i++;
	strcpy(stdColorName[i], "Orange"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 127; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Lime"); stdColor[i].index = i; stdColor[i].r = 191; stdColor[i].g = 255; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Olive"); stdColor[i].index = i; stdColor[i].r = 128; stdColor[i].g = 128; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Cyan"); stdColor[i].index = i; stdColor[i].r = 0; stdColor[i].g = 255; stdColor[i].b = 255; i++;
	strcpy(stdColorName[i], "Purple"); stdColor[i].index = i; stdColor[i].r = 128; stdColor[i].g = 0; stdColor[i].b = 128; i++;
	strcpy(stdColorName[i], "Yellow"); stdColor[i].index = i; stdColor[i].r = 255; stdColor[i].g = 255; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Maroon"); stdColor[i].index = i; stdColor[i].r = 128; stdColor[i].g = 0; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Grey"); stdColor[i].index = i; stdColor[i].r = 128; stdColor[i].g = 128; stdColor[i].b = 128; i++;
	strcpy(stdColorName[i], "Black"); stdColor[i].index = i; stdColor[i].r = 0; stdColor[i].g = 0; stdColor[i].b = 0; i++;
	strcpy(stdColorName[i], "Custom");

	i = 0;
	defaultColor[i].r = 128; defaultColor[i].g = 255; defaultColor[i].b = 128; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 128; defaultColor[i].b = 128; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 128; defaultColor[i].g = 255; defaultColor[i].b = 255; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 255; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 192; defaultColor[i].b = 203; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 191; defaultColor[i].g = 255; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 191; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 0; defaultColor[i].g = 255; defaultColor[i].b = 255; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 0; defaultColor[i].g = 255; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 0; defaultColor[i].g = 0; defaultColor[i].b = 255; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 0; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 128; defaultColor[i].g = 0; defaultColor[i].b = 128; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 128; defaultColor[i].g = 128; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 128; defaultColor[i].g = 0; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 127; defaultColor[i].b = 0; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 128; defaultColor[i].g = 128; defaultColor[i].b = 128; ColorName(defaultColor + i); i++;
	defaultColor[i].r = 255; defaultColor[i].g = 255; defaultColor[i].b = 255; ColorName(defaultColor + i); i++;

	defaultRegionData.color = stdColor[0];
	defaultRegionData.show = TRUE;
	defaultRegionData.name[0] = '\0';
	defaultRegionData.transp = 0.3;
	defaultRegionData.refl = 0.;

	memset(&defaultTrackSetData, 0, sizeof(TRACKSETDATA));

	for (bitMask[0] = 1, i = 1; i < NEVENTTYPES; i++) bitMask[i] = bitMask[i-1] * 2;

	defaultTrackFilter.trackNo[0] = 0; defaultTrackFilter.trackNo[1] = -1;
	defaultTrackFilter.historyNo[0] = 0; defaultTrackFilter.historyNo[1] = -1;
	defaultTrackFilter.nVert[0] = 1; defaultTrackFilter.nVert[1] = 0;
	defaultTrackFilter.gen[0] = 1; defaultTrackFilter.gen[1] = -1;
	defaultTrackFilter.originTypeMask = 0;
	for (i = 0; i < NEVENTTYPES - 3; i++) defaultTrackFilter.originTypeMask |= bitMask[i];
	defaultTrackFilter.Estart[0] = 0.; defaultTrackFilter.Estart[1] = -1.;
	defaultTrackFilter.Efinal[0] = 0.; defaultTrackFilter.Efinal[1] = -1.;
	defaultTrackFilter.Eloss[0] = 0.; defaultTrackFilter.Eloss[1] = -1.;
	defaultTrackFilter.relEloss[0] = 0.; defaultTrackFilter.relEloss[1] = 1.;
	defaultTrackFilter.displ[0] = 0.; defaultTrackFilter.displ[1] = -1.;
	defaultTrackFilter.peakDispl[0] = 0.; defaultTrackFilter.peakDispl[1] = -1.;
	defaultTrackFilter.path[0] = 0.; defaultTrackFilter.path[1] = -1.;

	defaultTrackSegmentSelection.iVert[0] = 0; defaultTrackSegmentSelection.iVert[1] = -1;
	defaultTrackSegmentSelection.E[0] = 0.; defaultTrackSegmentSelection.E[1] = -1.;
	defaultTrackSegmentSelection.Eloss[0] = 0.; defaultTrackSegmentSelection.Eloss[1] = -1.;
	defaultTrackSegmentSelection.relEloss[0] = 0.; defaultTrackSegmentSelection.relEloss[1] = 1.;
	defaultTrackSegmentSelection.path[0] = 0.; defaultTrackSegmentSelection.path[1] = -1.;

	strcpy(MGEDdistUnitName[0], "1 angstrom");
	strcpy(MGEDdistUnitName[1], "1 nm");
	strcpy(MGEDdistUnitName[2], "1 um");
	strcpy(MGEDdistUnitName[3], "1 mm");
	strcpy(MGEDdistUnitName[4], "1 cm");
	strcpy(MGEDdistUnitName[5], "1 m");
	strcpy(MGEDdistUnitName[6], "1 km");
	strcpy(MGEDdistUnitName[7], "1000 km");
	strcpy(MGEDdistUnitName[8], "1e6 km");
	strcpy(MGEDdistUnitName[9], "au");
	strcpy(MGEDdistUnitName[10], "1000 au");
	strcpy(MGEDdistUnitName[11], "pc");

	MGEDdistUnit[0] = 1e-8;
	MGEDdistUnit[1] = 1e-7;
	MGEDdistUnit[2] = 1e-4;
	MGEDdistUnit[3] = 0.1;
	MGEDdistUnit[4] = 1.;
	MGEDdistUnit[5] = 100.;
	MGEDdistUnit[6] = 1e5;
	MGEDdistUnit[7] = 1e8;
	MGEDdistUnit[8] = 1e11;
	MGEDdistUnit[9] = 1.4960e13;
	MGEDdistUnit[10] = 1.4960e16;
	MGEDdistUnit[11] = 3.0857e18;

	strcpy(energyUnits[0], "eV");
	strcpy(energyUnits[1], "keV");
	strcpy(energyUnits[2], "MeV");
	strcpy(distanceUnits[0], "um");
	strcpy(distanceUnits[1], "mm");
	strcpy(distanceUnits[2], "cm");
	strcpy(distanceUnits[3], "m");
	energyFactor[0] = 1.;
	energyFactor[1] = 1e-3;
	energyFactor[2] = 1e-6;
	distanceFactor[0] = 1e4;
	distanceFactor[1] = 10.;
	distanceFactor[2] = 1.;
	distanceFactor[3] = 0.01;

	fileLoaded = false;
	curParmSet = -1;
	GetCurrentDirectory(999, currentFolder); currentFolder[999] = '\0';
	int len = (int)strlen(currentFolder);
	if (len < 1 || currentFolder[len-1] != '\\') {
		currentFolder[len] = '\\';
		currentFolder[len+1] = '\0';
	}
	overwritePrompt = true;

	surf = new SURFACE[MAXSURFACES+13]; // the first 7 additional surfaces are the original faces of the "world" (including the cut-away plane), and the last 6 are the adjusted clipping planes (depending on a cell)
	macrobody = new MACROBODY[MAXSURFACES];
	arb_entries = new double*[MAXSURFACES];
	unionData = new UNION[MAXCELLS];
	materialID = new int[MAXMATERIALS];
	transform = new TRANSFORM[MAXTRANSFORMS];
	nSmax_cell = 0;
	edgeBuffer = NULL;
	nSurfaces = 0;
	memset(surf, 0, (MAXSURFACES + 13) * sizeof(SURFACE));
	nMacrobodies = 0;
	memset(macrobody, 0, MAXSURFACES * sizeof(MACROBODY));
	memset(arb_entries, 0, MAXSURFACES * sizeof(double*));
	nCells = 0;
	cell = (CELL *)malloc(sizeof(CELL));
	memset(cell, 0, sizeof(CELL));
	cellTransform = (TRANSFORM *)malloc(sizeof(TRANSFORM));
	nUnions = 0;
	memset(unionData, 0, MAXCELLS * sizeof(UNION));
	nMaterials = 0;
	memset(materialID, 0, MAXMATERIALS * sizeof(int));
	nTransforms = 0;
	memset(transform, 0, MAXTRANSFORMS * sizeof(TRANSFORM));

	nUsedMaterials = 0;
	usedMaterialNum = NULL;
	nMaterialsWithSource = 0;
	materialName = new char*[MAXMATERIALS];
	memset(materialName, 0, MAXMATERIALS * sizeof(char *));

	processingTrackData = false;
	trackColorsShown = false;
	systemVarsShown = false;

	zero = Init(0,0,0);
	xDir = Init(1,0,0);
	yDir = Init(0,1,0);
	zDir = Init(0,0,1);

	f_in = NULL;
	f_out = NULL;
}

CString ShortenPath(CString original, CString current)
{
	if (original[0] != '\\' && !strchr(original, ':') && (current[0] == '\\' || strchr(current, ':'))) original = (CString)currentFolder + original;
	else if ((original[0] == '\\' || strchr(original, ':')) && current[0] != '\\' && !strchr(current, ':')) current = (CString)currentFolder + current;
	int o = original.GetLength();
	int c = current.GetLength();
	if (o >= c && !original.Left(c).CompareNoCase(current)) return original.Right(o - c);
	else return original;
}

void CMCNScriptDlg::MakeCmdLine()
{
	if (MCNelectronExeName[0] == '\0') {
		this->cmdLine[0] = '\0';
		m_cmdLine.SetWindowText("");
		return;
	}
	CString cmdLine('\0', 7*FNLEN);
	CString current('\0', FNLEN+1);
	CString tmp('\0', FNLEN+1);
	if (strspn(inputFileFolder, " \t") == strlen(inputFileFolder)) strcpy(inputFileFolder, currentFolder);
	if (strspn(MCNelectronFolder, " \t") == strlen(MCNelectronFolder)) strcpy(MCNelectronFolder, currentFolder);
	if (strspn(cmdFolder, " \t") == strlen(cmdFolder)) strcpy(cmdFolder, currentFolder);
	if (!cmdFolderOption) current = inputFileFolder;
	else if (cmdFolderOption == 1) current = MCNelectronFolder;
	else if (cmdFolderOption == 2) current = currentFolder;
	else current = cmdFolder;

	CString outputFN('\0', FNLEN+1);
	if (suffix[0] != '\0') {
		char tmp[SUFFIXLEN+10];
		sprintf(tmp, suffix, curParmSet + 1);
		outputFN = (CString)inputFileFolder + fn1 + tmp + ".txt";
	}
	else outputFN = outputFileName;

	CString logFN('\0', FNLEN+1);
	if (logSuffix[0] != '\0') {
		char tmp[SUFFIXLEN+10];
		sprintf(tmp, logSuffix, curParmSet + 1);
		logFN = (CString)inputFileFolder + fn1 + tmp + ".txt";
	}
	else logFN = logFileName;

	cmdLine = ShortenPath((CString)MCNelectronFolder, current) + MCNelectronExeName;
	if (cmdLine.Find(' ') >= 0 || cmdLine.Find('\t') >= 0) cmdLine = (CString)"\"" + cmdLine + "\"";

	if (nThreads >= 0) {
		CString nThreadsString('\0', 10);
		nThreadsString.Format(_T(" tasks %d"), nThreads);
		cmdLine += nThreadsString;
	}

	if (!checkSyntax_MCNelectron) cmdLine += " check 0";
	else if (quitAfterChecking) cmdLine += " check -1";
	else if (checkSyntax_MCNelectron == 1) cmdLine += " check 1";

	if (!createLog) cmdLine += " writeLog 0";
	else if (createLog == 1) cmdLine += " writeLog 1";

	if (additionalArgs[0] != '\0') {
		cmdLine += " ";
		cmdLine += additionalArgs;
	}

	int i;
	for (i = 0; i < 10; i++) {
		if (!systemVars[i]) continue;
		tmp.Format(_T(" @%d %.16g"), i, systemVars[i]);
		cmdLine += tmp;
	}

	if (ENDFfolder[0] != '\0') {
		tmp = ShortenPath(ENDFfolder, MCNelectronFolder);
		if (tmp.Find(' ') >= 0 || tmp.Find('\t') >= 0) tmp = (CString)"\"" + tmp + "\"";
		else if (!tmp.GetLength()) tmp = ".\\";
		cmdLine += (" dir " + tmp);
	}
	int lastPos = (int)strlen(altCSinfoFileName) - 1;
	if (altCSinfoFileName[0] != '\0' && altCSinfoFileName[lastPos] != '\\' && altCSinfoFileName[lastPos] != ':') {
		tmp = ShortenPath(altCSinfoFileName, current);
		if (tmp.Find(' ') >= 0 || tmp.Find('\t') >= 0) tmp = (CString)"\"" + tmp + "\"";
		cmdLine += (" alt " + tmp);
	}
	if (fn1[0] != '\0') {
		tmp = ShortenPath(inputFileFolder, current) + fn;
		if (tmp.Find(' ') >= 0 || tmp.Find('\t') >= 0) tmp = (CString)"\"" + tmp + "\"";
		if (!MCNP_style_cmdLine) cmdLine += (" in " + tmp);
		else cmdLine += (" in= " + tmp);
	}
	if (createLog && strcmpi(logSuffix, "_log") && logFN[0] != '\0' && logFN.Right(1) != "\\" && logFN.Right(1) != ":") {
		tmp = ShortenPath(logFN, current);
		if (tmp.Find(' ') >= 0 || tmp.Find('\t') >= 0) tmp = (CString)"\"" + tmp + "\"";
		cmdLine += (" log " + tmp);
	}
	if (strcmpi(suffix, "_out") && outputFN[0] != '\0' && outputFN.Right(1) != "\\" && outputFN.Right(1) != ":") {
		tmp = ShortenPath(outputFN, current);
		if (tmp.Find(' ') >= 0 || tmp.Find('\t') >= 0) tmp = (CString)"\"" + tmp + "\"";
		if (!MCNP_style_cmdLine) cmdLine += (" out " + tmp);
		else cmdLine += (" out= " + tmp);
	}
	strcpy(this->cmdLine, cmdLine);
	m_cmdLine.SetWindowText(cmdLine);
}

void CMCNScriptDlg::ColorName(COLORDATA *color, bool includeBlack)
{
	int i, nStdColors;
	if (includeBlack) nStdColors = NSTDCOLORS;
	else nStdColors = NSTDCOLORS - 1;
	for (i = 0; i < nStdColors && (color->r != stdColor[i].r || color->g != stdColor[i].g || color->b != stdColor[i].b); i++);
	if (i < nStdColors) color->index = i;
	else color->index = NSTDCOLORS;
}

// Default values of parameters that have to be assigned when reading a MCNPRM file created with an older version of MCNScipt:
int nIterations_default = 20;
int createLog_default = 2; // not specified on the command line, i.e., the MCNelectron default (0) is used
char *logSuffix_default = "_log";
int checkSyntax_MCNelectron_default = 2; // not specified on the command line, i.e., the MCNelectron default (1) is used
BOOL quitAfterChecking_default = FALSE;
BOOL MCNP_style_cmdLine_default = FALSE;
BOOL checkSyntax_MCNScript_default = TRUE;
double systemVars_default[10] = {0., 0., 0., 0., 0., 0., 0., 0., 0., 0.};

void CMCNScriptDlg::DefaultParmSet(bool keepNames)
{
	int i;
	if (!keepNames) {
		strcpy(parmSetName, "");
		cmdFolderOption = 2;
		MCNelectronFolder[0] = '\0';
		inputFileFolder[0] = '\0';
		outputFileName[0] = '\0';
		logFileName[0] = '\0';
		altCSinfoFileName[0] = '\0';
		ENDFfolder[0] = '\0';
		cmdFolder[0] = '\0';
		strcpy(MCNelectronExeName, "MCNelectron.exe");
		fn[0] = '\0';
		fn1[0] = '\0';
		strcpy(suffix, "_out");
		additionalArgs[0] = '\0';
		trackFN[0] = '\0';
		scriptFolder[0] = '\0';
		strcpy(MGEDcmdLine, "mged.exe");
		useCustomTrackFN = FALSE;
		MGEDsuffix[0] = '\0';
//		strcpy(MGEDsuffix, "_%d");
		copyToClipboard = FALSE;
		trackFilter = defaultTrackFilter;
		trackSegmentSelection = defaultTrackSegmentSelection;
		trackFilteringBeforeSegmentSelection = TRUE;
		createLog = createLog_default;
		strcpy(logSuffix, logSuffix_default);
		checkSyntax_MCNelectron = checkSyntax_MCNelectron_default;
		quitAfterChecking = quitAfterChecking_default;
		checkSyntax_MCNScript = checkSyntax_MCNScript_default;
		MCNP_style_cmdLine = MCNP_style_cmdLine_default;
		memcpy(systemVars, systemVars_default, 10 * sizeof(double));
		cmdProcessID = 0;
		cmdTimerID = 0;
		cmdTimerIndex = -1;
		cmdWindowCreated = FALSE;
		mgedProcessID = 0;
		mgedTimerID = 0;
		mgedTimerIndex = -1;
		mgedWindowCreated = FALSE;
	}
	nRegions = NREGIONSINIT;
	regions.SetSize(nRegions);
	for (i = 0; i < nRegions; i++) {
		regions[i] = defaultRegionData;
		regions[i].color = defaultColor[i - i / NDEFAULTCOLORS * NDEFAULTCOLORS];
	}
	maximumObjectSizeRatio = 50.;
	sourceSizeRatio = 0.003;
	viewSize = 15;
	viewCenter = Init(0.,0.,0.);
	azimuth = 25.;
	elevation = 15.;
	twist = 0.;
	perspective = 0.;
	useCutawayPlane = FALSE;
	cutawayPlanePoint = Init(0.,0.,0.);
	cutawayPlaneNormal = Init(0.,0.,1.);
	CutawayPlaneCoefs();
	beamAxisColor.r = 255; beamAxisColor.g = 0; beamAxisColor.b = 0; ColorName(&beamAxisColor, true);
	backgroundColor.r = 255; backgroundColor.g = 255; backgroundColor.b = 255; ColorName(&beamAxisColor, true);
	DefaultTrackColors(trackColor);
	showWireframe = TRUE;
	showTracks = TRUE;
	showTrackSegments = FALSE;
	extendEscapeTracks = TRUE;
	keepTrackDataInMemory = TRUE;
	nVerticesTotalMax = 1000000;
	nVerticesTrackMax = 10000;
	raytraceInGraphicsWindow = FALSE;
	raytraceInLingeringWindow = TRUE;
	portNum = 1;
	nIterations = nIterations_default;
	unitEnergy = 0;
	unitDistance = 2;

	nThreads = -1;
}

void CMCNScriptDlg::DefaultTrackColors(COLORDATA *color)
{
	color[0].r = 128; color[0].g = 128; color[0].b = 128; ColorName(color, true);
	color[1].r = 255; color[1].g = 255; color[1].b = 0; ColorName(color + 1, true);
	color[2].r = 0; color[2].g = 255; color[2].b = 0; ColorName(color + 2, true);
	color[3].r = 255; color[3].g = 127; color[3].b = 0; ColorName(color + 3, true);
	color[4].r = 200; color[4].g = 50; color[4].b = 128; ColorName(color + 4, true);
	color[5].r = 255; color[5].g = 192; color[5].b = 203; ColorName(color + 5, true);
	color[6].r = 150; color[6].g = 150; color[6].b = 50; ColorName(color + 6, true);
	color[7].r = 50; color[7].g = 100; color[7].b = 255; ColorName(color + 7, true);
	color[8].r = 0; color[8].g = 255; color[8].b = 255; ColorName(color + 8, true);
	color[9].r = 200; color[9].g = 150; color[9].b = 100; ColorName(color + 9, true);
	color[10].r = 255; color[10].g = 100; color[10].b = 100; ColorName(color + 10, true);
	color[11].r = 191; color[11].g = 255; color[11].b = 0; ColorName(color + 11, true);
}

void CMCNScriptDlg::NewParameterSet()
{
	int i;
	if (!nViewParmSets) DefaultParmSet();
	viewParmSets.InsertAt(curParmSet, *((VIEWPARM *)&maximumObjectSizeRatio));
	trackSets.InsertAt(curParmSet, defaultTrackSetData);
	nViewParmSets++;

	regionData = (REGIONDATA **)realloc(regionData, nViewParmSets * sizeof(REGIONDATA *));
	if (curParmSet == nViewParmSets - 1) regionData[curParmSet] = (REGIONDATA *)malloc(nRegions * sizeof(REGIONDATA));
	else {
		for (i = nViewParmSets - 1; i > curParmSet; i--) {
			if (i == nViewParmSets - 1) regionData[i] = (REGIONDATA *)malloc(viewParmSets[i].nRegions * sizeof(REGIONDATA));
			else regionData[i] = (REGIONDATA *)realloc(regionData[i], viewParmSets[i].nRegions * sizeof(REGIONDATA));
			memcpy(regionData[i], regionData[i-1], viewParmSets[i].nRegions * sizeof(REGIONDATA));
		}
		regionData[curParmSet] = (REGIONDATA *)realloc(regionData[curParmSet], nRegions * sizeof(REGIONDATA));
	}
	REGIONDATA *r = regionData[curParmSet];
	for (i = 0; i < nRegions; i++) r[i] = regions[i];

	INT_PTR nCmdTimers = iParmSetFromCmdTimerIndex.GetSize();
	for (i = 0; i < nCmdTimers; i++) {
		if (iParmSetFromCmdTimerIndex[i] >= curParmSet) iParmSetFromCmdTimerIndex[i]++;
	}
	INT_PTR nMGEDtimers = iParmSetFromMGEDtimerIndex.GetSize();
	for (i = 0; i < nMGEDtimers; i++) {
		if (iParmSetFromMGEDtimerIndex[i] >= curParmSet) iParmSetFromMGEDtimerIndex[i]++;
	}
}

void CMCNScriptDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COLORS, m_colorsGrid);
	DDX_Control(pDX, IDC_VIEWCENTER, m_viewCenterGrid);
	DDX_Control(pDX, IDC_POINT, m_cutawayPlanePointGrid);
	DDX_Control(pDX, IDC_NORMAL, m_cutawayPlaneNormalGrid);
	DDX_Control(pDX, IDC_COORDSYS, m_coordSys);
	DDX_Control(pDX, IDC_BEAMAXISCOLORPIC, m_beamAxisColorPic);
	DDX_Control(pDX, IDC_BACKGROUNDCOLORPIC, m_backgroundColorPic);
	DDX_Control(pDX, IDC_TRACKFILTERING_TABCTRL, m_trackFilteringTabCtrl);
}
BEGIN_MESSAGE_MAP(CMCNScriptDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_LOADINPUTFILE, OnLoadInputFile)
	ON_BN_CLICKED(IDC_RELOAD, OnReloadInputFile)
	ON_EN_KILLFOCUS(IDC_INPUTFILENAME, OnInputFileName)
	ON_EN_KILLFOCUS(IDC_SUFFIX, OnSuffix)
	ON_EN_KILLFOCUS(IDC_OUTPUTFILENAME, OnOutputFileName)
	ON_BN_CLICKED(IDC_CREATELOG, OnCreateLog)
	ON_EN_KILLFOCUS(IDC_LOG_SUFFIX, OnLogSuffix)
	ON_EN_KILLFOCUS(IDC_LOGFILENAME, OnLogFileName)
	ON_BN_CLICKED(IDC_STARTMCNELECTRON, OnStartMCNelectron)
	ON_BN_CLICKED(IDC_USECUSTOMTRACKFN, OnUseCustomTrackFN)
	ON_BN_CLICKED(IDC_COPYTOCLIPBOARD, OnCopyToClipboard)
	ON_BN_CLICKED(IDC_LOADTRACKFILE, OnLoadTrackFile)
	ON_BN_CLICKED(IDC_TRACKSTATS, OnTrackStats)
	ON_EN_KILLFOCUS(IDC_TRACKFILENAME, OnTrackFileName)
	ON_BN_CLICKED(IDC_OPENINPUTFILE, OnOpenInputFile)
	ON_BN_CLICKED(IDC_OPENOUTPUTFILE, OnOpenOutputFile)
	ON_BN_CLICKED(IDC_OPENLOGFILE, OnOpenLogFile)
	ON_BN_CLICKED(IDC_OPENTRACKFILE, OnOpenTrackFile)
	ON_BN_CLICKED(IDC_OPENRTSCRIPT, OnOpenRTscript)
	ON_BN_CLICKED(IDC_MAKESCRIPT, OnMakeScript)
	ON_BN_CLICKED(IDC_SELECTFOLDER, OnSelectFolder)
	ON_EN_KILLFOCUS(IDC_MGEDCOMMANDLINE, OnMGEDCommandLine)
	ON_BN_CLICKED(IDC_SYSVARS, OnSystemVars)
	ON_CBN_SELCHANGE(IDC_PARMSET, OnParmSetSel)
	ON_BN_CLICKED(IDC_NEWPARMSET, OnNewParmSet)
	ON_BN_CLICKED(IDC_DELETEPARMSET, OnDeleteParmSet)
	ON_BN_CLICKED(IDC_REFRESH, OnRefresh)
	ON_BN_CLICKED(IDC_DEFAULTFILTERING, OnDefaultFiltering)
	ON_BN_CLICKED(IDC_EDITNAME, OnEditName)
	ON_BN_CLICKED(IDC_STARTMGED, OnStartMGED)
	ON_BN_CLICKED(IDC_BROWSE_MGED, OnBrowseMGED)
	ON_NOTIFY(NM_CLICK, IDC_COLORS, OnGridClick)
	ON_NOTIFY(GVN_BEGINLABELEDIT, IDC_COLORS, OnColorsGridStartEdit)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_COLORS, OnColorsGridEndEdit)
	ON_EN_KILLFOCUS(IDC_MAXOBJECTSIZE, OnMaximumObjectSizeRatioEdit)
	ON_EN_KILLFOCUS(IDC_SOURCESIZE, OnSourceSizeRatioEdit)
	ON_EN_KILLFOCUS(IDC_VIEWSIZE, OnViewSizeEdit)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_VIEWCENTER, OnViewCenterGridEndEdit)
	ON_EN_KILLFOCUS(IDC_AZIMUTH, OnAzimuthEdit)
	ON_EN_KILLFOCUS(IDC_ELEVATION, OnElevationEdit)
	ON_EN_KILLFOCUS(IDC_TWIST, OnTwistEdit)
	ON_NOTIFY(UDN_DELTAPOS, IDC_AZIMUTHSPIN, OnAzimuthSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_ELEVATIONSPIN, OnElevationSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_TWISTSPIN, OnTwistSpin)
	ON_EN_KILLFOCUS(IDC_PERSPECTIVE, OnPerspectiveEdit)
	ON_BN_CLICKED(IDC_USECUTAWAY, OnUseCutaway)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_POINT, OnCutawayPlanePointGridEndEdit)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_NORMAL, OnCutawayPlaneNormalGridEndEdit)
	ON_CBN_SELCHANGE(IDC_BEAMAXISCOLOR, OnBeamAxisColorSetSel)
	ON_CBN_SELCHANGE(IDC_BACKGROUNDCOLOR, OnBackgroundColorSetSel)
	ON_BN_CLICKED(IDC_TRACKCOLORS, OnShowTrackColors)
	ON_EN_KILLFOCUS(IDC_PORTNUM, OnPortNumEdit)
	ON_EN_KILLFOCUS(IDC_NITERATIONS, OnNiterationsEdit)
	ON_EN_KILLFOCUS(IDC_NVERT_TOTAL, OnNverticesTotalMaxEdit)
	ON_EN_KILLFOCUS(IDC_NVERT_TRACK, OnNverticesTrackMaxEdit)
	ON_EN_KILLFOCUS(IDC_MGEDSUFFIX, OnMGEDsuffixEdit)
	ON_BN_CLICKED(IDC_RAYTRACE_IN_GRAPHWIN, OnRaytraceInGraphicsWindow)
	ON_BN_CLICKED(IDC_WIREFRAME, OnShowWireframe)
	ON_BN_CLICKED(IDC_RAYTRACE_IN_LINGERINGWIN, OnRaytraceInLingeringWindow)
	ON_BN_CLICKED(IDC_SHOWTRACKS, OnShowTracks)
	ON_BN_CLICKED(IDC_KEEPTRACKDATAINMEMORY, OnKeepTrackDataInMemory)
	ON_BN_CLICKED(IDC_SHOWTRACKSEGMENTS, OnShowTrackSegments)
	ON_BN_CLICKED(IDC_EXTENDESCAPETRACKS, OnExtendEscapeTracks)
	ON_BN_CLICKED(IDC_OVERWRITEPROMPT, OnOverwritePrompt)
	ON_BN_CLICKED(IDC_CHECK_SYNTAX, OnCheckSyntax)
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_BN_CLICKED(IDC_HIDESHOW, OnHideShow)
	ON_BN_CLICKED(IDC_EDITCMDLINE, OnEditCmdLine)
	ON_WM_INITMENU()
	ON_COMMAND(ID_AUTOSIZEGRID, OnAutosizeGrid)
	ON_COMMAND(ID_HELP_MCNELECTRONUSERMANUAL, OnHelpMCNelectronUserManual)
	ON_COMMAND(ID_RELOAD, OnReload)
	ON_COMMAND(ID_CONSOLE_ACTIVATE, OnConsoleActivate)
	ON_COMMAND(ID_CONSOLE_CLOSE, OnConsoleClose)
	ON_WM_TIMER()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CColorsGridCtrl, CGridCtrl)
	ON_CBN_SELCHANGE(IDC_INPLACE_CONTROL, OnColorSelected)
	ON_COMMAND(ID_SHOWCOLORDIALOG, OnShowColorDialog)
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONDBLCLK()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CGridCtrlModified, CGridCtrl)
	ON_WM_KILLFOCUS()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CPicture, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CPicture, CWnd)

BEGIN_MESSAGE_MAP(CColorRect, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CColorRect, CWnd)

// Register the window class if it has not already been registered.
BOOL CColorRect::RegisterWindowClass()
{
    WNDCLASS wndcls;
    //HINSTANCE hInst = AfxGetInstanceHandle();
    HINSTANCE hInst = AfxGetResourceHandle();

    if (!(::GetClassInfo(hInst, COLORRECT_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
#ifndef _WIN32_WCE_NO_CURSOR
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
#else
        wndcls.hCursor          = 0;
#endif
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = COLORRECT_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

	return TRUE;
}

CColorRect::CColorRect()
{
    RegisterWindowClass();
}

BOOL CColorRect::Create(const RECT& rect, CWnd* pParentWnd, UINT nID, DWORD dwStyle)
{
    ASSERT(pParentWnd->GetSafeHwnd());

    if (!CWnd::Create(COLORRECT_CLASSNAME, NULL, dwStyle, rect, pParentWnd, nID))
        return FALSE;
    return TRUE;
}

void CColorRect::OnPaint()
{
	CWnd::OnPaint();
	CRect rect;
	GetClientRect(&rect);
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	CDC* pDC = GetDC();
	pDC->FillSolidRect(rect.left, rect.top, width, height, color);
	CPen pen;
	pen.CreatePen(PS_SOLID, 0, RGB(128,128,128));
	CPen *pOldPen = pDC->SelectObject(&pen);
	CPoint points[5];
	points[0] = CPoint(rect.left, rect.top);
	points[1] = CPoint(rect.left, rect.bottom - 1);
	points[2] = CPoint(rect.right - 1, rect.bottom - 1);
	points[3] = CPoint(rect.right - 1, rect.top);
	points[4] = points[0];
	pDC->Polyline(points, 5);
	pDC->SelectObject(pOldPen);
	ReleaseDC(pDC);
}

// Register the window class if it has not already been registered.
BOOL CPicture::RegisterWindowClass()
{
    WNDCLASS wndcls;
    //HINSTANCE hInst = AfxGetInstanceHandle();
    HINSTANCE hInst = AfxGetResourceHandle();

    if (!(::GetClassInfo(hInst, PICTURE_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
#ifndef _WIN32_WCE_NO_CURSOR
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
#else
        wndcls.hCursor          = 0;
#endif
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = PICTURE_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

	return TRUE;
}

CPicture::CPicture()
{
    RegisterWindowClass();
}

BOOL CPicture::Create(const RECT& rect, CWnd* pParentWnd, UINT nID, DWORD dwStyle)
{
    ASSERT(pParentWnd->GetSafeHwnd());

    if (!CWnd::Create(PICTURE_CLASSNAME, NULL, dwStyle, rect, pParentWnd, nID))
        return FALSE;
    return TRUE;
}

void CPicture::OnPaint()
{
	CWnd::OnPaint();
	CRect rect;
	GetClientRect(&rect);
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	double axisLength;
	if (height < width) axisLength = 0.45 * height;
	else axisLength = 0.45 * width;
	double coord[3][3];
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	Projection(p->azimuth, p->elevation, p->twist, axisLength, coord);
	int centerX = (rect.left + rect.right) / 2;
	int centerY = (rect.top + rect.bottom) / 2;
	COLORREF color[3];
	color[0] = RGB(0,0,0); // black
	color[1] = RGB(255,0,0); // red
	color[2] = RGB(0,0,255); // blue
	CDC* pDC = GetDC();
	int i, j[3], k;
	if (coord[0][2] <= coord[1][2] && coord[0][2] <= coord[2][2]) {
		j[0] = 0;
		if (coord[1][2] <= coord[2][2]) {
			j[1] = 1;
			j[2] = 2;
		}
		else {
			j[1] = 2;
			j[2] = 1;
		}
	}
	else if (coord[1][2] <= coord[0][2] && coord[1][2] <= coord[2][2]) {
		j[0] = 1;
		if (coord[0][2] <= coord[2][2]) {
			j[1] = 0;
			j[2] = 2;
		}
		else {
			j[1] = 2;
			j[2] = 0;
		}
	}
	else {
		j[0] = 2;
		if (coord[0][2] <= coord[1][2]) {
			j[1] = 0;
			j[2] = 1;
		}
		else {
			j[1] = 1;
			j[2] = 0;
		}
	}
	CPen pen[3], *pOldPen;
	pDC->SetTextAlign(pDC->GetTextAlign()|TA_CENTER);
	UINT nFlags = pDC->GetTextAlign();
	CString axisTitle_Zpositive[3] = {"X", "Y", "Z"};
	CString axisTitle_Znegative[3] = {"X'", "Y'", "Z'"};
	CString *axisTitle;
	pDC->SetBkColor(GetSysColor(COLOR_3DFACE));
	for (k = 0; k < 3; k++) {
		i = j[k];
		pen[i].CreatePen(PS_SOLID, 0, color[i]);
		if (!i) pOldPen = pDC->SelectObject(pen + i);
		else pDC->SelectObject(pen + i);
		pDC->SetTextColor(color[i]);
		pDC->MoveTo(centerX, centerY);
		pDC->LineTo((int)(centerX + coord[i][0] + 0.5), (int)(centerY - coord[i][1] + 0.5));
		if (coord[i][2] > -1e-8) axisTitle = axisTitle_Zpositive + i;
		else axisTitle = axisTitle_Znegative + i;
		if (coord[i][1] > 1e-8) pDC->SetTextAlign(nFlags|TA_BOTTOM);
		else pDC->SetTextAlign(nFlags|TA_TOP);
		pDC->TextOut((int)(centerX + coord[i][0] + 0.5), (int)(centerY - coord[i][1] + 0.5), *axisTitle);
	}
	pDC->SelectObject(pOldPen);
	ReleaseDC(pDC);
}

void CColorsGridCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CGridCtrl::OnLButtonDown(nFlags, point);
}

void CColorsGridCtrl::OnColorSelected()
{
	CCellID cellID =  GetFocusCell();
	CGridCellCombo *pCell = (CGridCellCombo *)GetCell(cellID.row, cellID.col);
	int i = cellID.row - 1;
	int j = pCell->GetCurSel();
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	if (j != p->prevSel) p->changed = true;
	if (j == NSTDCOLORS - 1) {
		currentRegionNum = i;
		PostMessage(WM_COMMAND, ID_SHOWCOLORDIALOG);
		return;
	}
	p->UpdateCellColor(i, j);
	p->UpdateCellRGB(i);
	Invalidate();
	UpdateWindow();
}

void CColorsGridCtrl::OnShowColorDialog()
{
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	int i = currentRegionNum;
	COLORREF color = p->GetRegionColor(i);
	CColorDialog colorDlg(color, CC_FULLOPEN);
	if (colorDlg.DoModal() == IDOK) {
		color = colorDlg.GetColor();
		if (color != p->prevColor) {
			p->changed = true;
			p->SetRegionColor(i, color);
			p->UpdateRegionColor(i);
			p->UpdateCellRGB(i);
			Invalidate();
			UpdateWindow();
		}
	}
}

void CColorsGridCtrl::OnKillFocus(CWnd *pNewWnd)
{
	CCellID cellID = GetFocusCell();
	CGridCtrl::OnKillFocus(pNewWnd);
	CGridCellCombo* pCell = (CGridCellCombo*)GetCell(cellID.row, 3);
	if (pCell && pCell->IsEditing()) return;
    SetSelectedRange(-1,-1,-1,-1);
	SetFocusCell(-1,-1);
}

void CGridCtrlModified::OnKillFocus(CWnd *pNewWnd)
{
	CGridCtrl::OnKillFocus(pNewWnd);
	SetSelectedRange(-1,-1,-1,-1);
	SetFocusCell(-1,-1);
}

void CMCNScriptDlg::SetupVectorGrid(CGridCtrlModified *pGrid)
{
	CRect rect;
	pGrid->GetClientRect(&rect);
	pGrid->SetColumnResize(FALSE);
//	pGrid->nAutoCols = 1;
//	pGrid->autoCol.SetSize(pGrid->nAutoCols);
//	pGrid->autoCol[0] = 1;
	pGrid->SetFont(GetFont());
	pGrid->m_disableCut = TRUE;
	pGrid->SetRowCount(2);
	pGrid->SetColumnCount(3);
	pGrid->isColReadOnly.SetSize(3);
	int i;
	for (i = 0; i < 3; i++) pGrid->isColReadOnly[i] = FALSE;
	pGrid->SetFixedRowCount(1);
	pGrid->SetFixedColumnCount(0);
	int height = rect.bottom - rect.top;
	pGrid->SetRowHeight(0, height / 2);
	pGrid->SetRowHeight(1, height - height / 2);
	double colWidth_dbl = (rect.right - rect.left) / 3.;
	int colWidth0 = (int)(colWidth_dbl + 0.5);
	pGrid->SetColumnWidth(0, colWidth0);
	int colWidth1 = (int)(2. * colWidth_dbl + 0.5) - colWidth0;
	pGrid->SetColumnWidth(1, colWidth1);
	pGrid->SetColumnWidth(2, rect.right - rect.left - colWidth0 - colWidth1);
	pGrid->SetItemFormat(0, 0, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemFormat(0, 1, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemFormat(0, 2, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemFormat(1, 0, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemFormat(1, 1, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemFormat(1, 2, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	pGrid->SetItemText(0, 0, "X");
	pGrid->SetItemText(0, 1, "Y");
	pGrid->SetItemText(0, 2, "Z");
}

BOOL CMCNScriptDlg::PreTranslateMessage(MSG *pMsg) 
{
	m_toolTip.RelayEvent(pMsg);
	if (pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST && m_hAccel != NULL) {
			if (::TranslateAccelerator(m_hWnd, m_hAccel, pMsg)) return TRUE;
	}
	return CDialogEx::PreTranslateMessage(pMsg);
}

BOOL CMCNScriptDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());
	m_hAccel = ::LoadAccelerators(NULL, MAKEINTRESOURCE(IDR_MAINFRAME));
	m_hAccel_trackStats = ::LoadAccelerators(NULL, MAKEINTRESOURCE(IDR_TRACKSTATS));

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	VERIFY(m_inputFileName.SubclassDlgItem(IDC_INPUTFILENAME, this));
	VERIFY(m_suffix.SubclassDlgItem(IDC_SUFFIX, this));
	VERIFY(m_outputFileName.SubclassDlgItem(IDC_OUTPUTFILENAME, this));
	VERIFY(m_createLog.SubclassDlgItem(IDC_CREATELOG, this));
	VERIFY(m_logSuffix.SubclassDlgItem(IDC_LOG_SUFFIX, this));
	VERIFY(m_logFileName.SubclassDlgItem(IDC_LOGFILENAME, this));
	VERIFY(m_cmdLine.SubclassDlgItem(IDC_CMDLINE, this));
	m_cmdLine.disable = TRUE;
	VERIFY(m_trackFileName.SubclassDlgItem(IDC_TRACKFILENAME, this));
	VERIFY(m_outputScriptName.SubclassDlgItem(IDC_OUTPUTSCRIPTNAME, this));
	VERIFY(m_MGEDcommandLine.SubclassDlgItem(IDC_MGEDCOMMANDLINE, this));
	m_outputScriptName.disable = TRUE;
	VERIFY(m_parmSetList.SubclassDlgItem(IDC_PARMSET, this));
	VERIFY(m_deleteParmSet.SubclassDlgItem(IDC_DELETEPARMSET, this));
	VERIFY(m_nTracks.SubclassDlgItem(IDC_NTRACKS, this));
	VERIFY(m_viewSizeEdit.SubclassDlgItem(IDC_VIEWSIZE, this));
	VERIFY(m_maximumObjectSizeRatioEdit.SubclassDlgItem(IDC_MAXOBJECTSIZE, this));
	VERIFY(m_sourceSizeRatioEdit.SubclassDlgItem(IDC_SOURCESIZE, this));
	VERIFY(m_azimuthEdit.SubclassDlgItem(IDC_AZIMUTH, this));
	VERIFY(m_elevationEdit.SubclassDlgItem(IDC_ELEVATION, this));
	VERIFY(m_twistEdit.SubclassDlgItem(IDC_TWIST, this));
	VERIFY(m_perspectiveEdit.SubclassDlgItem(IDC_PERSPECTIVE, this));
	VERIFY(m_cutawayPlaneCheck.SubclassDlgItem(IDC_USECUTAWAY, this));
	VERIFY(m_portNumEdit.SubclassDlgItem(IDC_PORTNUM, this));
	VERIFY(m_nIterationsEdit.SubclassDlgItem(IDC_NITERATIONS, this));
	VERIFY(m_nVerticesTotalMaxEdit.SubclassDlgItem(IDC_NVERT_TOTAL, this));
	VERIFY(m_nVerticesTrackMaxEdit.SubclassDlgItem(IDC_NVERT_TRACK, this));
	VERIFY(m_MGEDsuffixEdit.SubclassDlgItem(IDC_MGEDSUFFIX, this));
	VERIFY(m_beamAxisColor.SubclassDlgItem(IDC_BEAMAXISCOLOR, this));
	VERIFY(m_backgroundColor.SubclassDlgItem(IDC_BACKGROUNDCOLOR, this));
	VERIFY(m_raytraceInGraphicsWindowCheck.SubclassDlgItem(IDC_RAYTRACE_IN_GRAPHWIN, this));
	VERIFY(m_showWireframeCheck.SubclassDlgItem(IDC_WIREFRAME, this));
	VERIFY(m_raytraceInLingeringWindowCheck.SubclassDlgItem(IDC_RAYTRACE_IN_LINGERINGWIN, this));
	VERIFY(m_showTracksCheck.SubclassDlgItem(IDC_SHOWTRACKS, this));
	VERIFY(m_keepTrackDataInMemoryCheck.SubclassDlgItem(IDC_KEEPTRACKDATAINMEMORY, this));
	VERIFY(m_showTrackSegmentsCheck.SubclassDlgItem(IDC_SHOWTRACKSEGMENTS, this));
	VERIFY(m_extendEscapeTracksCheck.SubclassDlgItem(IDC_EXTENDESCAPETRACKS, this));
	VERIFY(m_overwritePromptCheck.SubclassDlgItem(IDC_OVERWRITEPROMPT, this));
	VERIFY(m_useCustomTrackFN.SubclassDlgItem(IDC_USECUSTOMTRACKFN, this));
	VERIFY(m_copyToClipboard.SubclassDlgItem(IDC_COPYTOCLIPBOARD, this));
	VERIFY(m_loadTrackFile.SubclassDlgItem(IDC_LOADTRACKFILE, this));
	VERIFY(m_startMCNelectron.SubclassDlgItem(IDC_STARTMCNELECTRON, this));
	m_startMCNelectron.pTimerID = &cmdTimerID;
	VERIFY(m_checkSyntax_MCNScript.SubclassDlgItem(IDC_CHECK_SYNTAX, this));
	VERIFY(m_makeRTscript.SubclassDlgItem(IDC_MAKESCRIPT, this));
	VERIFY(m_startMGED.SubclassDlgItem(IDC_STARTMGED, this));
	m_startMGED.pTimerID = &mgedTimerID;
	VERIFY(m_refresh.SubclassDlgItem(IDC_REFRESH, this));
	VERIFY(m_MGEDcmdLineCapt.SubclassDlgItem(IDC_MGEDCMDLINE_CAPT, this));

	UpdateData();

	if (m_toolTip.Create(this)) {
		m_toolTip.AddTool(&m_parmSetList, _T("The name of the current parameter set"));
		m_toolTip.AddTool(&m_deleteParmSet, _T("Delete the current parameter set"));
		m_toolTip.AddTool(&m_inputFileName, _T("The name of the MCNelectron input file"));
		m_toolTip.AddTool(&m_outputFileName, _T("The name of the MCNelectron output file"));
		m_toolTip.AddTool(&m_logFileName, _T("The name of the MCNelectron log file with all input directives (including the ones generated by executing the user code embedded in the MCNelectron input file)"));
		m_toolTip.AddTool(&m_cmdLine, _T("The command line used to start MCNelectron"));
		m_toolTip.AddTool(&m_suffix, _T("The file name suffix cannot have spaces. The two-character string '%d', if present, will be replaced by the sequence number of the parameter set. Clear this box to enable customizing the MCNelectron output file name."));
		m_toolTip.AddTool(&m_logSuffix, _T("The file name suffix cannot have spaces. The two-character string '%d', if present, will be replaced by the sequence number of the parameter set. Clear this box to enable customizing the MCNelectron log file name."));
		m_toolTip.AddTool(GetDlgItem(IDC_TRACKSTATS), _T("Click this button to open a dialog window with the track statistics and the track filtering controls"));
		m_toolTip.AddTool(&m_trackFileName, _T("The name of the file with track data created by MCNelectron"));
		m_toolTip.AddTool(&m_makeRTscript, _T("Click this button to create an .RT script file for subsequent loading into MGED (a part of the free software package BRL-CAD)"));
		m_toolTip.AddTool(&m_startMGED, _T("After starting MGED, the created script can be executed either by loading the corresponding .RT file (MGED menu command \"File / Load Script...\" or the MGED command \"source <file_name>\") or by pasting it into the MGED Command Window (keyboard shortcut Ctrl+V, followed by the key \"Enter\")"));
		m_toolTip.AddTool(&m_MGEDsuffixEdit, _T("The file name suffix cannot have spaces. The two-character string '%d', if present, will be replaced by the sequence number of the parameter set."));
		m_toolTip.AddTool(&m_copyToClipboard, _T("If this checkbox is checked, then the created RT script will be automatically copied to the clipboard"));
		m_toolTip.AddTool(&m_useCustomTrackFN, _T("Check this box to enable customizing the name of the track data file"));
		m_toolTip.AddTool(&m_outputScriptName, _T("The name of the file with the definition of the simulation geometry and the drawing commands used for rendering the view (this file must be loaded in MGED)"));
		m_toolTip.AddTool(&m_MGEDcommandLine, _T("The name of the MGED executable file (with optional command-line arguments)"));
		m_toolTip.AddTool(GetDlgItem(IDC_RESET), _T("Reset all graphical parameters of the current parameter set to their default values"));
		m_toolTip.AddTool(&m_perspectiveEdit, _T("The specified perspective angle will not be applied if the RT script is loaded using the MGED menu command \"File / Load Script...\". It will be applied if the RT script is executed using the MGED command \"source <file_name>\", or by pasting the script into the command window."));
		m_toolTip.AddTool(&m_coordSys, _T("The prime symbol in the axis title indicates that the positive half-axis is below the projection plane"));
		m_toolTip.AddTool(&m_viewSizeEdit, _T("The initial view size"));
		m_toolTip.AddTool(&m_maximumObjectSizeRatioEdit, _T("The ratio of the \"world size\" to the initial view size"));
		m_toolTip.AddTool(&m_sourceSizeRatioEdit, _T("The ratio of the diameter of the sphere representing the source to the initial view size (if this parameter is zero, the source wil not be displayed)"));
		m_toolTip.AddTool(&m_beamAxisColorPic, _T("The color of the line representing the symmetry axis of the angular distribution of the source particles"));
		m_toolTip.AddTool(&m_beamAxisColor, _T("The color of the line representing the symmetry axis of the angular distribution of the source particles"));
		m_toolTip.AddTool(&m_raytraceInGraphicsWindowCheck, _T("Use this option to create a raytraced image of the 3D objects in the MGED Graphics Window (optionally overlayed with the wireframe view). This option works only after activating the framebuffer (MGED menu command \"Modes / Framebuffer active\")"));
		m_toolTip.AddTool(&m_nVerticesTotalMaxEdit, _T("The maximum total number of vertices in all tracks shown (if the actual total number of vertices is greater than this number, then some of the last tracks in the current set of tracks will not be shown in the MGED Graphics Window). Clear this box to remove the limitation."));
		m_toolTip.AddTool(&m_nVerticesTrackMaxEdit, _T("The maximum number of vertices shown in a single track (if the actual number of vertices in a track is greater than this number, then some of the vertices will be skipped when drawing the track in the MGED Graphics Window). Clear this box to remove the limitation."));
		m_toolTip.AddTool(&m_portNumEdit, _T("The port number of the MGED internal framebuffer server associated with a given lingering window"));
		m_toolTip.AddTool(&m_nIterationsEdit, _T("The maximum number of iterations for adjustment of the positions of the flat faces of truncated cylinders and cones, and of the clipping planes of open polyhedra. If zero is entered here, then no adjustments will be made. This option affects only the wireframe view (not the raytraced image)."));
		m_toolTip.AddTool(&m_showTracksCheck, _T("Use this option to draw piecewice-linear curves representing the particle tracks or track segments. Only the tracks (or track segments) satisfying the indicated filtering criteria will be shown."));
		m_toolTip.AddTool(&m_showTrackSegmentsCheck, _T("If this checkbox is checked, then the track segments defined in the \"Track statistics\" dialog window will be shown instead of the complete tracks"));
		m_toolTip.AddTool(&m_extendEscapeTracksCheck, _T("If this checkbox is checked, then the last straight segment of the escape tracks will be extended beyond the clipping cube. Otherwise, the final point of the track will coincide with the escape point."));
		m_toolTip.AddTool(&m_showTracksCheck, _T("Use this option to draw piecewice-linear curves representing the particle tracks or track segments. Only the tracks (or track segments) satisfying the indicated filtering criteria will be shown."));
		m_toolTip.AddTool(&m_keepTrackDataInMemoryCheck, _T("If this checkbox is checked, then the entire dataset stored in the track data file will be kept in memory for subsequent use. Otherwise, the track data will be re-loaded from file every time when they are needed."));
		m_toolTip.AddTool(&m_refresh, _T("Recalculate the number of tracks or track segments satisfying the indicated filtering criteria"));
		m_toolTip.AddTool(&m_checkSyntax_MCNScript, _T("Check to enable reading the geometry information from the MCNelectron input file and creating the RT script (otherwise, the input file will be processed only by the MCNelectron executable after clicking \"Start MCNelectron\")."));
		m_toolTip.AddTool(&m_createLog, _T("If this checkbox is checked, MCNelectron will create the log file with the complete set of input directives (including the procedurally generated ones). The log file is functionally equivalent to the MCNelectron input file that was used to create it."));
		m_toolTip.AddTool(GetDlgItem(IDC_SYSVARS), _T("Click this button to modify the values of the MCNEcode system variables. The non-zero system variables will be specified on the MCNelectron command line. The system variables can be referenced in the user's programs embedded in the MCNelectron input file."));
		m_toolTip.SetDelayTime(TTDT_AUTOPOP, 20000);
		m_toolTip.Activate(TRUE);
	}

	CRect rect;
	m_colorsGrid.GetClientRect(&rect);
	colorsRowHeight = GetSystemMetrics(SM_CYVSCROLL) + 2;
	int colorsRowsVisible = rect.bottom / colorsRowHeight - 1;
	colorsHeaderHeight = rect.bottom - colorsRowsVisible * colorsRowHeight;
	int additionalHeight = (colorsHeaderHeight - colorsRowHeight) / colorsRowsVisible;
	colorsRowHeight += additionalHeight;
	colorsHeaderHeight -= colorsRowsVisible * additionalHeight;
	m_colorsGrid.SetColumnResize(FALSE);
	m_colorsGrid.nAutoCols = 1;
	m_colorsGrid.autoCol.SetSize(m_colorsGrid.nAutoCols);
	m_colorsGrid.autoCol[0] = 1;
	m_colorsGrid.SetFont(GetFont());
	m_colorsGrid.m_disableCut = TRUE;

	int i;
	for (i = 0; i <= NSTDCOLORS; i++) m_beamAxisColor.AddString(stdColorName[i]);
	m_beamAxisColor.AddString("None");
	for (i = 0; i <= NSTDCOLORS; i++) m_backgroundColor.AddString(stdColorName[i]);
	SetupVectorGrid(&m_viewCenterGrid);
	SetupVectorGrid(&m_cutawayPlanePointGrid);
	SetupVectorGrid(&m_cutawayPlaneNormalGrid);

	if (overwritePrompt) m_overwritePromptCheck.SetCheck(1);

	m_colorsGrid.SetRowCount(1);
	m_colorsGrid.SetColumnCount(10);
	m_colorsGrid.isColReadOnly.SetSize(10);
	for (i = 0; i < 3; i++) m_colorsGrid.isColReadOnly[i] = TRUE;
	for (i = 3; i < 9; i++) m_colorsGrid.isColReadOnly[i] = FALSE;
	m_colorsGrid.isColReadOnly[9] = TRUE;
	m_colorsGrid.SetFixedRowCount(1);
	m_colorsGrid.SetFixedColumnCount(1);
	m_colorsGrid.SetRowHeight(0, colorsHeaderHeight);
	m_colorsGrid.SetItemText(0, 0, "No.");
	m_colorsGrid.SetItemText(0, 1, "Material");
	m_colorsGrid.SetItemText(0, 2, "Color");
	m_colorsGrid.SetItemText(0, 3, "Color name");
	m_colorsGrid.SetItemText(0, 4, "R");
	m_colorsGrid.SetItemText(0, 5, "G");
	m_colorsGrid.SetItemText(0, 6, "B");
	m_colorsGrid.SetItemText(0, 7, "Transp.");
	m_colorsGrid.SetItemText(0, 8, "Refl.");
	m_colorsGrid.SetItemText(0, 9, "Show");

	f_par = NULL;

	CTabFilteringByTransportParameters *tab1 = (CTabFilteringByTransportParameters *)(m_trackFilteringTabCtrl.m_tabPages[0]);
	tab1->parent = this;
	tab1->parent2 = this;
	tab1->m_hAccel = m_hAccel_trackStats;
	tab1->pTrackFilter = &trackFilter;
	pTrackFilterGrid = &(tab1->m_trackFilterGrid);

	CTabFilteringByParticleType *tab2 = (CTabFilteringByParticleType *)(m_trackFilteringTabCtrl.m_tabPages[1]);
	tab2->parent = this;
	tab2->parent2 = this;
	tab2->pTrackFilter = &trackFilter;
	pOriginTypeCheckbox = tab2->m_originType;
	pGenMinEdit = &(tab2->m_genMin);
	pGenMaxEdit = &(tab2->m_genMax);

	m_trackFilteringTabCtrl.InsertItem(0, _T("Filtering by transport parameters"));
	m_trackFilteringTabCtrl.InsertItem(1, _T("Filtering by particle type"));
	m_trackFilteringTabCtrl.m_tabCurrent = 0;
	m_trackFilteringTabCtrl.Init();

	pSystemVarsDlg = new CSystemVarsDlg(this);
	pSystemVarsDlg->Create(IDD_SYSTEMVARS);
	pSystemVarsDlg->ShowWindow(SW_HIDE);
	pSystemVarsGrid = &(((CSystemVarsDlg *)pSystemVarsDlg)->m_systemVarsGrid);

	pTrackColorsDlg = new CTrackColorsDlg(this);
	pTrackColorsDlg->Create(IDD_TRACKCOLORS);
	pTrackColorsDlg->ShowWindow(SW_HIDE);
	pTrackColorsGrid = &(((CTrackColorsDlg *)pTrackColorsDlg)->m_trackColorsGrid);
	pTrackColorsDlg->pTrackFilter = &trackFilter;
	pTrackColorsDlg->pOriginTypeCheckbox = pOriginTypeCheckbox;

	OnOpen();

	CRect dlgRect, btnRect;
	GetWindowRect(&dlgRect);
	GetDlgItem(IDC_HIDESHOW)->GetWindowRect(&btnRect);
	initialWidth = dlgRect.right - dlgRect.left;
	initialHeight = dlgRect.bottom - dlgRect.top;
	clippedHeight = btnRect.bottom + 20 - dlgRect.top;

	graphicalParametersShown = true;

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CMCNScriptDlg::FreeAll()
{
	int i, j;
	for (i = 0; i < nViewParmSets; i++) {
		free(regionData[i]);
		if (viewParmSets[i].cmdTimerID) KillTimer(viewParmSets[i].cmdTimerID);
		if (viewParmSets[i].mgedTimerID) KillTimer(viewParmSets[i].mgedTimerID);
		if (trackSets[i].nTracks) {
			for (j = 0; j < trackSets[i].nTracks; j++) {
				delete trackSets[i].trackData[j].coord;
				delete trackSets[i].trackData[j].energy;
			}
			free(trackSets[i].trackData);
		}
	}
	free(regionData);
	viewParmSets.SetSize(0);
	trackSets.SetSize(0);
	nViewParmSets = 0;
	m_parmSetList.ResetContent();
	iParmSetFromCmdTimerIndex.SetSize(0);
	iParmSetFromMGEDtimerIndex.SetSize(0);
	timerCount = 0;
}

BOOL CMCNScriptDlg::OnOpen()
{
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());
	SetWindowText((CString)app->m_pszAppName + " [" + app->parFileName + "]");
	char line[1000], *ptr, tmp[1002];
	CString msg;
	bool err = false;
	int i = 0;
	regionData = (REGIONDATA **)malloc(0);
	if (f_par) fclose(f_par);
	f_par = fopen(app->parFileName, "r+t");
	if (!f_par) {
		if (parFileSpecified) MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for reading."), app->parFileName), msg), NULL, MB_OK | MB_ICONERROR);
		goto PARM_ERR;
	}
	int num;
	double numDbl;
	char outputString[100];
	fgets(line, 100, f_par); line[100] = '\0';
	ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nParmSets")) goto PARM_ERR;
	ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 10000); if (outputString[0] == '\0') goto PARM_ERR;
	nViewParmSets = num;
	viewParmSets.SetSize(nViewParmSets);
	trackSets.SetSize(nViewParmSets);
	regionData = (REGIONDATA **)malloc(nViewParmSets * sizeof(REGIONDATA *));
	fgets(line, 100, f_par); line[100] = '\0';
	ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "curParmSet")) goto PARM_ERR;
	ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, nViewParmSets - 1); if (outputString[0] == '\0') goto PARM_ERR;
	curParmSet = num;
	int format = 0;
	DWORD rgb = GetSysColor(COLOR_3DFACE);
	unsigned char r_std = GetRValue(rgb);
	unsigned char g_std = GetGValue(rgb);
	unsigned char b_std = GetBValue(rgb);
	for (i = 0; i < nViewParmSets; i++) {
		fgets(line, 2 + PARMSETNAMELEN, f_par); line[PARMSETNAMELEN] = '\0'; viewParmSets[i].parmSetName[0] = '\0'; ptr = strtok(line, "\n");
		if (ptr) strcpy(viewParmSets[i].parmSetName, ptr);
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "InputFileFolder")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].inputFileFolder[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].inputFileFolder, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "InputFN")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].fn[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].fn, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "cmdFolderOption")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 3); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cmdFolderOption = num;
		if (num == 3) {
			fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
			ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "cmdFolder")) goto PARM_ERR;
			ptr = strtok(NULL, "\n"); viewParmSets[i].cmdFolder[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].cmdFolder, ptr + strspn(ptr, " =\t"), FNLEN);
		}
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "MCNelectronFolder")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].MCNelectronFolder[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].MCNelectronFolder, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "MCNelectronExeName")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].MCNelectronExeName[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].MCNelectronExeName, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nThreads")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, -1, MAXTHREADS); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].nThreads = num;
		fgets(line, 11 + SUFFIXLEN, f_par); line[9+SUFFIXLEN] = '\0'; if (line[8+SUFFIXLEN] == '%') line[8+SUFFIXLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "suffix")) goto PARM_ERR;
		ptr = strtok(NULL, " =\n"); viewParmSets[i].suffix[0] = '\0'; if (ptr) strcpy(viewParmSets[i].suffix, ptr);
		if (viewParmSets[i].suffix[0] == '\0') {
			fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
			ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "OutputFileName")) goto PARM_ERR;
			ptr = strtok(NULL, "\n"); viewParmSets[i].outputFileName[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].outputFileName, ptr + strspn(ptr, " =\t"), FNLEN);
		}
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "AltCSinfoFN")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].altCSinfoFileName[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].altCSinfoFileName, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "ENDFfolder")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].ENDFfolder[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].ENDFfolder, ptr + strspn(ptr, " =\t"), FNLEN);
		int j;
		fgets(line, 100 + 2*FNLEN, f_par); line[100+2*FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		if (format < 105 && !strcmpi(ptr, "createLog")) format = 105;
		if (format >= 105) {
			if (strcmpi(ptr, "createLog")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].createLog = num;
			fgets(line, 14 + SUFFIXLEN, f_par); line[12+SUFFIXLEN] = '\0'; if (line[11+SUFFIXLEN] == '%') line[11+SUFFIXLEN] = '\0';
			ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "logSuffix")) goto PARM_ERR;
			ptr = strtok(NULL, " =\n"); viewParmSets[i].logSuffix[0] = '\0'; if (ptr) strcpy(viewParmSets[i].logSuffix, ptr);
			if (viewParmSets[i].logSuffix[0] == '\0') {
				fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
				ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "LogFileName")) goto PARM_ERR;
				ptr = strtok(NULL, "\n"); viewParmSets[i].logFileName[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].logFileName, ptr + strspn(ptr, " =\t"), FNLEN);
			}
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "checkSyntax_MCNelectron")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].checkSyntax_MCNelectron = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "quitAfterChecking")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].quitAfterChecking = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "checkSyntax_MCNScript")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].checkSyntax_MCNScript = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "MCNP_style_cmdLine")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].MCNP_style_cmdLine = num;
			for (j = 0; j < 10; j++) {
				sprintf(tmp, "@%d", j);
				fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, tmp)) goto PARM_ERR;
				ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -DBL_MAX, DBL_MAX); if (outputString[0] == '\0') goto PARM_ERR;
				viewParmSets[i].systemVars[j] = numDbl;
			}
			fgets(line, 100 + 2*FNLEN, f_par); line[100+2*FNLEN] = '\0';
			ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		}
		else {
			viewParmSets[i].createLog = createLog_default;
			strcpy(viewParmSets[i].logSuffix, logSuffix_default);
			viewParmSets[i].checkSyntax_MCNelectron = checkSyntax_MCNelectron_default;
			viewParmSets[i].quitAfterChecking = quitAfterChecking_default;
			viewParmSets[i].checkSyntax_MCNScript = checkSyntax_MCNScript_default;
			viewParmSets[i].MCNP_style_cmdLine = MCNP_style_cmdLine_default;
			memcpy(viewParmSets[i].systemVars, systemVars_default, 10 * sizeof(double));
		}
		if (strcmpi(ptr, "additionalArgs")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].additionalArgs[2*FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].additionalArgs, ptr + strspn(ptr, " =\t"), 2 * FNLEN);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "useCustomTrackFN")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].useCustomTrackFN = (num != 0);
		if (num) {
			fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
			ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "TrackFN")) goto PARM_ERR;
			ptr = strtok(NULL, "\n"); viewParmSets[i].trackFN[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].trackFN, ptr + strspn(ptr, " =\t"), FNLEN);
		}
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "ScriptFolder")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].scriptFolder[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].scriptFolder, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100 + FNLEN, f_par); line[100+FNLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "MGEDcmdLine")) goto PARM_ERR;
		ptr = strtok(NULL, "\n"); viewParmSets[i].MGEDcmdLine[FNLEN] = '\0'; if (ptr) strncpy(viewParmSets[i].MGEDcmdLine, ptr + strspn(ptr, " =\t"), FNLEN);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "copyToClipboard")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].copyToClipboard = (num != 0);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "maximumObjectSizeRatio")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 3.5, 1000.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].maximumObjectSizeRatio = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "sourceSizeRatio")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 0.1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].sourceSizeRatio = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "viewSize")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 5e-12, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].viewSize = numDbl;
		fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "viewCenter")) goto PARM_ERR;
		ptr = strtok(NULL, " =(,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].viewCenter.x = numDbl;
		ptr = strtok(NULL, " ,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].viewCenter.y = numDbl;
		ptr = strtok(NULL, " ,)\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].viewCenter.z = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "azimuth")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -360., 360.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].azimuth = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "elevation")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -90., 90.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].elevation = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "twist")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -360., 360.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].twist = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "perspective")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 90.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].perspective = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "useCutawayPlane")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].useCutawayPlane = (num != 0);
		fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "cutawayPlanePoint")) goto PARM_ERR;
		ptr = strtok(NULL, " =(,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlanePoint.x = numDbl;
		ptr = strtok(NULL, " ,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlanePoint.y = numDbl;
		ptr = strtok(NULL, " ,)\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlanePoint.z = numDbl;
		fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "cutawayPlaneNormal")) goto PARM_ERR;
		ptr = strtok(NULL, " =(,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlaneNormal.x = numDbl;
		ptr = strtok(NULL, " ,\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlaneNormal.y = numDbl;
		ptr = strtok(NULL, " ,)\t\n"); numDbl = CheckDouble(ptr, outputString, -1e300, 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].cutawayPlaneNormal.z = numDbl;
		if (!viewParmSets[i].cutawayPlaneNormal.x && !viewParmSets[i].cutawayPlaneNormal.y && !viewParmSets[i].cutawayPlaneNormal.z) goto PARM_ERR;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "portNum")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 10000); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].portNum = num;
		fgets(line, 100, f_par); line[100] = '\0';
		ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		if (format < 103 && !strcmpi(ptr, "nIterations")) format = 103;
		if (format >= 103) {
			if (strcmpi(ptr, "nIterations")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 50); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].nIterations = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		}
		else viewParmSets[i].nIterations = nIterations_default;
		if (strcmpi(ptr, "unitEnergy")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].unitEnergy = num;
		fgets(line, 100, f_par); line[100] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "unitDistance")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 3); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].unitDistance = num;
		fgets(line, 15 + SUFFIXLEN, f_par); line[13+SUFFIXLEN] = '\0'; if (line[12+SUFFIXLEN] == '%') line[12+SUFFIXLEN] = '\0';
		ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "MGEDsuffix")) goto PARM_ERR;
		ptr = strtok(NULL, " =\n"); viewParmSets[i].MGEDsuffix[0] = '\0'; if (ptr) strcpy(viewParmSets[i].MGEDsuffix, ptr);

		viewParmSets[i].trackFilter = defaultTrackFilter;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		if (format < 102 && !strcmpi(ptr, "historyNumStart")) format = 102;
		if (format >= 102) {
			if (strcmpi(ptr, "historyNumStart")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackFilter.historyNo[0] = num - 1;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "historyNumEnd")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackFilter.historyNo[1] = num - 1;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr) goto PARM_ERR;
		}
		if (strcmpi(ptr, "trackNumStart")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.trackNo[0] = num - 1;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackNumEnd")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.trackNo[1] = num - 1;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nVerticesMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.nVert[0] = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nVerticesMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.nVert[1] = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackEstartMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Estart[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackEstartMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Estart[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackEfinalMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Efinal[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackEfinalMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Efinal[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackElossMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Eloss[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackElossMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.Eloss[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackRelElossMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.relEloss[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackRelElossMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.relEloss[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackDisplacementMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.displ[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackDisplacementMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.displ[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackPeakDisplacementMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.peakDispl[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackPeakDisplacementMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.peakDispl[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackPathMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.path[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackPathMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilter.path[1] = numDbl;
		if (format >= 102) {
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "genMin")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackFilter.gen[0] = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "genMax")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, -1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackFilter.gen[1] = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "originTypeMask")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackFilter.originTypeMask = num;
		}

		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentCollMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.iVert[0] = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentCollMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, -1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.iVert[1] = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentEmin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.E[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentEmax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.E[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentElossMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.Eloss[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentElossMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.Eloss[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentRelElossMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.relEloss[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentRelElossMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.relEloss[1] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentPathMin")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.path[0] = numDbl;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "segmentPathMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, -1., 1e300); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackSegmentSelection.path[1] = numDbl;

		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackFilteringBeforeSegmentSelection")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].trackFilteringBeforeSegmentSelection = (num != 0);

		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "raytraceInGraphicsWindow")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].raytraceInGraphicsWindow = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "showWireframe")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].showWireframe = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "raytraceInLingeringWindow")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].raytraceInLingeringWindow = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "showTracks")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].showTracks = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "showTrackSegments")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].showTrackSegments = (num != 0);
		if (format >= 102) {
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "extendEscapeTracks")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].extendEscapeTracks = (num != 0);
		}
		else viewParmSets[i].extendEscapeTracks = true;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "keepTrackDataInMemory")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].keepTrackDataInMemory = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nVerticesTotalMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, -1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		if (num >= 0 && num < 10000) num = -1;
		viewParmSets[i].nVerticesTotalMax = num;
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nVerticesTrackMax")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, -1, 2147483647); if (outputString[0] == '\0') goto PARM_ERR;
		if (num >= 0 && num < 4) num = -1;
		viewParmSets[i].nVerticesTrackMax = num;
		DefaultTrackColors(viewParmSets[i].trackColor);
		if (format >= 102) {
			for (j = 0; j < NEVENTTYPES; j++) {
				sprintf(tmp, "trackColorRGB[%d]", j);
				fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, tmp)) goto PARM_ERR;
				ptr = strtok(NULL, " =(,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
				viewParmSets[i].trackColor[j].r = num;
				ptr = strtok(NULL, " ,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
				viewParmSets[i].trackColor[j].g = num;
				ptr = strtok(NULL, " ,)\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
				viewParmSets[i].trackColor[j].b = num;
				ColorName(viewParmSets[i].trackColor + j, true);
			}
		}
		else {
			fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "trackColorRGB")) goto PARM_ERR;
			ptr = strtok(NULL, " =(,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackColor[0].r = num;
			ptr = strtok(NULL, " ,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackColor[0].g = num;
			ptr = strtok(NULL, " ,)\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			viewParmSets[i].trackColor[0].b = num;
			ColorName(viewParmSets[i].trackColor, true);
		}
		bool showBeamAxis;
		fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "beamAxisColorRGB")) goto PARM_ERR;
		ptr = strtok(NULL, " =(,\t\n"); num = CheckInteger(ptr, outputString, -1, 255); if (outputString[0] == '\0') goto PARM_ERR;
		if (num >= 0) {
			viewParmSets[i].beamAxisColor.r = num;
			showBeamAxis = true;
		}
		else {
			viewParmSets[i].beamAxisColor.r = r_std;
			showBeamAxis = false;
		}
		ptr = strtok(NULL, " ,\t\n"); num = showBeamAxis ? CheckInteger(ptr, outputString, 0, 255) : CheckInteger(ptr, outputString, -1, -1); if (outputString[0] == '\0') goto PARM_ERR;
		if (showBeamAxis) viewParmSets[i].beamAxisColor.g = num;
		else viewParmSets[i].beamAxisColor.g = g_std;
		ptr = strtok(NULL, " ,)\t\n"); num = showBeamAxis ? CheckInteger(ptr, outputString, 0, 255) : CheckInteger(ptr, outputString, -1, -1); if (outputString[0] == '\0') goto PARM_ERR;
		if (showBeamAxis) viewParmSets[i].beamAxisColor.b = num;
		else viewParmSets[i].beamAxisColor.b = b_std;
		ColorName(&(viewParmSets[i].beamAxisColor), true);
		fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "backgroundColorRGB")) goto PARM_ERR;
		ptr = strtok(NULL, " =(,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].backgroundColor.r = num;
		ptr = strtok(NULL, " ,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].backgroundColor.g = num;
		ptr = strtok(NULL, " ,)\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].backgroundColor.b = num;
		ColorName(&(viewParmSets[i].backgroundColor), true);
		fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "nRows")) goto PARM_ERR;
		ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 1, 100000); if (outputString[0] == '\0') goto PARM_ERR;
		viewParmSets[i].nRegions = num;
		regionData[i] = (REGIONDATA *)malloc(viewParmSets[i].nRegions * sizeof(REGIONDATA));
		for (j = 0; j < viewParmSets[i].nRegions; j++) {
			fgets(line, 2 + REGIONNAMELEN, f_par); line[REGIONNAMELEN] = '\0'; regionData[i][j].name[0] = '\0'; ptr = strtok(line, "\n");
			if (ptr) strcpy(regionData[i][j].name, ptr);
			fgets(line, 200, f_par); line[200] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "colorRGB")) goto PARM_ERR;
			ptr = strtok(NULL, " =(,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].color.r = num;
			ptr = strtok(NULL, " ,\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].color.g = num;
			ptr = strtok(NULL, " ,)\t\n"); num = CheckInteger(ptr, outputString, 0, 255); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].color.b = num;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "transparency")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].transp = numDbl;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "reflectivity")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); numDbl = CheckDouble(ptr, outputString, 0., 1.); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].refl = numDbl;
			fgets(line, 100, f_par); line[100] = '\0'; ptr = strtok(line, " ="); if (!ptr || strcmpi(ptr, "show")) goto PARM_ERR;
			ptr = strtok(NULL, " =\t\n"); num = CheckInteger(ptr, outputString, 0, 1); if (outputString[0] == '\0') goto PARM_ERR;
			regionData[i][j].show = (num != 0);
		}
		viewParmSets[i].cmdProcessID = 0;
		viewParmSets[i].cmdTimerID = 0;
		viewParmSets[i].cmdTimerIndex = -1;
		viewParmSets[i].cmdWindowCreated = FALSE;
		viewParmSets[i].mgedProcessID = 0;
		viewParmSets[i].mgedTimerID = 0;
		viewParmSets[i].mgedTimerIndex = -1;
		viewParmSets[i].mgedWindowCreated = FALSE;
		trackSets[i] = defaultTrackSetData;
	}
	if (err) {
PARM_ERR:
		if (i) {
			MessageBox((LPCTSTR)(msg.Format(_T("Warning: Error while reading parameter set No. %d from file \"%s\" (incorrect file format)."), i + 1, app->parFileName), msg), NULL, MB_OK | MB_ICONERROR);
			nViewParmSets = i;
			viewParmSets.SetSize(nViewParmSets);
			trackSets.SetSize(nViewParmSets);
			regionData = (REGIONDATA **)realloc(regionData, nViewParmSets * sizeof(REGIONDATA *));
			if (curParmSet >= nViewParmSets) curParmSet = nViewParmSets - 1;
		}
		else {
			if (f_par) {
				MessageBox((LPCTSTR)(msg.Format(_T("Warning: File \"%s\" has incorrect format. A default parameter set will be created."), app->parFileName), msg), NULL, MB_OK | MB_ICONERROR);
			}
			nViewParmSets = 0;
			curParmSet = 0;
			NewParameterSet();
			strcpy(viewParmSets[0].parmSetName, parmSetName);
		}
	}
	parFileSpecified = true;
	if (i) {
		*((VIEWPARM *)&maximumObjectSizeRatio) = viewParmSets[curParmSet];
		REGIONDATA *r = regionData[curParmSet];
		regions.SetSize(nRegions);
		for (i = 0; i < nRegions; i++) regions[i] = r[i];
	}
	changed = false;
	iParmSetFromCmdTimerIndex.SetSize(0);
	iParmSetFromMGEDtimerIndex.SetSize(0);
	timerCount = 0;

	for (i = 0; i < nViewParmSets; i++) {
		sprintf(tmp, " (%d)  %s", i + 1, viewParmSets[i].parmSetName);
		m_parmSetList.AddString(tmp);
	}
	m_parmSetList.SetCurSel(curParmSet);
	UpdateViewParms();
	if (nViewParmSets == 1) m_deleteParmSet.EnableWindow(FALSE);
	else m_deleteParmSet.EnableWindow();
	m_startMCNelectron.SetWindowText("Start MCNelectron");
	m_startMGED.SetWindowText("Start MGED");
	m_nTracks.SetWindowText("");
	if (!err) {
		app->GetRecentFileList()->Add(app->parFileName);
		return TRUE;
	}
	else return FALSE;
}

void CMCNScriptDlg::OnInitMenu(CMenu *pMenu)
{
	m_MGEDcmdLineCapt.SetFocus();
	CMenu *pFileMenu = pMenu->GetSubMenu(0);
	int count = pFileMenu->GetMenuItemCount();
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());
	if (count == 5 && changed && f_par) {
		FILE *f_test = fopen(app->parFileName, "r+t");
		if (f_test) {
			fclose(f_test);
			pFileMenu->InsertMenu(1, MF_BYPOSITION, ID_RELOAD, "&Reload");
		}
	}
	else if (count == 6 && (!changed || !f_par)) pFileMenu->DeleteMenu(ID_RELOAD, MF_BYCOMMAND);
	CMenu *pConsoleMenu = pMenu->GetSubMenu(1);
	HWND consoleWndHandle = GetConsoleWindow();
	if (consoleWndHandle && IsWindow(consoleWndHandle)) {
		pConsoleMenu->EnableMenuItem(ID_CONSOLE_ACTIVATE, MF_BYCOMMAND | MF_ENABLED);
		pConsoleMenu->EnableMenuItem(ID_CONSOLE_CLOSE, MF_BYCOMMAND | MF_ENABLED);
	}
	else {
		pConsoleMenu->EnableMenuItem(ID_CONSOLE_ACTIVATE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		pConsoleMenu->EnableMenuItem(ID_CONSOLE_CLOSE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}
	CDialogEx::OnInitMenu(pMenu);
}

void CMCNScriptDlg::UpdateColorsView()
{
	char tmp[1000];
	int i;
	int nRows_prev = m_colorsGrid.GetRowCount();
    m_colorsGrid.SetSelectedRange(-1,-1,-1,-1);
	m_colorsGrid.SetFocusCell(-1,-1);
	m_colorsGrid.SetRowCount(nRegions + 1);
	for (i = 0; i < nRegions; i++) {
		int i1 = i + 1;
		m_colorsGrid.SetRowHeight(i1, colorsRowHeight);
		m_colorsGrid.SetCellType(i1, 3, RUNTIME_CLASS(CGridCellCombo));
		m_colorsGrid.SetCellType(i1, 9, RUNTIME_CLASS(CGridCellCheck));
	    m_colorsGrid.SetItemState(i1, 1, m_colorsGrid.GetItemState(i1, 1) | GVIS_READONLY);
	    m_colorsGrid.SetItemState(i1, 2, m_colorsGrid.GetItemState(i1, 2) | GVIS_READONLY);
	    m_colorsGrid.SetItemState(i1, 3, m_colorsGrid.GetItemState(i1, 3) | GVIS_NOPASTE);
		m_colorsGrid.SetItemState(i1, 9, m_colorsGrid.GetItemState(i1, 9) | GVIS_READONLY);
		sprintf(tmp, " %d ", i1);
		m_colorsGrid.SetItemText(i1, 0, tmp);
		m_colorsGrid.SetItemText(i1, 1, regions[i].name);
		UpdateRegionColor(i);
		UpdateCellRGB(i);
		sprintf(tmp, "%g", regions[i].transp); m_colorsGrid.SetItemText(i1, 7, tmp);
		sprintf(tmp, "%g", regions[i].refl); m_colorsGrid.SetItemText(i1, 8, tmp);
		CGridCellCheck *pCheck = (CGridCellCheck *)m_colorsGrid.GetCell(i1, 9);
		pCheck->m_bChecked = (regions[i].show != 0);
		UpdateShow(i);
	}
	m_colorsGrid.ResetWidth();
}

void CMCNScriptDlg::OnUseCustomTrackFN()
{
	if (m_useCustomTrackFN.GetCheck()) {
		useCustomTrackFN = TRUE;
		m_loadTrackFile.EnableWindow();
		int nBytes = m_trackFileName.GetWindowText(trackFN, FNLEN);
		trackFN[nBytes] = '\0';
		m_trackFileName.SetWindowText(ShortenPath(trackFN, currentFolder));
		m_trackFileName.disable = FALSE;
	}
	else {
		useCustomTrackFN = FALSE;
		m_loadTrackFile.EnableWindow(FALSE);
		trackFN[0] = '\0';
		CString trackFNstring('\0', FNLEN+1);
		if (suffix[0] != '\0') {
			char tmp[SUFFIXLEN+10];
			sprintf(tmp, suffix, curParmSet + 1);
			trackFNstring = (CString)inputFileFolder + "Files_" + fn1 + tmp + ".txt\\" + fn1 + tmp + "_tracks.txt";
		}
		else {
			char drive[1000], dir[1000], fname[1000], ext[1000];
			_splitpath(outputFileName, drive, dir, fname, ext);
			trackFNstring = (CString)drive + dir + "Files_" + fname + ext + "\\" + fname + "_tracks.txt";
		}
		m_trackFileName.SetWindowText(ShortenPath(trackFNstring, currentFolder));
		m_trackFileName.disable = TRUE;
	}
	changed = true;
}

void CMCNScriptDlg::OnCopyToClipboard()
{
	if (m_copyToClipboard.GetCheck()) copyToClipboard = TRUE;
	else copyToClipboard = FALSE;
	changed = true;
}

void CMCNScriptDlg::OnCreateLog()
{
	int ch = m_createLog.GetCheck();
	if (ch == BST_CHECKED) createLog = 1;
	else if (ch == BST_UNCHECKED) createLog = 0;
	else createLog = 2;
	changed = true;
	MakeCmdLine();
}

void CMCNScriptDlg::OnCheckSyntax_MCNScript()
{
	int ch = m_checkSyntax_MCNScript.GetCheck();
	if (ch == BST_CHECKED) checkSyntax_MCNScript = TRUE;
	else checkSyntax_MCNScript = FALSE;
	changed = true;
}

void CMCNScriptDlg::OnSuffix()
{
	char suffix_old[SUFFIXLEN+1], tmp[SUFFIXLEN+10];
	strcpy(suffix_old, suffix);
	char suffix_new[SUFFIXLEN+1];
	int nBytes = m_suffix.GetLine(0, suffix_new, SUFFIXLEN);
	suffix_new[nBytes] = '\0';
	char *ptr = strtok(suffix_new, " \t");
	if (!ptr) suffix_new[0] = '\0';
	else {
		strcpy(suffix_new, ptr);
		ptr = strchr(suffix_new, '%');
		if (ptr && (*(ptr + 1) != 'd' || *(ptr + 2) != '\0' && strchr(ptr + 2, '%'))) strcpy(suffix_new, suffix_old);
	}
	m_suffix.SetWindowText(suffix_new);
	if (!strcmp(suffix_new, suffix)) return;
	strcpy(suffix, suffix_new);
	changed = true;
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, curParmSet + 1);
		m_outputFileName.disable = TRUE;
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		outputFileName[0] = '\0';
		if (!useCustomTrackFN) m_trackFileName.SetWindowText(ShortenPath((CString)inputFileFolder + "Files_" + fn1 + tmp + ".txt\\" + fn1 + tmp + "_tracks.txt", currentFolder));
		m_nTracks.SetWindowText("");
	}
	else {
		m_outputFileName.disable = FALSE;
		int nBytes = m_outputFileName.GetWindowText(outputFileName, FNLEN);
		outputFileName[nBytes] = '\0';
		m_outputFileName.SetWindowText(ShortenPath(outputFileName, currentFolder));
	}
	MakeCmdLine();
}

void CMCNScriptDlg::OnLogSuffix()
{
	char logSuffix_old[SUFFIXLEN+1], tmp[SUFFIXLEN+10];
	strcpy(logSuffix_old, logSuffix);
	char logSuffix_new[SUFFIXLEN+1];
	int nBytes = m_logSuffix.GetLine(0, logSuffix_new, SUFFIXLEN);
	logSuffix_new[nBytes] = '\0';
	char *ptr = strtok(logSuffix_new, " \t");
	if (!ptr) logSuffix_new[0] = '\0';
	else {
		strcpy(logSuffix_new, ptr);
		ptr = strchr(logSuffix_new, '%');
		if (ptr && (*(ptr + 1) != 'd' || *(ptr + 2) != '\0' && strchr(ptr + 2, '%'))) strcpy(logSuffix_new, logSuffix_old);
	}
	m_logSuffix.SetWindowText(logSuffix_new);
	if (!strcmp(logSuffix_new, logSuffix)) return;
	strcpy(logSuffix, logSuffix_new);
	changed = true;
	if (logSuffix[0] != '\0') {
		sprintf(tmp, logSuffix, curParmSet + 1);
		m_logFileName.disable = TRUE;
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		logFileName[0] = '\0';
	}
	else {
		m_logFileName.disable = FALSE;
		int nBytes = m_logFileName.GetWindowText(logFileName, FNLEN);
		logFileName[nBytes] = '\0';
		m_logFileName.SetWindowText(ShortenPath(logFileName, currentFolder));
	}
	MakeCmdLine();
}

void CMCNScriptDlg::UpdateViewParms()
{
	int i, j;
    pSystemVarsGrid->SetSelectedRange(-1,-1,-1,-1);
	pSystemVarsGrid->SetFocusCell(-1,-1);
	for (i = 0; i < 10; i++) {
		char tmp[1000];
		sprintf(tmp, "%.16g", systemVars[i]);
		pSystemVarsGrid->SetItemText(i + 1, 1, tmp);
	}
	pSystemVarsGrid->ResetWidth();
	UpdateColorsView();
    pTrackColorsGrid->SetSelectedRange(-1,-1,-1,-1);
	pTrackColorsGrid->SetFocusCell(-1,-1);
	for (i = 0; i < NEVENTTYPES; i++) {
		UpdateTrackColor(i);
		UpdateCellRGB_tracks(i);
		CGridCellCheck *pCheck = (CGridCellCheck *)pTrackColorsGrid->GetCell(i + 1, 2);
		pCheck->m_bChecked = ((bitMask[i] & trackFilter.originTypeMask) != 0);
		UpdateShow_tracks(pCheck->m_bChecked, i);
	}
	pTrackColorsGrid->ResetWidth();
	nUsedDefaultColors = 0;
	memset(defaultColorUsed, 0, NDEFAULTCOLORS * sizeof(int));
	for (i = 0; i < nRegions; i++) {
		for (j = 0; j < NDEFAULTCOLORS && (regions[i].color.r != defaultColor[j].r || regions[i].color.g != defaultColor[j].g || regions[i].color.b != defaultColor[j].b); j++);
		if (j < NDEFAULTCOLORS) {
			if (!defaultColorUsed[j]) nUsedDefaultColors++;
			defaultColorUsed[j]++;
		}
	}
	UpdateBeamAxisColor();
	UpdateBackgroundColor();
	_splitpath(fn, NULL, NULL, fn1, NULL);
	UpdateFileNames();
	size_t cmdLineLength = strlen(MGEDcmdLine);
	char MGEDcmdLine_tmp[3*FNLEN+1];
	strcpy(MGEDcmdLine_tmp, MGEDcmdLine);
	char *MGEDpath = strtok(MGEDcmdLine_tmp, "\"");
	char *cmdLineArgs;
	if (MGEDpath && MGEDpath + strlen(MGEDpath) != MGEDcmdLine_tmp + cmdLineLength) cmdLineArgs = MGEDpath + strlen(MGEDpath) + 1;
	else cmdLineArgs = NULL;
	CString MGEDpath2 = ShortenPath(MGEDpath, currentFolder);
	if (cmdLineArgs) m_MGEDcommandLine.SetWindowText("\"" + MGEDpath2 + "\"" + cmdLineArgs);
	else m_MGEDcommandLine.SetWindowText(MGEDpath2);
	if (raytraceInGraphicsWindow) m_raytraceInGraphicsWindowCheck.SetCheck(1);
	else m_raytraceInGraphicsWindowCheck.SetCheck(0);
	if (showWireframe) m_showWireframeCheck.SetCheck(1);
	else m_showWireframeCheck.SetCheck(0);
	if (raytraceInLingeringWindow) {
		m_raytraceInLingeringWindowCheck.SetCheck(1);
		m_portNumEdit.EnableWindow();
	}
	else {
		m_raytraceInLingeringWindowCheck.SetCheck(0);
		m_portNumEdit.EnableWindow(FALSE);
	}
	if (showTracks) {
		m_showTracksCheck.SetCheck(1);
		m_nVerticesTotalMaxEdit.EnableWindow();
		m_nVerticesTrackMaxEdit.EnableWindow();
		m_showTrackSegmentsCheck.EnableWindow();
		m_extendEscapeTracksCheck.EnableWindow();
	}
	else {
		m_showTracksCheck.SetCheck(0);
		m_nVerticesTotalMaxEdit.EnableWindow(FALSE);
		m_nVerticesTrackMaxEdit.EnableWindow(FALSE);
		m_showTrackSegmentsCheck.EnableWindow(FALSE);
		m_extendEscapeTracksCheck.EnableWindow(FALSE);
	}
	if (showTrackSegments) m_showTrackSegmentsCheck.SetCheck(1);
	else m_showTrackSegmentsCheck.SetCheck(0);
	if (extendEscapeTracks) m_extendEscapeTracksCheck.SetCheck(1);
	else m_extendEscapeTracksCheck.SetCheck(0);
	if (keepTrackDataInMemory) m_keepTrackDataInMemoryCheck.SetCheck(1);
	else m_keepTrackDataInMemoryCheck.SetCheck(0);
	char tmp[100];
	sprintf(tmp, "%g", viewSize); m_viewSizeEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", maximumObjectSizeRatio); m_maximumObjectSizeRatioEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", sourceSizeRatio); m_sourceSizeRatioEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", viewCenter.x); m_viewCenterGrid.SetItemText(1, 0, tmp);
	sprintf(tmp, "%g", viewCenter.y); m_viewCenterGrid.SetItemText(1, 1, tmp);
	sprintf(tmp, "%g", viewCenter.z); m_viewCenterGrid.SetItemText(1, 2, tmp);
	sprintf(tmp, "%g", azimuth); m_azimuthEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", elevation); m_elevationEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", twist); m_twistEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", perspective); m_perspectiveEdit.SetWindowText(tmp);
	sprintf(tmp, "%g", cutawayPlanePoint.x); m_cutawayPlanePointGrid.SetItemText(1, 0, tmp);
	sprintf(tmp, "%g", cutawayPlanePoint.y); m_cutawayPlanePointGrid.SetItemText(1, 1, tmp);
	sprintf(tmp, "%g", cutawayPlanePoint.z); m_cutawayPlanePointGrid.SetItemText(1, 2, tmp);
	sprintf(tmp, "%g", cutawayPlaneNormal.x); m_cutawayPlaneNormalGrid.SetItemText(1, 0, tmp);
	sprintf(tmp, "%g", cutawayPlaneNormal.y); m_cutawayPlaneNormalGrid.SetItemText(1, 1, tmp);
	sprintf(tmp, "%g", cutawayPlaneNormal.z); m_cutawayPlaneNormalGrid.SetItemText(1, 2, tmp);
	UpdateCutawayGroup();
	sprintf(tmp, "%d", portNum); m_portNumEdit.SetWindowText(tmp);
	sprintf(tmp, "%d", nIterations); m_nIterationsEdit.SetWindowText(tmp);
	if (nVerticesTotalMax > 0) {
		sprintf(tmp, "%d", nVerticesTotalMax);
		m_nVerticesTotalMaxEdit.SetWindowText(tmp);
	}
	else m_nVerticesTotalMaxEdit.SetWindowText("");
	if (nVerticesTrackMax > 0) {
		sprintf(tmp, "%d", nVerticesTrackMax);
		m_nVerticesTrackMaxEdit.SetWindowText(tmp);
	}
	else m_nVerticesTrackMaxEdit.SetWindowText("");
	m_MGEDsuffixEdit.SetWindowText(MGEDsuffix);
    m_viewCenterGrid.SetSelectedRange(-1,-1,-1,-1);
	m_viewCenterGrid.SetFocusCell(-1,-1);
	m_viewCenterGrid.Invalidate();
	m_viewCenterGrid.UpdateWindow();
	m_coordSys.Invalidate();
	m_coordSys.UpdateWindow();
	UpdateTrackFilter(pTrackFilterGrid, pOriginTypeCheckbox, pGenMinEdit, pGenMaxEdit, &trackFilter);
}

int CALLBACK 
MyBrowseCallbackProc(HWND hwnd,UINT uMsg, LPARAM, LPARAM lpData)
{
    if (uMsg == BFFM_INITIALIZED) {
        CMCNScriptDlg* pThis= (CMCNScriptDlg*)lpData; // set in BROWSEINFO
        if (pThis && pThis->GetSafeHwnd()) { // forgot to set it?
			char path[1000] = {'\0'};
			if (pThis->scriptFolder[0] != '\\' && !strchr(pThis->scriptFolder, ':')) strcpy(path, currentFolder);
			strcat(path, pThis->scriptFolder);
		 	::SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)path);
        }
    }
    return 0;
}

void CMCNScriptDlg::OnSelectFolder()
{
    BROWSEINFO rBI = {0};
    rBI.hwndOwner = GetSafeHwnd();
	rBI.ulFlags = BIF_RETURNONLYFSDIRS | BIF_BROWSEINCLUDEFILES | BIF_USENEWUI;
	rBI.lpszTitle = "Browse for the folder to place the output RT script";
	rBI.lpfn = MyBrowseCallbackProc;
	rBI.lParam = (DWORD)this;   // used by static callback function
	LPITEMIDLIST pidl = SHBrowseForFolder(&rBI);
	if (!pidl) return;
	TCHAR szPath[MAX_PATH];
	BOOL fRet = SHGetPathFromIDList(pidl, szPath);
	FILE *f = fopen(szPath, "rb");
	if (f) {
		char drive[512], dir[512];
		_splitpath(szPath, drive, dir, NULL, NULL);
		sprintf(szPath, "%s%s", drive, dir);
	}
	else if (!strlen(szPath) || szPath[strlen(szPath)-1] != '\\') strcat(szPath, "\\");
	CString pathName = szPath;
	char scriptFolder_bak[1000];
	strcpy(scriptFolder_bak, scriptFolder);
	strcpy(scriptFolder, pathName.GetBuffer()); pathName.ReleaseBuffer();
	if (strcmp(ShortenPath(scriptFolder, currentFolder), ShortenPath(scriptFolder_bak, currentFolder))) changed = true;
	char tmp[100];
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
}

bool CompareTrackFilters(TRACKFILTERDATA *filter1, TRACKFILTERDATA *filter2) {
	int i;
	for (i = 0; i < 2; i++) {
		if (filter1->historyNo[i] != filter2->historyNo[i]) break;
		if (filter1->trackNo[i] != filter2->trackNo[i]) break;
		if (filter1->nVert[i] != filter2->nVert[i]) break;
		if (filter1->Estart[i] != filter2->Estart[i]) break;
		if (filter1->Efinal[i] != filter2->Efinal[i]) break;
		if (filter1->Eloss[i] != filter2->Eloss[i]) break;
		if (filter1->relEloss[i] != filter2->relEloss[i]) break;
		if (filter1->displ[i] != filter2->displ[i]) break;
		if (filter1->peakDispl[i] != filter2->peakDispl[i]) break;
		if (filter1->path[i] != filter2->path[i]) break;
		if (filter1->gen[i] != filter2->gen[i]) break;
		if (filter1->originTypeMask != filter2->originTypeMask) break;
	}
	return (i < 2);
}

bool CompareTrackSegmentData(TRACKSEGMENTDATA *segment1, TRACKSEGMENTDATA *segment2) {
	int i;
	for (i = 0; i < 2; i++) {
		if (segment1->iVert[i] != segment2->iVert[i]) break;
		if (segment1->E[i] != segment2->E[i]) break;
		if (segment1->Eloss[i] != segment2->Eloss[i]) break;
		if (segment1->relEloss[i] != segment2->relEloss[i]) break;
		if (segment1->path[i] != segment2->path[i]) break;
	}
	return (i < 2);
}

void CMCNScriptDlg::ColorSetSel(CComboBox *list, CColorRect *pic)
{
	COLORREF prevColor = pic->color;
	int prevSel = pic->sel;
	pic->sel = list->GetCurSel();
	int count = list->GetCount();
	if (pic->sel < NSTDCOLORS) pic->color = RGB(stdColor[pic->sel].r, stdColor[pic->sel].g, stdColor[pic->sel].b);
	else if (pic->sel == NSTDCOLORS) {
		unsigned char r = GetRValue(pic->color);
		unsigned char g = GetGValue(pic->color);
		unsigned char b = GetBValue(pic->color);
		DWORD rgb = GetSysColor(COLOR_3DFACE);
		unsigned char r_std = GetRValue(rgb);
		unsigned char g_std = GetGValue(rgb);
		unsigned char b_std = GetBValue(rgb);
		int i;
		if (count > NSTDCOLORS + 1 && r == r_std && g == g_std && b == b_std) i = NSTDCOLORS + 1;
		else {
			for (i = 0; i < NSTDCOLORS && (r != stdColor[i].r || g != stdColor[i].g || b != stdColor[i].b); i++);
		}
		list->SetCurSel(i);
		CColorDialog colorDlg(pic->color, CC_FULLOPEN);
		if (colorDlg.DoModal() == IDOK) {
			pic->color = colorDlg.GetColor();
			r = GetRValue(pic->color);
			g = GetGValue(pic->color);
			b = GetBValue(pic->color);
			if (count > NSTDCOLORS + 1 && r == r_std && g == g_std && b == b_std) i = NSTDCOLORS + 1;
			else {
				for (i = 0; i < NSTDCOLORS && (r != stdColor[i].r || g != stdColor[i].g || b != stdColor[i].b); i++);
			}
			list->SetCurSel(i);
			pic->sel = i;
		}
	}
	else {
		DWORD rgb = GetSysColor(COLOR_3DFACE);
		pic->color = RGB(GetRValue(rgb), GetGValue(rgb), GetBValue(rgb));
	}
	if (pic->color != prevColor || pic->sel != prevSel) {
		changed = true;
		pic->Invalidate();
		pic->UpdateWindow();
	}
}

void CMCNScriptDlg::OnBeamAxisColorSetSel()
{
	ColorSetSel(&m_beamAxisColor, &m_beamAxisColorPic);
	beamAxisColor.r = GetRValue(m_beamAxisColorPic.color);
	beamAxisColor.g = GetGValue(m_beamAxisColorPic.color);
	beamAxisColor.b = GetBValue(m_beamAxisColorPic.color);
}

void CMCNScriptDlg::OnBackgroundColorSetSel()
{
	ColorSetSel(&m_backgroundColor, &m_backgroundColorPic);
	backgroundColor.r = GetRValue(m_backgroundColorPic.color);
	backgroundColor.g = GetGValue(m_backgroundColorPic.color);
	backgroundColor.b = GetBValue(m_backgroundColorPic.color);
}

void CMCNScriptDlg::OnShowTrackColors()
{
	if (!trackColorsShown) {
		CRect colorsWindowRect, mainWindowRect;
		GetWindowRect(&mainWindowRect);
		pTrackColorsDlg->GetWindowRect(&colorsWindowRect);
		int width = colorsWindowRect.right - colorsWindowRect.left;
		int height = colorsWindowRect.bottom - colorsWindowRect.top;
		int hor_shift = (mainWindowRect.right - mainWindowRect.left - width) / 2;
		int vert_shift = (mainWindowRect.bottom - mainWindowRect.top - height) / 2;
		colorsWindowRect.left = mainWindowRect.left + hor_shift;
		colorsWindowRect.right = colorsWindowRect.left + width;
		colorsWindowRect.top = mainWindowRect.top + vert_shift;
		colorsWindowRect.bottom = colorsWindowRect.top + height;
		pTrackColorsDlg->SetWindowPos(&wndTop, colorsWindowRect.left, colorsWindowRect.top, colorsWindowRect.right - colorsWindowRect.left, colorsWindowRect.bottom - colorsWindowRect.top, SWP_SHOWWINDOW);
		trackColorsShown = true;
	}
	else {
		pTrackColorsDlg->ShowWindow(SW_SHOWNORMAL);
		pTrackColorsDlg->SetForegroundWindow();
	}
}

void CMCNScriptDlg::OnParmSetSel()
{
	int i;
	char nTracksStr[30];
	int nBytes = m_nTracks.GetWindowText(nTracksStr, 29);
	nTracksStr[nBytes] = '\0';
	char cmdLine_old[6*FNLEN];
	if (strspn(nTracksStr, " \t") != strlen(nTracksStr)) strcpy(cmdLine_old, cmdLine);
	else cmdLine_old[0] = '\0';
	TRACKFILTERDATA trackFilter_old = trackFilter;
	TRACKSEGMENTDATA trackSegmentSelection_old = showTrackSegments ? trackSegmentSelection : defaultTrackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection_old = trackFilteringBeforeSegmentSelection;
	viewParmSets[curParmSet] = *((VIEWPARM *)&maximumObjectSizeRatio);
	regionData[curParmSet] = (REGIONDATA *)realloc(regionData[curParmSet], nRegions * sizeof(REGIONDATA));
	REGIONDATA *r = regionData[curParmSet];
	for (i = 0; i < nRegions; i++) r[i] = regions[i];
	curParmSet = m_parmSetList.GetCurSel();
	*((VIEWPARM *)&maximumObjectSizeRatio) = viewParmSets[curParmSet];
	r = regionData[curParmSet];
	regions.SetSize(nRegions);
	for (i = 0; i < nRegions; i++) regions[i] = r[i];
	UpdateViewParms();
	TRACKSEGMENTDATA trackSegmentSelection_new = showTrackSegments ? trackSegmentSelection : defaultTrackSegmentSelection;
	if (cmdLine_old[0] != '\0' && strcmp(cmdLine_old, cmdLine) ||
		CompareTrackFilters(&trackFilter, &trackFilter_old) || CompareTrackSegmentData(&trackSegmentSelection_new, &trackSegmentSelection_old) ||
		trackFilteringBeforeSegmentSelection != trackFilteringBeforeSegmentSelection_old && CompareTrackFilters(&trackFilter, &defaultTrackFilter) && CompareTrackSegmentData(&trackSegmentSelection_new, &defaultTrackSegmentSelection)) m_nTracks.SetWindowText("");
	if (cmdProcessID) m_startMCNelectron.SetWindowText("Activate the cmd.exe window");
	else m_startMCNelectron.SetWindowText("Start MCNelectron");
	if (mgedProcessID) m_startMGED.SetWindowText("Activate the MGED window");
	else m_startMGED.SetWindowText("Start MGED");
}

void CMCNScriptDlg::OnNewParmSet()
{
	char parmSetName_bak[PARMSETNAMELEN+1];
	strcpy(parmSetName_bak, parmSetName);
	int curParmSet_bak = curParmSet;
	CNewParmSetDlg dlg(this);
	INT_PTR rc = dlg.DoModal();
	if (rc == IDCANCEL) return;
	int i;
	viewParmSets[curParmSet_bak] = *((VIEWPARM *)&maximumObjectSizeRatio);
	strcpy(viewParmSets[curParmSet_bak].parmSetName, parmSetName_bak);
	cmdProcessID = 0;
	cmdTimerID = 0;
	cmdTimerIndex = -1;
	cmdWindowCreated = FALSE;
	mgedProcessID = 0;
	mgedTimerID = 0;
	mgedTimerIndex = -1;
	mgedWindowCreated = FALSE;
	regionData[curParmSet_bak] = (REGIONDATA *)realloc(regionData[curParmSet_bak], nRegions * sizeof(REGIONDATA));
	REGIONDATA *r = regionData[curParmSet_bak];
	for (i = 0; i < nRegions; i++) r[i] = regions[i];
	strcpy(parmSetName_bak, parmSetName);
	NewParameterSet();
	strcpy(parmSetName, parmSetName_bak);
	strcpy(viewParmSets[curParmSet].parmSetName, parmSetName);
	if (nViewParmSets == 2) m_deleteParmSet.EnableWindow();
	char tmp[1000];
	for (i = nViewParmSets - 2; i >= curParmSet; i--) m_parmSetList.DeleteString(i);
	for (i = curParmSet; i < nViewParmSets; i++) {
		sprintf(tmp, " (%d)  %s", i + 1, viewParmSets[i].parmSetName);
		m_parmSetList.AddString(tmp);
	}
	m_parmSetList.SetCurSel(curParmSet);
	UpdateViewParms();
	m_startMCNelectron.SetWindowText("Start MCNelectron");
	m_startMGED.SetWindowText("Start MGED");
}

void CMCNScriptDlg::OnDeleteParmSet()
{
	char tmp[1000];
	int i, j;
	sprintf(tmp, "Do you really want to delete parameter set No. %d?", curParmSet + 1);
	if (MessageBox(tmp, NULL, MB_YESNOCANCEL | MB_ICONWARNING | MB_DEFBUTTON2) != IDYES) return;
	viewParmSets.RemoveAt(curParmSet);
	if (trackSets[curParmSet].nTracks) {
		for (j = 0; j < trackSets[curParmSet].nTracks; j++) {
			delete trackSets[curParmSet].trackData[j].coord;
			delete trackSets[curParmSet].trackData[j].energy;
		}
		free(trackSets[curParmSet].trackData);
	}
	trackSets.RemoveAt(curParmSet);
	nViewParmSets--;

	if (cmdTimerID) {
		KillTimer(cmdTimerID);
		iParmSetFromCmdTimerIndex.RemoveAt(cmdTimerIndex);
	}
	if (mgedTimerID) {
		KillTimer(mgedTimerID);
		iParmSetFromMGEDtimerIndex.RemoveAt(mgedTimerIndex);
	}

	INT_PTR nCmdTimers = iParmSetFromCmdTimerIndex.GetSize();
	for (i = 0; i < nCmdTimers; i++) {
		j = iParmSetFromCmdTimerIndex[i];
		if (j > curParmSet) {
			j--;
			iParmSetFromCmdTimerIndex[i] = j;
		}
		if (cmdTimerID && viewParmSets[j].cmdTimerIndex > cmdTimerIndex) viewParmSets[j].cmdTimerIndex--;
	}
	INT_PTR nMGEDtimers = iParmSetFromMGEDtimerIndex.GetSize();
	for (i = 0; i < nMGEDtimers; i++) {
		j = iParmSetFromMGEDtimerIndex[i];
		if (j > curParmSet) {
			j--;
			iParmSetFromMGEDtimerIndex[i] = j;
		}
		if (mgedTimerID && viewParmSets[j].mgedTimerIndex > mgedTimerIndex) viewParmSets[j].mgedTimerIndex--;
	}

	m_parmSetList.DeleteString(curParmSet);
	if (nViewParmSets == 1) m_deleteParmSet.EnableWindow(FALSE);
	free(regionData[curParmSet]);
	for (i = curParmSet; i < nViewParmSets; i++) regionData[i] = regionData[i+1];
	if (curParmSet == nViewParmSets) curParmSet--;
	else {
		for (i = nViewParmSets - 1; i >= curParmSet; i--) m_parmSetList.DeleteString(i);
		for (i = curParmSet; i < nViewParmSets; i++) {
			sprintf(tmp, " (%d)  %s", i + 1, viewParmSets[i].parmSetName);
			m_parmSetList.AddString(tmp);
		}
	}
	m_parmSetList.SetCurSel(curParmSet);
	*((VIEWPARM *)&maximumObjectSizeRatio) = viewParmSets[curParmSet];
	REGIONDATA *r = regionData[curParmSet];
	regions.SetSize(nRegions);
	for (i = 0; i < nRegions; i++) regions[i] = r[i];
	UpdateViewParms();
	if (cmdProcessID) m_startMCNelectron.SetWindowText("Activate the cmd.exe window");
	else m_startMCNelectron.SetWindowText("Start MCNelectron");
	if (mgedProcessID) m_startMGED.SetWindowText("Activate the MGED window");
	else m_startMGED.SetWindowText("Start MGED");
	changed = true;
}

void CMCNScriptDlg::KillCmdTimer()
{
	if (cmdTimerID) {
		KillTimer(cmdTimerID);
		iParmSetFromCmdTimerIndex.RemoveAt(cmdTimerIndex);
		INT_PTR nCmdTimers = iParmSetFromCmdTimerIndex.GetSize();
		for (int i = 0; i < nCmdTimers; i++) {
			int j = iParmSetFromCmdTimerIndex[i];
			if (viewParmSets[j].cmdTimerIndex > cmdTimerIndex) viewParmSets[j].cmdTimerIndex--;
		}
		cmdTimerID = 0;
		cmdTimerIndex = -1;
	}
	cmdProcessID = 0;
	cmdWindowCreated = FALSE;
	viewParmSets[curParmSet].cmdTimerID = 0;
	viewParmSets[curParmSet].cmdTimerIndex = -1;
	viewParmSets[curParmSet].cmdProcessID = 0;
	viewParmSets[curParmSet].cmdWindowCreated = FALSE;
	m_startMCNelectron.SetWindowText("Start MCNelectron");
}

void CMCNScriptDlg::KillMGEDtimer()
{
	if (mgedTimerID) {
		KillTimer(mgedTimerID);
		iParmSetFromMGEDtimerIndex.RemoveAt(mgedTimerIndex);
		INT_PTR nMGEDtimers = iParmSetFromMGEDtimerIndex.GetSize();
		for (int i = 0; i < nMGEDtimers; i++) {
			int j = iParmSetFromMGEDtimerIndex[i];
			if (viewParmSets[j].mgedTimerIndex > mgedTimerIndex) viewParmSets[j].mgedTimerIndex--;
		}
		mgedTimerID = 0;
		mgedTimerIndex = -1;
	}
	mgedProcessID = 0;
	mgedWindowCreated = FALSE;
	viewParmSets[curParmSet].mgedTimerID = 0;
	viewParmSets[curParmSet].mgedTimerIndex = -1;
	viewParmSets[curParmSet].mgedProcessID = 0;
	viewParmSets[curParmSet].mgedWindowCreated = FALSE;
	m_startMGED.SetWindowText("Start MGED");
}

void CStartButton::OnRButtonDown(UINT nFlags, CPoint point)
{
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	if (!(*pTimerID)) {
		CButton::OnRButtonDown(nFlags, point);
		return;
	}
	CMenu menu;
	menu.CreatePopupMenu();
	menu.AppendMenu(MF_ENABLED|MF_STRING, ID_RESETBUTTON, "Reset this button");
	CRect windowRect;
	GetWindowRect(&windowRect);
	menu.TrackPopupMenu(TPM_LEFTALIGN, windowRect.left + point.x, windowRect.top + point.y, this, NULL);
}

BEGIN_MESSAGE_MAP(CStartButton, CButton)
	ON_WM_RBUTTONDOWN()
END_MESSAGE_MAP()
BEGIN_MESSAGE_MAP(CStartButton_MCNelectron, CStartButton)
	ON_COMMAND(ID_RESETBUTTON, OnResetButton)
END_MESSAGE_MAP()
BEGIN_MESSAGE_MAP(CStartButton_MGED, CStartButton)
	ON_COMMAND(ID_RESETBUTTON, OnResetButton)
END_MESSAGE_MAP()

void CStartButton_MCNelectron::OnResetButton()
{
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	p->KillCmdTimer();
}

void CStartButton_MGED::OnResetButton()
{
	CMCNScriptDlg *p = (CMCNScriptDlg *)GetParent();
	p->KillMGEDtimer();
}

void CMCNScriptDlg::OnEditName()
{
	CEditNameDlg dlg(this);
	INT_PTR rc = dlg.DoModal();
	if (rc == IDCANCEL) return;
	m_parmSetList.DeleteString(curParmSet);
	char tmp[1000];
	sprintf(tmp, " (%d)  %s", curParmSet + 1, parmSetName);
	m_parmSetList.InsertString(curParmSet, tmp);
	m_parmSetList.SetCurSel(curParmSet);
}

void CMCNScriptDlg::OnStartMGED()
{
	if (mgedProcessID) {
		HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, mgedProcessID);
		if (hProcess) {
			CloseHandle(hProcess);
			HWND handles[NHANDLES_MAX];
//			int nWndHandles = find_main_window2(mgedProcessID, handles);
			int nWndHandles = find_main_window3(mgedProcessID, handles); // find_main_window3 is much faster than find_main_window2, but finds only the first-generation child processes
			if (nWndHandles > 0) {
				for (int i = nWndHandles - 1; i >= 0; i--) {
					CWnd *pWnd = CWnd::FromHandle(handles[i]);
					if (pWnd) {
						pWnd->ShowWindow(SW_RESTORE);
						pWnd->ActivateTopParent();
						pWnd->BringWindowToTop();
					}
				}
				return;
			}
			else if (!mgedWindowCreated || nWndHandles < 0) { // A failure to get the main window handle probably means that the window has not been created yet
				if (MessageBox("The mged.exe window could not be activated. Do you want to reset the \"Start MGED\" button?", NULL, MB_YESNO | MB_ICONINFORMATION | MB_DEFBUTTON2) != IDYES) return;
			}
		}
		KillMGEDtimer();
		return;
	}
	char MGEDexeName[1000] = {'\0'}, drive[1000] = {'\0'}, dir[1000] = {'\0'}, ext[1000] = {'\0'};
	size_t cmdLineLength = strlen(MGEDcmdLine);
	char *cmdLineArgs;
	char MGEDcmdLine_tmp[3*FNLEN+1];
	strcpy(MGEDcmdLine_tmp, MGEDcmdLine);
	char *MGEDpath = strtok(MGEDcmdLine_tmp + strspn(MGEDcmdLine_tmp, " \t\""), "\"");
	if (MGEDpath) {
		_splitpath(MGEDpath, drive, dir, MGEDexeName, ext);
		if (MGEDpath + strlen(MGEDpath) != MGEDcmdLine_tmp + cmdLineLength) cmdLineArgs = MGEDpath + strlen(MGEDpath) + 1;
		else cmdLineArgs = NULL;
	}
	else cmdLineArgs = NULL;
	if (strspn(MGEDexeName, " \t") == strlen(MGEDexeName)) {
		OnBrowseMGED();
		strcpy(MGEDcmdLine_tmp, MGEDcmdLine);
		MGEDpath = strtok(MGEDcmdLine_tmp, "\"");
		_splitpath(MGEDpath, drive, dir, MGEDexeName, ext);
		if (strspn(MGEDexeName, " \t") == strlen(MGEDexeName)) {
			MessageBox("The name of the executable file has not been specified. Enter the full path to the MGED executable file in order to start the Multi-Device Geometry Editor.", NULL, MB_OK | MB_ICONINFORMATION);
			m_MGEDcommandLine.SetFocus();
			m_MGEDcommandLine.SetSel(0, -1);
			return;
		}
	}
	strcat(MGEDexeName, ext);
	DWORD processes[10000], nProcesses;
	char tmp[1000];
	bool nameDifferent = (strcmpi(MGEDexeName, "mged.exe") != 0);
	if(EnumProcesses(processes, sizeof(processes), &nProcesses)) {
		unsigned int i;
		bool originalMGEDrunning = false;
		for(i = 0; i < nProcesses; i++) {
			if (!processes[i]) continue;
			HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, processes[i]);
			char processName[1000];
			GetModuleBaseNameA(hProcess, 0, processName, sizeof(processName));
			CloseHandle(hProcess);
			if (!strcmpi(MGEDexeName, processName)) break;
			if (nameDifferent && !strcmpi("mged.exe", processName)) originalMGEDrunning = true;
		}
		if (i < nProcesses || originalMGEDrunning) {
			sprintf(tmp, "The process \"%s\" is already running. Do you want to start another instance?", (i < nProcesses ? MGEDexeName : "mged.exe"));
			if (MessageBox(tmp, NULL, MB_YESNOCANCEL | MB_ICONQUESTION | MB_DEFBUTTON2) != IDYES) return;
		}
	}
	SHELLEXECUTEINFO shellExecuteInfo;
	memset(&shellExecuteInfo, 0, sizeof(SHELLEXECUTEINFO));
	shellExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	shellExecuteInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
	shellExecuteInfo.lpFile = MGEDpath;
	shellExecuteInfo.lpParameters = cmdLineArgs;
	if (scriptFolder[0] != '\0') shellExecuteInfo.lpDirectory = scriptFolder;
	else shellExecuteInfo.lpDirectory = currentFolder;
	shellExecuteInfo.nShow = SW_SHOW;
	if (ShellExecuteEx(&shellExecuteInfo) && (int)shellExecuteInfo.hInstApp > 32) {
		mgedProcessID = GetProcessId(shellExecuteInfo.hProcess);
		if (!mgedProcessID) {
			MessageBox("The process ID of the created MGED window could not be obtained.", NULL, MB_OK | MB_ICONWARNING);
			return;
		}
		timerCount++;
		mgedTimerID = SetTimer(timerCount, 200, NULL);
		mgedTimerIndex = iParmSetFromMGEDtimerIndex.GetSize();
		m_startMGED.SetWindowText("Activate the MGED window");
		viewParmSets[curParmSet].mgedTimerID = mgedTimerID;
		viewParmSets[curParmSet].mgedTimerIndex = mgedTimerIndex;
		viewParmSets[curParmSet].mgedProcessID = mgedProcessID;
		viewParmSets[curParmSet].mgedWindowCreated = FALSE;
		iParmSetFromMGEDtimerIndex.Add(curParmSet);
	}
	else MessageBox((CString)"An error occurred when attempting to run the program \"" + MGEDexeName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnBrowseMGED()
{
	size_t cmdLineLength = strlen(MGEDcmdLine);
	char MGEDcmdLine_tmp[3*FNLEN+1];
	strcpy(MGEDcmdLine_tmp, MGEDcmdLine);
	char *MGEDpath = strtok(MGEDcmdLine_tmp, "\"");
	char *cmdLineArgs;
	if (MGEDpath && MGEDpath + strlen(MGEDpath) != MGEDcmdLine_tmp + cmdLineLength) cmdLineArgs = MGEDpath + strlen(MGEDpath) + 1;
	else cmdLineArgs = NULL;
	char buffer[1000];
	GetFileName(&m_MGEDcommandLine, "Locate the executable file of the Multi-Device Geometry Editor (MGED)", "Programs (*.exe)|*.exe|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!MGEDpath || strcmp(ShortenPath(buffer, currentFolder), ShortenPath(MGEDpath, currentFolder))) changed = true;
	if (cmdLineArgs) strcpy(MGEDcmdLine, (CString)"\"" + buffer + "\"" + cmdLineArgs);
	else strcpy(MGEDcmdLine, buffer);
}

void CMCNScriptDlg::OnHideShow()
{
	CRect windowRect;
	GetWindowRect(&windowRect);
	int i;
	if (graphicalParametersShown) {
		for (i = IDC_STATIC1; i <= IDC_STATIC24; i++) GetDlgItem(i)->ShowWindow(SW_HIDE);
		m_nTracks.ShowWindow(SW_HIDE);
		m_coordSys.ShowWindow(SW_HIDE);
		m_viewSizeEdit.ShowWindow(SW_HIDE);
		m_maximumObjectSizeRatioEdit.ShowWindow(SW_HIDE);
		m_sourceSizeRatioEdit.ShowWindow(SW_HIDE);
		m_azimuthEdit.ShowWindow(SW_HIDE);
		m_elevationEdit.ShowWindow(SW_HIDE);
		m_twistEdit.ShowWindow(SW_HIDE);
		GetDlgItem(IDC_AZIMUTHSPIN)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_ELEVATIONSPIN)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_TWISTSPIN)->ShowWindow(SW_HIDE);
		m_perspectiveEdit.ShowWindow(SW_HIDE);
		m_cutawayPlaneCheck.ShowWindow(SW_HIDE);
		m_raytraceInGraphicsWindowCheck.ShowWindow(SW_HIDE);
		m_showWireframeCheck.ShowWindow(SW_HIDE);
		m_raytraceInLingeringWindowCheck.ShowWindow(SW_HIDE);
		m_showTracksCheck.ShowWindow(SW_HIDE);
		m_portNumEdit.ShowWindow(SW_HIDE);
		m_nIterationsEdit.ShowWindow(SW_HIDE);
		m_keepTrackDataInMemoryCheck.ShowWindow(SW_HIDE);
		m_nVerticesTotalMaxEdit.ShowWindow(SW_HIDE);
		m_nVerticesTrackMaxEdit.ShowWindow(SW_HIDE);
		GetDlgItem(IDC_RESET)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_DEFAULTFILTERING)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_REFRESH)->ShowWindow(SW_HIDE);
		m_colorsGrid.ShowWindow(SW_HIDE);
		m_beamAxisColor.ShowWindow(SW_HIDE);
		m_beamAxisColorPic.ShowWindow(SW_HIDE);
		m_backgroundColor.ShowWindow(SW_HIDE);
		m_backgroundColorPic.ShowWindow(SW_HIDE);
		GetDlgItem(IDC_TRACKCOLORS)->ShowWindow(SW_HIDE);
		m_viewCenterGrid.ShowWindow(SW_HIDE);
		m_cutawayPlanePointGrid.ShowWindow(SW_HIDE);
		m_cutawayPlaneNormalGrid.ShowWindow(SW_HIDE);
		m_showTrackSegmentsCheck.ShowWindow(SW_HIDE);
		m_extendEscapeTracksCheck.ShowWindow(SW_HIDE);
		windowRect.bottom = windowRect.top + clippedHeight;
		GetDlgItem(IDC_HIDESHOW)->SetWindowText("Show graphical parameters");
		m_trackFilteringTabCtrl.ShowWindow(SW_HIDE);
		graphicalParametersShown = false;
	}
	else {
		for (i = IDC_STATIC1; i <= IDC_STATIC24; i++) GetDlgItem(i)->ShowWindow(SW_SHOW);
		m_nTracks.ShowWindow(SW_SHOW);
		m_coordSys.ShowWindow(SW_SHOW);
		m_viewSizeEdit.ShowWindow(SW_SHOW);
		m_maximumObjectSizeRatioEdit.ShowWindow(SW_SHOW);
		m_sourceSizeRatioEdit.ShowWindow(SW_SHOW);
		m_azimuthEdit.ShowWindow(SW_SHOW);
		m_elevationEdit.ShowWindow(SW_SHOW);
		m_twistEdit.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_AZIMUTHSPIN)->ShowWindow(SW_SHOW);
		GetDlgItem(IDC_ELEVATIONSPIN)->ShowWindow(SW_SHOW);
		GetDlgItem(IDC_TWISTSPIN)->ShowWindow(SW_SHOW);
		m_perspectiveEdit.ShowWindow(SW_SHOW);
		m_cutawayPlaneCheck.ShowWindow(SW_SHOW);
		m_raytraceInGraphicsWindowCheck.ShowWindow(SW_SHOW);
		m_showWireframeCheck.ShowWindow(SW_SHOW);
		m_raytraceInLingeringWindowCheck.ShowWindow(SW_SHOW);
		m_showTracksCheck.ShowWindow(SW_SHOW);
		m_portNumEdit.ShowWindow(SW_SHOW);
		m_nIterationsEdit.ShowWindow(SW_SHOW);
		m_keepTrackDataInMemoryCheck.ShowWindow(SW_SHOW);
		m_nVerticesTotalMaxEdit.ShowWindow(SW_SHOW);
		m_nVerticesTrackMaxEdit.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_RESET)->ShowWindow(SW_SHOW);
		GetDlgItem(IDC_DEFAULTFILTERING)->ShowWindow(SW_SHOW);
		GetDlgItem(IDC_REFRESH)->ShowWindow(SW_SHOW);
		m_colorsGrid.ShowWindow(SW_SHOW);
		m_beamAxisColor.ShowWindow(SW_SHOW);
		m_beamAxisColorPic.ShowWindow(SW_SHOW);
		m_backgroundColor.ShowWindow(SW_SHOW);
		m_backgroundColorPic.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_TRACKCOLORS)->ShowWindow(SW_SHOW);
		m_viewCenterGrid.ShowWindow(SW_SHOW);
		m_cutawayPlanePointGrid.ShowWindow(SW_SHOW);
		m_cutawayPlaneNormalGrid.ShowWindow(SW_SHOW);
		m_showTrackSegmentsCheck.ShowWindow(SW_SHOW);
		m_extendEscapeTracksCheck.ShowWindow(SW_SHOW);
		m_trackFilteringTabCtrl.ShowWindow(SW_SHOW);
		windowRect.bottom = windowRect.top + initialHeight;
		GetDlgItem(IDC_HIDESHOW)->SetWindowText("Hide graphical parameters");
		graphicalParametersShown = true;
	}
	windowRect.right = windowRect.left + initialWidth;
	MoveWindow(&windowRect);
}

void CMCNScriptDlg::OnEditCmdLine()
{
	CCmdLineDlg dlg(this);
	if (dlg.DoModal() == IDOK) {
		UpdateFileNames();
		m_nTracks.SetWindowText("");
		changed = true;
	}
}

void CMCNScriptDlg::UpdateFileNames()
{
	char tmp[SUFFIXLEN+10];
	m_inputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn, currentFolder));
	m_suffix.SetWindowText(suffix);
	m_useCustomTrackFN.SetCheck(useCustomTrackFN);
	m_loadTrackFile.EnableWindow(useCustomTrackFN);
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, curParmSet + 1);
		m_outputFileName.disable = TRUE;
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		if (!useCustomTrackFN) {
			m_trackFileName.disable = TRUE;
			m_trackFileName.SetWindowText(ShortenPath((CString)inputFileFolder + "Files_" + fn1 + tmp + ".txt\\" + fn1 + tmp + "_tracks.txt", currentFolder));
		}
	}
	else {
		m_outputFileName.disable = FALSE;
		m_outputFileName.SetWindowText(ShortenPath(outputFileName, currentFolder));
		if (!useCustomTrackFN) {
			m_trackFileName.disable = TRUE;
			char drive[1000], dir[1000], fname[1000], ext[1000];
			_splitpath(outputFileName, drive, dir, fname, ext);
			m_trackFileName.SetWindowText(ShortenPath((CString)drive + dir + "Files_" + fname + ext + "\\" + fname + "_tracks.txt", currentFolder));
		}
	}
	if (useCustomTrackFN) {
		m_trackFileName.disable = FALSE;
		m_trackFileName.SetWindowText(ShortenPath(trackFN, currentFolder));
	}
	m_logSuffix.SetWindowText(logSuffix);
	if (createLog == 1) m_createLog.SetCheck(BST_CHECKED);
	else if (createLog == 0) m_createLog.SetCheck(BST_UNCHECKED);
	else m_createLog.SetCheck(BST_INDETERMINATE);
	if (logSuffix[0] != '\0') {
		sprintf(tmp, logSuffix, curParmSet + 1);
		m_logFileName.disable = TRUE;
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
	}
	else {
		m_logFileName.disable = FALSE;
		m_logFileName.SetWindowText(ShortenPath(logFileName, currentFolder));
	}
	if (checkSyntax_MCNScript) m_checkSyntax_MCNScript.SetCheck(BST_CHECKED);
	else m_checkSyntax_MCNScript.SetCheck(BST_UNCHECKED);
	m_copyToClipboard.SetCheck(copyToClipboard);
	MakeCmdLine();
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
}

void CMCNScriptDlg::OnCheckSyntax()
{
	if (m_checkSyntax_MCNScript.GetCheck() == BST_CHECKED) checkSyntax_MCNScript = TRUE;
	else checkSyntax_MCNScript = FALSE;
	changed = true;
}

void CMCNScriptDlg::OnAutosizeGrid()
{
	pTrackFilterGrid->ResetWidth();
}

void CMCNScriptDlg::OnReset()
{
	int rc = MessageBox("All graphical parameters of the current parameter set (colors, coordinates, angles, etc.) will be reset to their default values. Do you want to proceed?", NULL, MB_YESNOCANCEL | MB_ICONWARNING | MB_DEFBUTTON2);
	if (rc != IDYES) return;
	DefaultParmSet(true);
	viewParmSets[curParmSet] = *((VIEWPARM *)&maximumObjectSizeRatio);
	REGIONDATA *r = regionData[curParmSet];
	int i;
	for (i = 0; i < nRegions; i++) r[i] = regions[i];
	CRect windowRect;
	GetWindowRect(&windowRect);
	windowRect.bottom = windowRect.top + initialHeight;
	windowRect.right = windowRect.left + initialWidth;
	MoveWindow(&windowRect);
	UpdateViewParms();
	m_colorsGrid.EnsureVisible(0, 0);
	m_nTracks.SetWindowText("");
	changed = true;
}

void CMCNScriptDlg::OnMaximumObjectSizeRatioEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_maximumObjectSizeRatioEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, 3.5, 1000.);
	if (outputString[0] != '\0' && num != maximumObjectSizeRatio) {
		changed = true;
		maximumObjectSizeRatio = num;
	}
	sprintf(tmp, "%g", maximumObjectSizeRatio); m_maximumObjectSizeRatioEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnSourceSizeRatioEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_sourceSizeRatioEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, 0., 0.1);
	if (outputString[0] != '\0' && num != sourceSizeRatio) {
		changed = true;
		sourceSizeRatio = num;
	}
	sprintf(tmp, "%g", sourceSizeRatio); m_sourceSizeRatioEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnViewSizeEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_viewSizeEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, 5e-12, 1e300);
	if (outputString[0] != '\0' && num != viewSize) {
		changed = true;
		viewSize = num;
	}
	sprintf(tmp, "%g", viewSize); m_viewSizeEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnAzimuthEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_azimuthEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, -360., 360.);
	if (outputString[0] != '\0' && num != azimuth) {
		changed = true;
		azimuth = num;
		m_coordSys.Invalidate();
		m_coordSys.UpdateWindow();
	}
	sprintf(tmp, "%g", azimuth); m_azimuthEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnElevationEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_elevationEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, -90., 90.);
	if (outputString[0] != '\0' && num != elevation) {
		changed = true;
		elevation = num;
		m_coordSys.Invalidate();
		m_coordSys.UpdateWindow();
	}
	sprintf(tmp, "%g", elevation); m_elevationEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnTwistEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_twistEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, -360., 360.);
	if (outputString[0] != '\0' && num != twist) {
		changed = true;
		twist = num;
		m_coordSys.Invalidate();
		m_coordSys.UpdateWindow();
	}
	sprintf(tmp, "%g", twist); m_twistEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnAzimuthSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);
	m_MGEDcmdLineCapt.SetFocus();
	double increment = pNMUpDown->iDelta < 0 ? 5. : -5.;
	azimuth += increment;
	if (azimuth >= 360.) azimuth -= 720.;
	else if (azimuth <= -360.) azimuth += 720.;
	char tmp[100];
	changed = true;
	sprintf(tmp, "%g", azimuth); m_azimuthEdit.SetWindowText(tmp);
	m_coordSys.Invalidate();
	m_coordSys.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnElevationSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);
	m_MGEDcmdLineCapt.SetFocus();
	double increment = pNMUpDown->iDelta < 0 ? 5. : -5.;
	double newValue = elevation + increment;
	if (newValue > 90.) {
		if (newValue / 90. - 1. < 1e-8) elevation = 90.;
		else {
			*pResult = 1;
			return;
		}
	}
	else if (newValue < -90.) {
		if (-newValue / 90. - 1. < 1e-8) elevation = -90.;
		else {
			*pResult = 1;
			return;
		}
	}
	else elevation = newValue;
	char tmp[100];
	changed = true;
	sprintf(tmp, "%g", elevation); m_elevationEdit.SetWindowText(tmp);
	m_coordSys.Invalidate();
	m_coordSys.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnTwistSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);
	m_MGEDcmdLineCapt.SetFocus();
	double increment = pNMUpDown->iDelta < 0 ? 5. : -5.;
	twist += increment;
	if (twist >= 360.) twist -= 720.;
	else if (twist <= -360.) twist += 720.;
	char tmp[100];
	changed = true;
	sprintf(tmp, "%g", twist); m_twistEdit.SetWindowText(tmp);
	m_coordSys.Invalidate();
	m_coordSys.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnPerspectiveEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_perspectiveEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	double num = CheckDouble(tmp, outputString, 0., 90.);
	if (outputString[0] != '\0' && num != perspective) {
		changed = true;
		perspective = num;
	}
	sprintf(tmp, "%g", perspective); m_perspectiveEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnPortNumEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_portNumEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num = CheckInteger(tmp, outputString, 1, 10000);
	if (outputString[0] != '\0' && num != portNum) {
		changed = true;
		portNum = num;
	}
	sprintf(tmp, "%d", portNum); m_portNumEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnNiterationsEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_nIterationsEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num = CheckInteger(tmp, outputString, 0, 50);
	if (outputString[0] != '\0' && num != nIterations) {
		changed = true;
		nIterations = num;
	}
	sprintf(tmp, "%d", nIterations); m_nIterationsEdit.SetWindowText(tmp);
}

void CMCNScriptDlg::OnNverticesTotalMaxEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_nVerticesTotalMaxEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num;
	bool empty = (strspn(tmp, " \t") == strlen(tmp));
	if (empty) num = -1;
	else num = CheckInteger(tmp, outputString, -1, 2147483647);
	if ((empty || outputString[0] != '\0' && (num == -1 || num >= 10000)) && num != nVerticesTotalMax) {
		changed = true;
		nVerticesTotalMax = num;
	}
	if (nVerticesTotalMax > 0) {
		sprintf(tmp, "%d", nVerticesTotalMax);
		m_nVerticesTotalMaxEdit.SetWindowText(tmp);
	}
	else m_nVerticesTotalMaxEdit.SetWindowText("");
}

void CMCNScriptDlg::OnNverticesTrackMaxEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_nVerticesTrackMaxEdit.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num;
	bool empty = (strspn(tmp, " \t") == strlen(tmp));
	if (empty) num = -1;
	else num = CheckInteger(tmp, outputString, -1, 2147483647);
	if ((empty || outputString[0] != '\0' && (num == -1 || num >= 4)) && num != nVerticesTrackMax) {
		changed = true;
		nVerticesTrackMax = num;
	}
	if (nVerticesTrackMax > 0) {
		sprintf(tmp, "%d", nVerticesTrackMax);
		m_nVerticesTrackMaxEdit.SetWindowText(tmp);
	}
	else m_nVerticesTrackMaxEdit.SetWindowText("");
}

void CMCNScriptDlg::OnShowTrackSegments()
{
	if (m_showTrackSegmentsCheck.GetCheck()) showTrackSegments = true;
	else showTrackSegments = false;
	changed = true;
	if (CompareTrackFilters(&trackFilter, &defaultTrackFilter) && CompareTrackSegmentData(&trackSegmentSelection, &defaultTrackSegmentSelection)) m_nTracks.SetWindowText("");
}

void CMCNScriptDlg::OnExtendEscapeTracks()
{
	if (m_extendEscapeTracksCheck.GetCheck()) extendEscapeTracks = true;
	else extendEscapeTracks = false;
	changed = true;
}

void CMCNScriptDlg::OnMGEDsuffixEdit()
{
	char MGEDsuffix_bak[SUFFIXLEN+1];
	strcpy(MGEDsuffix_bak, MGEDsuffix);
	char tmp[SUFFIXLEN+10];
	int nBytes = m_MGEDsuffixEdit.GetLine(0, tmp, SUFFIXLEN);
	tmp[nBytes] = '\0';
	char *ptr = strtok(tmp, " \t");
	if (!ptr) MGEDsuffix[0] = '\0';
	else {
		strcpy(MGEDsuffix, ptr);
		ptr = strchr(MGEDsuffix, '%');
		if (ptr && (*(ptr + 1) != 'd' || *(ptr + 2) != '\0' && strchr(ptr + 2, '%'))) strcpy(MGEDsuffix, MGEDsuffix_bak);
	}
	m_MGEDsuffixEdit.SetWindowText(MGEDsuffix);
	if (strcmp(MGEDsuffix_bak, MGEDsuffix)) changed = true;
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
}

void CMCNScriptDlg::OnInputFileName()
{
	char inputFileName_old[FNLEN+1], inputFileName_new[FNLEN+1];
	strcpy(inputFileName_old, (CString)inputFileFolder + fn);
	if (!OnCommandLine(&m_inputFileName, inputFileName_old, inputFileName_new)) return;
	changed = true;
	fileLoaded = false;
	char drive[1000], dir[1000], ext[1000], tmp[SUFFIXLEN+10];
	_splitpath(inputFileName_new, drive, dir, fn1, ext);
	strcpy(inputFileFolder, (CString)drive + dir);
	strcpy(fn, (CString)fn1 + ext);
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, curParmSet + 1);
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		if (!useCustomTrackFN) m_trackFileName.SetWindowText(ShortenPath((CString)inputFileFolder + "Files_" + fn1 + tmp + ".txt\\" + fn1 + tmp + "_tracks.txt", currentFolder));
	}
	if (logSuffix[0] != '\0') {
		sprintf(tmp, logSuffix, curParmSet + 1);
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
	}
	MakeCmdLine();
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
	m_nTracks.SetWindowText("");
}

void CMCNScriptDlg::OnOutputFileName()
{
	if (m_outputFileName.disable) return;
	char outputFileName_old[FNLEN+1];
	strcpy(outputFileName_old, outputFileName);
	if (!OnCommandLine(&m_outputFileName, outputFileName_old, outputFileName)) return;
	if (!useCustomTrackFN) {
		char drive[1000], dir[1000], fname[1000], ext[1000];
		_splitpath(outputFileName, drive, dir, fname, ext);
		m_trackFileName.SetWindowText(ShortenPath((CString)drive + dir + "Files_" + fname + ext + "\\" + fname + "_tracks.txt", currentFolder));
	}
	changed = true;
	MakeCmdLine();
	m_nTracks.SetWindowText("");
}

void CMCNScriptDlg::OnLogFileName()
{
	if (m_logFileName.disable) return;
	char logFileName_old[FNLEN+1];
	strcpy(logFileName_old, logFileName);
	if (!OnCommandLine(&m_logFileName, logFileName_old, logFileName)) return;
	changed = true;
	MakeCmdLine();
}

void CMCNScriptDlg::OnTrackFileName()
{
	if (m_trackFileName.disable) return;
	char trackFN_old[FNLEN+1];
	strcpy(trackFN_old, trackFN);
	if (!OnCommandLine(&m_trackFileName, trackFN_old, trackFN)) return;
	changed = true;
	m_nTracks.SetWindowText("");
}

void CMCNScriptDlg::OnMGEDCommandLine()
{
	char MGEDcmdLine_old[3*FNLEN+1];
	strcpy(MGEDcmdLine_old, MGEDcmdLine);
	if (OnCommandLine(&m_MGEDcommandLine, MGEDcmdLine_old, MGEDcmdLine)) changed = true;
}

bool CMCNScriptDlg::OnCommandLine(CEdit *pEdit, char *cmdLine_old, char *cmdLine_new, bool isFolder)
{
	size_t cmdLineLength_old = strlen(cmdLine_old);
	char *path = strtok(cmdLine_old, "\"");
	char *cmdLineArgs_old;
	if (path && path + strlen(path) != cmdLine_old + cmdLineLength_old) cmdLineArgs_old = path + strlen(path) + 1;
	else cmdLineArgs_old = NULL;

	char cmdLine_new_tmp[1000], *path_new_char;
	CString path_old('\0', 1000), path_new('\0', 1000);
	if (path) path_old = ShortenPath((CString)path, currentFolder);

	int nBytes = pEdit->GetLine(0, cmdLine_new_tmp, 999);
	cmdLine_new_tmp[nBytes] = '\0';

	size_t cmdLineLength_new = strlen(cmdLine_new_tmp);
	path_new_char = strtok(cmdLine_new_tmp + strspn(cmdLine_new_tmp, " \t\""), "\"");
	char *cmdLineArgs_new;
	if (path_new_char && path_new_char + strlen(path_new_char) != cmdLine_new_tmp + cmdLineLength_new) {
		cmdLineArgs_new = path_new_char + strlen(path_new_char) + 1;
		if (strspn(cmdLineArgs_new, " \t\"") == strlen(cmdLineArgs_new)) cmdLineArgs_new = NULL;
	}
	else cmdLineArgs_new = NULL;

	if (path_new_char) path_new = path_new_char;
	path_new.TrimLeft();
	path_new.TrimRight();
	bool checkFolderName = isFolder && path_new.GetLength();
	if (checkFolderName && path_new.Right(1) != '\\') path_new += "\\";
	if (cmdLineArgs_new) strcpy(cmdLine_new, "\"" + path_new + "\"" + cmdLineArgs_new);
	else strcpy(cmdLine_new, path_new);

	path_new = ShortenPath(path_new, currentFolder);
	if (checkFolderName && !path_new.GetLength()) {
		path_new = (CString)currentFolder + ".\\";
		if (cmdLineArgs_new) strcpy(cmdLine_new, "\"" + path_new + "\"" + cmdLineArgs_new);
		else strcpy(cmdLine_new, path_new);
		path_new = ".\\";
	}
	if (cmdLineArgs_new) pEdit->SetWindowText("\"" + path_new + "\"" + cmdLineArgs_new);
	else pEdit->SetWindowText(path_new);

	if (path_new.CompareNoCase(path_old) ||
		!cmdLineArgs_old && cmdLineArgs_new || cmdLineArgs_old && !cmdLineArgs_new ||
		cmdLineArgs_old && cmdLineArgs_new && strcmp(cmdLineArgs_old, cmdLineArgs_new)) return true;
	else return false;
}

void CMCNScriptDlg::OnSystemVars()
{
	if (!systemVarsShown) {
		CRect systemVarsWindowRect, windowRect;
		GetWindowRect(&windowRect);
		pSystemVarsDlg->GetWindowRect(&systemVarsWindowRect);
		int width = systemVarsWindowRect.right - systemVarsWindowRect.left;
		int height = systemVarsWindowRect.bottom - systemVarsWindowRect.top;
		int hor_shift = (windowRect.right - windowRect.left - width) / 2;
		int vert_shift = (windowRect.bottom - windowRect.top - height) / 2;
		systemVarsWindowRect.left = windowRect.left + hor_shift;
		systemVarsWindowRect.right = systemVarsWindowRect.left + width;
		systemVarsWindowRect.top = windowRect.top + vert_shift;
		systemVarsWindowRect.bottom = systemVarsWindowRect.top + height;
		pSystemVarsDlg->SetWindowPos(&wndTop, systemVarsWindowRect.left, systemVarsWindowRect.top, systemVarsWindowRect.right - systemVarsWindowRect.left, systemVarsWindowRect.bottom - systemVarsWindowRect.top, SWP_SHOWWINDOW);
		systemVarsShown = true;
	}
	else {
		pSystemVarsDlg->ShowWindow(SW_SHOWNORMAL);
		pSystemVarsDlg->SetForegroundWindow();
	}
}

void CMCNScriptDlg::UpdateRegionNames()
{
	int i, j, k, m;
	memset(nCellsUsingTheMaterial, 0, nMaterials * sizeof(int));
	for (i = 0; i < nCells; i++) {
		int mat = cell[i].mat;
		if (!nCellsUsingTheMaterial[mat]) cellsUsingTheMaterial[mat] = (int *)malloc(sizeof(int));
		else cellsUsingTheMaterial[mat] = (int *)realloc(cellsUsingTheMaterial[mat], (nCellsUsingTheMaterial[mat] + 1) * sizeof(int));
		cellsUsingTheMaterial[mat][nCellsUsingTheMaterial[mat]] = i;
		nCellsUsingTheMaterial[mat]++;
	}
	if (usedMaterialNum) {
		delete usedMaterialNum;
		usedMaterialNum = NULL;
	}
	nUsedMaterials = 0;
	for (i = 0; i < nMaterials; i++) {
		if (nCellsUsingTheMaterial[i]) {
			if (!nUsedMaterials) usedMaterialNum = new int[nMaterials];
			usedMaterialNum[nUsedMaterials] = i;
			nUsedMaterials++;
		}
	}
	nMaterialsWithSource = nUsedMaterials;
	if (sourcePositionType < 3 && sourceSizeRatio > 0.) nMaterialsWithSource++;
	CString msg;
	for (k = 0, m = 0; m < nMaterialsWithSource; m++) {
		if (k == nRegions) {
			nRegions++;
			m_colorsGrid.InsertRow((LPCTSTR)(msg.Format(_T("%d"), nRegions), msg));
			if (nUsedDefaultColors == NDEFAULTCOLORS) j = nRegions - 1 - (nRegions - 1) / NDEFAULTCOLORS * NDEFAULTCOLORS;
			else {
				for (j = 0; j < NDEFAULTCOLORS && defaultColorUsed[j]; j++);
			}
			regions.Add(defaultRegionData);
			regions[k].color = defaultColor[j];
			if (!defaultColorUsed[j]) nUsedDefaultColors++;
			defaultColorUsed[j]++;
		}
		char prevName[REGIONNAMELEN+1];
		if (!changed) strcpy(prevName, regions[k].name);
		if (m < nUsedMaterials) {
			i = usedMaterialNum[m];
			if (materialID[i] > 0) j = sprintf(regions[k].name, "M%d", materialID[i]);
			else j = sprintf(regions[k].name, "V%d", -materialID[i]);
			if (materialName[i]) sprintf(regions[k].name + j, " (%s)", materialName[i]);
		}
		else strcpy(regions[k].name, "M_source");
		if (!changed && strcmp(regions[k].name, prevName)) changed = true;
		k++;
	}
	for (; k < nRegions; k++) regions[k].name[0] = '\0';
	m_colorsGrid.EnsureVisible(0, 0);
	UpdateColorsView();
}

void CMCNScriptDlg::OnMakeScript()
{
	OnReloadInputFile();
	if (!fileLoaded) return;
	CString msg;
	if (overwritePrompt) {
		FILE *f = fopen(outScriptFN, "r+t");
		if (f) {
			fclose(f);
			int rc = MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" already exists. Do you want to overwrite it?"), outScriptFN), msg), NULL, MB_YESNO | MB_ICONWARNING);
			if (rc == IDNO) return;
		}
	}
	f_out = fopen(outScriptFN, "wt");
	if (!f_out) {
		MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for writing."), outScriptFN), msg), NULL, MB_OK | MB_ICONERROR); return;
	}
	int i, j, k = 0, m;
	char scriptFN[1000], databaseFN[1000];
	if (scriptFolder[0] != '\\' && !strchr(scriptFolder, ':')) k = sprintf(scriptFN, currentFolder);
	char tmp[100];
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(scriptFN + k, "%s%s%s", scriptFolder, fn1, tmp);
	else sprintf(scriptFN + k, "%s%s", fn1, tmp);
	strcpy(databaseFN, scriptFN);
	strcat(databaseFN, ".g");
	for (i = 0; databaseFN[i] != '\0'; i++) {
		if (databaseFN[i] == '\\') databaseFN[i] = '/';
	}
	fprintf(f_out, "set glob_compat_mode 0\n");
	if (raytraceInLingeringWindow) fprintf(f_out, "catch {exec fbfree -F %d} fbfreeErr\n", portNum);
	fprintf(f_out, "closedb\n");
	if (raytraceInLingeringWindow) {
		fprintf(f_out, "catch {exec fbfree -F %d} fbfreeErr\n", portNum);
		fprintf(f_out, "catch {exec sleep 0.5} sleepErr\n");
	}
	fprintf(f_out, "catch {file delete \"%s\"} result\n", databaseFN);
	fprintf(f_out, "file delete \"%s\"\n", databaseFN);
	fprintf(f_out, "opendb \"%s\" y\n\n", databaseFN);

	int iUnit;
	for (iUnit = 0; iUnit < NMGEDDISTUNITS && MGEDdistUnit[iUnit] < viewSize; iUnit++);
	if (iUnit == NMGEDDISTUNITS || iUnit && (viewSize <= 100. * MGEDdistUnit[iUnit-1] || viewSize <= 0.01 * MGEDdistUnit[iUnit])) iUnit--;
	fprintf(f_out, "units cm\n");
//	fprintf(f_out, "units %s\n", MGEDdistUnitName[iUnit]);
	unitConversionFactor = 1. / MGEDdistUnit[iUnit];
	double worldSize0 = maximumObjectSizeRatio * this->viewSize;
	double halfSize0 = 0.5 * worldSize0;
	double minRadius0 = 0.0001 * this->viewSize;

	double worldSize = unitConversionFactor * worldSize0;
	double halfSize = unitConversionFactor * halfSize0;
	double minRadius = unitConversionFactor * minRadius0;
	double viewSize = unitConversionFactor * this->viewSize;
	VECTOR viewCenter = unitConversionFactor * this->viewCenter;

	CELL world;
	for (i = 0; i < 7; i++) {
		world.s[i] = nSurfaces + i;
		world.side[i] = -1;
	}
	SURFACE *s = surf + nSurfaces;
	for (i = 0; i < 6; i++) s[i].type = 1;
	s[0].dir = xDir;
	s[0].init = s[0].dir * halfSize0;
	s[1].dir = -xDir;
	s[1].init = s[1].dir * halfSize0;
	s[2].dir = yDir;
	s[2].init = s[2].dir * halfSize0;
	s[3].dir = -yDir;
	s[3].init = s[3].dir * halfSize0;
	s[4].dir = zDir;
	s[4].init = s[4].dir * halfSize0;
	s[5].dir = -zDir;
	s[5].init = s[5].dir * halfSize0;
	for (i = 0; i < 6; i++) s[i].D = -s[i].dir * s[i].init;
	bool removeSurface[MAXSURFACES+13], surfacesRemoved;
	bool removeSurface2[MAXSURFACES+13], surfacesRemoved2;
	bool removeClippingPlane[6], clippingPlanesRemoved;
	if (!useCutawayPlane) {
		world.nS = 6;
		fprintf(f_out, "in World rpp   %g %g %g %g %g %g\n", viewCenter.x - 0.5 * worldSize, viewCenter.x + 0.5 * worldSize, viewCenter.y - 0.5 * worldSize, viewCenter.y + 0.5 * worldSize, viewCenter.z - 0.5 * worldSize, viewCenter.z + 0.5 * worldSize);
	}
	else {
		world.nS = 7;
		s[6].type = 1;
		s[6].dir = *((VECTOR *)cutawayPlaneCoefs);
		s[6].init = cutawayPlanePoint;
		s[6].D = -cutawayPlaneCoefs[3];
		world.nContainedCells = 0;
		world.hostCell = -1;
		world.unionNum = -1;
		world.ID = 0;
		bool notImplemented = false;
		CELL *c = &world;
		if (!CheckCellGeometry2(c, distancePrecision, surfacesRemoved2, removeSurface2)) {
			MessageBox((LPCTSTR)(msg.Format(_T("Geometry error: The volume of the \"world\" is zero.")), msg), NULL, MB_OK | MB_ICONERROR); return;
		}
		surfacesRemoved = false;
		memset(removeSurface, 0, c->nS * sizeof(bool));
		for (j = 0; j < c->nS; j++) {
			c->side[j] = -c->side[j];
			if (!CheckCellGeometry2(c, distancePrecision, surfacesRemoved2, removeSurface2)) {
				removeSurface[j] = true;
				surfacesRemoved = true;
			}
			c->side[j] = -c->side[j];
		}
		if (surfacesRemoved) {
			for (k = 0, j = 0; j < c->nS; j++) {
				if (removeSurface[j]) continue;
				c->s[k] = c->s[j];
				c->side[k] = c->side[j];
				k++;
			}
			c->nS = k;
		}
		fprintf(f_out, "in World arbn %d", c->nS);
		for (j = 0; j < c->nS; j++) {
			s = surf + c->s[j];
			fprintf(f_out, "   %g %g %g %g", s->dir.x, s->dir.y, s->dir.z, -unitConversionFactor * s->D);
		}
		fprintf(f_out, "\n");
	}

	double height;
	VECTOR baseCenter, baseCenter2, heightVector;
	for (i = 0; i < nSurfaces; i++) {
		SURFACE *s = surf + i;
		if (s->type == 1 || s->mb_index >= 0) continue;
		if (s->type == 2) fprintf(f_out, "in S%d sph   %g %g %g   %g\n", s->ID, s->init.x * unitConversionFactor, s->init.y * unitConversionFactor, s->init.z * unitConversionFactor, s->R * unitConversionFactor);
		else {
			baseCenter = s->init * unitConversionFactor;
			if (s->type == 1) height = -2. * worldSize;
			else if (s->type == 3) height = 4. * worldSize;
			else height = 2. * worldSize;
			VECTOR diff = baseCenter - viewCenter;
			if (diff.x < -halfSize || diff.x > halfSize || diff.y < -halfSize || diff.y > halfSize || diff.z < -halfSize || diff.z > halfSize) height *= (1. + 0.5 * Norm(diff) / worldSize);
			heightVector = height * s->dir;
			if (s->type == 3) {
				baseCenter -= 0.5 * heightVector;
				fprintf(f_out, "in S%d rcc   %g %g %g   %g %g %g   %g\n", s->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, s->R * unitConversionFactor);
			}
			else if (s->type == 4) fprintf(f_out, "in S%d trc   %g %g %g   %g %g %g   %g %g\n", s->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, minRadius, height * tan(s->angle));
		}
	}
	for (i = 0; i < nMacrobodies; i++) {
		MACROBODY *mb = macrobody + i;
		if (!mb->type) continue; // identifiers of general macrobodies (mb->type = 0) are used only as shorthand notation for a set of two or more surface IDs
		VECTOR dir;
		if (mb->type < 3) {
			baseCenter = ProjectionToPlane(surf[mb->s[0]].init, surf + mb->s[2]) * unitConversionFactor;
			baseCenter2 = ProjectionToPlane(surf[mb->s[0]].init, surf + mb->s[1]) * unitConversionFactor;
			heightVector = baseCenter2 - baseCenter;
			dir = heightVector / Norm(heightVector);
		}
		if (mb->type == 1) { // RCC (right circular cylinder)
			fprintf(f_out, "in S%d rcc   %g %g %g   %g %g %g   %g\n", mb->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, surf[mb->s[0]].R * unitConversionFactor);
			height = 4. * worldSize;
			VECTOR diff = baseCenter - viewCenter;
			if (diff.x < -halfSize || diff.x > halfSize || diff.y < -halfSize || diff.y > halfSize || diff.z < -halfSize || diff.z > halfSize) height *= (1. + 0.5 * Norm(diff) / worldSize);
			heightVector = height * dir;
			baseCenter -= 0.5 * heightVector;
			fprintf(f_out, "in S%dF1 rcc   %g %g %g   %g %g %g   %g\n", mb->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, surf[mb->s[0]].R * unitConversionFactor);
		}
		else if (mb->type == 2) { // TRC (truncated right-angle cone)
			VECTOR coneTip = surf[mb->s[0]].init * unitConversionFactor;
			double tg = surf[mb->s[0]].tan;
			double R1 = Norm(coneTip - baseCenter2) * tg;
			double R2 = Norm(coneTip - baseCenter) * tg;
			fprintf(f_out, "in S%d trc   %g %g %g   %g %g %g   %g %g\n", mb->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, R2, R1);
			height = 2. * worldSize;
			VECTOR diff = baseCenter - viewCenter;
			if (diff.x < -halfSize || diff.x > halfSize || diff.y < -halfSize || diff.y > halfSize || diff.z < -halfSize || diff.z > halfSize) height *= (1. + 0.5 * Norm(diff) / worldSize);
			heightVector = height * surf[mb->s[0]].dir;
			fprintf(f_out, "in S%dF1 trc   %g %g %g   %g %g %g   %g %g\n", mb->ID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, minRadius, height * tan(surf[mb->s[0]].angle));
		}
		else if (mb->type < 5 || mb->type == 7) { // BOX, RPP or ARB4
			fprintf(f_out, "in S%d", mb->ID);
			int nEntries;
			if (mb->type == 3) {
				nEntries = 12;
				fprintf(f_out, " box");
			}
			else if (mb->type == 4) {
				nEntries = 6;
				fprintf(f_out, " rpp");
			}
			else {
				nEntries = 12;
				fprintf(f_out, " arb4");
			}
			for (j = 0; j < nEntries; j++) fprintf(f_out, " %g", unitConversionFactor * arb_entries[i][j]);
			fprintf(f_out, "\n");
		}
		else { // RHP or HEX (right hexagonal prism), WED (wedge), or ARB (a polyhedron with 4 - 8 corners and 4 - 6 sides defined in terms of triplets of corners)
			fprintf(f_out, "in S%d arbn %d", mb->ID, mb->nS);
			for (j = 0; j < mb->nS; j++) {
				SURFACE *s = surf + mb->s[j];
				double mult = mb->sense[j] < 0 ? 1. : -1.;
				fprintf(f_out, "   %g %g %g %g", mult * s->dir.x, mult * s->dir.y, mult * s->dir.z, -mult * unitConversionFactor * s->D);
			}
			fprintf(f_out, "\n");
		}
	}
	fprintf(f_out, "\n");
	char sourceString[100];
	sourceString[0] = '\0';
	if (sourcePositionType < 3 && sourceSizeRatio > 0.) {
		fprintf(f_out, "in S_source sph   %g %g %g    %g\n", sourceCenter.x * unitConversionFactor, sourceCenter.y * unitConversionFactor, sourceCenter.z * unitConversionFactor, 0.5 * sourceSizeRatio * viewSize);
		if (nRegions < nUsedMaterials + 1 || regions[nUsedMaterials].show) { // It is assumed that the default value of REGIONDATA::show is TRUE
			strcpy(sourceString, " - S_source");
		}
		fprintf(f_out, "\n");
	}
	strcat(sourceString, "\n");

	int iSurf_max = -1;
	LINE *intersectionLines[MAXSURFACES];
	memset(intersectionLines, 0, MAXSURFACES * sizeof(LINE *));
	SURFACE *clippingPlane[6];
	for (j = 0; j < 6; j++) clippingPlane[j] = surf + nSurfaces + 7 + j;
	for (i = 0; i < nCells; i++) {
		CELL *c = cell + i;
		CELL c2 = *c;
		for (k = c->nS, j = 0; j < world.nS; j++, k++) {
			c2.s[k] = world.s[j];
			c2.side[k] = world.side[j];
		}
		c2.nS = k;
		int k0 = -1;
		bool baseChanged[MAXSURFACES];
		memset(baseChanged, 0, c->nS * sizeof(bool));
		bool geomErr = !CheckCellGeometry2(&c2, distancePrecision, surfacesRemoved2, removeSurface2);
		if (surfacesRemoved2) {
			for (k = 0, j = 0; j < c2.nS; j++) {
				if (removeSurface2[j]) continue;
				c2.s[k] = c2.s[j];
				c2.side[k] = c2.side[j];
				k++;
			}
			c2.nS = k;
		}
		bool clipped, cut;
		clippingPlanesRemoved = false;
		char pointFound = c2.pointFound;
		VECTOR point = c2.point;
		if (!geomErr) {
			clipped = false;
			cut = false;
			for (j = 0; j < 6; j++) removeClippingPlane[j] = true;
			for (j = 0; j < c2.nS; j++) {
				if (c2.s[j] < nSurfaces) continue; // not a side of the "world"
				c2.side[j] = -c2.side[j];
				if (CheckCellGeometry2(&c2, distancePrecision, surfacesRemoved2, removeSurface2)) { // this side of the world is not redundant
					if (c2.s[j] < nSurfaces + 6) { // this side of the clipping cube is not redundant
						removeClippingPlane[c2.s[j]-nSurfaces] = false;
						clipped = true;
					}
					else cut = true; // the cell intersects with the cut-away plane
				}
				else if (c2.s[j] < nSurfaces + 6) clippingPlanesRemoved = true; // this side of the clipping cube is redundant
				c2.side[j] = -c2.side[j];
			}
		}
		else {
			for (j = 0; j < 6; j++) removeClippingPlane[j] = false;
			clipped = true;
			if (useCutawayPlane) cut = true;
			else cut = false;
		}
		c2.pointFound = pointFound;
		c2.point = point;
		if (clippingPlanesRemoved && !clipped) { // all sides of the clipping cube that were not removed during the initial geometry check (surfacesRemoved2) are redundant
			for (k = 0, j = 0; j < c2.nS; j++) {
				if (c2.s[j] >= nSurfaces && c2.s[j] < nSurfaces + 6) continue; // a side of the clipping cube
				c2.s[k] = c2.s[j];
				c2.side[k] = c2.side[j];
				k++;
			}
			c2.nS = k;
		}
		bool clippingPlanesAdjusted = false;
		int nCylCones_ZeroID = 0; // Number of "separate" cylinders and cones with zero IDs
		for (j = 0; j < c->nS; j++) {
			int mbNum = c->mb[j] > 0 ? c->mb[j] - 1 : -c->mb[j] - 1;
			if (c->i[j] > 0 && macrobody[mbNum].type) continue; // a part of a standard macrobody, specified as a whole (not using the detailed format "<mbNum>.<facetNum>")
			s = surf + c->s[j];
			if (s->type != 3 && s->type != 4) continue; // not a cylinder or a cone
			baseCenter = s->init;
			if (s->type == 3) height = 4. * worldSize0;
			else height = 2. * worldSize0;
			VECTOR diff = s->init - viewCenter;
			if (diff.x < -halfSize0 || diff.x > halfSize0 || diff.y < -halfSize0 || diff.y > halfSize0 || diff.z < -halfSize0 || diff.z > halfSize0) height *= (1. + 0.5 * Norm(diff) / worldSize0);
			heightVector = height * s->dir;
			if (s->type == 3) baseCenter -= 0.5 * heightVector;
			SURFACE base[2];
			base[0].type = 1; base[1].type = 1;
			base[0].init = baseCenter; base[1].init = baseCenter + heightVector;
			base[0].dir = s->dir; base[1].dir = -s->dir;
			base[0].D = -base[0].dir * base[0].init; base[1].D = -base[1].dir * base[1].init;
			if (nIterations && !geomErr) {
				for (k = 0; k < 2; k++) {
					if (AdjustPlanePosition(base + k, &c2, height, distancePrecision, iSurf_max, nIterations, intersectionLines, false, (c->side[j] > 0 ? c->s[j] : -1))) {
						baseChanged[j] = true;
						height = Norm(base[1].init - base[0].init);
					}
				}
			}
			if (baseChanged[j] || !s->ID) { // The bases have been adjusted, or the surface was created as a result of a coordinate transformation and was not assigned an ID
				char surfID[50];
				if (!s->ID) {
					nCylCones_ZeroID++;
					sprintf(surfID, "S0%dC%d", nCylCones_ZeroID, c->ID);
					s->ID = -nCylCones_ZeroID; // A temporary ID (it will be reset to zero at the end of processing of the current cell)
				}
				else if (c->i[j] >= 0) sprintf(surfID, "S%dC%d", s->ID, c->ID); // Not a part of a standard macrobody
				else sprintf(surfID, "S%dF%dC%d", macrobody[mbNum].ID, -c->i[j], c->ID);
				baseCenter = unitConversionFactor * base[0].init;
				heightVector = unitConversionFactor * height * base[0].dir;
				if (s->type == 3) fprintf(f_out, "in %s rcc   %g %g %g   %g %g %g   %g\n", surfID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, s->R * unitConversionFactor);
				else {
					double R1 = Norm(s->init - base[0].init) * tan(s->angle);
					if (R1 < minRadius0) R1 = minRadius0;
					R1 *= unitConversionFactor;
					double R2 = Norm(s->init - base[1].init) * tan(s->angle);
					if (R2 < minRadius0) R2 = minRadius0;
					R2 *= unitConversionFactor;
					fprintf(f_out, "in %s trc   %g %g %g   %g %g %g   %g %g\n", surfID, baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, R1, R2);
				}
			}
		}
		int mbPrev = -1;
		if (c->cellNum0 >= 0) {
			CELL *c0 = cell + c->cellNum0;
			TRANSFORM *tr = cellTransform + i;
			for (k = 0; k < c->nS; k++) {
				int mbNum = c->mb[k] > 0 ? c->mb[k] - 1 : -c->mb[k] - 1;
				if (!c->mb[k] || c->i[k] < 0 || !macrobody[mbNum].type) continue; // not a part of a macrobody, or specified using the detailed format "<mbNum>.<facetNum>", or not one of the standard macrobodies
				if (mbNum == mbPrev) continue; // the next part of a standard macrobody
				MACROBODY *mb = macrobody + mbNum;
				fprintf(f_out, "in S%dC%d", mb->ID, c->ID);
				if (mb->type < 3) { // RCC or TRC
					baseCenter = ProjectionToPlane(surf[mb->s[0]].init, surf + mb->s[2]);
					baseCenter2 = ProjectionToPlane(surf[mb->s[0]].init, surf + mb->s[1]);
					double R1, R2;
					if (mb->type == 2) {
						R1 = Norm(surf[mb->s[0]].init - baseCenter2) * surf[mb->s[0]].tan * unitConversionFactor;
						R2 = Norm(surf[mb->s[0]].init - baseCenter) * surf[mb->s[0]].tan * unitConversionFactor;
					}
					VECTOR heightVector = baseCenter2 - baseCenter;
					baseCenter = LocalToGlobal(tr->origin, tr->Q[0], baseCenter) * unitConversionFactor;
					heightVector = LocalToGlobal(zero, tr->Q[0], heightVector) * unitConversionFactor;
					if (mb->type == 1) fprintf(f_out, " rcc   %g %g %g   %g %g %g   %g\n", baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, surf[mb->s[0]].R * unitConversionFactor);
					else fprintf(f_out, " trc   %g %g %g   %g %g %g   %g %g\n", baseCenter.x, baseCenter.y, baseCenter.z, heightVector.x, heightVector.y, heightVector.z, R2, R1);
				}
				else if (mb->type < 5 || mb->type == 7) { // BOX, RPP or ARB4
					double entries[12];
					double *ptr = arb_entries[mbNum];
					VECTOR v[4];
					int nEntries;
					if (mb->type == 3) {
						nEntries = 12;
						fprintf(f_out, " box");
					}
					else if (mb->type == 4) {
						nEntries = 6;
						v[0] = Init(ptr[0], ptr[2], ptr[4]);
						v[1] = Init(ptr[1] - ptr[0], 0., 0.);
						v[2] = Init(0., ptr[3] - ptr[2], 0.);
						v[3] = Init(0., 0., ptr[5] - ptr[4]);
						for (m = 0, j = 0; j < 4; j++, m+=3) memcpy(entries + m, v + j, 3 * sizeof(double));
						ptr = entries;
					}
					else {
						nEntries = 12;
						fprintf(f_out, " arb4");
					}
					for (m = 0, j = 0; j < 4; j++, m+=3) {
						*((VECTOR *)(entries + m)) = LocalToGlobal(j && mb->type != 7 ? zero : tr->origin, tr->Q[0], *((VECTOR *)(ptr + m)));
					}
					if (mb->type == 4) {
						memcpy(v, entries, 4 * sizeof(VECTOR));
						int num[3];
						VECTOR dir[3];
						dir[0] = xDir;
						dir[1] = yDir;
						dir[2] = zDir;
						for (j = 1; j < 4; j++) {
							for (m = 0; m < 3 && Norm(v[j] & dir[m]) > anglePrecision; m++);
							if (m == 3) break;
							num[m] = j;
						}
						if (k == 6) { // All facets are orthogonal to the coordinate axes, so that there is no need to convert to "BOX"
							VECTOR oppositePoint = v[0] + v[1] + v[2] + v[3];
							if (v[num[0]].x > 0.) {
								entries[0] = v[0].x;
								entries[1] = oppositePoint.x;
							}
							else {
								entries[0] = oppositePoint.x;
								entries[1] = v[0].x;
							}
							if (v[num[1]].y > 0.) {
								entries[2] = v[0].y;
								entries[3] = oppositePoint.y;
							}
							else {
								entries[2] = oppositePoint.y;
								entries[3] = v[0].y;
							}
							if (v[num[2]].z > 0.) {
								entries[4] = v[0].z;
								entries[5] = oppositePoint.z;
							}
							else {
								entries[4] = oppositePoint.z;
								entries[5] = v[0].z;
							}
							fprintf(f_out, " rpp");
						}
						else { // Convert to "BOX"
							nEntries = 12;
							fprintf(f_out, " box");
						}
					}
					for (j = 0; j < nEntries; j++) fprintf(f_out, " %g", unitConversionFactor * entries[j]);
					fprintf(f_out, "\n");
				}
				else { // RHP or HEX (right hexagonal prism), WED (wedge), or ARB (a polyhedron with 4 - 8 corners and 4 - 6 sides defined in terms of triplets of corners)
					fprintf(f_out, " arbn %d", mb->nS);
					for (j = 0; j < mb->nS; j++) {
						SURFACE *s = surf + mb->s[j];
						double mult = mb->sense[j] < 0 ? 1. : -1.;
						VECTOR init = LocalToGlobal(tr->origin, tr->Q[0], s->init);
						VECTOR dir = LocalToGlobal(zero, tr->Q[0], s->dir);
						dir /= Norm(dir);
						double D = -dir * init;
						fprintf(f_out, "   %g %g %g %g", mult * dir.x, mult * dir.y, mult * dir.z, -mult * unitConversionFactor * D);
					}
					fprintf(f_out, "\n");
				}
				mbPrev = mbNum;
			}
		}
		for (j = 0; j < 6; j++) *(clippingPlane[j]) = surf[nSurfaces+j];
		if (!cut) {
			for (j = 0; j < c->nS; j++) {
				if (surf[c->s[j]].type != 1) continue; // not a plane
				int mbNum = c->mb[j] > 0 ? c->mb[j] - 1 : -c->mb[j] - 1;
				if (mbNum < 0 || c->i[j] < 0 || !macrobody[mbNum].type) break; // the plane is not a part of a macrobody (mbNum = -1), or it is specified using the detailed format (c->i[j] < 0), or it is a part of a general (non-standard) macrobody (macrobody[mbNum].type = 0)
			}
		}
		if (!cut && j == c->nS) { // the definition of the cell does not include any "separate" planes (i.e., the planes that are not parts of standard macrobodies, or that are specified using the detailed format "<mbNum.facetNum>")
			if (!clipped) { // all faces of the "world" are redundant
				for (k0 = 0; k0 < c->nS; k0++) { // at least one bounding surface must have negative sense (because the cell is finite and does not include separate planes)
					int mbNum = c->mb[k0] > 0 ? c->mb[k0] - 1 : -c->mb[k0] - 1;
					if (mbNum >= 0 && macrobody[mbNum].type) {
						if (c->i[k0] > 0) break; // a part of a standard macrobody, specified as a whole (not using the detailed format "<mbNum>.<facetNum>")
					}
					else if (c->side[k0] < 0) break; // or a separate surface with negative sense
				}
				fprintf(f_out, "comb C%d u", c->ID);
			}
			else { // the cell is clipped
				if (nIterations && clippingPlanesRemoved) { // some of the clipping planes are redundant
					for (j = 0; j < c2.nS; j++) {
						if (c2.s[j] < nSurfaces || c2.s[j] > nSurfaces + 5) continue; // not a side of the clipping cube
						int iSide = c2.s[j] - nSurfaces;
						if (!removeClippingPlane[iSide]) continue; // not redundant
						int iOpposite = (iSide / 2 * 2 == iSide) ? iSide + 1 : iSide - 1;
						height = fabs(clippingPlane[iSide]->D + clippingPlane[iOpposite]->D); // normals of the opposite sides have opposite directions
						if (AdjustPlanePosition(clippingPlane[iSide], &c2, height, distancePrecision, iSurf_max, nIterations, intersectionLines, true, -1)) clippingPlanesAdjusted = true;
					}
				}
				if (!clippingPlanesAdjusted) fprintf(f_out, "comb C%d u World", c->ID);
				else {
					fprintf(f_out, "in World%d rpp   %g %g %g %g %g %g\n", c->ID,
						unitConversionFactor * clippingPlane[1]->init.x, unitConversionFactor * clippingPlane[0]->init.x,
						unitConversionFactor * clippingPlane[3]->init.y, unitConversionFactor * clippingPlane[2]->init.y,
						unitConversionFactor * clippingPlane[5]->init.z, unitConversionFactor * clippingPlane[4]->init.z);
					fprintf(f_out, "comb C%d u World%d", c->ID, c->ID);
				}
			}
		}
		else {
			CELL arbn = world;
			int k1;
			if (!cut && world.s[world.nS-1] == nSurfaces + 6) k1 = world.nS - 1; // the cut-away plane is redundant, because the cell does not intersect with it
			else k1 = world.nS;
			for (k = k1, j = 0; j < c->nS; j++) {
				s = surf + c->s[j];
				if (s->type != 1) continue; // not a plane
				int mbNum = c->mb[j] > 0 ? c->mb[j] - 1 : -c->mb[j] - 1;
				if (mbNum >= 0 && c->i[j] > 0 && macrobody[mbNum].type) continue; // the plane is a part of a standard macrobody, and it is not specified using the detailed format "<mbNum>.<facetNum>"
				arbn.s[k] = c->s[j];
				arbn.side[k] = c->side[j];
				k++;
			}
			int nPlanes0 = k - k1;
			arbn.nS = k;
			surfacesRemoved = false;
			memset(removeSurface, 0, arbn.nS * sizeof(bool));
			bool geomErr_ARBN = false;
			if (geomErr) geomErr_ARBN = !CheckCellGeometry2(&arbn, distancePrecision, surfacesRemoved2, removeSurface2);
			if (!geomErr && nPlanes0 || geomErr && !geomErr_ARBN) {
				for (j = 0; j < arbn.nS; j++) {
					arbn.side[j] = -arbn.side[j];
					if (!CheckCellGeometry2(&arbn, distancePrecision, surfacesRemoved2, removeSurface2)) {
						removeSurface[j] = true;
						if (arbn.s[j] >= nSurfaces && arbn.s[j] < nSurfaces + 6) removeClippingPlane[arbn.s[j]-nSurfaces] = true; // a side of the clipping cube
						surfacesRemoved = true;
					}
					arbn.side[j] = -arbn.side[j];
				}
				if (surfacesRemoved) {
					for (k = 0, j = 0; j < arbn.nS; j++) {
						if (removeSurface[j]) continue;
						arbn.s[k] = arbn.s[j];
						arbn.side[k] = arbn.side[j];
						k++;
					}
					arbn.nS = k;
				}
			}
			int cutAwayPlaneNum = -1;
			if (!geomErr_ARBN) {
				for (j = 0; j < arbn.nS; j++) {
					if (arbn.s[j] == nSurfaces + 6) cutAwayPlaneNum = j; // the cut-away plane is used in the definition of the ARBN object
					if (!nIterations || arbn.s[j] < nSurfaces || arbn.s[j] > nSurfaces + 5) continue; // not a side of the clipping cube
					int iSide = arbn.s[j] - nSurfaces;
					s = clippingPlane[iSide];
					if (removeClippingPlane[iSide]) { // is redundant
						int iOpposite = (iSide / 2 * 2 == iSide) ? iSide + 1 : iSide - 1;
						height = fabs(clippingPlane[iSide]->D + clippingPlane[iOpposite]->D); // normals of the opposite sides have opposite directions
						if (AdjustPlanePosition(s, (!geomErr ? &c2 : &arbn), height, distancePrecision, iSurf_max, nIterations, intersectionLines, true, -1)) clippingPlanesAdjusted = true;
					}
				}
			}
			int excludeCutAwayPlane = 0;
			if (cutAwayPlaneNum >= 0) { // the cut-away plane is used in the definition of the ARBN object 
				CELL world2; // a temporary cell bounded by the clipping planes (which may have been adjusted) and the cut-away plane with the inverted sense
				world2.nS = 1;
				world2.s[0] = nSurfaces + 6; // the cut-away plane
				world2.side[0] = -arbn.side[cutAwayPlaneNum];
				for (k = 0; k < arbn.nS; k++) {
					if (arbn.s[k] < nSurfaces || arbn.s[k] > nSurfaces + 5) continue;
					world2.s[world2.nS] = arbn.s[k] + 7;
					world2.side[world2.nS] = arbn.side[k];
					world2.nS++;
				}
				if (!CheckCellGeometry2(&world2, distancePrecision, surfacesRemoved2, removeSurface2)) excludeCutAwayPlane = 1;
			}
			fprintf(f_out, "in ARBN%d arbn %d", c->ID, arbn.nS - excludeCutAwayPlane);
			for (j = 0; j < arbn.nS; j++) {
				if (arbn.s[j] == nSurfaces + 6 && excludeCutAwayPlane) continue;
				if (!nIterations || arbn.s[j] < nSurfaces || arbn.s[j] > nSurfaces + 5) s = surf + arbn.s[j];
				else s = surf + arbn.s[j] + 7; // an adjusted clipping plane
				double mult = arbn.side[j] < 0 ? 1. : -1.;
				fprintf(f_out, "   %g %g %g %g", mult * s->dir.x, mult * s->dir.y, mult * s->dir.z, -mult * unitConversionFactor * s->D);
			}
			fprintf(f_out, "\ncomb C%d u ARBN%d", c->ID, c->ID);
		}
		mbPrev = -1;
		if (k0 >= 0 && k0 < c->nS) k = k0;
		else k = 0;
		bool skip_k0 = false;
		for (; k < c->nS; k++) {
			if (k == k0 && skip_k0) continue;
			int mbNum = c->mb[k] > 0 ? c->mb[k] - 1 : -c->mb[k] - 1;
			if (!c->mb[k] || c->i[k] < 0 || !macrobody[mbNum].type) { // not a part of a macrobody, or specified using the detailed format "<mbNum>.<facetNum>", or not one of the standard macrobodies
				s = surf + c->s[k];
				if (s->type == 1) continue; // a "separate" plane, which has been incorporated into the polyhedron
				if (k != k0) fprintf(f_out, " %c", c->side[k] < 0 ? '+' : '-');
				if (s->ID < 0) { // The temporary ID assigned to a separate surface created by a transformation of a cell or a general macrobody (such surfaces are initially assigned zero ID)
					fprintf(f_out, " S0%dC%d", -s->ID, c->ID);
					s->ID = 0; // The original ID (zero) is restored
				}
				else if (baseChanged[k]) { // The bases have been adjusted
					if (c->i[k] >= 0) fprintf(f_out, " S%dC%d", s->ID, c->ID);
					else fprintf(f_out, " S%dF%dC%d", macrobody[mbNum].ID, -c->i[k], c->ID);
				}
				else if (c->i[k] >= 0) fprintf(f_out, " S%d", s->ID);
				else fprintf(f_out, " S%dF%d", macrobody[mbNum].ID, -c->i[k]);
				continue;
			}
			if (mbNum == mbPrev) continue; // the next part of a standard macrobody
			if (k != k0) fprintf(f_out, " + S%d", macrobody[mbNum].ID);
			else fprintf(f_out, " S%d", macrobody[mbNum].ID);
			if (c->cellNum0 >= 0) fprintf(f_out, "C%d", c->ID);
			mbPrev = mbNum;
			if (k == k0) {
				skip_k0 = true;
				k = -1;
			}
		}
		for (j = 0; j < c->nContainedCells; j++) fprintf(f_out, " - C%d", cell[c->containedCells[j]].ID);
		fprintf(f_out, "%s", sourceString);
	}
	for (i = 0; i <= iSurf_max; i++) {
		if (intersectionLines[i]) delete intersectionLines[i];
	}
	if (edgeBuffer) {
		delete edgeBuffer;
		edgeBuffer = NULL;
	}
	fprintf(f_out, "\n");
	for (k = 0; k < nUsedMaterials; k++) {
		i = usedMaterialNum[k];
		if (materialID[i] > 0) fprintf(f_out, "r M%d", materialID[i]);
		else fprintf(f_out, "r V%d", -materialID[i]);
		for (j = 0; j < nCellsUsingTheMaterial[i]; j++) fprintf(f_out, " u C%d", cell[cellsUsingTheMaterial[i][j]].ID);
		fprintf(f_out, "\n");
	}
	if (sourcePositionType < 3 && sourceSizeRatio > 0.) fprintf(f_out, "r M_source u S_source\n");
	if (raytraceInLingeringWindow) fprintf(f_out, "\nexec fbserv %d /dev/wgl &\n", portNum);
	fprintf(f_out, "Z\n");
	for (k = 0, j = 0; j < nMaterialsWithSource; j++) {
		if (j < nUsedMaterials) {
			i = usedMaterialNum[j];
			if (materialID[i] > 0) fprintf(f_out, "mater M%d", materialID[i]);
			else fprintf(f_out, "mater V%d", -materialID[i]);
		}	
		else fprintf(f_out, "mater M_source");
		fprintf(f_out, " \"plastic {tr %g re %g}\" %d %d %d 0\n", regions[k].transp, regions[k].refl, regions[k].color.r, regions[k].color.g, regions[k].color.b);
		k++;
	}
	fprintf(f_out, "draw");
	for (j = 0; j < nMaterialsWithSource; j++) {
		if (!regions[j].show) continue;
		if (j < nUsedMaterials) {
			i = usedMaterialNum[j];
			if (materialID[i] > 0) fprintf(f_out, " M%d", materialID[i]);
			else fprintf(f_out, " V%d", -materialID[i]);
		}	
		else fprintf(f_out, " M_source");
	}
	fprintf(f_out, "\n");
	fprintf(f_out, "\nae %g %g %g\n", azimuth, elevation, twist);
	fprintf(f_out, "set perspective %g\n", perspective);
	fprintf(f_out, "view center %g %g %g\n", viewCenter.x, viewCenter.y, viewCenter.z);
	fprintf(f_out, "view size %g\n", viewSize);
	if (MGEDdistUnit[iUnit] != 1.) fprintf(f_out, "puts \" \\n******************************************************************\\nCoordinate conversion: 1 MGED centimeter = %s = %g cm.\\n******************************************************************\\n \"\n", MGEDdistUnitName[iUnit], MGEDdistUnit[iUnit]);
	if (raytraceInLingeringWindow) {
		if (MGEDdistUnit[iUnit] != 1.) fprintf(f_out, "after 2000 ");
		fprintf(f_out, "rt -F:%d -R -s 512 -C%d/%d/%d\n", portNum, backgroundColor.r, backgroundColor.g, backgroundColor.b);
	}
	if (raytraceInGraphicsWindow) {
		fprintf(f_out, "set width_height [dm size]\n");
		fprintf(f_out, "set width [lindex $width_height 0]\n");
		fprintf(f_out, "set height [lindex $width_height 1]\n");
		fprintf(f_out, "set ratio [expr 1.0 * $width / $height]\n");
		if (MGEDdistUnit[iUnit] != 1.) fprintf(f_out, "after 2000 ");
		fprintf(f_out, "rt -F5559 -R -w $width -n $height -V $ratio -C%d/%d/%d -P16 -H0 -J0 -l0\n", backgroundColor.r, backgroundColor.g, backgroundColor.b);
	}
	if (!showWireframe) fprintf(f_out, "Z\n");
	if (sourcePositionType < 3 && sourceDirectionType < 3 && m_beamAxisColor.GetCurSel() <= NSTDCOLORS) {
		fprintf(f_out, "\nvdraw open beamAxis\n");
		fprintf(f_out, "vdraw params color %02x%02x%02x\n", beamAxisColor.r, beamAxisColor.g, beamAxisColor.b);
		fprintf(f_out, "vdraw write next 0 %.14g %.14g %.14g\n", 10. * sourceCenter.x * unitConversionFactor, 10. * sourceCenter.y * unitConversionFactor, 10. * sourceCenter.z * unitConversionFactor);
//		fprintf(f_out, "vdraw write next 0 %.14g %.14g %.14g\n", 10. * sourceCenter.x, 10. * sourceCenter.y, 10. * sourceCenter.z);
		VECTOR dir = Init(beam_A, beam_B, beam_C);
		dir /= Norm(dir);
		VECTOR finalPoint = 10. * (sourceCenter * unitConversionFactor + 2. * worldSize * dir);
//		VECTOR finalPoint = 10. * (sourceCenter + 2. * worldSize / unitConversionFactor * dir); // must be in millimeters
		fprintf(f_out, "vdraw write next 1 %.14g %.14g %.14g\n", finalPoint.x, finalPoint.y, finalPoint.z);
		fprintf(f_out, "vdraw send\n");
	}
	if (showTracks) {
		fflush(f_out);
		char **trackScriptFN = new char*[NEVENTTYPES];
		for (i = 0; i < NEVENTTYPES; i++) {
			trackScriptFN[i] = new char[1000];
			trackScriptFN[i][0] = '\0';
		}
		if (!showTrackSegments) LoadTrackFile(this, &trackStats, scriptFN, trackScriptFN, &trackFilter);
		else LoadTrackFile(this, &trackStats, scriptFN, trackScriptFN, &trackFilter, &trackSegmentSelection, trackFilteringBeforeSegmentSelection);
		for (i = 0; i < NEVENTTYPES; i++) {
			for (j = 0; trackScriptFN[i][j] != '\0'; j++) {
				if (trackScriptFN[i][j] == '\\') trackScriptFN[i][j] = '/';
			}
			if (j) fprintf(f_out, "source \"%s\"\n", trackScriptFN[i]);
			delete trackScriptFN[i];
		}
		sprintf(tmp, "%d", trackStats.nTracks);
		m_nTracks.SetWindowText(tmp);
	}
	fclose(f_out);
	f_out = NULL;
	if (copyToClipboard) CopyToClipboard();
}

int CMCNScriptDlg::AdjustPlanePosition(SURFACE *p, CELL *c, double height, double &distancePrecision, int &iSurf_max, int nIterations, LINE **intersectionLines, bool invertNormDir, int invertSenseSnum)
{ // If plane p does not intersect with cell c, then the position of the plane is iteratively adjusted so that it almost touches the cell
  // If "invertNormDir" is false, then the direction of the initial shift (the "positive" direction) coincides with the direction of the plane's normal. Otherwise, those directions are opposite.
  // If "invertSenseID" is nonzero, then the sense of the region with that ID in the definition of cell "c" will be temporarily inverted (this is necessary when "p" is the base plane of a cylinder or cone that is specified with the positive sense in the definition of the cell)
	bool invertSense = false;
	int invertSenseIndex = c->nS;
	if (invertSenseSnum >= 0) {
		for (invertSenseIndex = 0; invertSenseIndex < c->nS && c->s[invertSenseIndex] != invertSenseSnum; invertSenseIndex++);
		if (invertSenseIndex < c->nS) c->side[invertSenseIndex] = -c->side[invertSenseIndex];
	}
	int intersectionTypes[MAXSURFACES];
	LINE intParms;
	double initialShift = 10. * distancePrecision;
	double stepLength;
	if (initialShift < 0.5 * height) stepLength = initialShift;
	else stepLength = 0.5 * height;
	VECTOR dir0 = invertNormDir ? -p->dir : p->dir;
	VECTOR step = stepLength * dir0;
	char cellPosition_prev, cellPosition = 1;
	VECTOR lastPositivePosition = p->init;
	int iter;
	double mult = 0.5;
	for (iter = 0; iter < nIterations && stepLength > distancePrecision; iter++) {
		if (iter) {
			if (iter == 1 && initialShift < 0.5 * height) {
				stepLength = 0.5 * (height - initialShift);
				step = stepLength * dir0;
			}
			else {
				stepLength *= mult;
				if (cellPosition != cellPosition_prev && (cellPosition || cellPosition_prev > 0)) step *= -mult;
				else step *= mult;
			}
		}
		cellPosition_prev = cellPosition;
		p->init += step;
		p->D = -p->dir * p->init;
		bool intersection = false;
		int iSurf;
		SURFACE *s2;
		for (iSurf = 0; iSurf < c->nS; iSurf++) {
			s2 = surf + c->s[iSurf];
			intersectionTypes[iSurf] = Intersection(p, s2, intParms, distancePrecision);
			if (intersectionTypes[iSurf] > 0 && intersectionTypes[iSurf] < 12) {
				if (!intersectionLines[iSurf]) {
					intersectionLines[iSurf] = new LINE;
					if (iSurf > iSurf_max) iSurf_max = iSurf;
				}
				*(intersectionLines[iSurf]) = intParms;
			}
			if (!intersectionTypes[iSurf] ||
				(intersectionTypes[iSurf] == -1 || intersectionTypes[iSurf] == -3) && c->side[iSurf] < 0 ||
				(intersectionTypes[iSurf] == -2 || intersectionTypes[iSurf] == -4) && c->side[iSurf] > 0) break;
			if (intersectionTypes[iSurf] > 0) { // since one of the intersecting surfaces is a plane, the possible intersection types are 1 - 4 and 6 - 9
				intersection = true;
			}
		}
		if (iSurf < c->nS) { // the entire cell is on one side of the plane
			if (!intersectionTypes[iSurf]) {
				if ((s2->dir * p->dir) * c->side[iSurf] > 0.) cellPosition = 1;
				else cellPosition = -1;
			}
			else if (intersectionTypes[iSurf] > -3) cellPosition = 1;
			else cellPosition = -1;
			if (invertNormDir) cellPosition = -cellPosition;
			if (cellPosition > 0) lastPositivePosition = p->init;
			continue;
		}
		if (!intersection) { // the entire plane is inside the cell
			cellPosition = 0;
		}
		else {
			int inside = 0; // 1 - a part of the intersection line is inside (i.e., it intersects with one of the other bounding surfaces),
							// 2 - entire intersection line (or one branch of it) is inside (i.e., it does not intersect with any of the other bounding surfaces)
			for (iSurf = 0; iSurf < c->nS && !inside; iSurf++) {
				s2 = surf + c->s[iSurf];
				if (intersectionTypes[iSurf] < 1) continue; // since one of the intersecting surfaces is a plane, the possible intersection types are 1 - 4 and 6 - 9
				VECTOR intersectionPoint;
				int intersectionSurface;
				inside = LineCellIntersection(intersectionLines[iSurf], c, intersectionPoint, intersectionSurface, distancePrecision, iSurf);
			}
			if (inside || !c->pointFound) cellPosition = 0;
			else {
				double dist = DistanceToSurface(&(c->point), p);
				if (fabs(dist) <= distancePrecision) cellPosition = 0;
				else if (dist > 0.) cellPosition = 1;
				else cellPosition = -1;
			}
		}
		if (invertNormDir) cellPosition = -cellPosition;
		if (cellPosition > 0) lastPositivePosition = p->init;
		else if (!iter) break;
	}
	if (cellPosition <= 0) {
		p->init = lastPositivePosition;
		p->D = -p->dir * p->init;
	}
	if (invertSenseIndex < c->nS) c->side[invertSenseIndex] = -c->side[invertSenseIndex];
	return iter;
}

void CMCNScriptDlg::CopyToClipboard()
{
	_finddata_t fileInfo;	
	if (_findfirst(outScriptFN, &fileInfo) < 0) return;
	FILE *f = fopen(outScriptFN, "rb");
	if (!f) return;
	char *str = new char[fileInfo.size + 1];
	unsigned long n;
	for (n = 0; n < fileInfo.size; n++) {
		int c = fgetc(f);
		if (c == EOF) break;
		str[n] = c;
	}
	fclose(f);
	if (n < fileInfo.size) {
		delete str;
		return;
	}
	str[n] = '\0';
	LPCSTR lpString = (LPCSTR)str;
	if (OpenClipboard()) {
		EmptyClipboard();
		HGLOBAL hData = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, lstrlen(lpString) + 1);
		if (hData != NULL) {
			LPSTR pszData = (LPSTR) ::GlobalLock(hData);
			USES_CONVERSION;
			strcpy(pszData, T2A((LPTSTR) lpString));
			GlobalUnlock(hData);
			SetClipboardData(CF_TEXT, hData);
		}
		CloseClipboard();
	}
	delete str;
}

BOOL CMCNScriptDlg::OnSave()
{
	int i, j;
	viewParmSets[curParmSet] = *((VIEWPARM *)&maximumObjectSizeRatio);
	regionData[curParmSet] = (REGIONDATA *)realloc(regionData[curParmSet], nRegions * sizeof(REGIONDATA));
	REGIONDATA *r = regionData[curParmSet];
	for (i = 0; i < nRegions; i++) r[i] = regions[i];
	CString msg;
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());
	if (f_par) fclose(f_par);
	FILE *f = fopen(app->parFileName, "wt");
	if (!f) {
		MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for writing. View parameters have not been saved."), app->parFileName), msg), NULL, MB_OK | MB_ICONERROR);
		return FALSE;
	}
	DWORD rgb = GetSysColor(COLOR_3DFACE);
	unsigned char r_std = GetRValue(rgb);
	unsigned char g_std = GetGValue(rgb);
	unsigned char b_std = GetBValue(rgb);
	fprintf(f, "nParmSets = %d\n", nViewParmSets);
	fprintf(f, "curParmSet = %d\n", curParmSet);
	for (i = 0; i < nViewParmSets; i++) {
		fprintf(f, "%s\n", viewParmSets[i].parmSetName);
		fprintf(f, "InputFileFolder = %s\n", ShortenPath(viewParmSets[i].inputFileFolder, currentFolder));
		fprintf(f, "InputFN = %s\n", viewParmSets[i].fn);
		fprintf(f, "cmdFolderOption = %d\n", viewParmSets[i].cmdFolderOption);
		if (viewParmSets[i].cmdFolderOption == 3) fprintf(f, "cmdFolder = %s\n", ShortenPath(viewParmSets[i].cmdFolder, currentFolder));
		fprintf(f, "MCNelectronFolder = %s\n", ShortenPath(viewParmSets[i].MCNelectronFolder, currentFolder));
		fprintf(f, "MCNelectronExeName = %s\n", viewParmSets[i].MCNelectronExeName);
		fprintf(f, "nThreads = %d\n", viewParmSets[i].nThreads);
		fprintf(f, "suffix = %s\n", viewParmSets[i].suffix);
		if (viewParmSets[i].suffix[0] == '\0') fprintf(f, "OutputFileName = %s\n", ShortenPath(viewParmSets[i].outputFileName, currentFolder));
		fprintf(f, "AltCSinfoFN = %s\n", ShortenPath(viewParmSets[i].altCSinfoFileName, currentFolder));
		fprintf(f, "ENDFfolder = %s\n", ShortenPath(viewParmSets[i].ENDFfolder, currentFolder));
		fprintf(f, "createLog = %d\n", viewParmSets[i].createLog);
		fprintf(f, "logSuffix = %s\n", viewParmSets[i].logSuffix);
		if (viewParmSets[i].logSuffix[0] == '\0') fprintf(f, "LogFileName = %s\n", ShortenPath(viewParmSets[i].logFileName, currentFolder));
		fprintf(f, "checkSyntax_MCNelectron = %d\n", viewParmSets[i].checkSyntax_MCNelectron);
		fprintf(f, "quitAfterChecking = %d\n", (int)viewParmSets[i].quitAfterChecking);
		fprintf(f, "checkSyntax_MCNScript = %d\n", (int)viewParmSets[i].checkSyntax_MCNScript);
		fprintf(f, "MCNP_style_cmdLine = %d\n", (int)viewParmSets[i].MCNP_style_cmdLine);
		for (j = 0; j < 10; j++) fprintf(f, "@%d = %.16g\n", j, viewParmSets[i].systemVars[j]);
		fprintf(f, "additionalArgs = %s\n", viewParmSets[i].additionalArgs);
		fprintf(f, "useCustomTrackFN = %d\n", (int)viewParmSets[i].useCustomTrackFN);
		if (viewParmSets[i].useCustomTrackFN) fprintf(f, "TrackFN = %s\n", ShortenPath(viewParmSets[i].trackFN, currentFolder));
		fprintf(f, "ScriptFolder = %s\n", ShortenPath(viewParmSets[i].scriptFolder, currentFolder));
		char MGEDcmdLine_tmp[3*FNLEN+1];
		strcpy(MGEDcmdLine_tmp, viewParmSets[i].MGEDcmdLine);
		size_t cmdLineLength = strlen(MGEDcmdLine_tmp);
		char *MGEDpath = strtok(MGEDcmdLine_tmp, "\"");
		char *cmdLineArgs;
		if (MGEDpath && MGEDpath + strlen(MGEDpath) != MGEDcmdLine_tmp + cmdLineLength) cmdLineArgs = MGEDpath + strlen(MGEDpath) + 1;
		else cmdLineArgs = NULL;
		CString MGEDpath2 = ShortenPath(MGEDpath, currentFolder);
		if (cmdLineArgs) fprintf(f, "MGEDcmdLine = \"%s\"%s\n", MGEDpath2, cmdLineArgs);
		else fprintf(f, "MGEDcmdLine = %s\n", MGEDpath2);
		fprintf(f, "copyToClipboard = %d\n", (int)(viewParmSets[i].copyToClipboard));
		fprintf(f, "maximumObjectSizeRatio = %g\n", viewParmSets[i].maximumObjectSizeRatio);
		fprintf(f, "sourceSizeRatio = %g\n", viewParmSets[i].sourceSizeRatio);
		fprintf(f, "viewSize = %g\n", viewParmSets[i].viewSize);
		fprintf(f, "viewCenter = (%g, %g, %g)\n", viewParmSets[i].viewCenter.x, viewParmSets[i].viewCenter.y, viewParmSets[i].viewCenter.z);
		fprintf(f, "azimuth = %g\n", viewParmSets[i].azimuth);
		fprintf(f, "elevation = %g\n", viewParmSets[i].elevation);
		fprintf(f, "twist = %g\n", viewParmSets[i].twist);
		fprintf(f, "perspective = %g\n", viewParmSets[i].perspective);
		fprintf(f, "useCutawayPlane = %d\n", (int)(viewParmSets[i].useCutawayPlane));
		fprintf(f, "cutawayPlanePoint = (%g, %g, %g)\n", viewParmSets[i].cutawayPlanePoint.x, viewParmSets[i].cutawayPlanePoint.y, viewParmSets[i].cutawayPlanePoint.z);
		fprintf(f, "cutawayPlaneNormal = (%g, %g, %g)\n", viewParmSets[i].cutawayPlaneNormal.x, viewParmSets[i].cutawayPlaneNormal.y, viewParmSets[i].cutawayPlaneNormal.z);
		fprintf(f, "portNum = %d\n", viewParmSets[i].portNum);
		fprintf(f, "nIterations = %d\n", viewParmSets[i].nIterations);
		fprintf(f, "unitEnergy = %d\n", viewParmSets[i].unitEnergy);
		fprintf(f, "unitDistance = %d\n", viewParmSets[i].unitDistance);
		fprintf(f, "MGEDsuffix = %s\n", viewParmSets[i].MGEDsuffix);

		fprintf(f, "historyNumStart = %d\n", viewParmSets[i].trackFilter.historyNo[0] + 1);
		fprintf(f, "historyNumEnd = %d\n", viewParmSets[i].trackFilter.historyNo[1] + 1);
		fprintf(f, "trackNumStart = %d\n", viewParmSets[i].trackFilter.trackNo[0] + 1);
		fprintf(f, "trackNumEnd = %d\n", viewParmSets[i].trackFilter.trackNo[1] + 1);
		fprintf(f, "nVerticesMin = %d\n", viewParmSets[i].trackFilter.nVert[0]);
		fprintf(f, "nVerticesMax = %d\n", viewParmSets[i].trackFilter.nVert[1]);
		fprintf(f, "trackEstartMin = %g\n", viewParmSets[i].trackFilter.Estart[0]);
		fprintf(f, "trackEstartMax = %g\n", viewParmSets[i].trackFilter.Estart[1]);
		fprintf(f, "trackEfinalMin = %g\n", viewParmSets[i].trackFilter.Efinal[0]);
		fprintf(f, "trackEfinalMax = %g\n", viewParmSets[i].trackFilter.Efinal[1]);
		fprintf(f, "trackElossMin = %g\n", viewParmSets[i].trackFilter.Eloss[0]);
		fprintf(f, "trackElossMax = %g\n", viewParmSets[i].trackFilter.Eloss[1]);
		fprintf(f, "trackRelElossMin = %g\n", viewParmSets[i].trackFilter.relEloss[0]);
		fprintf(f, "trackRelElossMax = %g\n", viewParmSets[i].trackFilter.relEloss[1]);
		fprintf(f, "trackDisplacementMin = %g\n", viewParmSets[i].trackFilter.displ[0]);
		fprintf(f, "trackDisplacementMax = %g\n", viewParmSets[i].trackFilter.displ[1]);
		fprintf(f, "trackPeakDisplacementMin = %g\n", viewParmSets[i].trackFilter.peakDispl[0]);
		fprintf(f, "trackPeakDisplacementMax = %g\n", viewParmSets[i].trackFilter.peakDispl[1]);
		fprintf(f, "trackPathMin = %g\n", viewParmSets[i].trackFilter.path[0]);
		fprintf(f, "trackPathMax = %g\n", viewParmSets[i].trackFilter.path[1]);
		fprintf(f, "genMin = %d\n", viewParmSets[i].trackFilter.gen[0]);
		fprintf(f, "genMax = %d\n", viewParmSets[i].trackFilter.gen[1]);
		fprintf(f, "originTypeMask= %d\n", viewParmSets[i].trackFilter.originTypeMask);

		fprintf(f, "segmentCollMin = %d\n", viewParmSets[i].trackSegmentSelection.iVert[0]);
		fprintf(f, "segmentCollMax = %d\n", viewParmSets[i].trackSegmentSelection.iVert[1]);
		fprintf(f, "segmentEmin = %g\n", viewParmSets[i].trackSegmentSelection.E[0]);
		fprintf(f, "segmentEmax = %g\n", viewParmSets[i].trackSegmentSelection.E[1]);
		fprintf(f, "segmentElossMin = %g\n", viewParmSets[i].trackSegmentSelection.Eloss[0]);
		fprintf(f, "segmentElossMax = %g\n", viewParmSets[i].trackSegmentSelection.Eloss[1]);
		fprintf(f, "segmentRelElossMin = %g\n", viewParmSets[i].trackSegmentSelection.relEloss[0]);
		fprintf(f, "segmentRelElossMax = %g\n", viewParmSets[i].trackSegmentSelection.relEloss[1]);
		fprintf(f, "segmentPathMin = %g\n", viewParmSets[i].trackSegmentSelection.path[0]);
		fprintf(f, "segmentPathMax = %g\n", viewParmSets[i].trackSegmentSelection.path[1]);

		fprintf(f, "trackFilteringBeforeSegmentSelection = %d\n", (int)viewParmSets[i].trackFilteringBeforeSegmentSelection);

		fprintf(f, "raytraceInGraphicsWindow = %d\n", (int)viewParmSets[i].raytraceInGraphicsWindow);
		fprintf(f, "showWireframe = %d\n", (int)viewParmSets[i].showWireframe);
		fprintf(f, "raytraceInLingeringWindow = %d\n", (int)viewParmSets[i].raytraceInLingeringWindow);
		fprintf(f, "showTracks = %d\n", (int)viewParmSets[i].showTracks);
		fprintf(f, "showTrackSegments = %d\n", (int)viewParmSets[i].showTrackSegments);
		fprintf(f, "extendEscapeTracks = %d\n", (int)viewParmSets[i].extendEscapeTracks);
		fprintf(f, "keepTrackDataInMemory = %d\n", (int)viewParmSets[i].keepTrackDataInMemory);
		fprintf(f, "nVerticesTotalMax = %d\n", viewParmSets[i].nVerticesTotalMax);
		fprintf(f, "nVerticesTrackMax = %d\n", viewParmSets[i].nVerticesTrackMax);
		for (j = 0; j < NEVENTTYPES; j++) fprintf(f, "trackColorRGB[%d] = (%d, %d, %d)\n", j, (int)(viewParmSets[i].trackColor[j].r), (int)(viewParmSets[i].trackColor[j].g), (int)(viewParmSets[i].trackColor[j].b));
		if (viewParmSets[i].beamAxisColor.r != r_std || viewParmSets[i].beamAxisColor.g != g_std || viewParmSets[i].beamAxisColor.b != b_std) fprintf(f, "beamAxisColorRGB = (%d, %d, %d)\n", (int)(viewParmSets[i].beamAxisColor.r), (int)(viewParmSets[i].beamAxisColor.g), (int)(viewParmSets[i].beamAxisColor.b));
		else fprintf(f, "beamAxisColorRGB = (-1, -1, -1)\n");
		fprintf(f, "backgroundColorRGB = (%d, %d, %d)\n", (int)(viewParmSets[i].backgroundColor.r), (int)(viewParmSets[i].backgroundColor.g), (int)(viewParmSets[i].backgroundColor.b));
		fprintf(f, "nRows = %d\n", viewParmSets[i].nRegions);
		for (j = 0; j < viewParmSets[i].nRegions; j++) {
			REGIONDATA *r = regionData[i] + j;
			fprintf(f, "%s\n", regionData[i][j].name);
			fprintf(f, "colorRGB = (%d, %d, %d)\n", (int)(r->color.r), (int)(r->color.g), (int)(r->color.b));
			fprintf(f, "transparency = %g\n", r->transp);
			fprintf(f, "reflectivity = %g\n", r->refl);
			fprintf(f, "show = %d\n", (int)(r->show));
		}
	}
	fclose(f);
	f_par = fopen(app->parFileName, "r+t");
	if (!f_par) {
		MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be re-opened after saving."), app->parFileName), msg), NULL, MB_OK | MB_ICONWARNING);
	}
	changed = false;
	app->GetRecentFileList()->Add(app->parFileName);
	return TRUE;
}

BOOL CMCNScriptDlg::OnSaveAs()
{
	char drive[1000], dir[1000], fname[1000], ext[1000];
	char parFileName[1000], fileName[1000], path[1000];
	CMCNScriptApp *app = (CMCNScriptApp *)(AfxGetApp());

	size_t nBlanks = strspn(app->parFileName, " \t");
	if (app->parFileName[nBlanks] != '\\' && !strchr(app->parFileName + nBlanks, ':')) strcpy(parFileName, currentFolder);
	else parFileName[0] = '\0';
	strcat(parFileName, app->parFileName + nBlanks);

	bool nameIsEmpty = (strspn(parFileName, " \t") == strlen(parFileName));
	if (!nameIsEmpty) {
		_splitpath(parFileName, drive, dir, fname, ext);
		sprintf(fileName, "%s%s", fname, ext);
	}
	else fileName[0] = '\0';
	CFileDialog dlg(FALSE, "*.mcnprm", fileName, OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_EXPLORER|OFN_ENABLESIZING, "MCNScript files (*.mcnprm)|*.mcnprm|All files (*.*)|*.*||", NULL);
	if (!nameIsEmpty) {
		sprintf(path, "%s%s", drive, dir);
		dlg.m_ofn.lpstrInitialDir = path;
	}
	else dlg.m_ofn.lpstrInitialDir = NULL;
	dlg.m_ofn.lpstrTitle = "Save the parameter file";
	BOOL rc = FALSE;
	if (dlg.DoModal() == IDOK) {
		CString pathName = dlg.GetPathName();
		if (pathName.IsEmpty()) rc = OnSaveAs();
		else {
			char parFileName_bak[1002];
			strcpy(parFileName_bak, app->parFileName);
			strcpy(app->parFileName, pathName);
			SetWindowText((CString)app->m_pszAppName + " [" + app->parFileName + "]");
			rc = OnSave();
			if (!rc) {
				strcpy(app->parFileName, parFileName_bak);
				SetWindowText((CString)app->m_pszAppName + " [" + app->parFileName + "]");
			}
		}
	}
	if (rc) app->GetRecentFileList()->Add(app->parFileName);
	return rc;
}

void CMCNScriptDlg::OnTimer(UINT_PTR nIDEvent)
{
	int i, j;
	INT_PTR nCmdTimers = iParmSetFromCmdTimerIndex.GetSize();
	for (j = 0; j < nCmdTimers; j++) {
		i = iParmSetFromCmdTimerIndex[j];
		if (nIDEvent == viewParmSets[i].cmdTimerID) {
			HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, viewParmSets[i].cmdProcessID);
			if (hProcess) {
				HWND hWnd = find_main_window(viewParmSets[i].cmdProcessID);
				if (!hWnd && !viewParmSets[i].cmdWindowCreated) { // A failure to get the main window handle probably means that the window is not created yet
					CloseHandle(hProcess);
					return;
				}
				if (hWnd) {
					char processName[1000];
					int rc = GetModuleBaseNameA(hProcess, 0, processName, sizeof(processName));
					CloseHandle(hProcess);
					if (!rc && !viewParmSets[i].cmdWindowCreated) return; // A failure to get the process name probably means that the main window of the process is not created yet
					if (rc && !strcmpi(processName, "cmd.exe")) {
						viewParmSets[i].cmdWindowCreated = TRUE;
						if (i == curParmSet) cmdWindowCreated = TRUE;
						return;
					}
				}
				else CloseHandle(hProcess);
			}
			KillTimer(viewParmSets[i].cmdTimerID);
			iParmSetFromCmdTimerIndex.RemoveAt(viewParmSets[i].cmdTimerIndex);
			INT_PTR nCmdTimers = iParmSetFromCmdTimerIndex.GetSize();
			for (int k = 0; k < nCmdTimers; k++) {
				int m = iParmSetFromCmdTimerIndex[k];
				if (viewParmSets[m].cmdTimerIndex > viewParmSets[i].cmdTimerIndex) viewParmSets[m].cmdTimerIndex--;
			}
			viewParmSets[i].cmdTimerID = 0;
			viewParmSets[i].cmdTimerIndex = -1;
			viewParmSets[i].cmdProcessID = 0;
			viewParmSets[i].cmdWindowCreated = FALSE;
			if (i == curParmSet) {
				cmdTimerID = 0;
				cmdTimerIndex = -1;
				cmdProcessID = 0;
				cmdWindowCreated = FALSE;
				m_startMCNelectron.SetWindowText("Start MCNelectron");
			}
			return;
		}
	}
	INT_PTR nMGEDtimers = iParmSetFromMGEDtimerIndex.GetSize();
	for (j = 0; j < nMGEDtimers; j++) {
		i = iParmSetFromMGEDtimerIndex[j];
		if (nIDEvent == viewParmSets[i].mgedTimerID) {
			HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, viewParmSets[i].mgedProcessID);
			HWND hWnd;
			if (hProcess) {
				CloseHandle(hProcess);
				hWnd = find_main_window(viewParmSets[i].mgedProcessID);
				if (!hWnd && !viewParmSets[i].mgedWindowCreated) return; // A failure to get the main window handle probably means that the window is not created yet
				if (hWnd) {
					viewParmSets[i].mgedWindowCreated = TRUE;
					if (i == curParmSet) mgedWindowCreated = TRUE;
					return;
				}
			}
			if (!hProcess || !hWnd) {
				HWND handles[NHANDLES_MAX];
//				if (find_main_window2(viewParmSets[i].mgedProcessID, handles)) return;
				if (find_main_window3(viewParmSets[i].mgedProcessID, handles)) return; // find_main_window3 is much faster than find_main_window2, but finds only the first-generation child processes
			}
			KillTimer(viewParmSets[i].mgedTimerID);
			iParmSetFromMGEDtimerIndex.RemoveAt(viewParmSets[i].mgedTimerIndex);
			nMGEDtimers--;
			for (int k = 0; k < nMGEDtimers; k++) {
				int m = iParmSetFromMGEDtimerIndex[k];
				if (viewParmSets[m].mgedTimerIndex > viewParmSets[i].mgedTimerIndex) {
					viewParmSets[m].mgedTimerIndex--;
					if (m == curParmSet) mgedTimerIndex = viewParmSets[m].mgedTimerIndex;
				}
			}
			viewParmSets[i].mgedTimerID = 0;
			viewParmSets[i].mgedTimerIndex = -1;
			viewParmSets[i].mgedProcessID = 0;
			viewParmSets[i].mgedWindowCreated = FALSE;
			if (i == curParmSet) {
				mgedTimerID = 0;
				mgedTimerIndex = -1;
				mgedProcessID = 0;
				mgedWindowCreated = FALSE;
				m_startMGED.SetWindowText("Start MGED");
			}
			return;
		}
	}
	CDialogEx::OnTimer(nIDEvent);
}

void CMCNScriptDlg::OnStartMCNelectron()
{
	if (cmdProcessID) {
		HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, cmdProcessID);
		if (hProcess) {
			HWND hWnd = find_main_window(cmdProcessID);
			if (!hWnd && !cmdWindowCreated) { // A failure to get the cmd.exe window handle probably means that the cmd.exe window is not created yet
				CloseHandle(hProcess);
				MessageBox("It seems that the cmd.exe window is not created yet. Try again in a few seconds.", NULL, MB_OK | MB_ICONINFORMATION);
				return;
			}
			if (hWnd) {
				char processName[1000];
				int rc = GetModuleBaseNameA(hProcess, 0, processName, sizeof(processName));
				CloseHandle(hProcess);
				if (!rc && !cmdWindowCreated) { // A failure to get the process name probably means that the the main window of the process is not created yet
					MessageBox("It seems that the cmd.exe window is not created yet. Try again in a few seconds.", NULL, MB_OK | MB_ICONINFORMATION);
					return;
				}
				if (rc && !strcmpi(processName, "cmd.exe")) {
					CWnd *pWnd = CWnd::FromHandle(hWnd);
					if (pWnd) {
						pWnd->ShowWindow(SW_RESTORE);
						pWnd->ActivateTopParent();
						pWnd->BringWindowToTop();
						return;
					}
				}
			}
			else CloseHandle(hProcess);
		}
		KillCmdTimer();
		return;
	}
	char cmd[8*FNLEN];
	if (cmdLine[0] != '\0') {
		strcpy(cmd, "/K mode 170 && ");
		strcat(cmd, cmdLine);
	}
	else strcpy(cmd, "/K mode 170");
	char current[FNLEN+1];
	if (!cmdFolderOption) strcpy(current, inputFileFolder);
	else if (cmdFolderOption == 1) strcpy(current, MCNelectronFolder);
	else if (cmdFolderOption == 2) strcpy(current, currentFolder);
	else strcpy(current, cmdFolder);
	if (strspn(current, " \t") == strlen(current)) strcpy(current, currentFolder);
	SHELLEXECUTEINFO shellExecuteInfo;
	memset(&shellExecuteInfo, 0, sizeof(SHELLEXECUTEINFO));
	shellExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	shellExecuteInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
	shellExecuteInfo.lpFile = "cmd.exe";
	shellExecuteInfo.lpParameters = cmd;
	shellExecuteInfo.lpDirectory = current;
	shellExecuteInfo.nShow = SW_SHOW;
	if (ShellExecuteEx(&shellExecuteInfo) && (int)shellExecuteInfo.hInstApp > 32) {
		m_nTracks.SetWindowText("");
		cmdProcessID = GetProcessId(shellExecuteInfo.hProcess);
		if (!cmdProcessID) MessageBox("The process ID of the created cmd.exe window could not be obtained.", NULL, MB_OK | MB_ICONWARNING);
		else {
			timerCount++;
			cmdTimerID = SetTimer(timerCount, 200, NULL);
			cmdTimerIndex = iParmSetFromCmdTimerIndex.GetSize();
			m_startMCNelectron.SetWindowText("Activate the cmd.exe window");
			viewParmSets[curParmSet].cmdTimerID = cmdTimerID;
			viewParmSets[curParmSet].cmdTimerIndex = cmdTimerIndex;
			viewParmSets[curParmSet].cmdProcessID = cmdProcessID;
			viewParmSets[curParmSet].cmdWindowCreated = FALSE;
			iParmSetFromCmdTimerIndex.Add(curParmSet);
		}
	}
	else MessageBox((CString)"An error occurred when attempting to execute the command \"cmd.exe " + cmd + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnReloadInputFile()
{
	if (strspn(fn, " \t") == strlen(fn)) OnLoadInputFile();
	else {
		LoadInputFile();
		if (!checkSyntax_MCNScript) {
			MessageBox((CString)"In order to read the geometry information contained in the MCNelectron input file, check the checkbox \"Check syntax\".", NULL, MB_OK | MB_ICONINFORMATION);
		}
		if (fileLoaded) UpdateRegionNames();
		else if (f_in) {
			fclose(f_in);
			f_in = NULL;
			if (edgeBuffer) {
				delete edgeBuffer;
				edgeBuffer = NULL;
			}
		}
	}
}

void CMCNScriptDlg::OnLoadInputFile()
{
	char buffer[1000];
	GetFileName(&m_inputFileName, "Select the MCNelectron input file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (strcmp(ShortenPath(buffer, currentFolder), ShortenPath((CString)inputFileFolder + fn, currentFolder))) changed = true;
	char drive[1000], dir[1000], ext[1000], tmp[SUFFIXLEN+10];
	_splitpath(buffer, drive, dir, fn1, ext);
	strcpy(inputFileFolder, (CString)drive + dir);
	strcpy(fn, (CString)fn1 + ext);
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, curParmSet + 1);
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		if (!useCustomTrackFN) m_trackFileName.SetWindowText(ShortenPath((CString)inputFileFolder + "Files_" + fn1 + tmp + ".txt\\" + fn1 + tmp + "_tracks.txt", currentFolder));
	}
	if (suffix[0] != '\0') {
		sprintf(tmp, logSuffix, curParmSet + 1);
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
	}
	MakeCmdLine();
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
	LoadInputFile();
	if (fileLoaded) UpdateRegionNames();
	else if (f_in) {
		fclose(f_in);
		f_in = NULL;
		if (edgeBuffer) {
			delete edgeBuffer;
			edgeBuffer = NULL;
		}
	}
}

void CMCNScriptDlg::OnLoadTrackFile()
{
	char buffer[1000];
	GetFileName(&m_trackFileName, "Select the track data file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (strcmp(ShortenPath(buffer, currentFolder), ShortenPath(trackFN, currentFolder))) changed = true;
	strcpy(trackFN, buffer);
	OnRefresh();
}

void CMCNScriptDlg::LoadTrackFile(CWnd *parent, TRACKSTATS *pTrackStats, char *scriptFN, char **trackScriptFN, TRACKFILTERDATA *pTrackFilter, TRACKSEGMENTDATA *pTrackSegmentSelection, BOOL trackFilteringBeforeSegmentSelection)
{
	progressDlg = new CTrackDataFileProgressDlg(parent);
	progressDlg->nSteps = 1000;
	progressDlg->pTrackStats = pTrackStats;
	progressDlg->scriptFN = scriptFN;
	progressDlg->trackScriptFN = trackScriptFN;
	progressDlg->pTrackFilter = pTrackFilter;
	progressDlg->pTrackSegmentSelection = pTrackSegmentSelection;
	progressDlg->trackFilteringBeforeSegmentSelection = trackFilteringBeforeSegmentSelection;
	progressDlg->DoModal();
	delete progressDlg;
}

CTrackDataFileProgressDlg::CTrackDataFileProgressDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTrackDataFileProgressDlg::IDD, pParent)
{
}

void LoadTrackFile1(void *pMainWnd) {
	((CMCNScriptDlg *)pMainWnd)->LoadTrackFile2();
}

BOOL CTrackDataFileProgressDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	VERIFY(m_progress.SubclassDlgItem(IDC_PROGRESS, this));
	m_progress.SetRange(0, nSteps);
	m_progress.SetStep(1);
	CRect parentWindowRect;
	CWnd *parent = GetParent();
	if (parent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg)) || ((CTrackStatsDlg *)parent)->dialogInitialized) parent->GetWindowRect(&parentWindowRect);
	else parent->GetParent()->GetWindowRect(&parentWindowRect);
	CRect windowRect;
	GetWindowRect(&windowRect);
	int width = windowRect.right - windowRect.left;
	int height = windowRect.bottom - windowRect.top;
	int hor_shift = (parentWindowRect.right - parentWindowRect.left - width) / 2;
	int vert_shift = (parentWindowRect.bottom - parentWindowRect.top - height) / 2;
	m_visible = FALSE;
	SetWindowPos(NULL, parentWindowRect.left + hor_shift, parentWindowRect.top + vert_shift, width, height, SWP_HIDEWINDOW);
	m_progress.SetFocus();
	CWnd *pMainWnd = parent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg)) ? parent : parent->GetParent();
	_beginthread(LoadTrackFile1, 0, pMainWnd);
	return FALSE;
}

BOOL CTrackDataFileProgressDlg::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN && (pMsg->wParam == VK_RETURN || pMsg->wParam == VK_ESCAPE)) return TRUE;
	return CDialog::PreTranslateMessage(pMsg);
}

BEGIN_MESSAGE_MAP(CTrackDataFileProgressDlg, CDialog)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_WM_WINDOWPOSCHANGING()
END_MESSAGE_MAP()

void CTrackDataFileProgressDlg::OnStop()
{
	CWnd *parent = GetParent();
	CMCNScriptDlg *pMainWnd = parent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg)) ? (CMCNScriptDlg *)parent : ((CTrackStatsDlg *)parent)->parent;
	pMainWnd->stopReadingTrackData = true;
}

void CTrackDataFileProgressDlg::OnWindowPosChanging(WINDOWPOS FAR* lpwndpos)
{
	if (!m_visible) lpwndpos->flags &= ~SWP_SHOWWINDOW;
	CDialog::OnWindowPosChanging(lpwndpos);
}

void CMCNScriptDlg::LoadTrackFile2()
{
	TRACKSTATS *pTrackStats = progressDlg->pTrackStats;
	char *scriptFN = progressDlg->scriptFN;
	char **trackScriptFN = progressDlg->trackScriptFN;
	TRACKFILTERDATA *pTrackFilter = progressDlg->pTrackFilter;
	TRACKSEGMENTDATA *pTrackSegmentSelection = progressDlg->pTrackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection = progressDlg->trackFilteringBeforeSegmentSelection;
	int i, j;
	char line[101];
	CString msg;
	CString trackFileName('\0', 1000);
	int nBytes = m_trackFileName.GetWindowText(trackFileName.GetBuffer(), 999); trackFileName.ReleaseBuffer();
	if (trackFileName.GetAt(nBytes) != '\0') trackFileName.SetAt(nBytes, '\0');
	trackFileName.TrimLeft();
	trackFileName.TrimRight();
	if (trackFileName.GetAt(0) != '\\' && trackFileName.Find(':') < 0) trackFileName = currentFolder + trackFileName;

	bool trackDataChanged = true;
	bool err = false, nVerticesTotalMaxExceeded = false, verticesSkipped = false;
	__finddata64_t fileinfo;
	intptr_t handle = _findfirst64(trackFileName, &fileinfo);
	if (keepTrackDataInMemory) {
		FILETIME lastWriteTime;
		if (handle < 0) err = true;
		else {
			HANDLE hFile = CreateFile(trackFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
			if (hFile == INVALID_HANDLE_VALUE) err = true;
			else {
				FILETIME creationTime, lastAccessTime;
				GetFileTime(hFile, &creationTime, &lastAccessTime, &lastWriteTime);
				CloseHandle(hFile);
			}
		}
		if (!err) {
			trackDataChanged = !trackSets[curParmSet].nTracks ||
//			strcmpi(trackFileName, trackSets[curParmSet].trackFN) ||
			fileinfo.size != trackSets[curParmSet].fileSize ||
			lastWriteTime.dwLowDateTime != trackSets[curParmSet].lastWriteTime.dwLowDateTime ||
			lastWriteTime.dwHighDateTime != trackSets[curParmSet].lastWriteTime.dwHighDateTime;
			if (trackDataChanged) {
				strcpy(trackSets[curParmSet].trackFN, trackFileName);
				trackSets[curParmSet].fileSize = fileinfo.size;
				trackSets[curParmSet].lastWriteTime = lastWriteTime;
				if (trackSets[curParmSet].nTracks) {
					for (i = 0; i < trackSets[curParmSet].nTracks; i++) {
						delete trackSets[curParmSet].trackData[i].coord;
						delete trackSets[curParmSet].trackData[i].energy;
					}
					free(trackSets[curParmSet].trackData);
					trackSets[curParmSet].nTracks = 0;
				}
			}
		}
	}
	else trackSets[curParmSet].fileSize = 0;
	stopReadingTrackData = false;

	memset(pTrackStats, 0, sizeof(TRACKSTATS));
	pTrackStats->lastHistory = -1;
	char *trackFileSuffix[NEVENTTYPES] = {"_tracks0_source", "_tracks1_brems", "_tracks2_annih",
		"_tracks3_x-ray_el", "_tracks4_x-ray_phot", "_tracks5_pp_el", "_tracks6_pp_pos", "_tracks7_Compton_recoil",
		"_tracks8_photoel", "_tracks9_Auger_phot", "_tracks10_Auger_el", "_tracks11_knock-on"};
	FILE *f_track[NEVENTTYPES];
	bool trackOutputError[NEVENTTYPES];
	memset(f_track, 0, NEVENTTYPES * sizeof(FILE *));
	memset(trackOutputError, 0, NEVENTTYPES * sizeof(bool));

	int nVerticesTotal = 0, nTracksWritten = 0;
	int add1, add2;
	VECTOR initialPoint, finalPoint;
	int iHist = -1;
	bool oldFormatWarning = false;
	if (!trackDataChanged) {
		processingTrackData = true;
		int nHistories = trackSets[curParmSet].nHistories;
		int nTracks = trackSets[curParmSet].nTracks;
		for (i = pTrackFilter->historyNo[0]; i < nHistories && trackSets[curParmSet].histStartTrackNum[i] <= pTrackFilter->trackNo[0]; i++);
		int startHistory = (i > pTrackFilter->historyNo[0] ? i - 1 : pTrackFilter->historyNo[0]);
		int endHistory = -1, lastTrackNo;
		if (pTrackFilter->historyNo[0] < nHistories) {
			lastTrackNo = (pTrackFilter->trackNo[1] >= 0 && pTrackFilter->trackNo[1] < nTracks ? pTrackFilter->trackNo[1] : nTracks - 1);
			for (i = (pTrackFilter->historyNo[1] >= 0 && pTrackFilter->historyNo[1] < nHistories ? pTrackFilter->historyNo[1] : nHistories - 1); i >= startHistory && trackSets[curParmSet].histStartTrackNum[i] > lastTrackNo; i--);
			endHistory = i;
		}
		if (startHistory <= endHistory) {
			int startTrackNo = (pTrackFilter->trackNo[0] >= trackSets[curParmSet].histStartTrackNum[startHistory] ? pTrackFilter->trackNo[0] : trackSets[curParmSet].histStartTrackNum[startHistory]);
			int endTrackNo = (lastTrackNo < trackSets[curParmSet].histStartTrackNum[endHistory+1] ? lastTrackNo : trackSets[curParmSet].histStartTrackNum[endHistory+1] - 1);
			iHist = startHistory;
			int nextHistoryStart = trackSets[curParmSet].histStartTrackNum[iHist+1];
			for (i = startTrackNo; i <= endTrackNo; i++) {
				if (i == nextHistoryStart) {
					iHist++;
					nextHistoryStart = trackSets[curParmSet].histStartTrackNum[iHist+1];
				}
				TRACKDATA *pTrack = trackSets[curParmSet].trackData + i;
				int iStart, iEnd;
				bool escape = (pTrack->type > 0);
				bool success = UpdateTrackStats(pTrackStats, iHist, i, pTrack->nVertices, pTrack->gen, pTrack->originType, pTrack->coord, pTrack->energy, pTrack->type, iStart, iEnd, add1, initialPoint, add2, finalPoint, pTrackFilter, pTrackSegmentSelection, trackFilteringBeforeSegmentSelection);
				if (scriptFN && success && !nVerticesTotalMaxExceeded) {
					int jMax = escape ? pTrack->nVertices - 1 : iEnd;
					int nVert0 = jMax - iStart + 1 + add1 + add2;
					int nVert = (nVerticesTrackMax < 0 || nVert0 <= nVerticesTrackMax) ? nVert0 : nVerticesTrackMax;
					if (nVerticesTotalMax - nVerticesTotal >= nVert) {
						char ot = pTrack->ot;
						if (!trackOutputError[ot] && !f_track[ot]) {
							CString msg;
							sprintf(trackScriptFN[ot], "%s%s.rt", scriptFN, trackFileSuffix[ot]);
							f_track[ot] = fopen(trackScriptFN[ot], "wt");
							if (f_track[ot]) {
								fprintf(f_track[ot], "vdraw open tracks%d\n", ot);
								fprintf(f_track[ot], "vdraw params color %02x%02x%02x\n", trackColor[ot].r, trackColor[ot].g, trackColor[ot].b);
							}
							else {
								trackOutputError[ot] = true;
								trackScriptFN[ot][0] = '\0';
								MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for writing."), trackScriptFN[ot]), msg), NULL, MB_OK | MB_ICONERROR);
							}
						}
						if (!trackOutputError[ot]) {
							FILE *f_script = f_track[ot];
							fprintf(f_script, "vdraw write next 0 %.14g %.14g %.14g\n", 10. * initialPoint.x * unitConversionFactor, 10. * initialPoint.y * unitConversionFactor, 10. * initialPoint.z * unitConversionFactor);
							j = iStart + 1 - add1;
							if (nVert == nVert0 && j <= jMax || nVert != nVert0 && j < iEnd + add2) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * pTrack->coord[j].x * unitConversionFactor, 10. * pTrack->coord[j].y * unitConversionFactor, 10. * pTrack->coord[j].z * unitConversionFactor);
							if (nVert == nVert0) {
								if (j <= jMax) {
									for (j++; j <= jMax; j++) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * pTrack->coord[j].x * unitConversionFactor, 10. * pTrack->coord[j].y * unitConversionFactor, 10. * pTrack->coord[j].z * unitConversionFactor);
								}
							}
							else {
								if (j < iEnd + add2) {
									double increment = (double)(iEnd - iStart - 1 + add1 + add2) / (nVert - 2 - (jMax - iEnd));
									double jDbl = j + increment;
									j = (int)(jDbl + 0.5);
									for (; j < iEnd + add2; jDbl += increment, j = (int)(jDbl + 0.5)) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * pTrack->coord[j].x * unitConversionFactor, 10. * pTrack->coord[j].y * unitConversionFactor, 10. * pTrack->coord[j].z * unitConversionFactor);
								}
								for (j = iEnd + add2; j <= jMax; j++) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * pTrack->coord[j].x * unitConversionFactor, 10. * pTrack->coord[j].y * unitConversionFactor, 10. * pTrack->coord[j].z * unitConversionFactor);
								verticesSkipped = true;
							}
							if (j <= jMax + add2) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * finalPoint.x * unitConversionFactor, 10. * finalPoint.y * unitConversionFactor, 10. * finalPoint.z * unitConversionFactor);
							nVerticesTotal += nVert;
							if (escape) {
								if (extendEscapeTracks && trackSets[curParmSet].format >= 102) { // Last "vertex" of an escape track
									VECTOR finalSegment = pTrack->coord[pTrack->nVertices-1] - pTrack->coord[pTrack->nVertices-2];
									finalSegment = 2. * maximumObjectSizeRatio * viewSize * (finalSegment / Norm(finalSegment));
									VECTOR finalPoint2 = pTrack->coord[pTrack->nVertices-1] + finalSegment;
									fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * finalPoint2.x * unitConversionFactor, 10. * finalPoint2.y * unitConversionFactor, 10. * finalPoint2.z * unitConversionFactor);
								}
								else if (!extendEscapeTracks && trackSets[curParmSet].format == 101) oldFormatWarning = true;
							}
							nTracksWritten++;
						}
					}
					else nVerticesTotalMaxExceeded = true;
				}
			}
		}
	}
	else if (!err) {
		FILE *f_tracks = fopen(trackFileName, "r+t");
		if (!f_tracks) {
			if (!scriptFN || strspn(trackFileName, " \t") != strlen(trackFileName)) {
				MessageBox((LPCTSTR)(msg.Format(_T("Error: Track data file \"%s\" could not be opened for reading. Particle tracks will not be shown."), ShortenPath(trackFileName, currentFolder)), msg), NULL, MB_OK | MB_ICONERROR);
			}
			progressDlg->PostMessage(WM_CLOSE);
			return;
		}
		processingTrackData = true;
		int chunkSize = 10000;
		if (keepTrackDataInMemory) {
			trackSets[curParmSet].trackData = (TRACKDATA *)malloc(chunkSize * sizeof(TRACKDATA));
			trackSets[curParmSet].histStartTrackNum = (int *)malloc(chunkSize * sizeof(int));
			trackSets[curParmSet].nHistories = 0;
		}
		char *ptr, *endPtr;
		int historyNum, iTrack = -1;
		long long sourceParticleNum = 0;
		long gen;
		unsigned long originType;
		char ot;
		int format = 0;
		bool skipRemainingTracks = false;
		size_t nBytesLoaded = 0;
		double timeFactor = 1. / CLOCKS_PER_SEC;
		double startTime = timeFactor * clock();
		double elapsedTime_prev = 0.;
		while(!feof(f_tracks)) {
			if (!skipRemainingTracks && pTrackFilter->trackNo[1] >= 0 && iTrack >= pTrackFilter->trackNo[1]) {
				if (!keepTrackDataInMemory) break;
				else skipRemainingTracks = true;
			}
			line[0] = '\0'; fgets(line, 100, f_tracks); nBytesLoaded += strlen(line);
			if (line[0] == '\0') break;
			iTrack++;
			line[100] = '\0';
			char *ptr0; 
			if (!format || format >= 102) ptr0 = strchr(line, ':');
			if ((!format || format >= 102) && ptr0 && !strnicmp(ptr0 + 2, "History No.", 11) || (!format || format == 101) && !strnicmp(line, "Track No.", 9)) {
				if (!skipRemainingTracks && pTrackFilter->historyNo[1] >= 0 && iHist >= pTrackFilter->historyNo[1]) {
					if (!keepTrackDataInMemory) {
						iTrack--;
						break;
					}
					else skipRemainingTracks = true;
				}
				if (!format) {
					if (ptr0 && !strnicmp(ptr0 + 2, "History No.", 11)) format = 102;
					else format = 101;
				}
				if (format >= 102) {
					*ptr0 = '\0';
					int num = strtol(line, &endPtr, 10);
					if (*endPtr != '\0' && *endPtr != ' ' && *endPtr != '\t' || num != iTrack + 1) { err = true; break; }
					ptr = strtok(ptr0 + 13, "\n");
				}
				else ptr = strtok(line + 9, "\n");
				historyNum = strtol(ptr, &endPtr, 10);
				if (historyNum < 1 || *endPtr != '\0') { err = true; break; }
				iHist++;
				if (keepTrackDataInMemory) {
					if (iHist && iHist / chunkSize * chunkSize == iHist) trackSets[curParmSet].histStartTrackNum = (int *)realloc(trackSets[curParmSet].histStartTrackNum, (iHist + chunkSize) * sizeof(int));
					trackSets[curParmSet].histStartTrackNum[iHist] = iTrack;
				}
				gen = 0;
				originType = 1;
				ot = 0;
				line[0] = '\0'; fgets(line, 100, f_tracks); nBytesLoaded += strlen(line);
				if (line[0] == '\0') { iHist--; err = true; break; }
				line[100] = '\0';
				ptr = strtok(line, "\n");
				double sourceParticleNum_dbl = strtod(ptr, &endPtr);
				if (sourceParticleNum_dbl <= 0.5 || sourceParticleNum_dbl > 9223372036854775807. || *endPtr != '\0')  { err = true; break; }
				sourceParticleNum = (long long)(sourceParticleNum_dbl + 0.5);
			}
			else if (format >= 102) {
				strtok(line, "\n");
				char *ptr = strchr(line, ':');
				if (!ptr) { err = true; break; }
				*ptr = '\0';
				int num = strtol(line, &endPtr, 10);
				if (*endPtr != '\0' && *endPtr != ' ' && *endPtr != '\t' || num != iTrack + 1) { err = true; break; }
				char *ptr2 = strchr(ptr + 1, '/');
				if (!ptr2) { err = true; break; }
				*ptr2 = '\0';
				gen = strtol(ptr + 1, &endPtr, 10);
				if (*endPtr != '\0' && *endPtr != ' ' && *endPtr != '\t') { err = true; break; }
				originType = strtoul(ptr2 + 1, &endPtr, 10);
				if (*endPtr != '\0' && *endPtr != ' ' && *endPtr != '\t' || originType < 1 || originType >= NEVENTTYPES) { err = true; break; }
				ot = (char)originType;
				originType = bitMask[ot];
			}
			else { err = true; break; }
			if (keepTrackDataInMemory && iTrack && iTrack / chunkSize * chunkSize == iTrack) trackSets[curParmSet].trackData = (TRACKDATA *)realloc(trackSets[curParmSet].trackData, (iTrack + chunkSize) * sizeof(TRACKDATA));
			line[0] = '\0'; fgets(line, 100, f_tracks); nBytesLoaded += strlen(line);
			if (line[0] == '\0') { err = true; break; }
			line[100] = '\0';
			ptr = strtok(line, "\n");
			int nVertices = strtol(ptr, &endPtr, 10);
			if (format >= 102 && (nVertices < 1 || !gen && nVertices == 1) || format == 101 && nVertices < 2 && nVertices > -2 || *endPtr != '\0') { err = true; break; }
			bool truncated, escape = false;
			if (nVertices < 0) {
				nVertices = -nVertices;
				truncated = true;
			}
			else truncated = false;
			VECTOR *coord;
			double *energy;
			if (iHist >= pTrackFilter->historyNo[0] && iTrack >= pTrackFilter->trackNo[0] || keepTrackDataInMemory) {
				coord = new VECTOR[nVertices];
				energy = new double[nVertices];
			}
			for (i = 0; i < nVertices && !feof(f_tracks); i++) {
				double elapsedTime = timeFactor * clock() - startTime;
				if (elapsedTime - elapsedTime_prev >= 1.) {
					if (!progressDlg->m_visible) {
						progressDlg->m_visible = true;
						progressDlg->SetWindowPos(&wndTop, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
					}
					char tmp[50];
					double fraction = (double)nBytesLoaded / fileinfo.size;
					sprintf(tmp, "Reading the track data file (%.2f %%)", 100. * fraction);
					progressDlg->SetWindowText(tmp);
					progressDlg->m_progress.SetPos((int)(progressDlg->nSteps * fraction + 0.5));
					elapsedTime_prev = elapsedTime;
				}
				line[0] = '\0'; fgets(line, 100, f_tracks); nBytesLoaded += strlen(line);
				if (line[0] == '\0') break;
				if ((iHist < pTrackFilter->historyNo[0] || iTrack < pTrackFilter->trackNo[0]) && !keepTrackDataInMemory) continue;
				line[100] = '\0';
				ptr = strtok(line, " \t\n");
				double *coord_tmp = (double *)&(coord[i].x);
				for (j = 0; j < 3 && ptr; j++) {
					coord_tmp[j] = strtod(ptr, &endPtr);
					if (*endPtr != '\0') break;
					ptr = strtok(NULL, " \t\n");
				}
				if (j < 3 || !ptr) break;
				energy[i] = strtod(ptr, &endPtr);
				if (*endPtr != '\0') break;
				if (stopReadingTrackData && i < nVertices - 1) nVertices = i + 1;
				if (i == nVertices - 1) {
					if (format >= 102 || stopReadingTrackData) {
						if (energy[i] < 0.) {
							escape = true;
							energy[i] = -energy[i];
						}
						else if (energy[i] > 0.) truncated = true;
					}
					else if (!truncated && energy[i] > 0.) { // Last "vertex" of an escape track (old format)
						escape = true;
						VECTOR finalSegment = 2. * maximumObjectSizeRatio * viewSize * (coord[i] / Norm(coord[i]));
						coord[i] = coord[i-1] + finalSegment;
					}
				}
			}
			if (i < nVertices) err = true;
			if (!skipRemainingTracks && iHist >= pTrackFilter->historyNo[0] && iTrack >= pTrackFilter->trackNo[0]) {
				if (!err) {
					int iStart, iEnd;
					char type;
					if (truncated) type = -1;
					else if (escape) type = 1;
					else type = 0;
					bool success = UpdateTrackStats(pTrackStats, iHist, iTrack, nVertices, gen, originType, coord, energy, type, iStart, iEnd, add1, initialPoint, add2, finalPoint, pTrackFilter, pTrackSegmentSelection, trackFilteringBeforeSegmentSelection);
					if (scriptFN && success && !nVerticesTotalMaxExceeded) {
						int jMax = escape ? nVertices - 1 : iEnd;
						int nVert0 = jMax - iStart + 1 + add1 + add2;
						int nVert = (nVerticesTrackMax < 0 || nVert0 <= nVerticesTrackMax) ? nVert0 : nVerticesTrackMax;
						if (nVerticesTotalMax - nVerticesTotal >= nVert) {
							if (!trackOutputError[ot] && !f_track[ot]) {
								CString msg;
								sprintf(trackScriptFN[ot], "%s%s.rt", scriptFN, trackFileSuffix[ot]);
								f_track[ot] = fopen(trackScriptFN[ot], "wt");
								if (f_track[ot]) {
									fprintf(f_track[ot], "vdraw open tracks%d\n", ot);
									fprintf(f_track[ot], "vdraw params color %02x%02x%02x\n", trackColor[ot].r, trackColor[ot].g, trackColor[ot].b);
								}
								else {
									trackOutputError[ot] = true;
									trackScriptFN[ot][0] = '\0';
									MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for writing."), trackScriptFN[ot]), msg), NULL, MB_OK | MB_ICONERROR);
								}
							}
							if (!trackOutputError[ot]) {
								FILE *f_script = f_track[ot];
								fprintf(f_script, "vdraw write next 0 %.14g %.14g %.14g\n", 10. * initialPoint.x * unitConversionFactor, 10. * initialPoint.y * unitConversionFactor, 10. * initialPoint.z * unitConversionFactor);
								j = iStart + 1 - add1;
								if (nVert == nVert0 && j <= jMax || nVert != nVert0 && j < iEnd + add2) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * coord[j].x * unitConversionFactor, 10. * coord[j].y * unitConversionFactor, 10. * coord[j].z * unitConversionFactor);
								if (nVert == nVert0) {
									if (j <= jMax) {
										for (j++; j <= jMax; j++) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * coord[j].x * unitConversionFactor, 10. * coord[j].y * unitConversionFactor, 10. * coord[j].z * unitConversionFactor);
									}
								}
								else {
									if (j < iEnd + add2) {
										double increment = (double)(iEnd - iStart - 1 + add1 + add2) / (nVert - 2 - (jMax - iEnd));
										double jDbl = j + increment;
										j = (int)(jDbl + 0.5);
										for (; j < iEnd + add2; jDbl += increment, j = (int)(jDbl + 0.5)) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * coord[j].x * unitConversionFactor, 10. * coord[j].y * unitConversionFactor, 10. * coord[j].z * unitConversionFactor);
									}
									for (j = iEnd + add2; j <= jMax; j++) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * coord[j].x * unitConversionFactor, 10. * coord[j].y * unitConversionFactor, 10. * coord[j].z * unitConversionFactor);
									verticesSkipped = true;
								}
								if (j <= jMax + add2) fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * finalPoint.x * unitConversionFactor, 10. * finalPoint.y * unitConversionFactor, 10. * finalPoint.z * unitConversionFactor);
								nVerticesTotal += nVert;
								if (escape) {
									if (extendEscapeTracks && format >= 102) { // Last "vertex" of an escape track
										VECTOR finalSegment = coord[nVertices-1] - coord[nVertices-2];
										finalSegment = 2. * maximumObjectSizeRatio * viewSize * (finalSegment / Norm(finalSegment));
										VECTOR finalPoint2 = coord[nVertices-1] + finalSegment;
										fprintf(f_script, "vdraw write next 1 %.14g %.14g %.14g\n", 10. * finalPoint2.x * unitConversionFactor, 10. * finalPoint2.y * unitConversionFactor, 10. * finalPoint2.z * unitConversionFactor);
									}
									else if (!extendEscapeTracks && format == 101) oldFormatWarning = true;
								}
								nTracksWritten++;
							}
						}
						else nVerticesTotalMaxExceeded = true;
					}
				}
				if (!keepTrackDataInMemory) {
					delete coord;
					delete energy;
				}
			}
			if (err) break;
			if (keepTrackDataInMemory) {
				TRACKDATA *pTrack = trackSets[curParmSet].trackData + iTrack;
				pTrack->sourceParticleNum = sourceParticleNum;
				pTrack->nVertices = nVertices;
				pTrack->gen = gen;
				pTrack->originType = originType;
				pTrack->ot = ot;
				if (truncated) pTrack->type = -1;
				else if (escape) pTrack->type = 1;
				else pTrack->type = 0;
				pTrack->coord = coord;
				pTrack->energy = energy;
			}
			if (stopReadingTrackData) break;
		}
		if (stopReadingTrackData && !err && !feof(f_tracks)) {
			line[0] = '\0'; fgets(line, 100, f_tracks);
			if (line[0] != '\0') trackSets[curParmSet].fileSize = -1;
		}
		fclose(f_tracks);
		if (trackSets[curParmSet].fileSize < 0) {
			MessageBox((LPCTSTR)(msg.Format(_T("The process of reading the track data from file \"%s\" has been interrupted. The track statistics that will be shown will correspond to the incomplete dataset."), ShortenPath(trackFileName, currentFolder)), msg), "Reading of the track dataset has been interrupted", MB_OK | MB_ICONWARNING);
		}
		if (err) {
			MessageBox((LPCTSTR)(msg.Format(_T("Error: File \"%s\" has incorrect format (error occurred while reading history No. %d, corresponding to source particle No. %lld)."), ShortenPath(trackFileName, currentFolder), iHist + 1, sourceParticleNum), msg), NULL, MB_OK | MB_ICONERROR);
		}
		if (keepTrackDataInMemory) {
			trackSets[curParmSet].nHistories = iHist + 1;
			if (!err) trackSets[curParmSet].nTracks = iTrack + 1;
			else trackSets[curParmSet].nTracks = iTrack;
			if (trackSets[curParmSet].nTracks) {
				trackSets[curParmSet].trackData = (TRACKDATA *)realloc(trackSets[curParmSet].trackData, trackSets[curParmSet].nTracks * sizeof(TRACKDATA));
				trackSets[curParmSet].histStartTrackNum = (int *)realloc(trackSets[curParmSet].histStartTrackNum, (trackSets[curParmSet].nHistories + 1) * sizeof(int));
				trackSets[curParmSet].histStartTrackNum[trackSets[curParmSet].nHistories] = trackSets[curParmSet].nTracks;
			}
			else {
				free(trackSets[curParmSet].trackData);
				free(trackSets[curParmSet].histStartTrackNum);
			}
			if (format) trackSets[curParmSet].format = format;
			else trackSets[curParmSet].format = 102;
		}
	}
	else if (!scriptFN || strspn(trackFileName, " \t") != strlen(trackFileName)) {
		MessageBox((LPCTSTR)(msg.Format(_T("Error: Track data file \"%s\" could not be opened for reading. Particle tracks will not be shown."), ShortenPath(trackFileName, currentFolder)), msg), NULL, MB_OK | MB_ICONERROR);
		progressDlg->PostMessage(WM_CLOSE);
		return;
	}
	if (scriptFN) {
		for (i = 0; i < NEVENTTYPES; i++) {
			if (f_track[i]) {
				fprintf(f_track[i], "vdraw send\n");
				fclose(f_track[i]);
			}
		}
	}
	if (oldFormatWarning) {
		MessageBox((LPCTSTR)(msg.Format(_T("Warning: File \"%s\" was created with an older version of MCNScript and has no escape point coordinates. Consequently, the escape tracks will be extended beyond the clipping cube."), ShortenPath(trackFileName, currentFolder)), msg), NULL, MB_OK | MB_ICONWARNING);
	}
	char trackMsg[500] = {'\0'};
	int k = 0;
	if (nVerticesTotalMaxExceeded) k = sprintf(trackMsg, "The maximum allowed total number of vertices (%d) has been exceeded. %d out of %d tracks will be shown.\n", nVerticesTotalMax, nTracksWritten, pTrackStats->nTracks);
	if (verticesSkipped) sprintf(trackMsg + k, "Some tracks have more than %d vertices. Some vertices in those tracks will not be shown.", nVerticesTrackMax);
	if (trackMsg[0] != '\0') MessageBox(trackMsg, NULL, MB_OK | MB_ICONWARNING);

	if (pTrackStats->nTracks > 0) {
		double factor = 1. / pTrackStats->nTracks;
		pTrackStats->nColl0_avg = factor * pTrackStats->nColl0_sum;
		pTrackStats->nColl_avg = factor * pTrackStats->nColl_sum;
		pTrackStats->startE_avg = factor * pTrackStats->startE_sum;
		pTrackStats->finalE_avg = factor * pTrackStats->finalE_sum;
		pTrackStats->Eloss0_avg = factor * pTrackStats->Eloss0_sum;
		pTrackStats->Eloss_avg = factor * pTrackStats->Eloss_sum;
		pTrackStats->relEloss0_avg = factor * pTrackStats->relEloss0_sum;
		pTrackStats->relEloss_avg = factor * pTrackStats->relEloss_sum;
		pTrackStats->displ0_avg = factor * pTrackStats->displ0_sum;
		pTrackStats->path0_avg = factor * pTrackStats->path0_sum;
		if (pTrackStats->nTracksWithTwoOrMoreVertices > 0) {
			factor = 1. / pTrackStats->nTracksWithTwoOrMoreVertices;
			pTrackStats->displ_avg = factor * pTrackStats->displ_sum;
			pTrackStats->peakDispl_avg = factor * pTrackStats->peakDispl_sum;
			pTrackStats->path_avg = factor * pTrackStats->path_sum;
		}
		if (pTrackStats->nTracksWithCollisions > 0) {
			factor = 1. / pTrackStats->nTracksWithCollisions;
			pTrackStats->meanFreePath_avg = factor * pTrackStats->meanFreePath_sum;
			pTrackStats->meanElossPerCollision_avg = factor * pTrackStats->meanElossPerCollision_sum;
		}
		if (pTrackStats->nTracks > 1) {
			factor = 1. / (pTrackStats->nTracks - 1) / pTrackStats->nTracks;
			double tmp = pTrackStats->nColl0_sum2 - pTrackStats->nColl0_avg * pTrackStats->nColl0_sum; if (tmp < 0. || pTrackStats->nColl0_min >= pTrackStats->nColl0_max) tmp = 0.;
			pTrackStats->nColl0_SD = sqrt(factor * tmp);
			tmp = pTrackStats->nColl_sum2 - pTrackStats->nColl_avg * pTrackStats->nColl_sum; if (tmp < 0. || pTrackStats->nColl_min >= pTrackStats->nColl_max) tmp = 0.;
			pTrackStats->nColl_SD = sqrt(factor * tmp);
			tmp = pTrackStats->startE_sum2 - pTrackStats->startE_avg * pTrackStats->startE_sum; if (tmp < 0. || pTrackStats->startE_min && pTrackStats->startE_max && pTrackStats->startE_max / pTrackStats->startE_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->startE_SD = sqrt(factor * tmp);
			tmp = pTrackStats->finalE_sum2 - pTrackStats->finalE_avg * pTrackStats->finalE_sum; if (tmp < 0. || pTrackStats->finalE_min && pTrackStats->finalE_max && pTrackStats->finalE_max / pTrackStats->finalE_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->finalE_SD = sqrt(factor * tmp);
			tmp = pTrackStats->Eloss0_sum2 - pTrackStats->Eloss0_avg * pTrackStats->Eloss0_sum; if (tmp < 0. || pTrackStats->Eloss0_min && pTrackStats->Eloss0_max && pTrackStats->Eloss0_max / pTrackStats->Eloss0_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->Eloss0_SD = sqrt(factor * tmp);
			tmp = pTrackStats->Eloss_sum2 - pTrackStats->Eloss_avg * pTrackStats->Eloss_sum; if (tmp < 0. || pTrackStats->Eloss_min && pTrackStats->Eloss_max && pTrackStats->Eloss_max / pTrackStats->Eloss_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->Eloss_SD = sqrt(factor * tmp);
			tmp = pTrackStats->relEloss0_sum2 - pTrackStats->relEloss0_avg * pTrackStats->relEloss0_sum; if (tmp < 0. || pTrackStats->relEloss0_min && pTrackStats->relEloss0_max && pTrackStats->relEloss0_max / pTrackStats->relEloss0_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->relEloss0_SD = sqrt(factor * tmp);
			tmp = pTrackStats->relEloss_sum2 - pTrackStats->relEloss_avg * pTrackStats->relEloss_sum; if (tmp < 0. || pTrackStats->relEloss_min && pTrackStats->relEloss_max && pTrackStats->relEloss_max / pTrackStats->relEloss_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->relEloss_SD = sqrt(factor * tmp);
			tmp = pTrackStats->displ0_sum2 - pTrackStats->displ0_avg * pTrackStats->displ0_sum; if (tmp < 0. || pTrackStats->displ0_min && pTrackStats->displ0_max && pTrackStats->displ0_max / pTrackStats->displ0_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->displ0_SD = sqrt(factor * tmp);
			tmp = pTrackStats->path0_sum2 - pTrackStats->path0_avg * pTrackStats->path0_sum; if (tmp < 0. || pTrackStats->path0_min && pTrackStats->path0_max && pTrackStats->path0_max / pTrackStats->path0_min - 1. < 2e-15) tmp = 0.;
			pTrackStats->path0_SD = sqrt(factor * tmp);
			if (pTrackStats->nTracksWithTwoOrMoreVertices > 1) {
				factor = 1. / (pTrackStats->nTracksWithTwoOrMoreVertices - 1) / pTrackStats->nTracksWithTwoOrMoreVertices;
				tmp = pTrackStats->displ_sum2 - pTrackStats->displ_avg * pTrackStats->displ_sum; if (tmp < 0. || pTrackStats->displ_min && pTrackStats->displ_max && pTrackStats->displ_max / pTrackStats->displ_min - 1. < 2e-15) tmp = 0.;
				pTrackStats->displ_SD = sqrt(factor * tmp);
				tmp = pTrackStats->peakDispl_sum2 - pTrackStats->peakDispl_avg * pTrackStats->peakDispl_sum; if (tmp < 0. || pTrackStats->peakDispl_min && pTrackStats->peakDispl_max && pTrackStats->peakDispl_max / pTrackStats->peakDispl_min - 1. < 2e-15) tmp = 0.;
				pTrackStats->peakDispl_SD = sqrt(factor * tmp);
				tmp = pTrackStats->path_sum2 - pTrackStats->path_avg * pTrackStats->path_sum; if (tmp < 0. || pTrackStats->path_min && pTrackStats->path_max && pTrackStats->path_max / pTrackStats->path_min - 1. < 2e-15) tmp = 0.;
				pTrackStats->path_SD = sqrt(factor * tmp);
			}
			if (pTrackStats->nTracksWithCollisions > 1) {
				factor = 1. / (pTrackStats->nTracksWithCollisions - 1) / pTrackStats->nTracksWithCollisions;
				tmp = pTrackStats->meanFreePath_sum2 - pTrackStats->meanFreePath_avg * pTrackStats->meanFreePath_sum; if (tmp < 0. || pTrackStats->meanFreePath_min && pTrackStats->meanFreePath_max && pTrackStats->meanFreePath_max / pTrackStats->meanFreePath_min - 1. < 2e-15) tmp = 0.;
				pTrackStats->meanFreePath_SD = sqrt(factor * tmp);
				tmp = pTrackStats->meanElossPerCollision_sum2 - pTrackStats->meanElossPerCollision_avg * pTrackStats->meanElossPerCollision_sum; if (tmp < 0. || pTrackStats->meanElossPerCollision_min && pTrackStats->meanElossPerCollision_max && pTrackStats->meanElossPerCollision_max / pTrackStats->meanElossPerCollision_min - 1. < 2e-15) tmp = 0.;
				pTrackStats->meanElossPerCollision_SD = sqrt(factor * tmp);
			}
		}
		if (pTrackSegmentSelection && pTrackSegmentSelection->path[1] > pTrackSegmentSelection->path[0]) {
			double pathLength = pTrackSegmentSelection->path[1] - pTrackSegmentSelection->path[0];
			pTrackStats->stoppingPower_min = pTrackStats->Eloss_min / pathLength;
			pTrackStats->stoppingPower_max = pTrackStats->Eloss_max / pathLength;
			pTrackStats->stoppingPower_avg = pTrackStats->Eloss_avg / pathLength;
			pTrackStats->stoppingPower_SD = pTrackStats->Eloss_SD / pathLength;
		}
	}
	progressDlg->PostMessage(WM_CLOSE);
	processingTrackData = false;
}

void CMCNScriptDlg::OnRefresh()
{
	pTrackFilterGrid->ResetWidth();
	if (useCustomTrackFN && strspn(trackFN, " \t") == strlen(trackFN)) OnLoadTrackFile();
	if (!useCustomTrackFN || strspn(trackFN, " \t") != strlen(trackFN)) {
		if (!showTrackSegments) LoadTrackFile(this, &trackStats, NULL, NULL, &trackFilter);
		else LoadTrackFile(this, &trackStats, NULL, NULL, &trackFilter, &trackSegmentSelection, trackFilteringBeforeSegmentSelection);
	}
	else memset(&trackStats, 0, sizeof(TRACKSTATS));
	char tmp[100];
	sprintf(tmp, "%d", trackStats.nTracks);
	m_nTracks.SetWindowText(tmp);
}

void CMCNScriptDlg::OnDefaultFiltering()
{
	if (CompareTrackFilters(&trackFilter, &defaultTrackFilter) || CompareTrackSegmentData(&trackSegmentSelection, &defaultTrackSegmentSelection) || !trackFilteringBeforeSegmentSelection) {
		changed = true;
		m_nTracks.SetWindowText("");
		trackFilter = defaultTrackFilter;
		trackSegmentSelection = defaultTrackSegmentSelection;
		trackFilteringBeforeSegmentSelection = TRUE;
	}
	UpdateTrackFilter(pTrackFilterGrid, pOriginTypeCheckbox, pGenMinEdit, pGenMaxEdit, &trackFilter);
}

void CMCNScriptDlg::OnTrackStats()
{
	CTrackStatsDlg dlg(this);
	dlg.m_hAccel = m_hAccel_trackStats;
	int unitEnergy_bak = unitEnergy;
	int unitDistance_bak = unitDistance;
	TRACKFILTERDATA trackFilter_bak = trackFilter;
	TRACKSEGMENTDATA trackSegmentSelection_bak = trackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection_bak = trackFilteringBeforeSegmentSelection;
	__int64 fileSize_bak = trackSets[curParmSet].fileSize;
	INT_PTR rc = dlg.DoModal();
	pTrackColorsDlg->pTrackFilter = &trackFilter;
	pTrackColorsDlg->pOriginTypeCheckbox = pOriginTypeCheckbox;
	if (rc == IDOK) {
		if (unitEnergy != unitEnergy_bak || unitDistance != unitDistance_bak) changed = true;
		if (CompareTrackFilters(&trackFilter, &trackFilter_bak) ||
			CompareTrackSegmentData(&trackSegmentSelection, &trackSegmentSelection_bak) ||
			trackFilteringBeforeSegmentSelection != trackFilteringBeforeSegmentSelection_bak) {
			changed = true;
			m_nTracks.SetWindowText("");
		}
		else if (trackSets[curParmSet].fileSize < 0) m_nTracks.SetWindowText("");
	}
	UpdateTrackFilter(pTrackFilterGrid, pOriginTypeCheckbox, pGenMinEdit, pGenMaxEdit, &trackFilter);
}

void CMCNScriptDlg::UpdateTrackFilter(CGridCtrlModified *pGrid, CButton *pCheck, CEdit *pGenMin, CEdit *pGenMax, TRACKFILTERDATA *pTrackFilter)
{
	pGrid->SetItemText(0, 0, "Bounded parameter");
	pGrid->SetItemText(0, 1, "Minimum");
	pGrid->SetItemText(0, 2, "Maximum");
	pGrid->SetItemText(1, 0, "History number");
	pGrid->SetItemText(2, 0, "Track number");
	pGrid->SetItemText(3, 0, "Number of collisions in a track");
	pGrid->SetItemText(4, 0, (CString)"Initial energy (" + energyUnits[unitEnergy] + ")");
	pGrid->SetItemText(5, 0, (CString)"Final energy (" + energyUnits[unitEnergy] + ")");
	pGrid->SetItemText(6, 0, (CString)"Energy loss (" + energyUnits[unitEnergy] + ")");
	pGrid->SetItemText(7, 0, "Relative energy loss (%)");
	pGrid->SetItemText(8, 0, (CString)"Displacement (" + distanceUnits[unitDistance] + ")");
	pGrid->SetItemText(9, 0, (CString)"Peak displacement (" + distanceUnits[unitDistance] + ")");
	pGrid->SetItemText(10, 0, (CString)"Path length (" + distanceUnits[unitDistance] + ")");
	char tmp[100];
	sprintf(tmp, "%d", pTrackFilter->historyNo[0] + 1); pGrid->SetItemText(1, 1, tmp);
	if (pTrackFilter->historyNo[1] >= 0) {
		sprintf(tmp, "%d", pTrackFilter->historyNo[1] + 1); pGrid->SetItemText(1, 2, tmp);
	}
	else pGrid->SetItemText(1, 2, "");
	sprintf(tmp, "%d", pTrackFilter->trackNo[0] + 1); pGrid->SetItemText(2, 1, tmp);
	if (pTrackFilter->trackNo[1] >= 0) {
		sprintf(tmp, "%d", pTrackFilter->trackNo[1] + 1); pGrid->SetItemText(2, 2, tmp);
	}
	else pGrid->SetItemText(2, 2, "");
	sprintf(tmp, "%d", pTrackFilter->nVert[0] - 1); pGrid->SetItemText(3, 1, tmp);
	if (pTrackFilter->nVert[1] > 0) {
		sprintf(tmp, "%d", pTrackFilter->nVert[1] - 1); pGrid->SetItemText(3, 2, tmp);
	}
	else pGrid->SetItemText(3, 2, "");
	sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Estart[0]); pGrid->SetItemText(4, 1, tmp);
	if (pTrackFilter->Estart[1] >= 0.) {
		sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Estart[1]); pGrid->SetItemText(4, 2, tmp);
	}
	else pGrid->SetItemText(4, 2, "");
	sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Efinal[0]); pGrid->SetItemText(5, 1, tmp);
	if (pTrackFilter->Efinal[1] >= 0.) {
		sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Efinal[1]); pGrid->SetItemText(5, 2, tmp);
	}
	else pGrid->SetItemText(5, 2, "");
	sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Eloss[0]); pGrid->SetItemText(6, 1, tmp);
	if (pTrackFilter->Eloss[1] >= 0.) {
		sprintf(tmp, "%g", energyFactor[unitEnergy] * pTrackFilter->Eloss[1]); pGrid->SetItemText(6, 2, tmp);
	}
	else pGrid->SetItemText(6, 2, "");
	sprintf(tmp, "%g", 100. * pTrackFilter->relEloss[0]); pGrid->SetItemText(7, 1, tmp);
	sprintf(tmp, "%g", 100. * pTrackFilter->relEloss[1]); pGrid->SetItemText(7, 2, tmp);
	sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->displ[0]); pGrid->SetItemText(8, 1, tmp);
	if (pTrackFilter->displ[1] >= 0.) {
		sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->displ[1]); pGrid->SetItemText(8, 2, tmp);
	}
	else pGrid->SetItemText(8, 2, "");
	sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->peakDispl[0]); pGrid->SetItemText(9, 1, tmp);
	if (pTrackFilter->peakDispl[1] >= 0.) {
		sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->peakDispl[1]); pGrid->SetItemText(9, 2, tmp);
	}
	else pGrid->SetItemText(9, 2, "");
	sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->path[0]); pGrid->SetItemText(10, 1, tmp);
	if (pTrackFilter->path[1] >= 0.) {
		sprintf(tmp, "%g", distanceFactor[unitDistance] * pTrackFilter->path[1]); pGrid->SetItemText(10, 2, tmp);
	}
	else pGrid->SetItemText(10, 2, "");
	pGrid->ResetWidth();

	int i;
	for (i = 0; i < NEVENTTYPES; i++) {
		pCheck[i].SetCheck(pTrackFilter->originTypeMask & bitMask[i]);
		CGridCellCheck *pCheck2 = (CGridCellCheck *)pTrackColorsGrid->GetCell(i + 1, 2);
		pCheck2->m_bChecked = ((bitMask[i] & pTrackFilter->originTypeMask) != 0);
		UpdateShow_tracks(pCheck2->m_bChecked, i);
	}
	pTrackColorsGrid->Invalidate();
	pTrackColorsGrid->UpdateWindow();
	sprintf(tmp, "%d", pTrackFilter->gen[0]);
	pGenMin->SetWindowText(tmp);
	pTrackColorsDlg->m_genMin.SetWindowText(tmp);
	if (pTrackFilter->gen[1] >= 0) {
		sprintf(tmp, "%d", pTrackFilter->gen[1]);
		pGenMax->SetWindowText(tmp);
		pTrackColorsDlg->m_genMax.SetWindowText(tmp);
	}
	else {
		pGenMax->SetWindowText("");
		pTrackColorsDlg->m_genMax.SetWindowText("");
	}
}

BEGIN_MESSAGE_MAP(CTrackStatsDlg, CDialogEx)
	ON_CBN_SELCHANGE(IDC_UNITENERGY, OnEnergyUnitSelected)
	ON_CBN_SELCHANGE(IDC_UNITDISTANCE, OnDistanceUnitSelected)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_TRACKSEGMENT, OnTrackSegmentGridEndEdit)
	ON_BN_CLICKED(IDC_REFRESH, OnRefresh)
	ON_BN_CLICKED(IDC_DEFAULTFILTERING, OnDefaultFiltering)
	ON_BN_CLICKED(IDC_KEEPTRACKDATAINMEMORY, OnKeepTrackDataInMemory)
	ON_BN_CLICKED(IDC_TRACKFILTERBEFORESEGMENTSELECTION, OnTrackFilteringBeforeSegmentSelection)
	ON_BN_CLICKED(IDC_SEGMENTSELECTIONBEFORETRACKFILTER, OnSegmentSelectionBeforeTrackFiltering)
	ON_COMMAND(ID_AUTOSIZEGRID, OnAutosizeGrid)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CTrackStatsDlg, CDialogEx)

CTrackStatsDlg::CTrackStatsDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CTrackStatsDlg::IDD, pParent)
{
	parent = (CMCNScriptDlg *)pParent;
	nRows = 19;
	nCols = 5;
	trackSegmentSelection = parent->trackSegmentSelection;
	trackFilter = parent->trackFilter;
	trackStats = parent->trackStats;
	trackFilteringBeforeSegmentSelection = parent->trackFilteringBeforeSegmentSelection;
	dialogInitialized = false;
}

BOOL CTrackStatsDlg::PreTranslateMessage(MSG *pMsg) 
{
	m_toolTip.RelayEvent(pMsg);
	if (pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST && m_hAccel != NULL) {
			if (::TranslateAccelerator(m_hWnd, m_hAccel, pMsg)) return TRUE;
	}
	return CDialogEx::PreTranslateMessage(pMsg);
}

void CTrackStatsDlg::UpdateTrackStats()
{
	int j, k;
	char tmp[100];
	CGridCtrl *pGrid = &m_trackStatsGrid;
	sprintf(tmp, "%d", trackStats.nHistories); m_nHistories.SetWindowText(tmp);
	sprintf(tmp, "%d", trackStats.nTracks); m_nTracks.SetWindowText(tmp);
	sprintf(tmp, "%d", trackStats.nTracksEscape); m_nTracksEscape.SetWindowText(tmp);
	sprintf(tmp, "%d", trackStats.nTracksTruncated); m_nTracksTruncated.SetWindowText(tmp);
	if (trackStats.nTracks > 0) {
		sprintf(tmp, "%d", trackStats.iHist_min + 1); pGrid->SetItemText(1, 1, tmp);
		sprintf(tmp, "%d", trackStats.iHist_max + 1); pGrid->SetItemText(1, 2, tmp);
		sprintf(tmp, "%d", trackStats.iTrack_min + 1); pGrid->SetItemText(2, 1, tmp);
		sprintf(tmp, "%d", trackStats.iTrack_max + 1); pGrid->SetItemText(2, 2, tmp);
		sprintf(tmp, "%d", trackStats.nColl0_min); pGrid->SetItemText(3, 1, tmp);
		sprintf(tmp, "%d", trackStats.nColl0_max); pGrid->SetItemText(3, 2, tmp);
		sprintf(tmp, "%.4g", trackStats.nColl0_avg); pGrid->SetItemText(3, 3, tmp);
		sprintf(tmp, "%.4g", trackStats.nColl0_SD); pGrid->SetItemText(3, 4, tmp);
		sprintf(tmp, "%d", trackStats.nColl_min); pGrid->SetItemText(4, 1, tmp);
		sprintf(tmp, "%d", trackStats.nColl_max); pGrid->SetItemText(4, 2, tmp);
		sprintf(tmp, "%.4g", trackStats.nColl_avg); pGrid->SetItemText(4, 3, tmp);
		sprintf(tmp, "%.4g", trackStats.nColl_SD); pGrid->SetItemText(4, 4, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss0_min); pGrid->SetItemText(9, 1, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss0_max); pGrid->SetItemText(9, 2, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss0_avg); pGrid->SetItemText(9, 3, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss0_SD); pGrid->SetItemText(9, 4, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss_min); pGrid->SetItemText(10, 1, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss_max); pGrid->SetItemText(10, 2, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss_avg); pGrid->SetItemText(10, 3, tmp);
		sprintf(tmp, "%.4g", 100. * trackStats.relEloss_SD); pGrid->SetItemText(10, 4, tmp);
	}
	else {
		for (j = 1; j < nRows; j++) {
		  for (k = 1; k < nCols; k++) pGrid->SetItemText(j, k, "");
		}
	}
	UpdateUnits();
}

void CTrackStatsDlg::UpdateUnits()
{
	int j, k;
	CGridCtrl *pGrid = &m_trackStatsGrid;
	pGrid->SetItemText(5, 0, (CString)"Initial energy (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(6, 0, (CString)"Final energy (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(7, 0, (CString)"Initial energy loss (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(8, 0, (CString)"Energy loss (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(11, 0, (CString)"Initial displacement (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(12, 0, (CString)"Displacement (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(13, 0, (CString)"Peak displacement (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(14, 0, (CString)"Initial path length (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(15, 0, (CString)"Path length (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(16, 0, (CString)"Mean free path (" + parent->distanceUnits[parent->unitDistance] + ")");
	pGrid->SetItemText(17, 0, (CString)"Mean energy loss per collision (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(18, 0, (CString)"Stopping power (" + parent->energyUnits[parent->unitEnergy] + "/" + parent->distanceUnits[parent->unitDistance] + ")");
	char tmp[100];
	if (trackStats.nTracks > 0) {
		double *arrayPtr = (double *)&(trackStats.startE_min);
		double factor = parent->energyFactor[parent->unitEnergy];
		for (j = 5; j < 9; j++) {
			for (k = 1; k < nCols; k++, arrayPtr++) {
				sprintf(tmp, "%.4g", factor * *arrayPtr);
				pGrid->SetItemText(j, k, tmp);
			}
		}
		arrayPtr += 2 * (nCols - 1);
		factor = parent->distanceFactor[parent->unitDistance];
		if (trackStats.nTracksWithTwoOrMoreVertices > 0) {
			for (j = 11; j < 16; j++) {
				for (k = 1; k < nCols; k++, arrayPtr++) {
					sprintf(tmp, "%.4g", factor * *arrayPtr);
					pGrid->SetItemText(j, k, tmp);
				}
			}
			if (trackStats.nTracksWithCollisions > 0) {
				for (k = 1; k < nCols; k++, arrayPtr++) {
					sprintf(tmp, "%.4g", factor * *arrayPtr);
					pGrid->SetItemText(16, k, tmp);
				}
				factor = parent->energyFactor[parent->unitEnergy];
				for (k = 1; k < nCols; k++, arrayPtr++) {
					sprintf(tmp, "%.4g", factor * *arrayPtr);
					pGrid->SetItemText(17, k, tmp);
				}
			}
			else {
				for (j = 16; j < nRows - 1; j++) {
				 for (k = 1; k < nCols; k++) pGrid->SetItemText(j, k, "");
				}
			}
		}
		else {
			for (k = 1; k < nCols; k++, arrayPtr++) {
				sprintf(tmp, "%.4g", factor * *arrayPtr);
				pGrid->SetItemText(11, k, tmp);
			}
			for (j = 12; j < 14; j++) {
			  for (k = 1; k < nCols; k++) pGrid->SetItemText(j, k, "");
			}
			arrayPtr += 2 * (nCols - 1);
			for (k = 1; k < nCols; k++, arrayPtr++) {
				sprintf(tmp, "%.4g", factor * *arrayPtr);
				pGrid->SetItemText(14, k, tmp);
			}
			for (j = 15; j < nRows - 1; j++) {
			  for (k = 1; k < nCols; k++) pGrid->SetItemText(j, k, "");
			}
		}
		factor = parent->energyFactor[parent->unitEnergy] / parent->distanceFactor[parent->unitDistance];
		if (trackSegmentSelection.path[1] > trackSegmentSelection.path[0] && fabs(trackStats.path_min / (trackSegmentSelection.path[1] - trackSegmentSelection.path[0]) - 1.) < 1e-4) {
			sprintf(tmp, "%.4g", factor * trackStats.stoppingPower_min);
			pGrid->SetItemText(nRows - 1, 1, tmp);
			sprintf(tmp, "%.4g", factor * trackStats.stoppingPower_max);
			pGrid->SetItemText(nRows - 1, 2, tmp);
			sprintf(tmp, "%.4g", factor * trackStats.stoppingPower_avg);
			pGrid->SetItemText(nRows - 1, 3, tmp);
			sprintf(tmp, "%.4g", factor * trackStats.stoppingPower_SD);
			pGrid->SetItemText(nRows - 1, 4, tmp);
		}
		else {
			pGrid->SetItemText(nRows - 1, 1, "");
			pGrid->SetItemText(nRows - 1, 2, "");
			pGrid->SetItemText(nRows - 1, 3, "");
			pGrid->SetItemText(nRows - 1, 4, "");
		}
	}
	pGrid->ResetWidth();
	parent->UpdateTrackFilter(pTrackFilterGrid, pOriginTypeCheckbox, pGenMinEdit, pGenMaxEdit, &trackFilter);
	UpdateTrackSegmentGrid();
}

void CTrackStatsDlg::OnAutosizeGrid()
{
	pTrackFilterGrid->ResetWidth();
	m_trackSegmentGrid.ResetWidth();
}

BOOL CTrackStatsDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	VERIFY(m_trackFilteringBeforeSegmentSelection.SubclassDlgItem(IDC_TRACKFILTERBEFORESEGMENTSELECTION, this));
	VERIFY(m_segmentSelectionBeforeTrackFiltering.SubclassDlgItem(IDC_SEGMENTSELECTIONBEFORETRACKFILTER, this));
	VERIFY(m_unitEnergy.SubclassDlgItem(IDC_UNITENERGY, this));
	VERIFY(m_unitDistance.SubclassDlgItem(IDC_UNITDISTANCE, this));
	VERIFY(m_nHistories.SubclassDlgItem(IDC_NHISTORIES, this));
	VERIFY(m_nTracks.SubclassDlgItem(IDC_NTRACKS, this));
	VERIFY(m_nTracksEscape.SubclassDlgItem(IDC_NESCAPED, this));
	VERIFY(m_nTracksTruncated.SubclassDlgItem(IDC_NTRUNCATED, this));
	VERIFY(m_keepTrackDataInMemoryCheck.SubclassDlgItem(IDC_KEEPTRACKDATAINMEMORY, this));
	UpdateData();
	if (m_toolTip.Create(this)) {
		m_toolTip.AddTool(&m_trackSegmentGrid, _T("This grid control allows replacing each track by its segment satisfying the indicated constraints. The point of the first collision of that segment is treated as the source point (i.e., it is not included in the total number of collisions)."));
		m_toolTip.AddTool(&m_keepTrackDataInMemoryCheck, _T("If this checkbox is checked, then the entire dataset stored in the track data file will be kept in memory for subsequent use. Otherwise, the track data will be re-loaded from file every time when they are needed."));
		m_toolTip.SetDelayTime(TTDT_AUTOPOP, 20000);
		m_toolTip.Activate(TRUE);
	}

	int i, j, k;
	for (i = 0; i < 3; i++) m_unitEnergy.AddString(parent->energyUnits[i]);
	m_unitEnergy.SetCurSel(parent->unitEnergy);
	for (i = 0; i < 4; i++) m_unitDistance.AddString(parent->distanceUnits[i]);
	m_unitDistance.SetCurSel(parent->unitDistance);
	if (parent->keepTrackDataInMemory) m_keepTrackDataInMemoryCheck.SetCheck(1);
	else m_keepTrackDataInMemoryCheck.SetCheck(0);

	CGridCtrlModified *pGrid = &m_trackStatsGrid;
	CRect rect;
	pGrid->GetClientRect(&rect);
	pGrid->SetRowCount(nRows);
	pGrid->SetColumnCount(nCols);
	pGrid->isColReadOnly.SetSize(nCols);
	for (k = 0; k < nCols; k++) pGrid->isColReadOnly[k] = TRUE;
	pGrid->SetFixedRowCount(1);
	pGrid->SetFixedColumnCount(0);
	pGrid->SetColumnResize(FALSE);
	pGrid->nAutoCols = 1;
	pGrid->autoCol.SetSize(pGrid->nAutoCols);
	pGrid->autoCol[0] = 0;
	int rowHeight = rect.bottom / nRows;
	int nAddHeight = rect.bottom - nRows * rowHeight;
	char tmp[100];
	for (j = 0; j < nRows; j++) {
		if (j < nAddHeight) pGrid->SetRowHeight(j, rowHeight + 1);
		else pGrid->SetRowHeight(j, rowHeight);
	    for (k = 0; k < nCols; k++) {
			pGrid->SetItemState(j, k, pGrid->GetItemState(j, k) | GVIS_READONLY);
			if (!j || k) pGrid->SetItemFormat(j, k, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
		}
	}
	pGrid->SetItemText(0, 0, "Statistic");
	pGrid->SetItemText(0, 1, "Minimum");
	pGrid->SetItemText(0, 2, "Maximum");
	pGrid->SetItemText(0, 3, "Average");
	pGrid->SetItemText(0, 4, "Std. dev.");
	pGrid->SetItemText(1, 0, "History sequence number");
	pGrid->SetItemText(2, 0, "Track sequence number");
	pGrid->SetItemText(3, 0, "Initial number of collisions in a track");
	pGrid->SetItemText(4, 0, "Number of collisions in a track");
	pGrid->SetItemText(9, 0, "Initial relative energy loss (%)");
	pGrid->SetItemText(10, 0, "Relative energy loss (%)");
	for (i = 0; i < nCols; i++) {
		pGrid->SetItemBkColour(3, i, RGB(224, 224, 224));
		pGrid->SetItemBkColour(5, i, RGB(224, 224, 224));
		pGrid->SetItemBkColour(7, i, RGB(224, 224, 224));
		pGrid->SetItemBkColour(9, i, RGB(224, 224, 224));
		pGrid->SetItemBkColour(11, i, RGB(224, 224, 224));
		pGrid->SetItemBkColour(14, i, RGB(224, 224, 224));
	}
	pGrid->SetFont(GetFont());

	CTabFilteringByTransportParameters *tab1 = (CTabFilteringByTransportParameters *)(m_trackFilteringTabCtrl.m_tabPages[0]);
	tab1->parent = parent;
	tab1->parent2 = this;
	tab1->m_hAccel = m_hAccel;
	tab1->pTrackFilter = &trackFilter;
	pTrackFilterGrid = &(tab1->m_trackFilterGrid);

	CTabFilteringByParticleType *tab2 = (CTabFilteringByParticleType *)(m_trackFilteringTabCtrl.m_tabPages[1]);
	tab2->parent = parent;
	tab2->parent2 = this;
	tab2->pTrackFilter = &trackFilter;
	pOriginTypeCheckbox = tab2->m_originType;
	pGenMinEdit = &(tab2->m_genMin);
	pGenMaxEdit = &(tab2->m_genMax);

	m_trackFilteringTabCtrl.InsertItem(0, _T("Filtering by transport parameters"));
	m_trackFilteringTabCtrl.InsertItem(1, _T("Filtering by particle type"));

	int curTab = parent->m_trackFilteringTabCtrl.GetCurSel();
	m_trackFilteringTabCtrl.SetCurSel(curTab);
	m_trackFilteringTabCtrl.m_tabCurrent = curTab;
	m_trackFilteringTabCtrl.Init();

	pGrid = &m_trackSegmentGrid;
	pGrid->GetClientRect(&rect);
	pGrid->SetRowCount(6);
	pGrid->SetColumnCount(3);
	pGrid->isColReadOnly.SetSize(3);
	pGrid->isColReadOnly[0] = TRUE;
	pGrid->isColReadOnly[1] = FALSE;
	pGrid->isColReadOnly[2] = FALSE;
	pGrid->SetFixedRowCount(1);
	pGrid->SetFixedColumnCount(0);
	pGrid->SetColumnResize(TRUE);
	pGrid->nAutoCols = 1;
	pGrid->autoCol.SetSize(pGrid->nAutoCols);
	pGrid->autoCol[0] = 0;
	rowHeight = rect.bottom / 6;
	nAddHeight = rect.bottom - 6 * rowHeight;
	for (j = 0; j < 6; j++) {
		if (j < nAddHeight) pGrid->SetRowHeight(j, rowHeight + 1);
		else pGrid->SetRowHeight(j, rowHeight);
		pGrid->SetItemState(j, 0, pGrid->GetItemState(j, 0) | GVIS_READONLY);
	    for (k = 0; k < 3; k++) if (!j || k) pGrid->SetItemFormat(j, k, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX);
	}
	pGrid->SetItemText(0, 0, "Bounded parameter");
	pGrid->SetItemText(0, 1, "Minimum");
	pGrid->SetItemText(0, 2, "Maximum");
	pGrid->SetItemText(1, 0, "Collision number");
	pGrid->SetItemText(4, 0, "Relative energy loss (%)");
	sprintf(tmp, "%d", trackSegmentSelection.iVert[0] + 1); pGrid->SetItemText(1, 1, tmp);
	if (trackSegmentSelection.iVert[1] >= 0) {
		sprintf(tmp, "%d", trackSegmentSelection.iVert[1] + 1); pGrid->SetItemText(1, 2, tmp);
	}
	sprintf(tmp, "%g", trackSegmentSelection.E[0]); pGrid->SetItemText(2, 1, tmp);
	if (trackSegmentSelection.E[1] >= 0.) {
		sprintf(tmp, "%g", trackSegmentSelection.E[1]); pGrid->SetItemText(2, 2, tmp);
	}
	pGrid->SetFont(GetFont());
	pGrid->ResetWidth();

	if (trackFilteringBeforeSegmentSelection) m_trackFilteringBeforeSegmentSelection.SetCheck(1);
	else m_segmentSelectionBeforeTrackFiltering.SetCheck(1);

	parent->pTrackColorsDlg->pTrackFilter = &trackFilter;
	parent->pTrackColorsDlg->pOriginTypeCheckbox = pOriginTypeCheckbox;

	OnRefresh();

	dialogInitialized = true;

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTrackStatsDlg::OnRefresh()
{
	pTrackFilterGrid->ResetWidth();
	m_trackSegmentGrid.ResetWidth();
	if (!parent->useCustomTrackFN) parent->LoadTrackFile(this, &trackStats, NULL, NULL, &trackFilter, &trackSegmentSelection, trackFilteringBeforeSegmentSelection);
	else {
		if (strspn(parent->trackFN, " \t") == strlen(parent->trackFN)) {
			char buffer[1000];
			parent->GetFileName(&(parent->m_trackFileName), "Select the track data file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
			if (buffer[0] != '\0') {
				if (strcmp(buffer, parent->trackFN)) parent->changed = true;
				strcpy(parent->trackFN, buffer);
			}
		}
		if (strspn(parent->trackFN, " \t") != strlen(parent->trackFN)) parent->LoadTrackFile(this, &trackStats, NULL, NULL, &trackFilter, &trackSegmentSelection, trackFilteringBeforeSegmentSelection);
		else memset(&trackStats, 0, sizeof(TRACKSTATS));
	}
	UpdateTrackStats();
}

void CTrackStatsDlg::OnDefaultFiltering()
{
	if (CompareTrackFilters(&trackFilter, &(parent->defaultTrackFilter)) || CompareTrackSegmentData(&trackSegmentSelection, &(parent->defaultTrackSegmentSelection))) {
		Clear_nTracks();
	}
	trackFilter = parent->defaultTrackFilter;
	trackSegmentSelection = parent->defaultTrackSegmentSelection;
	OnTrackFilteringBeforeSegmentSelection();
	parent->UpdateTrackFilter(pTrackFilterGrid, pOriginTypeCheckbox, pGenMinEdit, pGenMaxEdit, &trackFilter);
	UpdateTrackSegmentGrid();
}

void CTrackStatsDlg::OnKeepTrackDataInMemory()
{
	if (m_keepTrackDataInMemoryCheck.GetCheck()) parent->m_keepTrackDataInMemoryCheck.SetCheck(1);
	else parent->m_keepTrackDataInMemoryCheck.SetCheck(0);
	parent->OnKeepTrackDataInMemory();
}

void CTrackStatsDlg::OnTrackFilteringBeforeSegmentSelection()
{
	trackFilteringBeforeSegmentSelection = TRUE;
	m_trackFilteringBeforeSegmentSelection.SetCheck(1);
	m_segmentSelectionBeforeTrackFiltering.SetCheck(0);
	if (CompareTrackSegmentData(&trackSegmentSelection, &(parent->defaultTrackSegmentSelection))) {
		int i;
		for (i = 0; i < 2; i++) {
			if (trackFilter.nVert[i] != parent->defaultTrackFilter.nVert[i]) break;
			if (trackFilter.Estart[i] != parent->defaultTrackFilter.Estart[i]) break;
			if (trackFilter.Efinal[i] != parent->defaultTrackFilter.Efinal[i]) break;
			if (trackFilter.Eloss[i] != parent->defaultTrackFilter.Eloss[i]) break;
			if (trackFilter.relEloss[i] != parent->defaultTrackFilter.relEloss[i]) break;
			if (trackFilter.displ[i] != parent->defaultTrackFilter.displ[i]) break;
			if (trackFilter.peakDispl[i] != parent->defaultTrackFilter.peakDispl[i]) break;
			if (trackFilter.path[i] != parent->defaultTrackFilter.path[i]) break;
		}
		if (i < 2) Clear_nTracks();
	}
}

void CTrackStatsDlg::OnSegmentSelectionBeforeTrackFiltering()
{
	trackFilteringBeforeSegmentSelection = FALSE;
	m_trackFilteringBeforeSegmentSelection.SetCheck(0);
	m_segmentSelectionBeforeTrackFiltering.SetCheck(1);
	if (CompareTrackSegmentData(&trackSegmentSelection, &(parent->defaultTrackSegmentSelection))) {
		int i;
		for (i = 0; i < 2; i++) {
			if (trackFilter.nVert[i] != parent->defaultTrackFilter.nVert[i]) break;
			if (trackFilter.Estart[i] != parent->defaultTrackFilter.Estart[i]) break;
			if (trackFilter.Efinal[i] != parent->defaultTrackFilter.Efinal[i]) break;
			if (trackFilter.Eloss[i] != parent->defaultTrackFilter.Eloss[i]) break;
			if (trackFilter.relEloss[i] != parent->defaultTrackFilter.relEloss[i]) break;
			if (trackFilter.displ[i] != parent->defaultTrackFilter.displ[i]) break;
			if (trackFilter.peakDispl[i] != parent->defaultTrackFilter.peakDispl[i]) break;
			if (trackFilter.path[i] != parent->defaultTrackFilter.path[i]) break;
		}
		if (i < 2) Clear_nTracks();
	}
}

void CTrackStatsDlg::UpdateTrackSegmentGrid()
{
	CGridCtrlModified *pGrid = &m_trackSegmentGrid;
	pGrid->SetItemText(2, 0, (CString)"Particle energy (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(3, 0, (CString)"Energy loss (" + parent->energyUnits[parent->unitEnergy] + ")");
	pGrid->SetItemText(5, 0, (CString)"Path length (" + parent->distanceUnits[parent->unitDistance] + ")");
	char tmp[100];
	sprintf(tmp, "%d", trackSegmentSelection.iVert[0]); pGrid->SetItemText(1, 1, tmp);
	if (trackSegmentSelection.iVert[1] >= 0) {
		sprintf(tmp, "%d", trackSegmentSelection.iVert[1]); pGrid->SetItemText(1, 2, tmp);
	}
	else pGrid->SetItemText(1, 2, "");
	sprintf(tmp, "%g", parent->energyFactor[parent->unitEnergy] * trackSegmentSelection.E[0]); pGrid->SetItemText(2, 1, tmp);
	if (trackSegmentSelection.E[1] >= 0.) {
		sprintf(tmp, "%g", parent->energyFactor[parent->unitEnergy] * trackSegmentSelection.E[1]); pGrid->SetItemText(2, 2, tmp);
	}
	else pGrid->SetItemText(2, 2, "");
	sprintf(tmp, "%g", parent->energyFactor[parent->unitEnergy] * trackSegmentSelection.Eloss[0]); pGrid->SetItemText(3, 1, tmp);
	if (trackSegmentSelection.Eloss[1] >= 0.) {
		sprintf(tmp, "%g", parent->energyFactor[parent->unitEnergy] * trackSegmentSelection.Eloss[1]); pGrid->SetItemText(3, 2, tmp);
	}
	else pGrid->SetItemText(3, 2, "");
	sprintf(tmp, "%g", 100. * trackSegmentSelection.relEloss[0]); pGrid->SetItemText(4, 1, tmp);
	sprintf(tmp, "%g", 100. * trackSegmentSelection.relEloss[1]); pGrid->SetItemText(4, 2, tmp);
	sprintf(tmp, "%g", parent->distanceFactor[parent->unitDistance] * trackSegmentSelection.path[0]); pGrid->SetItemText(5, 1, tmp);
	if (trackSegmentSelection.path[1] >= 0.) {
		sprintf(tmp, "%g", parent->distanceFactor[parent->unitDistance] * trackSegmentSelection.path[1]); pGrid->SetItemText(5, 2, tmp);
	}
	else pGrid->SetItemText(5, 2, "");
	pGrid->ResetWidth();
}

void CTrackStatsDlg::OnTrackSegmentGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	CGridCtrlModified *pGrid = &m_trackSegmentGrid;
	CGridCellBase* pCell = pGrid->GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = pGrid->GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	char outputString[100];
	bool filterChanged = false;
	if (pItem->iRow == 1){
		int num;
		if (pItem->iColumn == 2 && strspn(tmp, " \t") == strlen(tmp)) num = -1;
		else {
			if (pItem->iColumn == 1) num = parent->CheckInteger(tmp, outputString, 0, 2147483647);
			else num = parent->CheckInteger(tmp, outputString, -2147483647, 2147483647);
			if (outputString[0] == '\0') {
				*pResult = -1;
				return;
			}
			if (num < 0) num = -1;
		}
		int prev = trackSegmentSelection.iVert[pItem->iColumn-1];
		if (pItem->iColumn == 2 && num < 0) {
			trackSegmentSelection.iVert[1] = num;
			pGrid->SetItemText(pItem->iRow, pItem->iColumn, "");
		}
		else {
			trackSegmentSelection.iVert[pItem->iColumn-1] = num;
			pGrid->SetItemText(pItem->iRow, pItem->iColumn, outputString);
		}
		if (num != prev) filterChanged = true;
	}
	else {
		double num;
		if (pItem->iColumn == 2 && strspn(tmp, " \t") == strlen(tmp)) {
			if (pItem->iRow != 4) num = -1.;
			else num = 1.;
		}
		else {
			if (pItem->iRow == 4) num = 0.01 * parent->CheckDouble(tmp, outputString, 0., 100.);
			else if (pItem->iColumn == 1) num = parent->CheckDouble(tmp, outputString, 0., 1e300);
			else num = parent->CheckDouble(tmp, outputString, -1., 1e300);
			if (outputString[0] == '\0') {
				*pResult = -1;
				return;
			}
		}
		double prev;
		if (pItem->iRow == 2) prev = trackSegmentSelection.E[pItem->iColumn-1];
		else if (pItem->iRow == 3) prev = trackSegmentSelection.Eloss[pItem->iColumn-1];
		else if (pItem->iRow == 4) prev = trackSegmentSelection.relEloss[pItem->iColumn-1];
		else if (pItem->iRow == 5) prev = trackSegmentSelection.path[pItem->iColumn-1];
		if (pItem->iColumn == 2 && (pItem->iRow != 4 && num < 0. || pItem->iRow == 4 && num == 1.)) {
			if (num < 0.) num = -1.;
			if (pItem->iRow == 2) trackSegmentSelection.E[1] = -1.;
			else if (pItem->iRow == 3) trackSegmentSelection.Eloss[1] = -1.;
			else if (pItem->iRow == 4) trackSegmentSelection.relEloss[1] = 1.;
			else if (pItem->iRow == 5) trackSegmentSelection.path[1] = -1.;
			if (pItem->iRow != 4) pGrid->SetItemText(pItem->iRow, pItem->iColumn, "");
			else pGrid->SetItemText(pItem->iRow, pItem->iColumn, "100");
		}
		else {
			if (pItem->iRow < 4) num /= parent->energyFactor[parent->unitEnergy];
			else if (pItem->iRow > 4) num /= parent->distanceFactor[parent->unitDistance];
			if (pItem->iRow == 2) trackSegmentSelection.E[pItem->iColumn-1] = num;
			else if (pItem->iRow == 3) trackSegmentSelection.Eloss[pItem->iColumn-1] = num;
			else if (pItem->iRow == 4) trackSegmentSelection.relEloss[pItem->iColumn-1] = num;
			else if (pItem->iRow == 5) trackSegmentSelection.path[pItem->iColumn-1] = num;
			pGrid->SetItemText(pItem->iRow, pItem->iColumn, outputString);
		}
		if (num != prev) filterChanged = true;
	}
	pGrid->Invalidate();
	pGrid->UpdateWindow();
	if (filterChanged && trackStats.nTracks) {
		memset(&trackStats, 0, sizeof(TRACKSTATS));
		Clear_nTracks();
	}
}

void CTrackStatsDlg::OnEnergyUnitSelected()
{
	parent->unitEnergy = m_unitEnergy.GetCurSel();
	UpdateUnits();
}

void CTrackStatsDlg::OnDistanceUnitSelected()
{
	parent->unitDistance = m_unitDistance.GetCurSel();
	UpdateUnits();
}

void CTrackStatsDlg::Clear_nTracks()
{
	CString text;
	m_nHistories.GetWindowText(text);
	if (!text.IsEmpty() && text.Find('*') < 0) {
		m_nHistories.SetWindowText(text + "*");
		m_nTracks.GetWindowText(text);
		m_nTracks.SetWindowText(text + "*");
		m_nTracksEscape.GetWindowText(text);
		m_nTracksEscape.SetWindowText(text + "*");
		m_nTracksTruncated.GetWindowText(text);
		m_nTracksTruncated.SetWindowText(text + "*");
	}
}

void CTrackStatsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TRACKSTATS, m_trackStatsGrid);
	DDX_Control(pDX, IDC_TRACKSEGMENT, m_trackSegmentGrid);
	DDX_Control(pDX, IDC_TRACKFILTERING_TABCTRL, m_trackFilteringTabCtrl);
}

void CTrackStatsDlg::OnOK()
{
	parent->trackFilter = trackFilter;
	parent->trackSegmentSelection = trackSegmentSelection;
	parent->trackFilteringBeforeSegmentSelection = trackFilteringBeforeSegmentSelection;
	int curTab = m_trackFilteringTabCtrl.GetCurSel();
	parent->m_trackFilteringTabCtrl.SetCurSel(curTab);
	parent->m_trackFilteringTabCtrl.m_tabCurrent = curTab;
	parent->m_trackFilteringTabCtrl.SetRectangle();
	EndDialog(IDOK);
}

Compiler *comp = NULL;

#include <fcntl.h>
HANDLE stdHandle_out, stdHandle_in;
bool nextLine;
unsigned long lineCount;
char *CMCNScriptDlg::NextToken(char *line, char *strToken, char *strDelimit)
{
	char *ptr = NULL;
	size_t len = 0;
	if (!nextLine) {
		ptr = strtok(strToken, strDelimit);
		if (ptr) {
			char *ptr2 = strchr(ptr, '&');
			if (ptr2) {
				if (((unsigned int)(ptr2 - ptr) < strlen(ptr) - 1 || strtok(NULL, strDelimit)) && f_out) printf("Warning: The line-continuation character '&' in line No. %d is not the last character of the line. The remainder of the line will be ignored.\n", lineCount);
				*ptr2 = '\0';
				nextLine = true;
			}
			len = strlen(ptr);
		}
	}
	if (!len && nextLine) {
		nextLine = false;
		bool breakLoop = false;
		while (!len && (comp->currentProgram >= 0 && !breakLoop || comp->currentProgram < 0 && !feof(f_in))) {
			do {
				line[0] = '\0';
				if (comp->aPtr1->calculationInProgress) { // pass No. 0
					do {
						if (!comp->aPtr1->terminate) {
							char ch = 0;
							bool keyPressed = false;
							if (comp->checkKeyboard) {
//								while(_kbhit()) {
//									keyPressed = true;
//									ch = _getch();
//								}
								INPUT_RECORD inputRecord;
								DWORD nChars;
								do {
									nChars = 0;
									PeekConsoleInput(stdHandle_in, &inputRecord, 1, &nChars);
									if (nChars && ReadConsoleInput(stdHandle_in, &inputRecord, 1, &nChars) &&
										   inputRecord.EventType == KEY_EVENT &&
										   inputRecord.Event.KeyEvent.bKeyDown) {
										keyPressed = true;
										if (inputRecord.Event.KeyEvent.wVirtualKeyCode == VK_ESCAPE) {
											ch = 27;
										}
										else ch = inputRecord.Event.KeyEvent.uChar.AsciiChar;
									}
								} while(nChars);
							}
							if (!comp->escapeKeyPressed) {
								if (ch == 27) {
									if (comp->f_write) fflush(comp->f_write);
									printf("\nThe escape key has been pressed. Press Q to attempt to interrupt execution of the user program, or any other key except 'Esc' to cancel the escape key.\n");
									comp->escapeKeyPressed = true;
									comp->feedback = true;
								}
							}
							else if (ch == 'q' || ch == 'Q') {
								printf("\nThe 'Esc' and 'Q' keys have been pressed. Interrupting execution of the user program.\n");
								comp->aPtr1->terminate = true;
							}
							else if (keyPressed && ch != 27) {
								printf("\nThe escape key has been cancelled. Press 'Esc' again to attempt to interrupt execution of the user program.\n");
								comp->escapeKeyPressed = false;
							}
						}
						if (!comp->feedback) {
							double curTime = (double)clock() / CLOCKS_PER_SEC;
							if (curTime - comp->aPtr1->startTime > comp->subroutine2[comp->currentProgram].waitTime) {
								if (comp->f_write) fflush(comp->f_write);
								printf("\nExecution time of the user program exceeds %g seconds. Press the escape key and then 'Q' to attempt to interrupt execution of the user program.\n", comp->subroutine2[comp->currentProgram].waitTime);
								comp->feedback = true;
							}
						}
						Sleep(0); // needed to avoid deadlock between the main thread and the compiler thread
					} while(!comp->inputLineCreated && comp->aPtr1->calculationInProgress); // Wait until the compiler thread creates the string
					if (!comp->inputLineCreated) goto CALCULATION_DONE;
					strcpy(line, comp->curStringArray->data[comp->curStringArray->nElem-1]);
					comp->inputLineCreated = false;
					comp->inputLineCopied = true; // Notify the compiler thread that the string has been copied
				}
				else if (comp->currentProgram >= 0) {
					if (comp->waiting) { // pass No. 0
CALCULATION_DONE:
						int nStrings = comp->curStringArray->nElem;
						if (!comp->inputLineCreated && !comp->newLine && nStrings) {
							comp->curStringArray->data[nStrings-1] += "\n";
							strcpy(line, comp->curStringArray->data[nStrings-1]);
							comp->newLine = true;
						}
						comp->PostProcessing(f_in);
						comp->waiting = FALSE;
						if (comp->fatalError) return NULL;
						line[0] = '\0';
						fgets(line, 1000, f_in);
					}
					else { // pass No. 1
						if (comp->currentLineNum == comp->curStringArray->nElem - 1) {
							breakLoop = true;
							break;
						}
						comp->currentLineNum++;
						strcpy(line, comp->curStringArray->data[comp->currentLineNum].data);
					}
				}
				else fgets(line, 1000, f_in); // pass No. 0 or No. 1
				if (line[0] == '\0') continue;
				line[1000] = '\0';
				lineCount++;
				char *slashPtr = strchr(line, '/');
				while (slashPtr) {
					slashPtr++;
					if (*slashPtr == '/') break;
					if (*slashPtr == '\0') slashPtr = NULL;
					else slashPtr = strchr(slashPtr + 1, '/');
				}
				if (slashPtr) *(slashPtr - 1) = '\0';
			} while (strspn(line, strDelimit) == strlen(line) && (comp->currentProgram >= 0 || !feof(f_in)));
			ptr = strtok(line, strDelimit);
			if (ptr) {
				char *ptr2 = strchr(ptr, '&');
				if (ptr2) {
					if (((unsigned int)(ptr2 - ptr) < strlen(ptr) - 1 || strtok(NULL, strDelimit)) && f_out) printf("Warning: The line-continuation character '&' in line No. %d is not the last character of the line. The remainder of the line will be ignored.\n", lineCount);
					*ptr2 = '\0';
					if (ptr2 != ptr) nextLine = true;
				}
				len = strlen(ptr);
			}
		}
		if (!len) ptr = NULL;
	}
	if (!ptr) nextLine = true;
	return ptr;
}

void CMCNScriptDlg::LoadInputFile()
{
	if (f_in) {
		fclose(f_in);
		f_in = NULL;
	}
	if (f_out) {
		fclose(f_out);
		f_in = NULL;
	}
	fileLoaded = false;
	if (!checkSyntax_MCNScript) return;
	CString msg;
	f_in = fopen((CString)inputFileFolder + fn, "r+t");
	if (!f_in) {
		MessageBox((LPCTSTR)(msg.Format(_T("File \"%s\" could not be opened for reading."), (CString)inputFileFolder + fn), msg), NULL, MB_OK | MB_ICONERROR); return;
	}
	int i, m;
	if (nSurfaces) memset(surf, 0, nSurfaces * sizeof(SURFACE));
	if (edgeBuffer) {
		delete edgeBuffer;
		edgeBuffer = NULL;
	}
	nSmax_cell = 0;
	nSurfaces = 0;
	if (nMacrobodies) {
		for (i = 0; i < nMacrobodies; i++) {
			free(macrobody[i].s);
			free(macrobody[i].sense);
			if (macrobody[i].mbNum0 < 0) {
				free(macrobody[i].mb);
				free(macrobody[i].i);
			}
			if (arb_entries[i]) delete arb_entries[i];
		}
		memset(macrobody, 0, nMacrobodies * sizeof(MACROBODY));
		memset(arb_entries, 0, nMacrobodies * sizeof(double *));
		nMacrobodies = 0;
	}
	if (nCells) {
		for (i = 0; i < nCells; i++) {
			if (cell[i].cellNum0 >= 0) continue;
			free(cell[i].mb);
			free(cell[i].i);
		}
		memset(cell, 0, nCells * sizeof(CELL));
		nCells = 0;
	}
	if (nUnions) memset(unionData, 0, nUnions * sizeof(UNION));
	nUnions = 0;
	if (nMaterials) {
		memset(materialID, 0, nMaterials * sizeof(int));
		for (i = 0; i < nMaterials; i++) {
			if (materialName[i]) delete materialName[i];
		}
		memset(materialName, 0, nMaterials * sizeof(char *));
		nMaterials = 0;
		usedMaterialNum = NULL;
	}
	if (nTransforms) {
		memset(transform, 0, nTransforms * sizeof(TRANSFORM));
		nTransforms = 0;
	}

	HWND consoleWndHandle = GetConsoleWindow();
	if (consoleWndHandle && IsWindow(consoleWndHandle)) {
		FreeConsole();
		::SendMessage(consoleWndHandle, WM_CLOSE, 0, 0);
	}
	AllocConsole();
	stdHandle_out = GetStdHandle(STD_OUTPUT_HANDLE);
	stdHandle_in = GetStdHandle(STD_INPUT_HANDLE);
	int hConsole = _open_osfhandle((long)stdHandle_out, _O_TEXT);
	if (hConsole != -1) {
		FILE* fp = _fdopen(hConsole, "w+");
		freopen_s( &fp, "CONOUT$", "w+", stdout);
		consoleWndHandle = GetConsoleWindow();
		CWnd *consoleWnd = CWnd::FromHandle(consoleWndHandle);
		consoleWnd->ShowWindow(SW_SHOWNORMAL);
		consoleWnd->SetForegroundWindow();
		CMenu* pSysMenu = consoleWnd->GetSystemMenu(FALSE);
		pSysMenu->DeleteMenu(SC_CLOSE,MF_BYCOMMAND);
	}

	comp = new Compiler;
	memcpy(comp->sysVarValues, systemVars, 10 * sizeof(double));

	char tmp[1001], arrays_errMsg[STRLEN_MAX+1];
	int arrays_errLineNum = -1;
	arrays_errMsg[0] = '\0';

	double worldSize = maximumObjectSizeRatio * viewSize;
	double halfSize = 0.5 * worldSize;
	CELL world;
	world.nS = 7;

	int j, k;
	char line[1001], *ptr, *endPtr;
	line[1000] = '\0';
	bool sourcePositionSpecified = false, sourceDirectionSpecified = false;
	sourceCell = -1;
	complexGeometry = -1;
	double w = 0.;
	bool notImplemented = false;

	for (int iPass = 0; iPass < 2; iPass++) { // pass No. 0 is for running the user programs and for reading the values of distancePrecision, anglePrecision and maxDistance_ratio

	int keywordNum = -1;
	int keywordUsed[3000];
	char keyword[1001];
	memset(keywordUsed, 0, 3000 * sizeof(int));
	bool success = true;
	if (iPass) {
		if (comp->f_write) fflush(comp->f_write);
		rewind(f_in);
	}
	lineCount = 0;
	comp->currentProgram = -1;
	comp->aPtr1->calculationInProgress = FALSE;
	comp->waiting = FALSE;

	fpos_t curPos;

	bool isProgram;

	while (iPass && (comp->currentProgram >= 0 || !success) || comp->waiting || !feof(f_in)) {
		OPDATA *op;
		EQDATA *eq;
		if (iPass) {
			if (comp->currentProgram < 0) {
				if (!success) { delete comp; comp = NULL; return; }
			}
			else {
				bool lastLine = (comp->currentLineNum == comp->outputLines[comp->currentProgram].GetSize() - 1);
				if (!success || lastLine) {
					eq = comp->subroutine + comp->currentProgram;
					if (!isProgram) comp->PostProcessing(f_in, comp->currentProgram); // In the case "isProgram = true", PostProcessing() is called immediately after finishing execution of the program
					if (!success || comp->currentProgram == comp->nSubroutines - 1 && eq->errorType > 1) { delete comp; comp = NULL; return; }
					if (lastLine) {
						comp->currentProgram++; // the sequence number of the next program
						comp->currentProgram = -comp->currentProgram - 1;
					}
				}
			}
		}
		success = false;
		if (keywordNum >= 0) {
			keywordUsed[keywordNum]++;
			if (keywordUsed[keywordNum] > 1) {
				if (!iPass) break;
				printf("Error: Keyword \"%s\" is used more than once in the input file.\n", strupr(keyword)); continue;
			}
			keywordNum = -1;
		}
		nextLine = true;
		fgetpos(f_in, &curPos);
		ptr = NextToken(line, line, " \t\n");
		if (!iPass && comp->fatalError) break;
		if (!ptr) {
			success = true;
			continue;
		}
		if ((!iPass && !comp->waiting || iPass && comp->currentProgram < 0) &&
			(!strcmpi(ptr, "PROGRAM") || !strcmpi(ptr, "PARAMETERS") || !strcmpi(ptr, "SUBROUTINE") || !strcmpi(ptr, "ARRAYS"))) {
			success = true;
			int len;
			char header[11];
			strcpy(header, strupr(ptr));
			if (!strcmpi(ptr, "ARRAYS")) {
				if (!iPass) {
					if (comp->nSubroutines) break;
					if (comp->nArrays0) {
						strcpy(arrays_errMsg, "******** Error: only one \"ARRAYS\" directive is allowed. ********");
						break;
					}
				}
				else if (-comp->currentProgram - 1) {
					printf("******** Error: the \"ARRAYS\" directive must precede the definitions of global parameters, programs and subroutines. ********"); delete comp; comp = NULL; return;
				}
				else if (arrays_errLineNum < 0 && arrays_errMsg[0] != '\0') {
					printf("\n%s\n", arrays_errMsg);
					delete comp; comp = NULL; return;
				}
				len = 6;
				String code;
				line[0] = '\0';
				fsetpos(f_in, &curPos);
				do {
					fgetpos(f_in, &curPos);
					fgets(line, 1000, f_in);
					ptr = line + strspn(line, " \t");
				} while(strnicmp(ptr, header, len));
				fpos_t startPos = curPos;
				ptr += len;
				ptr += strspn(ptr, " \t");
				curPos += (ptr - line);
				fsetpos(f_in, &curPos);
				line[0] = '\0';
				while (!feof(f_in)) {
					fgets(line, 1000, f_in);
					if (line[0] == '\0') break;
					line[1000] = '\0';
					ptr = strtok(line, "\n");
					if (ptr) {
						char *ptr2 = ptr + strspn(ptr, " \t");
						if (!strnicmp(ptr2, "END", 3) && (ptr2[3] == ' ' || ptr2[3] == '\t')) {
							ptr2 += 3;
							ptr2 += strspn(ptr2, " \t");
							if (!strnicmp(ptr2, header, len) && (ptr2[len] == '\0' || ptr2[len] == ' ' || ptr2[len] == '\t')) {
								break;
							}
						}
					}
					else {
						line[0] = '\0';
						ptr = line;
					}
					sprintf(tmp, "%s%c%c", ptr, 0x0d, 0x0a);
					code += tmp;
					line[0] = '\0';
					fgetpos(f_in, &curPos);
				}
				if (!iPass) {
					arrays_errLineNum = comp->Arrays(code, arrays_errMsg);
					if (arrays_errLineNum >= 0) break;
				}
				else if (arrays_errLineNum >= 0) {
					printf("\n%s\n", arrays_errMsg);
					delete comp; comp = NULL; return;
				}
				continue;
			}
			else if (!strcmpi(ptr, "PROGRAM")) {
				isProgram = true;
				len = 7;
			}
			else if (!strcmpi(ptr, "PARAMETERS")) {
				if (!iPass) {
					if (comp->nSubroutines) break;
				}
				else if (-comp->currentProgram - 1) {
					if (comp->subroutine[-comp->currentProgram-2].gParms) printf("Error: only one \"PARAMETERS\" directive is allowed.\n");
					else printf("Error: the \"PARAMETERS\" directive must precede the definitions of all other programs and subroutines.\n");
					delete comp; comp = NULL; return;
				}
				isProgram = true;
				len = 10;
			}
			else {
				isProgram = false;
				len = 10;
			}
			size_t nameLen = 0;
			if (strcmpi(ptr, "PARAMETERS")) {
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					if (!iPass) break;
					printf("Error: User program name has not been specified.\n"); delete comp; comp = NULL; return;
				}
				char *ptr2 = strchr(ptr, '(');
				if (ptr2) *ptr2 = '\0';
				nameLen = strlen(ptr);
				int nConf = 0;
				int *typeConf = (int *)malloc(0);
				int *numConf = (int *)malloc(0);
				char **nameConf = (char **)malloc(0);
				char *nameConfBuffer = (char *)malloc(0);
				char message[STRLEN_MAX+1];
				int num = !iPass ? -1 : -comp->currentProgram - 1;
				BOOL rc = comp->CheckIdentifier(ptr, 3, num, "Program name", nConf, typeConf, numConf, nameConf, nameConfBuffer, comp->S1);
				if (rc && nConf) {
					rc = FALSE;
					sprintf(comp->S1, "Error: the program name \"%s\" coincides with the name of a previously-defined %s.", ptr, comp->types[typeConf[0]]);
				}
				free(typeConf);
				free(numConf);
				free(nameConf);
				free(nameConfBuffer);
				if (!rc) {
					if (!iPass) break;
					sprintf(message, "******** %s ********", comp->S1);
					printf("%s\n", message);
					delete comp; comp = NULL; return;
				}
			}
			comp->currentLineNum = -1;
			if (iPass) {
				comp->currentProgram = -comp->currentProgram - 1;
				op = comp->subroutine2 + comp->currentProgram;
				eq = comp->subroutine + comp->currentProgram;
				comp->curStringArray = comp->outputLines + comp->currentProgram;
				comp->curNumberArray = comp->outputLineNumbers + comp->currentProgram;
				continue;
			}
			comp->nSubroutines++;
			i = comp->nSubroutines - 1;
			comp->subroutine = (EQDATA *)realloc(comp->subroutine, comp->nSubroutines * sizeof(EQDATA));
			comp->subroutine2 = (OPDATA *)realloc(comp->subroutine2, comp->nSubroutines * sizeof(OPDATA));
			memset(comp->subroutine2 + i, 0, sizeof(OPDATA));
			OPDATA opData;
			comp->subroutine2[i] = opData;
			comp->subroutineArgInfo = (FUNCARGINFO *)realloc(comp->subroutineArgInfo, comp->nSubroutines * sizeof(FUNCARGINFO));
			memset(comp->subroutineArgInfo + i, 0, sizeof(FUNCARGINFO));
			comp->subroutineArgInfo[i].list = "";
			comp->subroutineArgInfo[i].list_singleLine = "";
			comp->subroutineArgInfo[i].count = 0;
			comp->outputLines = (Array_String *)realloc(comp->outputLines, comp->nSubroutines * sizeof(Array_String));
			comp->outputLineNumbers = (Array_int *)realloc(comp->outputLineNumbers, comp->nSubroutines * sizeof(Array_int));
			comp->fpos = (fpos_t **)realloc(comp->fpos, comp->nSubroutines * sizeof(fpos_t *));
			comp->fpos[i] = (fpos_t *)malloc(0);
			comp->startPos = (fpos_t *)realloc(comp->startPos, comp->nSubroutines * sizeof(fpos_t));
			comp->endPos = (fpos_t *)realloc(comp->endPos, comp->nSubroutines * sizeof(fpos_t));
			Array_String stringArray;
			comp->outputLines[i] = stringArray;
			Array_int intArray;
			comp->outputLineNumbers[i] = intArray;
			comp->curStringArray = comp->outputLines + i;
			comp->curNumberArray = comp->outputLineNumbers + i;
			op = comp->subroutine2 + i;
			eq = comp->subroutine + i;
			if (strcmpi(header, "PARAMETERS")) {
				eq->gParms = false;
				strcpy(eq->name, ptr);
			}
			else {
				eq->gParms = true;
				eq->name[0] = '\0';
			}
			String userCode;
			int nCodeLines = 0;
			fpos_t programPos = curPos;
			line[0] = '\0';
			fsetpos(f_in, &programPos);
			do {
				fgetpos(f_in, &programPos);
				fgets(line, 1000, f_in);
				ptr = line + strspn(line, " \t");
			} while(strnicmp(ptr, header, len));
			ptr += len;
			ptr += strspn(ptr, " \t");
			if (nameLen) {
				while(strncmp(ptr, eq->name, nameLen)) {
					fgetpos(f_in, &programPos);
					fgets(line, 1000, f_in);
					ptr = line + strspn(line, " \t");
				}
				ptr += nameLen;
				ptr += strspn(ptr, " \t");
			}
			programPos += (ptr - line);
			bool formalParmsSpecified;
			if (!isProgram && *ptr == '(') {
				formalParmsSpecified = true;
				ptr++;
				programPos++;
			}
			else formalParmsSpecified = false;
			comp->startPos[i] = programPos;
			fsetpos(f_in, &programPos);
			line[0] = '\0';
			while (!feof(f_in)) {
				fgets(line, 1000, f_in);
				if (line[0] == '\0') break;
				line[1000] = '\0';
				ptr = strtok(line, "\n");
				if (ptr) {
					char *ptr2 = ptr + strspn(ptr, " \t");
					if (!strnicmp(ptr2, "END", 3) && (ptr2[3] == ' ' || ptr2[3] == '\t')) {
						ptr2 += 3;
						ptr2 += strspn(ptr2, " \t");
						if (!strnicmp(ptr2, header, len) && (ptr2[len] == '\0' || ptr2[len] == ' ' || ptr2[len] == '\t')) {
							break;
						}
					}
				}
				else {
					line[0] = '\0';
					ptr = line;
				}
				comp->fpos[i] = (fpos_t *)realloc(comp->fpos[i], (nCodeLines + 1) * sizeof(fpos_t));
				comp->fpos[i][nCodeLines] = programPos;
				sprintf(tmp, "%s%c%c", ptr, 0x0d, 0x0a);
				userCode += tmp;
				nCodeLines++;
				line[0] = '\0';
				fgetpos(f_in, &programPos);
			}
			comp->endPos[i] = programPos;
			comp->subroutineCode.Add(userCode);
			op->operands = NULL;
			op->nInterm = 0;
			op->nXbreaks = 0;
			op->parmValues = NULL;
			op->valuesUpdated = 0;
			eq->message[0] = '\0';
			eq->errorType = 0;
			eq->nParm = 0;
			comp->nNewGParm = 0;
			if (eq->gParms) comp->newGlobalParms = (PARMDATA *)malloc(0);
			int iLine = 0;
			if (formalParmsSpecified) {
				iLine = comp->FormalParameters(i);
				if (eq->errorType < 2) userCode = (char *)(comp->subroutineCode[i]);
			}
			if (eq->errorType < 2) {
				comp->Compile(userCode, iLine, eq, op, i);
				if (eq->gParms) {
					if (comp->nNewGParm) {
						memcpy(comp->globalParms + comp->nGParm, comp->newGlobalParms, comp->nNewGParm * sizeof(PARMDATA));
						memset(comp->gParmValues + comp->nGParm, 0, comp->nNewGParm * sizeof(double));
						comp->nGParm += comp->nNewGParm;
						comp->Compile(userCode, 0, eq, op, i);
					}
					else if (!eq->errorType) {
						sprintf(eq->message, "Warning: the \"PARAMETERS\" code does not define any global parameters.", eq->name);
						op->warnLineNum = nCodeLines - 1;
					}
				}
			}
			if (eq->gParms) free(comp->newGlobalParms);
			comp->aPtr1->iEq = -1;
			comp->aPtr1->eq = eq;
			comp->aPtr1->op = op;
			comp->currentProgram = i;
			if (eq->errorType) {
				comp->PostProcessing(f_in);
				if (eq->errorType > 1) break;
			}
			if (!isProgram) {
				if (!iPass) comp->currentProgram = -1;
				continue;
			}
			if (op->operands) {
				op->operands[0] = comp->indeterminateValue;
				op->operands[1] = comp->indeterminateValue;
			}
			comp->aPtr1->arg = 0.;
			comp->aPtr1->it = -1;
			comp->aPtr1->t = 0.;
			if (iPass && op->warnLineNum >= 0 && eq->message[0] != '\0') printf("%s\n", eq->message);
			comp->aPtr1->iFunc = -1;
			comp->aPtr1->calculationInProgress = TRUE;
			comp->waiting = TRUE;
			comp->aPtr1->terminate = FALSE;
			comp->aPtr1->startTime = (double)clock() / CLOCKS_PER_SEC;
			comp->callDepth = 0;
			comp->feedback = false;
			comp->checkKeyboard = true;
			comp->escapeKeyPressed = false;
			comp->inputLineCopied = false;
			_clearfp();
			_beginthread(RunProgram0, 0, NULL);
			continue;
		}
		strcpy(keyword, ptr);
		if (!strcmpi(ptr, "COMPLEX_GEOMETRY")) {
			keywordNum = 3;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			ptr = NextToken(line, NULL, " \t\n");
			if (ptr) {
				int test = atoi(ptr);
				if (test < 0) test = -test;
				if (test && !complexGeometry) {
					printf("Error: Setting the complex geometry option is not allowed here, because the previous directives imply simple geometry.\n"); continue;
				}
				if (!test && complexGeometry > 0) {
					printf("Error: Setting the simple geometry option is not allowed here, because the previous directives imply complex geometry.\n"); continue;
				}
				complexGeometry = test;
			}
			else {
				printf("Error: Value of the \"complex geometry\" switch is missing.\n"); continue;
			}
		}
		else if (!strcmpi(ptr, "DISTANCE_PRECISION")) {
			keywordNum = 5;
			ptr = NextToken(line, NULL, " \t\n");
			if (ptr) {
				double test = atof(ptr);
				if (test > 0.) {
					if (!iPass) {
						distancePrecision = test;
						continue;
					}
				}
				else if (iPass) {
					printf("Error: Illegal value \"%s\" of the distance precision (it must be positive).\n", ptr); continue;
				}
				else break;
			}
			else if (iPass) {
				printf("Error: Distance precision value is missing.\n"); continue;
			}
			else break;
			if (!complexGeometry) {
				printf("Error: Keyword \"DISTANCE_PRECISION\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
		}
		else if (!strcmpi(ptr, "ANGLE_PRECISION")) {
			keywordNum = 6;
			ptr = NextToken(line, NULL, " \t\n");
			if (ptr) {
				double test = atof(ptr);
				if (test > 0. && test <= 1e-5) {
					if (!iPass) {
						anglePrecision = test;
						continue;
					}
				}
				else if (iPass) {
					printf("Error: Illegal value \"%s\" of the angle precision (it must be positive and less than 1e-5).\n", ptr); continue;
				}
				else break;
			}
			else if (iPass) {
				printf("Error: Angle precision value is missing.\n"); continue;
			}
			else break;
			if (!complexGeometry) {
				printf("Error: Keyword \"ANGLE_PRECISION\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
		}
		else if (!strcmpi(ptr, "MAXDISTANCE_RATIO")) {
			keywordNum = 7;
			ptr = NextToken(line, NULL, " \t\n");
			if (ptr) {
				double test = atof(ptr);
				if (test >= 1e6 && test <= 1e24) {
					if (!iPass) {
						maxDistance_ratio = test;
						continue;
					}
				}
				else if (iPass) {
					printf("Error: Illegal value \"%s\" of the ratio of the maximum distance and the distance precision (it must be between 1e6 and 1e24).\n", ptr); continue;
				}
				else break;
			}
			else if (iPass) {
				printf("Error: The value of the ratio of the maximum distance and the distance precision is missing.\n"); continue;
			}
			else break;
			if (!complexGeometry) {
				printf("Error: Keyword \"MAXDISTANCE_RATIO\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
		}
		else if (!strnicmp(ptr, "M", 1) && strlen(ptr) > 1 && atoi(ptr + 1) > 0) {
			keywordNum = -1;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			double value_test = strtod(ptr + 1, &endPtr);
			if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
				printf("Error in specification of the material \"%s\" (the first keyword format must be \"M<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", ptr); continue;
			}
			int matID = (int)(value_test + 0.5);
			if (!complexGeometry) {
				printf("Error: Keyword \"M<n>\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			if (nMaterials == MAXMATERIALS) {
				printf("Error: The number of materials is greater than %d.\n", MAXMATERIALS); continue;
			}
			for (i = 0; i < nMaterials && materialID[i] != matID; i++);
			if (i < nMaterials) {
				printf("Error: Duplicate material identifier \"M%d\".\n", materialID); continue;
			}
			char *ptr1 = ptr + strlen(ptr) + 1;
			ptr1 += strspn(ptr1, " \t");
			if (*ptr1 == '"') {
				ptr1++;
				char *ptr2 = strchr(ptr1, '"');
				if (!ptr2) {
					printf("Error: Missing closing quote in the name of a material.\n"); continue;
				}
				*ptr2 = '\0';
				CString matName = ptr1;
				matName.TrimLeft();
				matName.TrimRight();
				size_t len = strlen(matName);
				if (len) {
					if (len <= REGIONNAMELEN) {
						materialName[nMaterials] = new char[len + 1];
						strcpy(materialName[nMaterials], matName);
					}
					else {
						materialName[nMaterials] = new char[REGIONNAMELEN + 1];
						strncpy(materialName[nMaterials], matName, REGIONNAMELEN);
						materialName[nMaterials][REGIONNAMELEN] = '\0';
					}
				}
			}
			materialID[nMaterials] = matID;
			nMaterials++;
		}
		else if (!strcmpi(ptr, "MAT")) {
			keywordNum = 4;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (complexGeometry > 0) {
				printf("Error: Keyword \"MAT\" can be used only in simple geometry mode, but the previous directives imply complex geometry.\n"); continue;
			}
			complexGeometry = 0;
			char *ptr1 = ptr + strlen(ptr) + 1;
			ptr1 += strspn(ptr1, " \t");
			if (*ptr1 == '"') {
				ptr1++;
				char *ptr2 = strchr(ptr1, '"');
				if (!ptr2) {
					printf("Error: Missing closing quote in the name of the material.\n"); continue;
				}
				*ptr2 = '\0';
				CString matName = ptr1;
				matName.TrimLeft();
				matName.TrimRight();
				size_t len = strlen(matName);
				if (len) {
					if (len <= REGIONNAMELEN) {
						materialName[0] = new char[len + 1];
						strcpy(materialName[0], matName);
					}
					else {
						materialName[0] = new char[REGIONNAMELEN + 1];
						strncpy(materialName[0], matName, REGIONNAMELEN);
						materialName[0][REGIONNAMELEN] = '\0';
					}
				}
			}
			materialID[0] = 1;
			nMaterials = 1;
		}
		else if (!strnicmp(ptr, "C", 1) && strlen(ptr) > 1 && atoi(ptr + 1) > 0) {
			keywordNum = -1;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			double value_test = strtod(ptr + 1, &endPtr);
			if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
				printf("Error in specification of the cell \"%s\" (the first keyword format must be \"C<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", ptr); continue;
			}
			int cellID = (int)(value_test + 0.5);
			if (!complexGeometry) {
				printf("Error: Keyword \"C<n>\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			if (nCells == MAXCELLS) {
				printf("Error: The number of cells is greater than %d.\n", MAXCELLS); continue;
			}
			for (i = 0; i < nCells && cell[i].ID != cellID; i++);
			if (i < nCells) {
				printf("Error: Duplicate cell identifier \"C%d\".\n", cellID); continue;
			}
			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of a cell.\n"); continue;
			}
			if (nCells / CHUNKSIZE_CELLS * CHUNKSIZE_CELLS == nCells) {
				cell = (CELL *)realloc(cell, (nCells + CHUNKSIZE_CELLS) * sizeof(CELL));
				memset(cell + nCells, 0, CHUNKSIZE_CELLS * sizeof(CELL));
				cellTransform = (TRANSFORM *)realloc(cellTransform, (nCells + CHUNKSIZE_CELLS) * sizeof(TRANSFORM));
			}
			CELL *c = cell + nCells;
			TRANSFORM *cTr = cellTransform + nCells;
			c->ID = cellID;
			int matID = strtol(ptr, &endPtr, 10);
			if (*endPtr != '\0') {
				printf("Error: Invalid material identifier \"%s\" in the definition of a cell.\n", ptr); continue;
			}
			else if (matID > 0) {
				for (i = 0; i < nMaterials && materialID[i] != matID; i++);
				if (i == nMaterials) {
					printf("Error: Undefined material identifier \"%d\".\n", matID); continue;
				}
				c->mat = i;
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					printf("Error: Missing concentration value in the definition of a cell.\n", ptr); continue;
				}
				c->concentration = strtod(ptr, &endPtr);
				if (*endPtr != '\0') {
					printf("Error: Invalid concentration value \"%s\" in the definition of a cell.\n", ptr); continue;
				}
				else if (!c->concentration) {
					printf("Error: Zero concentration value in the definition of a cell.\n"); continue;
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			else {
				for (i = 0; i < nMaterials && materialID[i] != matID; i++);
				if (i == nMaterials) {
					materialID[nMaterials] = matID;
					nMaterials++;
				}
				c->mat = i;
				c->concentration = 0.;
				char *ptr1 = ptr + strlen(ptr) + 1;
				ptr1 += strspn(ptr1, " \t");
				if (*ptr1 == '"') {
					ptr1++;
					char *ptr2 = strchr(ptr1, '"');
					if (!ptr2) {
						printf("Error: Missing closing quote in the name of a material.\n"); continue;
					}
					*ptr2 = '\0';
					if (materialName[c->mat]) {
						delete materialName[c->mat];
						materialName[c->mat] = NULL;
					}
					CString matName = ptr1;
					matName.TrimLeft();
					matName.TrimRight();
					size_t len = strlen(matName);
					if (len) {
						if (len <= REGIONNAMELEN) {
							materialName[c->mat] = new char[len + 1];
							strcpy(materialName[c->mat], matName);
						}
						else {
							materialName[c->mat] = new char[REGIONNAMELEN + 1];
							strncpy(materialName[c->mat], matName, REGIONNAMELEN);
							materialName[c->mat][REGIONNAMELEN] = '\0';
						}
					}
					ptr = NextToken(line, ptr2 + 1, " \t\n");
				}
				else ptr = NextToken(line, NULL, " \t\n");
			}
			if (!ptr) {
				printf("Error: There are no bounding surfaces listed in the definition of cell %d.\n", cellID); continue;
			}
			c->nS = 0;
			c->nContainedCells = 0;
			c->cutoffEnergy = -1.;
			c->cutoffEnergy_photon = -1.;
			double cellVolume = -1.;
			c->mb = (short int *)malloc(0);
			c->i = (short int *)malloc(0);
			c->cellNum0 = -1;
			c->force = -1;
			int chunkSize = 10;
			while (ptr) {
				if (!strnicmp(ptr, "TR", 2) && strlen(ptr) > 2 && atoi(ptr + 2) > 0) {
					double value_test = strtod(ptr + 2, &endPtr);
					if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
						printf("Error: Invalid identifier of coordinate transformation \"%s\" in the definition of a cell.\n", ptr); goto CONTINUE;
					}
					int transformID = (int)(value_test + 0.5);
					for (i = 0; i < nTransforms && transform[i].ID != transformID; i++);
					if (i == nTransforms) {
						printf("Error: Unknown coordinate transformation \"%s\" in the definition of a cell.\n", ptr); goto CONTINUE;
					}
					if (c->cellNum0 < 0 && c->nS) {
						printf("Error: A coordinate transformation of the original cell cannot be specified in the definition of a cell after specifying the bounding surfaces.\n"); goto CONTINUE;
					}
					if (c->cellNum0 >= 0) {
						printf("Error: Multiple coordinate transformations of the original cell specified in the definition of a cell.\n"); goto CONTINUE;
					}
					TRANSFORM *tr = transform + i;
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing identifier of the original cell after \"TR%d\" in the definition of a cell.\n", transformID); goto CONTINUE;
					}
					value_test = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
						printf("Error: Invalid identifier of the original cell \"%s\" after \"TR%d\" in the definition of a cell.\n", ptr, transformID); goto CONTINUE;
					}
					int cellID0 = (int)(value_test + 0.5);
					for (i = 0; i < nCells && cell[i].ID != cellID0; i++);
					if (i == nCells) {
						printf("Error: Unknown identifier of the original cell \"%s\" after \"TR%d\" in the definition of a cell.\n", ptr, transformID); goto CONTINUE;
					}
					CELL *c0 = cell + i;
					c->cellNum0 = c0->cellNum0 < 0 ? i : c0->cellNum0;
					*cTr = *tr;
					if (c0->cellNum0 >= 0) {
						cTr->origin = LocalToGlobal(cTr->origin, cTr->Q[0], cellTransform[i].origin);
						MatrixProduct(cTr->Q, cellTransform[i].Q);
					}
					c->nS = c0->nS;
					free(c->mb);
					c->mb = c0->mb;
					free(c->i);
					c->i = c0->i;
					memcpy(c->side, c0->side, c0->nS * sizeof(char));
					for (j = 0; j < c0->nS; j++) {
						SURFACE s = surf[c0->s[j]];
						s.init = LocalToGlobal(tr->origin, tr->Q[0], s.init);
						if (s.type != 2) {
							s.dir = LocalToGlobal(zero, tr->Q[0], s.dir);
							s.dir /= Norm(s.dir);
						}
						if (s.type == 1) s.D = -s.dir * s.init;
						for (i = 0; i < nSurfaces && !SurfacesCoincide(&s, surf + i, distancePrecision); i++);
						c->s[j] = i;
						if (i == nSurfaces) {
							if (nSurfaces == MAXSURFACES) {
								printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); goto CONTINUE;
							}
							s.ID = 0;
							surf[nSurfaces] = s;
							nSurfaces++;
						}
						else if (s.type == 1 && s.dir * surf[i].dir < 0.) c->side[j] = -c->side[j];
					}
				}
				else if (*ptr == '#') {
					int cellID2 = strtol(ptr + 1, &endPtr, 10);
					if (*endPtr != '\0' || cellID2 < 1) {
						printf("Error in specification of a cell \"%s\" after the complement operator \"#\".\n", ptr); goto CONTINUE;
					}
					for (i = 0; i < nCells && cell[i].ID != cellID2; i++);
					if (i == nCells) {
						printf("Error: The cell identifier \"%d\" after the complement operator \"#\" is undefined.\n", cellID2); goto CONTINUE;
					}
					if (cell[i].hostCell >= 0) {
						printf("Error: Cell %d cannot be inside cell %d, because it is inside cell %d.\n", cellID2, cellID, cell[cell[i].hostCell].ID); goto CONTINUE;
					}
					if (cell[i].unionNum >= 0) {
						printf("Error: Cell %d cannot be specified after the complement operator, because it is a part of a union of cells.\n", cellID2); goto CONTINUE;
					}
					c->containedCells[c->nContainedCells] = i;
					c->nContainedCells++;
					cell[i].hostCell = nCells;
				}
				else if (c->cutoffEnergy < 0. && !strcmpi(ptr, "CUT_E") || c->cutoffEnergy_photon < 0. && !strcmpi(ptr, "CUT_P")) {
					if (!strcmpi(ptr, "CUT_E")) c->cutoffEnergy = 1.;
					else c->cutoffEnergy_photon = 1.;
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the cutoff energy in the definition of a cell.\n", ptr); goto CONTINUE;
					}
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || value <= 0.) {
						printf("Error: Illegal value \"%s\" of the cutoff energy in the definition of a cell (it must be a positive number).\n", ptr); goto CONTINUE;
					}
					if (c->cutoffEnergy > 0.) c->cutoffEnergy = value;
					else c->cutoffEnergy_photon = value;
				}
				else if (cellVolume < 0. && !strcmpi(ptr, "VOL")) {
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the cell volume in the definition of a cell.\n", ptr); goto CONTINUE;
					}
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || value <= 0.) {
						printf("Error: Illegal value \"%s\" of the cell volume in the definition of a cell (it must be a positive number).\n", ptr); goto CONTINUE;
					}
					cellVolume = value;
				}
				else if (c->force < 0 && !strcmpi(ptr, "FORCE")) {
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the option to apply interaction forcing in a cell.\n", ptr); goto CONTINUE;
					}
					long value = strtol(ptr, &endPtr, 10);
					if (*endPtr != '\0' || value < 0 || value > 1) {
						printf("Error: Illegal value \"%s\" of the option to apply interaction forcing in a cell (it must be 0 or 1).\n", ptr); goto CONTINUE;
					}
					c->force = (char)value;
				}
				else {
					unsigned short *ptr_s = c->s;
					char *ptr_side = c->side;
					int rc = CheckSurfaceID(line, ptr, true, c->ID, c->nS, ptr_s, ptr_side, c->mb, c->i, chunkSize);
					if (!rc) goto CONTINUE;
					if (rc < 0) continue;
					if (c->cellNum0 >= 0) {
						printf("Error: Bounding surfaces cannot be specified in the definition of a cell after specifying the coordinate transformation of the original cell.\n"); goto CONTINUE;
					}
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			if (!c->nS) {
				printf("Error: There are no bounding surfaces listed in the definition of cell %d.\n", cellID); continue;
			}
			c->pointFound = 0;
			c->hostCell = -1;
			c->unionNum = -1;
			notImplemented = false;
			bool removeSurface[MAXSURFACES_CELL], surfacesRemoved = false;
			memset(removeSurface, 0, c->nS * sizeof(bool));
			for (j = 0; j < c->nS; j++) {
				if (removeSurface[j]) continue;
				for (k = j + 1; k < c->nS; k++) {
					if (removeSurface[k]) continue;
					int rc = CheckCellGeometry0(c->s, j, k, c->side, distancePrecision, notImplemented);
					if (rc == -1) {
						removeSurface[j] = true;
						surfacesRemoved = true;
						break;
					}
					else if (rc == -2) {
						removeSurface[k] = true;
						surfacesRemoved = true;
					}
				}
			}
			if (surfacesRemoved) {
				for (k = 0, j = 0; j < c->nS; j++) {
					if (removeSurface[j]) continue;
					c->s[k] = c->s[j];
					c->side[k] = c->side[j];
					c->mb[k] = c->mb[j];
					c->i[k] = c->i[j];
					k++;
				}
				c->nS = k;
			}
			if (c->nS > nSmax_cell) nSmax_cell = c->nS;
			nCells++;
		}
		else if (!strnicmp(ptr, "UNION", 5) && strlen(ptr) > 5 && atoi(ptr + 5) > 0) {
			keywordNum = -1;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			double value_test = strtod(ptr + 5, &endPtr);
			if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
				printf("Error in specification of the union \"%s\" (the first keyword format must be \"UNION<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", ptr); continue;
			}
			int unionID = (int)(value_test + 0.5);
			if (!complexGeometry) {
				printf("Error: Keyword \"UNION\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of a union of cells.\n"); continue;
			}
			int matID = strtol(ptr, &endPtr, 10);
			int mat;
			double concentration;
			if (*endPtr != '\0') {
				printf("Error: Invalid material identifier \"%s\" in the definition of a union of cells.\n", ptr); continue;
			}
			else if (matID > 0) {
				for (i = 0; i < nMaterials && materialID[i] != matID; i++);
				if (i == nMaterials) {
					printf("Error: Undefined material identifier \"%d\".\n", matID); continue;
				}
				mat = i;
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					printf("Error: Missing concentration value in the definition of a union of cells.\n", ptr); continue;
				}
				concentration = strtod(ptr, &endPtr);
				if (*endPtr != '\0') {
					printf("Error: Invalid concentration value \"%s\" in the definition of a union of cells.\n", ptr); continue;
				}
				else if (!concentration) {
					printf("Error: Zero concentration value in the definition of a union of cells.\n"); continue;
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			else {
				for (i = 0; i < nMaterials && materialID[i] != matID; i++);
				if (i == nMaterials) {
					materialID[nMaterials] = matID;
					nMaterials++;
				}
				mat = i;
				concentration = 0.;
				char *ptr1 = ptr + strlen(ptr) + 1;
				ptr1 += strspn(ptr1, " \t");
				if (*ptr1 == '"') {
					ptr1++;
					char *ptr2 = strchr(ptr1, '"');
					if (!ptr2) {
						printf("Error: Missing closing quote in the name of a material.\n"); continue;
					}
					*ptr2 = '\0';
					if (materialName[mat]) {
						delete materialName[mat];
						materialName[mat] = NULL;
					}
					CString matName = ptr1;
					matName.TrimLeft();
					matName.TrimRight();
					size_t len = strlen(matName);
					if (len) {
						if (len <= REGIONNAMELEN) {
							materialName[mat] = new char[len + 1];
							strcpy(materialName[mat], matName);
						}
						else {
							materialName[mat] = new char[REGIONNAMELEN + 1];
							strncpy(materialName[mat], matName, REGIONNAMELEN);
							materialName[mat][REGIONNAMELEN] = '\0';
						}
					}
					ptr = NextToken(line, ptr2 + 1, " \t\n");
				}
				else ptr = NextToken(line, NULL, " \t\n");
			}
			if (!ptr) {
				printf("Error: There are no cells listed in the definition of a union of cells.\n"); continue;
			}
			unionData[nUnions].ID = unionID;
			unionData[nUnions].parts = (short int *)malloc(0);
			unionData[nUnions].nParts = 0;
			double electronCutoffEnergy = -1., photonCutoffEnergy = -1., volume = -1.;
			char forceOpt = -1;
			while (ptr) {
				if (electronCutoffEnergy < 0. && !strcmpi(ptr, "CUT_E") || photonCutoffEnergy < 0. && !strcmpi(ptr, "CUT_P")) {
					if (!strcmpi(ptr, "CUT_E")) electronCutoffEnergy = 1.;
					else photonCutoffEnergy = 1.;
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the cutoff energy in the definition of a union of cells.\n", ptr); goto CONTINUE;
					}
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || value <= 0.) {
						printf("Error: Illegal value \"%s\" of the cutoff energy in the definition of a union of cells (it must be a positive number).\n", ptr); goto CONTINUE;
					}
					if (electronCutoffEnergy > 0.) electronCutoffEnergy = value;
					else photonCutoffEnergy = value;
				}
				else if (volume < 0. && !strcmpi(ptr, "VOL")) {
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the cell volume in the definition of a union of cells.\n", ptr); goto CONTINUE;
					}
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || value <= 0.) {
						printf("Error: Illegal value \"%s\" of the cell volume in the definition of a union of cells (it must be a positive number).\n", ptr); goto CONTINUE;
					}
					volume = value;
				}
				else if (forceOpt < 0 && !strcmpi(ptr, "FORCE")) {
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the option to apply interaction forcing in a union of cells.\n", ptr); goto CONTINUE;
					}
					long value = strtol(ptr, &endPtr, 10);
					if (*endPtr != '\0' || value < 0 || value > 1) {
						printf("Error: Illegal value \"%s\" of the option to apply interaction forcing in a union of cells (it must be 0 or 1).\n", ptr); goto CONTINUE;
					}
					forceOpt = (char)value;
				}
				else {
					int cellID = strtol(ptr, &endPtr, 10);
					if (*endPtr != '\0' || cellID < 1) {
						printf("Error: Invalid cell identifier \"%s\".\n", ptr); goto CONTINUE;
					}
					for (i = 0; i < nCells && cell[i].ID != cellID; i++);
					if (i == nCells) {
						printf("Error: Undefined cell identifier \"%d\".\n", cellID); goto CONTINUE;
					}
					if (i == sourceCell) {
						printf("Error: Cell %d, which is used in the definition of a union, was previously defined as a source cell (the source cell that is a union of cells must be defined after the union, and its definition must specify only the first cell of the union).\n", cellID); goto CONTINUE;
					}
					CELL *c2 = cell + i;
					if (cell[i].unionNum >= 0) {
						printf("Error: Cell %d is already included in another union of cells.\n", cellID); goto CONTINUE;
					}
					for (j = 0; j < unionData[nUnions].nParts; j++) {
						CELL *c1 = cell + unionData[nUnions].parts[j];
						if (c2->hostCell != c1->hostCell) {
							if (c2->hostCell < 0 || c1->hostCell < 0 || cell[c2->hostCell].hostCell != cell[c1->hostCell].hostCell && (cell[c2->hostCell].unionNum < 0 || cell[c2->hostCell].unionNum != cell[c1->hostCell].unionNum)) {
								printf("Geometry error: Cells %d and %d cannot be parts of the same union of cells, because their depth levels are different, or because they are inside different cells, which are not themselves parts of a union of cells.\n", c1->ID, c2->ID);
								goto CONTINUE;
							}
						}
					}
					if (c2->usedInCellEntryTallies[0] || c2->usedInCellEntryTallies[1]) {
						printf("Error: Cell %d, which is used in the definition of a union, was previously specified in the definition of a cell-entry tally (the cell-entry tally for a cell that is a part of a union must be defined after the union, and can be defined only for the first cell of the union).\n", cellID); goto CONTINUE;
					}
					if (c2->nPulseHeight) {
						printf("Error: Cell %d, which is used in the definition of a union, was previously specified in the definition of a pulse-height tally (the pulse-height tally for a cell that is a part of a union must be defined after the union, and can be defined only for the first cell of the union).\n", cellID); goto CONTINUE;
					}
					cell[i].unionNum = nUnions;
					cell[i].mat = mat;
					cell[i].concentration = concentration;
					unionData[nUnions].parts = (short int *)realloc(unionData[nUnions].parts, (unionData[nUnions].nParts + 1) * sizeof(short int));
					unionData[nUnions].parts[unionData[nUnions].nParts] = i;
					unionData[nUnions].nParts++;
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			if (unionData[nUnions].nParts == 1) {
				printf("Error: Only one cell is listed in the definition of a union of cells.\n"); continue;
			}
			for (i = 0; i < unionData[nUnions].nParts; i++) {
				CELL *c = cell + unionData[nUnions].parts[i];
				c->cutoffEnergy = electronCutoffEnergy;
				c->cutoffEnergy_photon = photonCutoffEnergy;
				c->force = forceOpt;
			}
			nUnions++;
		}
		else if (!strnicmp(ptr, "S", 1) && strlen(ptr) > 1 && atoi(ptr + 1) > 0) {
			keywordNum = -1;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			double value_test = strtod(ptr + 1, &endPtr);
			if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
				printf("Error in specification of the surface \"%s\": The first keyword format must be \"S<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", ptr); continue;
			}
			int surfaceID = (int)(value_test + 0.5);
			if (!complexGeometry) {
				printf("Error: Keyword \"S<n>\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			for (i = 0; i < nSurfaces && (surf[i].mb_index >= 0 || surf[i].ID != surfaceID); i++);
			if (i == nSurfaces) {
				for (j = 0; j < nMacrobodies && macrobody[j].ID != surfaceID; j++);
			}
			if (i < nSurfaces || j < nMacrobodies) {
				printf("Error: Duplicate surface identifier \"S%d\".\n", surfaceID); continue;
			}
			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of a surface.\n"); continue;
			}
			TRANSFORM *tr = NULL;
			value_test = strtod(ptr, &endPtr);
			if (*endPtr == '\0') {
				if (value_test <= 0.5 || value_test >= 2147483647.5) {
					printf("Error in specification of surface \"S%d\": Invalid identifier of coordinate transformation \"%s\".\n", surfaceID, ptr); continue;
				}
				int transformID = (int)(value_test + 0.5);
				for (i = 0; i < nTransforms && transform[i].ID != transformID; i++);
				if (i == nTransforms) {
					printf("Error in specification of surface \"S%d\": Unknown coordinate transformation \"%s\".\n", surfaceID, ptr); continue;
				}
				tr = transform + i;
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					printf("Error: Missing specification of a surface.\n"); continue;
				}
			}
			SURFACE s, p[8];
			MACROBODY mb;
			bool isMb;
			char *name[12] = {"plane", "sphere", "cylinder", "cone", "right circular cylinder", "truncated right-angle cone", "box", "rectangular parallelepiped", "right hexagonal prism", "wedge", "four-sided polyhedron", "polyhedron"};
			int nSurfaces_bak;
			value_test = strtod(ptr, &endPtr);
			if (*endPtr == '\0') {
				if (value_test <= 0.5 || value_test >= 2147483647.5) {
					printf("Error in specification of surface \"S%d\": Invalid identifier of the original surface \"%s\".\n", surfaceID, ptr); continue;
				}
				int surfaceID0 = (int)(value_test + 0.5);
				ptr = NextToken(line, NULL, " \t\n");
				if (ptr) {
					printf("Error in specification of surface \"S%d\": Extra entries after the identifier of the original surface %d.\n", surfaceID, surfaceID0); continue;
				}
				for (i = 0; i < nSurfaces && (surf[i].mb_index >= 0 || surf[i].ID != surfaceID0); i++);
				if (i < nSurfaces) {
					s = surf[i];
					s.init = LocalToGlobal(tr->origin, tr->Q[0], s.init);
					if (s.type != 2) {
						s.dir = LocalToGlobal(zero, tr->Q[0], s.dir);
						s.dir /= Norm(s.dir);
					}
					if (s.type == 1) s.D = -s.dir * s.init;
					goto CHECK_FOR_COINCIDING_SURFACES;
				}
				else {
					for (j = 0; j < nMacrobodies && macrobody[j].ID != surfaceID0; j++);
					if (j == nMacrobodies) {
						printf("Error in specification of surface \"S%d\": Unknown identifier of the original surface %d.\n", surfaceID, surfaceID0); continue;
					}
					mb = macrobody[j];
					mb.ID = surfaceID;
					if (!macrobody[j].type) {
						if (mb.mbNum0 < 0) mb.mbNum0 = j;
						mb.s = (unsigned short int *)malloc(mb.nS * sizeof(unsigned short int));
						mb.sense = (char *)malloc(mb.nS * sizeof(char));
						MACROBODY *mb0 = macrobody + j;
						memcpy(mb.sense, mb0->sense, mb.nS * sizeof(char));
						for (k = 0; k < mb.nS; k++) {
							SURFACE s = surf[mb0->s[k]];
							s.init = LocalToGlobal(tr->origin, tr->Q[0], s.init);
							if (s.type != 2) {
								s.dir = LocalToGlobal(zero, tr->Q[0], s.dir);
								s.dir /= Norm(s.dir);
							}
							if (s.type == 1) s.D = -s.dir * s.init;
							for (m = 0; m < nSurfaces && !SurfacesCoincide(&s, surf + m, distancePrecision); m++);
							mb.s[k] = m;
							if (m == nSurfaces) {
								if (nSurfaces == MAXSURFACES) {
									printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); goto CONTINUE;
								}
								s.ID = 0;
								surf[nSurfaces] = s;
								nSurfaces++;
							}
							else if (s.type == 1 && s.dir * surf[m].dir < 0.) mb.sense[k] = -mb.sense[k];
						}
						macrobody[nMacrobodies] = mb;
						nMacrobodies++;
						success = true; continue;
					}
					for (k = 0; k < mb.nS; k++) {
						p[k] = surf[mb.s[k]];
						p[k].init = LocalToGlobal(tr->origin, tr->Q[0], p[k].init);
						if (s.type != 2) {
							p[k].dir = LocalToGlobal(zero, tr->Q[0], p[k].dir);
							p[k].dir /= Norm(p[k].dir);
						}
						if (p[k].type == 1) p[k].D = -p[k].dir * p[k].init;
					}
					if (mb.type == 4) {
						if (ConvertRPPtoBOX(p, arb_entries[nMacrobodies])) mb.type = 3;
						goto FINALIZE_MACROBODY;
					}
					VECTOR v[4];
					if (mb.type == 3 || mb.type == 7) { // BOX or ARB4
						for (k = 0, i = 0; i < 4; i++, k+=3) memcpy(v + i, arb_entries[j] + k, 3 * sizeof(double));
					}
					else goto FINALIZE_MACROBODY;
					v[0] = LocalToGlobal(tr->origin, tr->Q[0], v[0]);
					if (mb.type == 7) {
						for (i = 1; i < 4; i++) v[i] = LocalToGlobal(tr->origin, tr->Q[0], v[i]);
					}
					else {
						for (i = 1; i < 4; i++) v[i] = LocalToGlobal(zero, tr->Q[0], v[i]);
					}
					arb_entries[nMacrobodies] = new double[12];
					for (k = 0, i = 0; i < 4; i++, k+=3) memcpy(arb_entries[nMacrobodies] + k, v + i, 3 * sizeof(double));
					goto FINALIZE_MACROBODY;
				}
			}
			if (!strcmpi(ptr, "MB")) { // general macrobody (a shorthand notation for a set of two or more regular surfaces)
				if (tr) {
					printf("Error: An attempt to define general macrobody \"S%d\" in an auxiliary coordinate system (this is allowed only for regular surfaces and standard macrobodies).\n", surfaceID); continue;
				}
				mb.type = 0;
				mb.ID = surfaceID;
				mb.nS = 0;
				mb.s = (unsigned short int *)malloc(0);
				mb.sense = (char *)malloc(0);
				mb.mb = (short int *)malloc(0);
				mb.i = (short int *)malloc(0);
				mb.mbNum0 = -1;
				int chunkSize = 10, count = 0;
				ptr = NextToken(line, NULL, " \t\n");
				while(ptr) {
					int rc = CheckSurfaceID(line, ptr, false, surfaceID, mb.nS, mb.s, mb.sense, mb.mb, mb.i, chunkSize);
					if (!rc) goto CONTINUE;
					if (rc < 0) continue;
					count++;
					ptr = NextToken(line, NULL, " \t\n");
				}
				if (count < 2) {
					printf("Error: The number of surfaces specified in the definition of general macrobody \"S%d\" is less than 2.\n", surfaceID); continue;
				}
				notImplemented = false;
				bool removeSurface[MAXSURFACES_CELL];
				int nSurfacesRemoved = 0;
				memset(removeSurface, 0, mb.nS * sizeof(bool));
				char regionID1[100], regionID2[100];
				for (j = 0; j < mb.nS; j++) {
					if (removeSurface[j]) continue;
					for (k = j + 1; k < mb.nS; k++) {
						if (removeSurface[k]) continue;
						int rc = CheckCellGeometry0(mb.s, j, k, mb.sense, distancePrecision, notImplemented);
						if (rc == -1) {
							removeSurface[j] = true;
							if (!nSurfacesRemoved) SurfaceIDstring(surf + mb.s[j], mb.sense[j], mb.mb[j], mb.i[j], regionID1);
							else if (nSurfacesRemoved == 1) SurfaceIDstring(surf + mb.s[j], mb.sense[j], mb.mb[j], mb.i[j], regionID2);
							nSurfacesRemoved++;
							break;
						}
						else if (rc == -2) {
							removeSurface[k] = true;
							if (!nSurfacesRemoved) SurfaceIDstring(surf + mb.s[k], mb.sense[k], mb.mb[k], mb.i[k], regionID1);
							else if (nSurfacesRemoved == 1) SurfaceIDstring(surf + mb.s[k], mb.sense[k], mb.mb[k], mb.i[k], regionID2);
							nSurfacesRemoved++;
						}
					}
				}
				if (nSurfacesRemoved) {
					if (nSurfacesRemoved == 1) printf("Warning: Region %s was removed from the definition of general macrobody \"S%d\".\n", regionID1, surfaceID);
					else if (nSurfacesRemoved == 2) printf("Warning: Regions %s and %s were removed from the definition of general macrobody \"S%d\".\n", regionID1, regionID2, surfaceID);
					else printf("Warning: %d regions (%s, %s, ...) were removed from the definition of general macrobody \"S%d\".\n", nSurfacesRemoved, regionID1, regionID2, surfaceID);
					for (k = 0, j = 0; j < mb.nS; j++) {
						if (removeSurface[j]) continue;
						mb.s[k] = mb.s[j];
						mb.sense[k] = mb.sense[j];
						mb.mb[k] = mb.mb[j];
						mb.i[k] = mb.i[j];
						k++;
					}
					mb.nS = k;
				}
				macrobody[nMacrobodies] = mb;
				nMacrobodies++;
			}
			else {
				int num, nEntries;
				if (!strcmpi(ptr, "P")) { // plane
					s.type = 1;
					nEntries = 9;
					num = 0;
				}
				else if (!strcmpi(ptr, "S")) { // sphere
					s.type = 2;
					nEntries = 4;
					num = 1;
				}
				else if (!strcmpi(ptr, "CY")) { // circular cylinder
					s.type = 3;
					nEntries = 7;
					num = 2;
				}
				else if (!strcmpi(ptr, "CO")) { // circular one-sheet cone
					s.type = 4;
					nEntries = 7;
					num = 3;
				}
				else if (!strcmpi(ptr, "RCC")) { // right circular cylinder
					mb.type = 1;
					nEntries = 7;
					num = 4;
				}
				else if (!strcmpi(ptr, "TRC")) { // truncated right-angle cone
					mb.type = 2;
					nEntries = 8;
					num = 5;
				}
				else if (!strcmpi(ptr, "BOX")) { // general parallelepiped
					mb.type = 3;
					nEntries = 12;
					num = 6;
				}
				else if (!strcmpi(ptr, "RPP")) { // rectangular parallelepiped, surfaces normal to major axes
					mb.type = 4;
					nEntries = 6;
					num = 7;
				}
				else if (!strcmpi(ptr, "RHP") || !strcmpi(ptr, "HEX")) { // right hexagonal prism
					mb.type = 5;
					nEntries = 9;
					num = 8;
				}
				else if (!strcmpi(ptr, "WED")) { // wedge
					mb.type = 6;
					nEntries = 12;
					num = 9;
				}
				else if (!strcmpi(ptr, "ARB4")) { // four-sided polyhedron defined in terms of vertex coordinates
					mb.type = 7;
					nEntries = 12;
					num = 10;
				}
				else if (!strcmpi(ptr, "ARB")) { // polyhedron with 4 - 8 corners and 4 - 6 sides defined in terms of triplets of corners
					mb.type = 8;
					nEntries = 30;
					num = 11;
				}
				else {
					printf("Error in specification of the surface \"S%d\": the surface type must be \"P\", \"S\", \"CY\", \"CO\", \"MB\", \"RCC\", \"TRC\", \"BOX\", \"RPP\", \"RHP\", \"HEX\", \"WED\", \"ARB4\" or \"ARB\".\n", surfaceID); continue;
				}
				isMb = (num > 3);
				double init_dir[9];
				double mb_entries[30];
				ptr = NextToken(line, NULL, " \t\n");
				for (i = 0; ptr && i < nEntries; i++) {
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0') {
						printf("Error in specification of the %s \"S%d\": incorrect number format \"%s\".\n", name[num], surfaceID, ptr); goto CONTINUE;
					}
					if (num < 6) { // Individual surfaces, or macrobodies including curved surfaces
						if (!num || num < 5 && i < nEntries - 1 || num == 5 && i < nEntries - 2) {
							if (!isMb) init_dir[i] = value;
						}
						else if (num != 3) {
							if (value <= 0.) {
								printf("Error in specification of the %s \"S%d\": radius is not a positive number (%g).\n", name[num], surfaceID, value); goto CONTINUE;
							}
							if (!isMb) s.R = value;
						}
						else {
							s.angle = value * pi / 180.;
						}
					}
					if (isMb) mb_entries[i] = value;
					ptr = NextToken(line, NULL, " \t\n");
				}
				if (!num) {
					if (i != 4 && i != 6 && i != 9) {
						printf("Error: Incorrect number of entries in the definition of the %s \"S%d\" (it must be 4, 6 or 9).\n", name[num], surfaceID); continue;
					}
					if (i == 4) { // Coefficients of the plane equation A * x + B * y + C * z - D = 0
						double norm = Norm(*((VECTOR *)init_dir));
						if (!norm) {
							printf("Error in specification of the %s \"S%d\": all components of the direction vector are zero.\n", name[num], surfaceID); continue;
						}
						init_dir[0] /= norm;
						init_dir[1] /= norm;
						init_dir[2] /= norm;
						init_dir[3] /= norm;
						double D = init_dir[3];
						memcpy(init_dir + 3, init_dir, 3 * sizeof(double));
						init_dir[0] *= D;
						init_dir[1] *= D;
						init_dir[2] *= D;
					}
					else if (i == 9) { // Coordinates of three points on the plane
						VECTOR *point = (VECTOR *)init_dir;
						int next[3] = {1, 2, 0};
						VECTOR v[3];
						for (j = 0; j < 3; j++) {
							v[j] = point[j] - point[next[j]];
							double norm = Norm(v[j]);
							if (norm <= distancePrecision) {
								printf("Error: Distance between points No. %d and No. %d in the definition of the %s \"S%d\" is less than %g cm.\n", j + 1, next[j] + 1, name[num], surfaceID, distancePrecision); continue;
							}
							v[j] /= norm; 
						}
						VECTOR dir = v[0] & v[1];
						double norm = Norm(dir);
						if (norm <= anglePrecision) {
							printf("Error in specification of the %s \"S%d\": the three points are collinear.\n", name[num], surfaceID); continue;
						}
						dir /= norm;
						double D = point[0] * dir;
						if (fabs(D) > distancePrecision) {
							if (D < 0.) dir = -dir; // the normal must be directed away from the origin
						}
						else if (fabs(dir.z) > anglePrecision) {
							if (dir.z < 0.) dir = -dir;
						}
						else if (fabs(dir.y) > anglePrecision) {
							if (dir.y < 0.) dir = -dir;
						}
						else if (dir.x < 0.) dir = -dir;
						point[1] = dir;
					}
				}
				else if (i < nEntries) {
					printf("Error: The number of entries in the definition of the %s \"S%d\" is less than %d.\n", name[num], surfaceID, nEntries); continue;
				}
				if (!isMb) memcpy((double *)&s + 2, init_dir, 6 * sizeof(double));
				if (i == nEntries && ptr) {
					printf("Error: The number of entries in the definition of the %s \"S%d\" is greater than %d.\n", name[num], surfaceID, nEntries); continue;
				}
				if (num < 6 || num == 8) { // Individual surfaces, or macrobodies including curved surfaces, or a right hexagonal prism
					if (!isMb && num != 1 && !Norm(s.dir)) {
						printf("Error in specification of the %s \"S%d\": all components of the direction vector are zero.\n", name[num], surfaceID); continue;
					}
					if (isMb && Norm(*((VECTOR *)(mb_entries + 3))) <= distancePrecision) {
						printf("Error in specification of the %s \"S%d\": the norm of the height vector is less than %g cm.\n", name[num], surfaceID, distancePrecision); continue;
					}
					if (num == 5 && fabs(mb_entries[6] - mb_entries[7]) <= distancePrecision) {
						printf("Error in specification of the %s \"S%d\": radii of both bases are equal.\n", name[num], surfaceID); continue;
					}
				}
				if (!isMb) {
					if (s.type != 2) s.dir /= Norm(s.dir);
					if (tr) {
						s.init = LocalToGlobal(tr->origin, tr->Q[0], s.init);
						if (s.type != 2) {
							s.dir = LocalToGlobal(zero, tr->Q[0], s.dir);
							s.dir /= Norm(s.dir);
						}
					}
					if (s.type == 1) s.D = -s.dir * s.init;
					if (s.type == 4) {
						if (s.angle <= anglePrecision || s.angle >= 0.5 * pi - anglePrecision) {
							double degrees = s.angle * 180. / pi;
							double precision = anglePrecision * 180. / pi;
							printf("Error in specification of the cone \"S%d\": incorrect angle %g (it must be between %.15g and %.15g).\n", surfaceID, degrees, precision, 90. - precision); continue;
						}
						s.sin = sin(s.angle);
						s.cos = cos(s.angle);
						s.tan = tan(s.angle);
					}
CHECK_FOR_COINCIDING_SURFACES:
					bool warn_coincidingSurfaces = false;
					for (i = 0; i < nSurfaces; i++) {
						if (SurfacesCoincide(&s, surf + i, distancePrecision)) {
							if (surf[i].mb_index >= 0) break;
							if (!warn_coincidingSurfaces) {
								printf("Warning: Surfaces %d and %d coincide.\n", surfaceID, surf[i].ID);
								warn_coincidingSurfaces = true;
								if (!nMacrobodies) i = nSurfaces - 1;
							}
						}
					}
					if (i == nSurfaces) {
						if (nSurfaces == MAXSURFACES) {
							printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); continue;
						}
						s.ID = surfaceID;
						s.mb_index = -1;
						s.nMb = 0;
						surf[nSurfaces] = s;
						nSurfaces++;
					}
					else { // Replace the original surface identifier (defined in terms of the macrobody number) by the overall identifier
						int mbNum = surf[i].mb_index;
						int sNumInsideMb = surf[i].ID - 1;
						surf[i].mb_index = -1;
						surf[i].ID = surfaceID;
						if (s.type == 1 && s.dir * surf[i].dir < 0.) { // If the user-defined direction of the plane's normal does not match the original direction, then flip the direction of the normal and change the sense of this plane in all macrobodies and cells using it
							surf[i].dir = -surf[i].dir;
							surf[i].D = -surf[i].D;
							MACROBODY *mb = macrobody + mbNum;
							mb->sense[sNumInsideMb] = -mb->sense[sNumInsideMb];
							if (surf[i].nMb > 1) {
								for (j = 0; j < nMacrobodies; j++) {
									if (j == mbNum) continue;
									mb = macrobody + j;
									for (k = 0; k < mb->nS && mb->s[k] != i; k++);
									if (k < mb->nS) mb->sense[k] = -mb->sense[k];
								}
							}
							for (j = 0; j < nCells; j++) {
								CELL *c = cell + j;
								for (k = 0; k < c->nS && c->s[k] != i; k++);
								if (k < c->nS) c->side[k] = -c->side[k];
							}
						}
					}
				}
				else {
					nSurfaces_bak = nSurfaces;
					mb.ID = surfaceID;
					mb.mbNum0 = -1;
					if (mb.type < 3) { // RCC or TRC
						mb.nS = 3;
						mb.s = (unsigned short int *)malloc(mb.nS * sizeof(unsigned short int));
						mb.sense = (char *)malloc(mb.nS * sizeof(char));
						s.nMb = 0;
						VECTOR baseCenter = *((VECTOR *)mb_entries);
						VECTOR dir = *((VECTOR *)(mb_entries + 3));
						if (tr) {
							baseCenter = LocalToGlobal(tr->origin, tr->Q[0], baseCenter);
							dir = LocalToGlobal(zero, tr->Q[0], dir);
						}
						double height = Norm(dir);
						dir /= height;
						// Cylinder or cone
						if (mb.type == 1) { // RCC
							s.type = 3;
							s.init = baseCenter;
							s.dir = dir;
							s.R = mb_entries[6];
						}
						else if (mb.type == 2) { // TRC
							s.type = 4;
							double R1 = mb_entries[6];
							double R2 = mb_entries[7];
							if (R1 < R2) {
								s.tan = (R2 - R1) / height;
								s.init = baseCenter - dir * R1 / s.tan;
								s.dir = dir;
							}
							else {
								s.tan = (R1 - R2) / height;
								s.init = baseCenter + dir * (height + R2 / s.tan);
								s.dir = -dir;
							}
							s.angle = atan(s.tan);
							if (s.angle <= anglePrecision || s.angle >= 0.5 * pi - anglePrecision) {
								double degrees = s.angle * 180. / pi;
								double precision = anglePrecision * 180. / pi;
								printf("Error in specification of the %s \"S%d\": incorrect cone angle %g (it must be between %.15g and %.15g).\n", name[num], surfaceID, degrees, precision, 90. - precision); continue;
							}
							s.sin = sin(s.angle);
							s.cos = cos(s.angle);
						}
						for (i = 0; i < nSurfaces && !SurfacesCoincide(&s, surf + i, distancePrecision); i++);
						mb.s[0] = i;
						mb.sense[0] = -1;
						if (i == nSurfaces) {
							if (nSurfaces == MAXSURFACES) {
								printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); continue;
							}
							s.ID = 1;
							s.mb_index = nMacrobodies;
							surf[nSurfaces] = s;
							nSurfaces++;
						}
						surf[i].nMb++;
						// Top plane
						s.type = 1;
						s.init = baseCenter + dir * height;
						s.dir = dir;
						s.D = -s.dir * s.init;
						for (i = 0; i < nSurfaces && !SurfacesCoincide(&s, surf + i, distancePrecision); i++);
						mb.s[1] = i;
						if (i == nSurfaces) {
							if (nSurfaces == MAXSURFACES) {
								printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); continue;
							}
							s.ID = 2;
							s.mb_index = nMacrobodies;
							surf[nSurfaces] = s;
							nSurfaces++;
							mb.sense[1] = -1;
						}
						else mb.sense[1] = (s.dir * surf[i].dir < 0.) ? 1 : -1;
						surf[i].nMb++;
						// Bottom plane
						s.type = 1;
						s.init = baseCenter;
						s.dir = -dir;
						s.D = -s.dir * s.init;
						for (i = 0; i < nSurfaces && !SurfacesCoincide(&s, surf + i, distancePrecision); i++);
						mb.s[2] = i;
						if (i == nSurfaces) {
							if (nSurfaces == MAXSURFACES) {
								printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); continue;
							}
							s.ID = 3;
							s.mb_index = nMacrobodies;
							surf[nSurfaces] = s;
							nSurfaces++;
							mb.sense[2] = -1;
						}
						else mb.sense[2] = (s.dir * surf[i].dir < 0.) ? 1 : -1;
						surf[i].nMb++;
						// Finalization
						if (nSurfaces == nSurfaces_bak) {
							for (i = 0; i < nMacrobodies; i++) {
								MACROBODY *mb0 = macrobody + i;
								if (mb0->type != mb.type) continue;
								if (mb.s[0] == mb0->s[0] &&
									(mb.s[1] == mb0->s[1] && mb.sense[1] == mb0->sense[1] && mb.s[2] == mb0->s[2] && mb.sense[2] == mb0->sense[2] ||
									 mb.s[1] == mb0->s[2] && mb.sense[1] == mb0->sense[2] && mb.s[2] == mb0->s[1] && mb.sense[2] == mb0->sense[1])) {
									printf("Warning: Surfaces %d and %d coincide.\n", surfaceID, mb0->ID);
									break;
								}
							}
						}
					}
					else if (mb.type < 5) { // BOX or RPP
						mb.nS = 6;
						VECTOR v[4];
						if (mb.type == 3) { // BOX
							for (k = 0, i = 0; i < 4; i++, k+=3) {
								v[i] = *((VECTOR *)(mb_entries + k));
								if (i && Norm(v[i]) <= distancePrecision) {
									printf("Error in specification of the %s \"S%d\": the norm of the edge vector No. %d is less than %g cm.\n", name[num], surfaceID, i, distancePrecision); goto CONTINUE;
								}
							}
						}
						else { // RPP
							v[0] = Init(mb_entries[0], mb_entries[2], mb_entries[4]);
							v[1] = Init(mb_entries[1] - mb_entries[0], 0., 0.);
							v[2] = Init(0., mb_entries[3] - mb_entries[2], 0.);
							v[3] = Init(0., 0., mb_entries[5] - mb_entries[4]);
							if (Norm(v[1]) <= distancePrecision || Norm(v[2]) <= distancePrecision || Norm(v[3]) <= distancePrecision) {
								printf("Error: The size of the %s \"S%d\" is less than %g cm.\n", name[num], surfaceID, distancePrecision); continue;
							}
						}
						if (tr) {
							v[0] = LocalToGlobal(tr->origin, tr->Q[0], v[0]);
							for (i = 1; i < 4; i++) v[i] = LocalToGlobal(zero, tr->Q[0], v[i]);
						}
						VECTOR edgeDir[3];
						VECTOR oppositePoint  = v[0] + v[1] + v[2] + v[3];
						for (i = 0; i < 6; i+=2) {
							p[i].init = oppositePoint;
							p[i+1].init = v[0];
						}
						for (i = 0; i < 3; i++) edgeDir[i] = v[i+1] / Norm(v[i+1]);
						int next[4] = {1, 2, 0, 1};
						for (i = 0; i < 3; i++) {
							VECTOR dir = edgeDir[next[i]] & edgeDir[next[i+1]];
							double norm = Norm(dir);
							if (norm <= anglePrecision) {
								printf("Error in specification of the %s \"S%d\": edges No. %d and No. %d are collinear.\n", name[num], surfaceID, next[i] + 1, next[i] + 2); goto CONTINUE;
							}
							dir /= norm;
							double distanceToPlane = dir * v[i+1];
							if (fabs(distanceToPlane) <= distancePrecision) {
								printf("Error in specification of the %s \"S%d\": all three edges are coplanar.\n", name[num], surfaceID); goto CONTINUE;
							}
							if (distanceToPlane < 0.) dir = -dir;
							p[2*i].dir = dir;
							p[2*i+1].dir = -dir;
						}
						if (mb.type == 3) {
							arb_entries[nMacrobodies] = new double[nEntries];
							for (k = 0, i = 0; i < 4; i++, k+=3) memcpy(arb_entries[nMacrobodies] + k, v + i, 3 * sizeof(double));
						}
						else if (!tr) {
							arb_entries[nMacrobodies] = new double[nEntries];
							memcpy(arb_entries[nMacrobodies], mb_entries, nEntries * sizeof(double));
						}
						else if (ConvertRPPtoBOX(p, arb_entries[nMacrobodies])) mb.type = 3;
					}
					else if (mb.type == 5) { // RHP or HEX
						mb.nS = 8;
						VECTOR axisToSide = *((VECTOR *)(mb_entries + 6));
						double R = Norm(axisToSide);
						if (R <= distancePrecision) {
							printf("Error in specification of the %s \"S%d\": Distance from the symmetry axis to the side is less than %g cm.\n", name[num], surfaceID, distancePrecision); continue;
						}
						VECTOR heightVector = *((VECTOR *)(mb_entries + 3));
						VECTOR dir = heightVector / Norm(heightVector);
						VECTOR dir1 = axisToSide / R;
						if (fabs(dir * dir1) > anglePrecision) {
							printf("Error: Vectors defined by the second and third triplets of entries in specification of %s \"S%d\" are not orthogonal.\n", name[num], surfaceID); continue;
						}
						VECTOR baseCenter = *((VECTOR *)mb_entries);
						if (tr) {
							axisToSide = LocalToGlobal(zero, tr->Q[0], axisToSide);
							heightVector = LocalToGlobal(zero, tr->Q[0], heightVector);
							baseCenter = LocalToGlobal(tr->origin, tr->Q[0], baseCenter);
							dir = heightVector / Norm(heightVector);
							dir1 = axisToSide / R;
						}
						p[7].init = baseCenter; p[7].dir = -dir;
						p[6].init = baseCenter + heightVector; p[6].dir = dir;
						VECTOR sideCenter = baseCenter + axisToSide;
						p[0].init = sideCenter; p[0].dir = dir1;
						p[1].init = baseCenter - axisToSide; p[1].dir = -dir1;
						double Q[3][3];
						RotationMatrix(Q, dir, dir1);
						VECTOR local;
						local.z = 0.;
						double angle = pi / 3.;
						for (j = 2, i = 1; i <= 2; i++, j+=2) {
							local.x = R * cos(i * angle);
							local.y = R * sin(i * angle);
							p[j].init = LocalToGlobal(baseCenter, Q[0], local);
							axisToSide = p[j].init - baseCenter; p[j].dir = axisToSide / Norm(axisToSide);
							p[j+1].init = baseCenter - axisToSide; p[j+1].dir = -p[j].dir;
						}
					}
					else if (mb.type == 6) { // WED
						mb.nS = 5;
						VECTOR v[4];
						for (k = 0, i = 0; i < 4; i++, k+=3) {
							v[i] = *((VECTOR *)(mb_entries + k));
							if (i) {
								if (Norm(v[i]) <= distancePrecision) {
									printf("Error in specification of the %s \"S%d\": the norm of the edge vector No. %d is less than %g cm.\n", name[num], surfaceID, i, distancePrecision); goto CONTINUE;
								}
								if (tr) v[i] = LocalToGlobal(zero, tr->Q[0], v[i]);
							}
							else if (tr) v[0] = LocalToGlobal(tr->origin, tr->Q[0], v[0]);
						}
						VECTOR edgeDir[3];
						for (i = 0; i < 3; i++) edgeDir[i] = v[i+1] / Norm(v[i+1]);
						int next[4] = {1, 2, 0, 1};
						for (i = 0; i < 3; i++) {
							VECTOR dir = edgeDir[next[i]] & edgeDir[next[i+1]];
							double norm = Norm(dir);
							if (norm <= anglePrecision) {
								printf("Error in specification of the %s \"S%d\": edges No. %d and No. %d are collinear.\n", name[num], surfaceID, next[i] + 1, next[i] + 2); goto CONTINUE;
							}
							double distanceToPlane = (dir / norm) * v[i+1];
							if (fabs(distanceToPlane) <= distancePrecision) {
								printf("Error in specification of the %s \"S%d\": all three edges are coplanar.\n", name[num], surfaceID); goto CONTINUE;
							}
						}
						VECTOR hypotenuse = v[2] - v[1];
						p[0].init = v[0] + v[1];
						VECTOR dir = hypotenuse & v[3];
						p[0].dir = dir / Norm(dir);
						if (v[1] * p[0].dir < 0.) p[0].dir = -p[0].dir;

						p[1].init = v[0];
						dir = v[2] & v[3];
						p[1].dir = dir / Norm(dir);
						if (v[1] * p[1].dir > 0.) p[1].dir = -p[1].dir;

						p[2].init = v[0];
						dir = v[1] & v[3];
						p[2].dir = dir / Norm(dir);
						if (v[2] * p[2].dir > 0.) p[2].dir = -p[2].dir;

						p[3].init = v[0] + v[3];
						dir = v[1] & v[2];
						p[3].dir = dir / Norm(dir);
						if (v[3] * p[3].dir < 0.) p[3].dir = -p[3].dir;

						p[4].init = v[0];
						p[4].dir = -p[3].dir;
					}
					else if (mb.type == 7) { // ARB4
						mb.nS = 4;
						VECTOR v[4], edge[3];
						for (k = 0, i = 0; i < 4; i++, k+=3) {
							v[i] = *((VECTOR *)(mb_entries + k));
							if (tr) v[i] = LocalToGlobal(tr->origin, tr->Q[0], v[i]);
							if (i) {
								edge[i-1] = v[i] - v[0];
								if (Norm(edge[i-1]) <= distancePrecision) {
									printf("Error in specification of the %s \"S%d\": the norm of the edge vector No. %d is less than %g cm.\n", name[num], surfaceID, i, distancePrecision); goto CONTINUE;
								}
							}
						}
						VECTOR edgeDir[3];
						for (i = 0; i < 3; i++) {
							p[i].init = v[0];
							edgeDir[i] = edge[i] / Norm(edge[i]);
						}
						p[3].init = (v[1] + v[2] + v[3]) / 3.;
						int next[4] = {1, 2, 0, 1};
						for (i = 0; i < 3; i++) {
							VECTOR dir = edgeDir[next[i]] & edgeDir[next[i+1]];
							double norm = Norm(dir);
							if (norm <= anglePrecision) {
								printf("Error in specification of the %s \"S%d\": all vertices belong to the same plane.\n", name[num], surfaceID); goto CONTINUE;
							}
							dir /= norm;
							double distanceToPlane = dir * edge[i];
							if (fabs(distanceToPlane) <= distancePrecision) {
								printf("Error in specification of the %s \"S%d\": all vertices belong to the same plane.\n", name[num], surfaceID); goto CONTINUE;
							}
							if (distanceToPlane > 0.) dir = -dir;
							p[i].dir = dir;
						}
						VECTOR edge3 = v[2] - v[1];
						VECTOR edge4 = v[3] - v[2];
						VECTOR dir = (edge3 / Norm(edge3)) & (edge4 / Norm(edge4));
						dir /= Norm(dir);
						if (dir * edge[0] < 0.) dir = -dir;
						p[3].dir = dir;
						arb_entries[nMacrobodies] = new double[nEntries];
						for (k = 0, i = 0; i < 4; i++, k+=3) memcpy(arb_entries[nMacrobodies] + k, v + i, 3 * sizeof(double));
					}
					else if (mb.type == 8) { // ARB
						int n[6];
						mb.nS = 0;
						bool isUsed[8];
						memset(isUsed, 0, 8 * sizeof(bool));
						int c[8][4], iEntry;
						for (iEntry = 24, i = 0; i < 6; i++, iEntry++) {
							double d = mb_entries[iEntry];
							if (d && (d < 1229.999999 || d > 8765.000001)) {
								printf("Error in specification of the %s \"S%d\": invalid entry %g (entries No. 25 - 30 that are not zero must be integers in the range from 1230 to 8765, with all digits different and less than 9 and the first three digits non-zero).\n", name[num], surfaceID, d); goto CONTINUE;
							}
							n[i] = (int)(d + 0.5);
							if (!n[i]) continue;
							int remainder = n[i];
							int div[4] = {1000, 100, 10, 1};
							for (j = 0; j < 4; j++) {
								c[i][j] = remainder / div[j];
								if (c[i][j] == 9 || !c[i][j] && j < 3) break;
								for (k = 0; k < j && c[i][k] != c[i][j]; k++);
								if (k < j) break;
								if (j < 3) {
									isUsed[c[i][j]-1] = true;
									remainder -= c[i][j] * div[j];
								}
							}
							if (j < 4) {
								printf("Error in specification of the %s \"S%d\": invalid entry %d (entries No. 25 - 30 that are not zero must be integers in the range from 1230 to 8765, with all digits different and less than 9 and the first three digits non-zero).\n", name[num], surfaceID, n[i]); goto CONTINUE;
							}
							mb.nS++;
						}
						if (mb.nS < 4) {
							printf("Error in specification of the %s \"S%d\": the number of planes is less than 4.\n", name[num], surfaceID); continue;
						}
						VECTOR v[8];
						for (k = 0, i = 0; i < 8; i++, k+=3) {
							v[i] = *((VECTOR *)(mb_entries + k));
							if (tr) v[i] = LocalToGlobal(tr->origin, tr->Q[0], v[i]);
						}
						for (i = 0; i < 8; i++) {
							if (!isUsed[i]) continue;
							for (j = i + 1; j < 8; j++) {
								if (!isUsed[j]) continue;
								if (Norm(v[i] - v[j]) <= distancePrecision) {
									printf("Error in specification of the %s \"S%d\": the distance between vertices No. %d and %d is less than %g cm.\n", name[num], surfaceID, i + 1, j + 1, distancePrecision); goto CONTINUE;
								}
							}
						}
						for (j = 0, i = 0; i < 6; i++) {
							if (!n[i]) continue;
							p[j].init = v[c[i][1]-1];
							VECTOR edge1 = v[c[i][0]-1] - p[j].init;
							VECTOR edge2 = v[c[i][2]-1] - p[j].init;
							p[j].dir = (edge1 / Norm(edge1)) & (edge2 / Norm(edge2));
							double norm = Norm(p[j].dir);
							if (norm <= anglePrecision) {
								printf("Error in specification of the %s \"S%d\": the three corners described by the entry \"%d\" are collinear.\n", name[num], surfaceID, n[i]); goto CONTINUE;
							}
							p[j].dir /= norm;
							if (c[i][3] && fabs(p[j].dir * (v[c[i][3]-1] - p[j].init)) > distancePrecision) {
								printf("Warning: In specification of the %s \"S%d\", the entry \"%d\" is inconsistent (corner No. %d does not lie in the plane containing corners No. %d, %d and %d).\n", name[num], surfaceID, n[i], c[i][3], c[i][0], c[i][1], c[i][2]);
							}
							j++;
						}
						for (i = 0; i < mb.nS; i++) {
							for (j = 0; j < i; j++) {
								if (Norm(p[i].dir & p[j].dir) <= anglePrecision && fabs((p[i].init - p[j].init) * p[i].dir) <= distancePrecision) {
									printf("Error in specification of the %s \"S%d\": some of the planes coincide.\n", name[num], surfaceID); goto CONTINUE;
								}
							}
						}
						for (i = 0; i < 8; i++) {
							if (!isUsed[i]) continue;
							for (k = 0, j = 0; j < mb.nS; j++) {
								if (fabs(p[j].dir * (v[i] - p[j].init)) <= distancePrecision) {
									k++;
									if (k == 3) break;
								}
							}
							if (j == mb.nS) {
								printf("Error in specification of the %s \"S%d\": vertex No. %d (%g, %g, %g) belongs to less than 3 planes.\n", name[num], surfaceID, i + 1, v[i].x, v[i].y, v[i].z); goto CONTINUE;
							}
						}
						for (i = 0; i < mb.nS; i++) {
							int sign = 0;
							for (k = 0, j = 0; j < 8; j++) {
								if (!isUsed[j]) continue;
								double dist = p[i].dir * (v[j] - p[i].init);
								if (fabs(dist) <= distancePrecision) k++;
								else if (!sign) sign = (dist > 0 ? 1 : -1);
								else if (sign * dist < 0) break;
							}
							if (j < 8) {
								printf("Error in specification of the %s \"S%d\": the vertices that do not belong to plane No. %d are not all on the same side of it.\n", name[num], surfaceID, i + 1); goto CONTINUE;
							}
							if (k < 3) {
								printf("Error in specification of the %s \"S%d\": plane No. %d contains less than 3 vertices.\n", name[num], surfaceID, i + 1); goto CONTINUE;
							}
							if (sign > 0) p[i].dir = -p[i].dir;
						}
					}
					if (mb.type > 2) { // all types of polyhedra
						for (j = 0; j < mb.nS; j++) {
							p[j].type = 1;
							p[j].D = -p[j].dir * p[j].init;
						}
FINALIZE_MACROBODY:
						mb.s = (unsigned short int *)malloc(mb.nS * sizeof(unsigned short int));
						mb.sense = (char *)malloc(mb.nS * sizeof(char));
						for (j = 0; j < mb.nS; j++) {
							for (i = 0; i < nSurfaces && !SurfacesCoincide(p + j, surf + i, distancePrecision); i++);
							mb.s[j] = i;
							if (i == nSurfaces) {
								if (nSurfaces == MAXSURFACES) {
									printf("Error: The number of surfaces is greater than %d.\n", MAXSURFACES); goto CONTINUE;
								}
								p[j].ID = j + 1;
								p[j].mb_index = nMacrobodies;
								p[j].nMb = 1;
								surf[nSurfaces] = p[j];
								nSurfaces++;
								mb.sense[j] = -1;
							}
							else {
								surf[i].nMb++;
								if (p[j].type == 1) mb.sense[j] = (p[j].dir * surf[i].dir < 0.) ? 1 : -1;
							}
						}
					}
					mb.mb = (short int *)malloc(mb.nS * sizeof(short int)); memset(mb.mb, 0, mb.nS * sizeof(short int));
					mb.i = (short int *)malloc(mb.nS * sizeof(short int)); memset(mb.i, 0, mb.nS * sizeof(short int));
					macrobody[nMacrobodies] = mb;
					nMacrobodies++;
				}
			}
		}
		else if (!strnicmp(ptr, "TR", 2) && strlen(ptr) > 2 && atoi(ptr + 2) > 0 ||
			!strnicmp(ptr, "*TR", 3) && strlen(ptr) > 3 && atoi(ptr + 3) > 0) {
			keywordNum = -1;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			bool anglesSpecified = !strnicmp(ptr, "*TR", 3);
			double value_test = !anglesSpecified ? strtod(ptr + 2, &endPtr) : strtod(ptr + 3, &endPtr);
			if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
				printf("Error in specification of the coordinate transformation \"%s\": the first keyword format must be \"TR<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", ptr); continue;
			}
			int transformID = (int)(value_test + 0.5);
			if (!complexGeometry) {
				printf("Error: Keyword \"TR<n>\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			for (i = 0; i < nTransforms && transform[i].ID != transformID; i++);
			if (i < nTransforms) {
				printf("Error: Duplicate coordinate transformation identifier \"TR%d\".\n", transformID); continue;
			}
			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of a coordinate transformation.\n"); continue;
			}
			if (nTransforms == MAXTRANSFORMS) {
				printf("Error: The number of coordinate transformations is greater than %d.\n", MAXTRANSFORMS); continue;
			}
			TRANSFORM *tr = transform + nTransforms;
			tr->ID = transformID;
			TRANSFORM *tr0 = NULL;
			if (!strnicmp(ptr, "TR", 2) && strlen(ptr) > 2 && atoi(ptr + 2) > 0) {
				value_test = strtod(ptr + 2, &endPtr);
				if (*endPtr != '\0' || value_test <= 0.5 || value_test >= 2147483647.5) {
					printf("Error in specification of the coordinate transformation \"TR%d\": the identifier of the primary coordinate transformation must be \"TR<n>\", where \"<n>\" is a positive integer number less than 2^31 = 2147483648.\n", transformID); continue;
				}
				int transformID0 = (int)(value_test + 0.5);
				for (i = 0; i < nTransforms && transform[i].ID != transformID0; i++);
				if (i == nTransforms) {
					printf("Error: Unknown identifier of the primary coordinate transformation \"%s\" in the definition of the coordinate transformation \"TR%d\".\n", ptr, transformID); continue;
				}
				tr0 = transform + i;
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					printf("Error: Missing specification of a coordinate transformation.\n"); continue;
				}
			}
			int nEntriesMax = 9;
			double entries[9];
			for (i = 0; ptr && i < nEntriesMax; i++) {
				double value = strtod(ptr, &endPtr);
				if (*endPtr != '\0') {
					printf("Error in specification of the coordinate transformation \"TR%d\": incorrect number format \"%s\".\n", transformID, ptr); goto CONTINUE;
				}
				entries[i] = value;
				ptr = NextToken(line, NULL, " \t\n");
			}
			if (ptr || i < 3 || i == 4 || i > nEntriesMax) {
				printf("Error in specification of the coordinate transformation \"TR%d\": incorrect number of numerical entries (it must be 3, 5, 6, 7, 8 or %d).\n", transformID, nEntriesMax); continue;
			}
			bool azimuthSpecified = (i == 5 || i == 7 || i == 8);
			int nEntries = i;
			for (i = 0; i < nEntries; i++) {
				if (i < 3) *((double *)&(tr->origin) + i) = entries[i];
				else if (!azimuthSpecified || i > 4) {
					int i1 = !azimuthSpecified ? i : i + 1;
					j = i1 / 3 - 1; // 0-based sequence number of the unit vector of the auxiliary coordinate system
					k = i1 - (j + 1) * 3; // 0-based sequence number of the component of the unit vector of the auxiliary coordinate system, expressed in the main coordinate system
					if (!anglesSpecified) tr->Q[k][j] = entries[i];
					else {
						if (entries[i] < -720. || entries[i] > 720.) {
							printf("Error in specification of the coordinate transformation \"TR%d\": invalid angle %.14g (it must be between -720 and +720 degrees).\n", transformID, entries[i]); goto CONTINUE;
						}
						tr->Q[k][j] = cos(entries[i] / 180. * pi);
					}
					if (k == 2) { // in the case nEntries = 7, this part is not executed for the Y' axis (because k < 2)
						double norm2 = tr->Q[0][j] * tr->Q[0][j] + tr->Q[1][j] * tr->Q[1][j] + tr->Q[2][j] * tr->Q[2][j];
						if (norm2 <= anglePrecision * anglePrecision) {
							printf("Error in specification of the coordinate transformation \"TR%d\": the norm of unit vector No. %d is less than %g.\n", transformID, j + 1, anglePrecision); goto CONTINUE;
						}
						double norm = sqrt(norm2);
						if (anglesSpecified && fabs(norm - 1.) > anglePrecision) {
							printf("Error in specification of the coordinate transformation \"TR%d\": the norm of unit vector No. %d is not equal to 1.\n", transformID, j + 1); goto CONTINUE;
						}
						for (int n = 0; n < 3; n++) tr->Q[n][j] /= norm;
					}
				}
				else if (i == 3) {
					if (entries[i] < -720. || entries[i] > 720.) {
						printf("Error in specification of the coordinate transformation \"TR%d\": invalid azimuth angle %.14g (it must be between -720 and +720 degrees).\n", transformID, entries[i]); goto CONTINUE;
					}
				}
				else {
					if (entries[i] < -90. || entries[i] > 90.) {
						printf("Error in specification of the coordinate transformation \"TR%d\": invalid elevation angle %.14g (it must be between -90 and 90 degrees).\n", transformID, entries[i]); goto CONTINUE;
					}
					double azimuthAngle = entries[i-1] / 180. * pi;
					double elevationAngle = entries[i] / 180. * pi;
					double R = cos(elevationAngle);
					tr->Q[0][0] = R * cos(azimuthAngle);
					tr->Q[1][0] = R * sin(azimuthAngle);
					tr->Q[2][0] = sin(elevationAngle);
				}
			}
			if (nEntries == 7) {
				if (entries[6] < -90. || entries[6] > 90.) {
					printf("Error in specification of the coordinate transformation \"TR%d\": invalid elevation angle %.14g (it must be between -90 and 90 degrees).\n", transformID, entries[6]); goto CONTINUE;
				}
				double azimuthAngle = entries[5] / 180. * pi;
				double elevationAngle = entries[6] / 180. * pi;
				double R = cos(elevationAngle);
				tr->Q[0][1] = R * cos(azimuthAngle);
				tr->Q[1][1] = R * sin(azimuthAngle);
				tr->Q[2][1] = sin(elevationAngle);
			}
			VECTOR xPrime, yPrime;
			if (nEntries == 3) { // Pure translation
				memset(tr->Q[0], 0, 9 * sizeof(double));
				tr->Q[0][0] = 1.;
				tr->Q[1][1] = 1.;
				tr->Q[2][2] = 1.;
			}
			else {
				if (nEntries < 7) { // Only the X' axis direction is provided. Direction of the Y' axis is chosen arbitrarily, and direction of the Z' axis is calculated as the vector product.
					if (fabs(tr->Q[2][0]) <= anglePrecision && fabs(tr->Q[0][0]) <= anglePrecision) {
						if (tr->Q[1][0] > 0.) tr->Q[2][1] = 1.;
						else tr->Q[2][1] = -1.;
						tr->Q[1][1] = 0.;
						tr->Q[0][1] = 0.;
					}
					else if (fabs(tr->Q[1][0]) <= anglePrecision) {
						tr->Q[1][1] = 1.;
						tr->Q[2][1] = 0.;
						tr->Q[0][1] = 0.;
					}
					else {
						tr->Q[1][1] = tr->Q[1][0] - 1. / tr->Q[1][0];
						tr->Q[2][1] = tr->Q[2][0];
						tr->Q[0][1] = tr->Q[0][0];
						double norm = sqrt(tr->Q[1][1] * tr->Q[1][1] + tr->Q[2][1] * tr->Q[2][1] + tr->Q[0][1] * tr->Q[0][1]);
						tr->Q[1][1] /= norm;
						tr->Q[2][1] /= norm;
						tr->Q[0][1] /= norm;
					}
				}
				xPrime = Init(tr->Q[0][0], tr->Q[1][0], tr->Q[2][0]);
				yPrime = Init(tr->Q[0][1], tr->Q[1][1], tr->Q[2][1]);
				VECTOR zPrime = xPrime & yPrime;
				if (Norm(zPrime) <= anglePrecision) {
					printf("Error in specification of the coordinate transformation \"TR%d\": the unit vectors of the auxiliary coordinate system are collinear.\n", transformID); continue;
				}
				double dotProduct = xPrime * yPrime;
				bool adjustYprime = (fabs(dotProduct) > anglePrecision);
				if (adjustYprime) {
					printf("Warning: the unit vectors of the coordinate transformation \"TR%d\" are not orthogonal. The Y' unit vector will be rotated so that it is orthogonal to the X' unit vector.\n", transformID);
				}
				zPrime /= Norm(zPrime);
				tr->Q[0][2] = zPrime.x;
				tr->Q[1][2] = zPrime.y;
				tr->Q[2][2] = zPrime.z;
				if (adjustYprime) {
					yPrime = zPrime & xPrime;
					yPrime /= Norm(yPrime);
					tr->Q[0][1] = yPrime.x;
					tr->Q[1][1] = yPrime.y;
					tr->Q[2][1] = yPrime.z;
				}
			}
			if (tr0) {
				tr->origin = LocalToGlobal(tr0->origin, tr0->Q[0], tr->origin);
				MatrixProduct(tr->Q, tr0->Q);
			}
			nTransforms++;
		}
		else if (!strcmpi(ptr, "Surface")) {
			keywordNum = 87;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (complexGeometry > 0) {
				printf("Error: Keyword \"Surface\" can be used only in simple geometry mode, but the previous directives imply complex geometry.\n"); continue;
			}
			complexGeometry = 0;
			nSurfaces = 2;
			surf[0].type = 1;
			surf[0].ID = 1;
			ptr = NextToken(line, NULL, " \t\n");
			for(k = 0; k < 6 && ptr; k++) {
				if (!k) surf[0].init.x = strtod(ptr, &endPtr);
				else if (k == 1) surf[0].init.y = strtod(ptr, &endPtr);
				else if (k == 2) surf[0].init.z = strtod(ptr, &endPtr);
				else if (k == 3) surf[0].dir.x = strtod(ptr, &endPtr);
				else if (k == 4) surf[0].dir.y = strtod(ptr, &endPtr);
				else surf[0].dir.z = strtod(ptr, &endPtr);
				if (*endPtr != '\0') {
					printf("Error: Invalid number \"%s\" in specification of the front surface of the layer.\n", ptr); goto CONTINUE;
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			if (k < 6) {
				printf("Error: Number of entries in specification of the front surface of the layer is less than 6.\n"); continue;
			}
			if (!surf[0].dir.x && !surf[0].dir.y) {
				if (!surf[0].dir.z) {
					printf("Error in values of components of the normal vector of the front surface of the layer (at least one of them must be non-zero).\n"); continue;
				}
				if (surf[0].dir.z > 0.) surf[0].dir.z = 1.;
				else surf[0].dir.z = -1.;
			}
			else surf[0].dir /= Norm(surf[0].dir);
			surf[0].D = -surf[0].dir * surf[0].init;
		}
		else if (!strcmpi(ptr, "THICK")) {
			keywordNum = 66;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (complexGeometry > 0) {
				printf("Error: Keyword \"THICK\" can be used only in simple geometry mode, but the previous directives imply complex geometry.\n"); continue;
			}
			complexGeometry = 0;
			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Thickness value is missing.\n"); continue;
			}
			w = atof(ptr);
		}
		else if (!strcmpi(ptr, "BEAM")) {
			keywordNum = 88;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (sourcePositionSpecified || sourceDirectionSpecified) {
				printf("Error: Keyword \"BEAM\" cannot be used after specifying the initial position or the direction of the source particles.\n"); continue;
			}
			ptr = NextToken(line, NULL, " \t\n");
			for(k = 0; k < 6 && ptr; k++) {
				if (!k) beam_X = strtod(ptr, &endPtr);
				else if (k == 1) beam_Y = strtod(ptr, &endPtr);
				else if (k == 2) beam_Z = strtod(ptr, &endPtr);
				else if (k == 3) beam_A = strtod(ptr, &endPtr);
				else if (k == 4) beam_B = strtod(ptr, &endPtr);
				else beam_C = strtod(ptr, &endPtr);
				if (*endPtr != '\0') {
					printf("Error: Invalid number \"%s\" in specification of the incident beam.\n", ptr); goto CONTINUE;
				}
				ptr = NextToken(line, NULL, " \t\n");
			}
			if (k < 6) {
				printf("Error: Number of entries in specification of the incident beam is less than 6.\n"); continue;
			}
			if (!beam_A && !beam_B && !beam_C) {
				printf("Error in values of components of the direction vector of the incident beam (at least one of them must be non-zero).\n"); continue;
			}
			sourceCenter = Init(beam_X, beam_Y, beam_Z);
			double norm = sqrt(beam_A * beam_A + beam_B * beam_B + beam_C * beam_C);
			beam_A /= norm;
			beam_B /= norm;
			beam_C /= norm;
			sourcePositionType = 1;
			sourceDirectionType = 1;
			sourcePositionSpecified = true;
			sourceDirectionSpecified = true;
		}
		else if (!strcmpi(ptr, "SOURCE_POSITION")) {
			keywordNum = 89;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (!complexGeometry) {
				printf("Error: Keyword \"SOURCE_POSITION\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			if (sourcePositionSpecified) {
				printf("Error: Keyword \"SOURCE_POSITION\" cannot be used after specifying the position of the source.\n"); continue;
			}

			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of the source position type (it should be 1 for a point source, 2 for a rotationally symmetric distribution on a plane, or 3 for a uniform distribution inside an arbitrary cell with finite dimensions).\n"); continue;
			}
			sourcePositionType = strtol(ptr, &endPtr, 10);
			if (*endPtr != '\0' || sourcePositionType < 1 || sourcePositionType > 3) {
				printf("Error: Invalid specification of the source position type \"%s\" (it should be 1 for a point source, 2 for a rotationally symmetric distribution on a plane, or 3 for a uniform distribution inside an arbitrary cell with finite dimensions).\n", ptr); continue;
			}

			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of the source position.\n"); continue;
			}
			if (sourcePositionType < 3) { // the source should have a defined center
				for (k = 0; k < 3 && ptr; k++) {
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0') {
						printf("Error: \"%s\" is not a valid value of a source coordinate.\n", ptr); goto CONTINUE;
					}
					if (!k) sourceCenter.x = value;
					else if (k == 1) sourceCenter.y = value;
					else sourceCenter.z = value;
					ptr = NextToken(line, NULL, " \t\n");
				}
				if (k < 2) {
					printf("Error: At least one of the three source center coordinates has not been specified.\n", ptr); continue;
				}
				if (sourcePositionType == 2) {
					if (!ptr) {
						printf("Error: Missing specification of the source surface normal.\n"); continue;
					}
					for (k = 0; k < 3 && ptr; k++) {
						double value = strtod(ptr, &endPtr);
						if (*endPtr != '\0') {
							printf("Error: \"%s\" is not a valid value of a component of a source surface normal.\n", ptr); goto CONTINUE;
						}
						if (!k) sourceNormal.x = value;
						else if (k == 1) sourceNormal.y = value;
						else sourceNormal.z = value;
						ptr = NextToken(line, NULL, " \t\n");
					}
					if (k < 2) {
						printf("Error: At least one of the three components of the source surface normal has not been specified.\n", ptr); continue;
					}
					if (!sourceNormal.x && !sourceNormal.y && !sourceNormal.z) {
						printf("Error in values of components of the source surface normal vector (at least one of them must be non-zero).\n"); continue;
					}
					double norm = Norm(sourceNormal);
					sourceNormal.x /= norm;
					sourceNormal.y /= norm;
					sourceNormal.z /= norm;
					if (!ptr) {
						printf("Error: Missing value of the radius of the area with constant surface density of the source.\n"); continue;
					}
					sourceConstantDensityRadius = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || sourceConstantDensityRadius < 0.) {
						printf("Error: \"%s\" is not a valid value of the radius of the area with constant surface density of the source.\n", ptr); continue;
					}
					ptr = NextToken(line, NULL, " \t\n");
					if (!ptr) {
						printf("Error: Missing value of the source Gaussian RMS width.\n"); continue;
					}
					sourceRMSwidth = strtod(ptr, &endPtr);
					if (*endPtr != '\0' || sourceRMSwidth < 0.) {
						printf("Error: \"%s\" is not a valid value of the source Gaussian RMS width.\n", ptr); continue;
					}
					if (!sourceConstantDensityRadius && !sourceRMSwidth) sourcePositionType = 1;
				}
				if (sourcePositionType == 1) {
					beam_X = sourceCenter.x;
					beam_Y = sourceCenter.y;
					beam_Z = sourceCenter.z;
				}
			}
			else {
				if (!ptr) {
					printf("Error: Missing specification of the source cell.\n"); continue;
				}
				int sourceCellID = strtol(ptr, &endPtr, 10);
				if (*endPtr != '\0' || sourceCellID < 1) {
					printf("Error: \"%s\" is not a valid identifier of the source cell.\n", ptr); continue;
				}
				for (i = 0; i < nCells && cell[i].ID != sourceCellID; i++);
				if (i == nCells) {
					printf("Error: Undefined cell identifier \"%s\" in the definition of the source cell.\n", ptr); continue;
				}
				if (cell[i].unionNum >= 0 && i != unionData[cell[i].unionNum].parts[0]) {
					printf("Error: A cell that is a part of a union can be defined as a source cell only if it the first cell of the union.\n"); continue;
				}
				sourceCell = i;
			}
			sourcePositionSpecified = true;
		}
		else if (!strcmpi(ptr, "SOURCE_DIRECTION")) {
			keywordNum = 90;
			if (!iPass) { do { ptr = NextToken(line, NULL, " \t\n"); } while(ptr); continue; }
			if (!complexGeometry) {
				printf("Error: Keyword \"SOURCE_DIRECTION\" can be used only in complex geometry mode, but the previous directives imply simple geometry.\n"); continue;
			}
			complexGeometry = 1;
			if (sourceDirectionSpecified) {
				printf("Error: Keyword \"SOURCE_DIRECTION\" cannot be used after specifying the direction of the source particles.\n"); continue;
			}

			ptr = NextToken(line, NULL, " \t\n");
			if (!ptr) {
				printf("Error: Missing specification of the source direction type (it should be 1 for a parallel beam, 2 for a uniform distribution inside a cone, or 3 for the isotropic distribution).\n"); continue;
			}
			sourceDirectionType = strtol(ptr, &endPtr, 10);
			if (*endPtr != '\0' || sourceDirectionType < 1 || sourceDirectionType > 3) {
				printf("Error: Invalid specification of the source direction type \"%s\" (it should be 1 for a parallel beam, 2 for a uniform distribution inside a cone, or 3 for the isotropic distribution).\n", ptr); continue;
			}

			if (sourceDirectionType < 3) {
				ptr = NextToken(line, NULL, " \t\n");
				if (!ptr) {
					printf("Error: Missing specification of the source direction vector.\n"); continue;
				}
				for (k = 0; k < 3 && ptr; k++) {
					double value = strtod(ptr, &endPtr);
					if (*endPtr != '\0') {
						printf("Error: \"%s\" is not a valid value of a component of the source direction vector.\n", ptr); goto CONTINUE;
					}
					if (!k) beam_A = value;
					else if (k == 1) beam_B = value;
					else beam_C = value;
					ptr = NextToken(line, NULL, " \t\n");
				}
				if (k < 2) {
					printf("Error: At least one of three components of the source direction vector has not been specified.\n", ptr); continue;
				}
				if (!beam_A && !beam_B && !beam_C) {
					printf("Error in values of components of the direction vector of the incident beam (at least one of them must be non-zero).\n"); continue;
				}
				double norm = sqrt(beam_A * beam_A + beam_B * beam_B + beam_C * beam_C);
				beam_A /= norm;
				beam_B /= norm;
				beam_C /= norm;
				if (sourceDirectionType == 2) {
					if (!ptr) {
						printf("Error: Missing specification of the source cone angle.\n"); continue;
					}
					double sourceConeAngle = strtod(ptr, &endPtr) * pi / 180.;
					if (*endPtr != '\0' || sourceConeAngle <= 0. || sourceConeAngle >= pi) {
						printf("Error: \"%s\" is not a valid value of the source cone angle (it must be between 0 and 180 degrees).\n", ptr); continue;
					}
					sourceConeCos = cos(sourceConeAngle);
				}
			}
			sourceDirectionSpecified = true;
		}
		success = true;
CONTINUE:
		continue;
	}

	if (iPass && keywordNum >= 0 && keywordUsed[keywordNum]) {
		printf("Error: Keyword \"%s\" is used more than once in the input file.\n", strupr(keyword)); delete comp; comp = NULL; return;
	}

	}

	fclose(f_in);
	f_in = NULL;

	if (!complexGeometry && !nSurfaces) {
		nSurfaces = 2;
		surf[0].type = 1;
		surf[0].ID = 1;
		surf[0].init = Init(0., 0., 0.);
		surf[0].dir = Init(0., 0., 1.);
		surf[0].D = -surf[0].dir * surf[0].init;
	}
	if (!complexGeometry && nSurfaces == 2) {
		nCells = 1;
		cell[0].ID = 1;
		cell[0].mat = 0;
		cell[0].s[0] = 0;
		cell[0].side[0] = 1;
		if (w > 0.) {
			surf[1].type = 1;
			surf[1].ID = 2;
			surf[1].init = surf[0].init + w * surf[0].dir;
			surf[1].dir = surf[0].dir;
			surf[1].D = -surf[1].dir * surf[1].init;
			cell[0].nS = 2;
			cell[0].s[1] = 1;
			cell[0].side[1] = -1;
		}
		else {
			nSurfaces = 1;
			cell[0].nS = 1;
		}
		cell[0].mb = (short int *)malloc(nSurfaces * sizeof(short int));
		cell[0].i = (short int *)malloc(nSurfaces * sizeof(short int));
		for (i = 0; i < nSurfaces; i++) {
			cell[0].mb[i] = 0;
			cell[0].i[i] = 0;
		}
	}
	if (!nCells) {
		printf("Error: There are no cells defined.\n"); delete comp; comp = NULL; return;
	}
	if (!sourcePositionSpecified) {
		sourcePositionType = 1;
		sourceCenter = Init(0., 0., 0.);
	}
	if (!sourceDirectionSpecified) {
		sourceDirectionType = 1;
		beam_A = 0.;
		beam_B = 0.;
		beam_C = 1.;
	}
	char ext[1000];
	_splitpath(fn, NULL, NULL, fn1, ext);
	sprintf(tmp, MGEDsuffix, curParmSet + 1);
	if (scriptFolder[0] != '\0') sprintf(outScriptFN, "%s%s%s.rt", scriptFolder, fn1, tmp);
	else sprintf(outScriptFN, "%s%s%s.rt", currentFolder, fn1, tmp);
	m_outputScriptName.SetWindowText(ShortenPath(outScriptFN, currentFolder));
	fileLoaded = true;
	if (nSmax_cell > 1) edgeBuffer = new VECTOR[nSmax_cell * (nSmax_cell - 1) / 2];
	else if (nSmax_cell == 1) edgeBuffer = new VECTOR[1];

	delete comp; comp = NULL;

	consoleWndHandle = GetConsoleWindow();
	if (consoleWndHandle && IsWindow(consoleWndHandle)) {
		CONSOLE_SCREEN_BUFFER_INFO screenBufferInfo;
		GetConsoleScreenBufferInfo(stdHandle_out, &screenBufferInfo);
		DWORD length = screenBufferInfo.dwSize.X * screenBufferInfo.dwSize.Y;
		DWORD nChars = 0;
		COORD xy;
		xy.X = 0; xy.Y = 0;
		char *ch = new char[length];
		BOOL rc = ReadConsoleOutputCharacter(stdHandle_out, ch, length, xy, &nChars);
		DWORD count;
		for (count = 0; count < nChars && count < length && ch[count] == ' '; count++);
		delete ch;
		if (count == nChars || count == length) {
			FreeConsole();
			::ShowWindow(consoleWndHandle, SW_HIDE);
			::PostMessage(consoleWndHandle, WM_CLOSE, 0, 0);
		}
	}
}

void CMCNScriptDlg::GetFileName(CEdit *pEdit, char *title, char *filter, char *buffer)
{
	char fname0[1000], fileName[1000];
	int nBytes = pEdit->GetLine(0, fname0, 999);
	fname0[nBytes] = '\0';
	char *fname1 = strtok(fname0, "\"");
	if (fname1) {
		size_t nBlanks = strspn(fname1, " \t");
		if (fname1[nBlanks] != '\\' && !strchr(fname1 + nBlanks, ':')) strcpy(fileName, currentFolder);
		else fileName[0] = '\0';
		strcat(fileName, fname1 + nBlanks);
	}
	else strcpy(fileName, currentFolder);
	char *cmdLineArgs;
	if (fname1 && fname1 + strlen(fname1) != fname0 + nBytes) cmdLineArgs = fname1 + strlen(fname1) + 1;
	else cmdLineArgs = NULL;
	char drive[1000], dir[1000], fname[1000], ext[1000];
	char path[1000], fname2[1000];
	bool nameIsEmpty = (strspn(fileName, " \t") == strlen(fileName));
	if (!nameIsEmpty) {
		_splitpath(fileName, drive, dir, fname, ext);
		sprintf(path, "%s%s", drive, dir);
		sprintf(fname2, "%s%s", fname, ext);
		if (path[0] != '\0') {
			_finddata_t fileinfo;
			char path2[1000];
			strcpy(path2, path);
			int lastPos = (int)strlen(path2) - 1;
			if (path2[lastPos] == '\\') path2[lastPos] = '\0';
			intptr_t handle = _findfirst(path2, &fileinfo);
			if (handle < 0 || fileinfo.attrib != _A_SUBDIR) strcpy(path, currentFolder);
		}
		else strcpy(path, currentFolder);
	}
	else {
		strcpy(path, currentFolder);
		fname2[0] = '\0';
	}

//	CFileDialog dlg(TRUE, "*.txt", NULL, OFN_EXPLORER|OFN_ENABLESIZING, filter, this);
	CFileDialog dlg(TRUE, "*.txt", fname2, OFN_EXPLORER|OFN_ENABLESIZING, filter, this);

	if (!nameIsEmpty) dlg.m_ofn.lpstrInitialDir = path;
	else dlg.m_ofn.lpstrInitialDir = NULL;
	dlg.m_ofn.lpstrTitle = title;
	INT_PTR rc = dlg.DoModal();
	if (rc == IDOK) {
		CString path = dlg.GetPathName();
		CString path2 = ShortenPath(path, currentFolder);
		if (cmdLineArgs) pEdit->SetWindowText("\"" + path2 + "\"" + cmdLineArgs);
		else pEdit->SetWindowText(path2);
		strcpy(buffer, path.GetBuffer()); path.ReleaseBuffer();
		buffer[999] = '\0';
	}
	else buffer[0] = '\0';
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMCNScriptDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMCNScriptDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CMCNScriptDlg::OnGridClick(NMHDR *pNotifyStruct, LRESULT* /*pResult*/)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	if (pItem->iColumn != 9 || pItem->iRow < 1) return;
	CGridCellCheck *pCheck = (CGridCellCheck *)m_colorsGrid.GetCell(pItem->iRow, 9);
	pCheck->m_bChecked = !pCheck->m_bChecked;
	int i = pItem->iRow - 1;
	regions[i].show = (pCheck->m_bChecked != 0);
	m_colorsGrid.InvalidateRect(pCheck->m_Rect);
	UpdateShow(i);
	m_colorsGrid.Invalidate();
	m_colorsGrid.UpdateWindow();
	changed = true;
}

int CMCNScriptDlg::CheckInteger(char *inputString, char *outputString, int minValue, int maxValue)
{
	outputString[0] = '\0';
	if (!inputString) return 0;
	char *endPtr;
	int num = strtol(inputString, &endPtr, 10);
	if ((*endPtr == '\0' || *endPtr == ' ' || *endPtr == '\t') && num >= minValue && num <= maxValue) sprintf(outputString, "%d", num);
	return num;
}

double CMCNScriptDlg::CheckDouble(char *inputString, char *outputString, double minValue, double maxValue)
{
	outputString[0] = '\0';
	if (!inputString) return 0.;
	char *endPtr;
	double num = strtod(inputString, &endPtr);
	if ((*endPtr == '\0' || *endPtr == ' ' || *endPtr == '\t') && num >= minValue && num <= maxValue) sprintf(outputString, "%g", num);
	return num;
}

void CMCNScriptDlg::OnColorsGridStartEdit(NMHDR *pNotifyStruct, LRESULT* pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	*pResult = 0;
	if (pItem->iColumn < 3 || pItem->iColumn > 6) return;
	int j;
	int i = pItem->iRow - 1;
	for (j = 0; j < NSTDCOLORS - 1 && (regions[i].color.r != stdColor[j].r || regions[i].color.g != stdColor[j].g || regions[i].color.b != stdColor[j].b); j++);
	prevSel = j;
	if (j == NSTDCOLORS - 1) prevColor = RGB(regions[i].color.r, regions[i].color.g, regions[i].color.b);
	for (j = 0; j < NDEFAULTCOLORS && (regions[i].color.r != defaultColor[j].r || regions[i].color.g != defaultColor[j].g || regions[i].color.b != defaultColor[j].b); j++);
	prevDefaultColor = j;
}

void CMCNScriptDlg::OnColorsGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	if (pItem->iColumn == 3) {
		CGridCellCombo* pCell = (CGridCellCombo*)m_colorsGrid.GetCell(pItem->iRow, pItem->iColumn);
		int j = pCell->GetCurSel();
		if (j != prevSel || j == NSTDCOLORS - 1 && RGB(regions[pItem->iRow-1].color.r, regions[pItem->iRow-1].color.g, regions[pItem->iRow-1].color.b) != prevColor) {
			if (prevDefaultColor < NDEFAULTCOLORS) {
				defaultColorUsed[prevDefaultColor]--;
				if (!defaultColorUsed[prevDefaultColor]) nUsedDefaultColors--;
			}
			for (j = 0; j < NDEFAULTCOLORS && (regions[pItem->iRow-1].color.r != defaultColor[j].r || regions[pItem->iRow-1].color.g != defaultColor[j].g || regions[pItem->iRow-1].color.b != defaultColor[j].b); j++);
			if (j < NDEFAULTCOLORS) {
				if (!defaultColorUsed[j]) nUsedDefaultColors++;
				defaultColorUsed[j]++;
			}
			changed = true;
		}
		*pResult = 0;
		return;
	}
	if (pItem->iColumn == 9) {
		*pResult = -1;
		return;
	}
    CGridCellBase* pCell = m_colorsGrid.GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = m_colorsGrid.GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	int i = pItem->iRow - 1;
	char outputString[100];
	if (pItem->iColumn >= 4 && pItem->iColumn <= 6) {
		int num = CheckInteger(tmp, outputString, 0, 255);
		if (outputString[0] == '\0') {
			*pResult = -1;
			return;
		}
		int prev = *(&(regions[i].color.r) + (pItem->iColumn - 4));
		*(&(regions[i].color.r) + (pItem->iColumn - 4)) = num;
		if (num != prev) {
			if (prevDefaultColor < NDEFAULTCOLORS) {
				defaultColorUsed[prevDefaultColor]--;
				if (!defaultColorUsed[prevDefaultColor]) nUsedDefaultColors--;
			}
			int j;
			for (j = 0; j < NDEFAULTCOLORS && (regions[i].color.r != defaultColor[j].r || regions[i].color.g != defaultColor[j].g || regions[i].color.b != defaultColor[j].b); j++);
			if (j < NDEFAULTCOLORS) {
				if (!defaultColorUsed[j]) nUsedDefaultColors++;
				defaultColorUsed[j]++;
			}
			changed = true;
		}
		m_colorsGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
		UpdateRegionColor(i);
		m_colorsGrid.Invalidate();
		m_colorsGrid.UpdateWindow();
	}
	else if (pItem->iColumn == 7 || pItem->iColumn == 8){
		double num = CheckDouble(tmp, outputString, 0., 1.);
		if (outputString[0] == '\0') {
			*pResult = -1;
			return;
		}
		double prev = *(&(regions[i].transp) + (pItem->iColumn - 7));
		*(&(regions[i].transp) + (pItem->iColumn - 7)) = num;
		if (num != prev) changed = true;
		m_colorsGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
		m_colorsGrid.Invalidate();
		m_colorsGrid.UpdateWindow();
	}
	*pResult = 0;
}

bool CMCNScriptDlg::TrackFilterGridEndEdit(TRACKFILTERDATA *pTrackFilter, CGridCtrlModified *pGrid, int iRow, int iColumn, char *tmp, LRESULT *pResult)
{
	char outputString[100];
	bool filterChanged = false;
	if (iRow >= 1 && iRow <= 3){
		int num;
		if (iColumn == 2 && strspn(tmp, " \t") == strlen(tmp)) num = -1;
		else {
			if (iColumn == 1) {
				if (iRow < 3) num = CheckInteger(tmp, outputString, 1, 2147483647);
				else num = CheckInteger(tmp, outputString, 0, 2147483647);
			}
			else num = CheckInteger(tmp, outputString, -2147483647, 2147483647);
			if (outputString[0] == '\0') {
				*pResult = -1;
				return filterChanged;
			}
			if (iRow < 3) num--;
			if (num < 0) num = -1;
			if (iRow == 3) num++;
		}
		int prev;
		if (iRow == 1) prev = pTrackFilter->historyNo[iColumn-1];
		else if (iRow == 2) prev = pTrackFilter->trackNo[iColumn-1];
		else prev = pTrackFilter->nVert[iColumn-1];
		if (iColumn == 2 && (iRow < 3 && num < 0 || iRow == 3 && num < 1)) {
			if (iRow == 1) pTrackFilter->historyNo[1] = num;
			else if (iRow == 2) pTrackFilter->trackNo[1] = num;
			else pTrackFilter->nVert[1] = num;
			pGrid->SetItemText(iRow, iColumn, "");
		}
		else {
			if (iRow == 1) pTrackFilter->historyNo[iColumn-1] = num;
			else if (iRow == 2) pTrackFilter->trackNo[iColumn-1] = num;
			else pTrackFilter->nVert[iColumn-1] = num;
			pGrid->SetItemText(iRow, iColumn, outputString);
		}
		if (num != prev) filterChanged = true;
	}
	else {
		double num;
		if (iColumn == 2 && strspn(tmp, " \t") == strlen(tmp)) {
			if (iRow != 7) num = -1.;
			else num = 1.;
		}
		else {
			if (iRow == 7) num = 0.01 * CheckDouble(tmp, outputString, 0., 100.);
			else if (iColumn == 1) num = CheckDouble(tmp, outputString, 0., 1e300);
			else num = CheckDouble(tmp, outputString, -1., 1e300);
			if (outputString[0] == '\0') {
				*pResult = -1;
				return filterChanged;
			}
		}
		double prev;
		if (iRow == 4) prev = pTrackFilter->Estart[iColumn-1];
		else if (iRow == 5) prev = pTrackFilter->Efinal[iColumn-1];
		else if (iRow == 6) prev = pTrackFilter->Eloss[iColumn-1];
		else if (iRow == 7) prev = pTrackFilter->relEloss[iColumn-1];
		else if (iRow == 8) prev = pTrackFilter->displ[iColumn-1];
		else if (iRow == 9) prev = pTrackFilter->peakDispl[iColumn-1];
		else if (iRow == 10) prev = pTrackFilter->path[iColumn-1];
		if (iColumn == 2 && (iRow != 7 && num < 0. || iRow == 7 && num == 1.)) {
			if (num < 0.) num = -1.;
			if (iRow == 4) pTrackFilter->Estart[1] = -1.;
			else if (iRow == 5) pTrackFilter->Efinal[1] = -1.;
			else if (iRow == 6) pTrackFilter->Eloss[1] = -1.;
			else if (iRow == 7) pTrackFilter->relEloss[1] = 1.;
			else if (iRow == 8) pTrackFilter->displ[1] = -1.;
			else if (iRow == 9) pTrackFilter->peakDispl[1] = -1.;
			else if (iRow == 10) pTrackFilter->path[1] = -1.;
			if (iRow != 7) pGrid->SetItemText(iRow, iColumn, "");
			else pGrid->SetItemText(iRow, iColumn, "100");
		}
		else {
			if (iRow < 7) num /= energyFactor[unitEnergy];
			else if (iRow > 7) num /= distanceFactor[unitDistance];
			if (iRow == 4) pTrackFilter->Estart[iColumn-1] = num;
			else if (iRow == 5) pTrackFilter->Efinal[iColumn-1] = num;
			else if (iRow == 6) pTrackFilter->Eloss[iColumn-1] = num;
			else if (iRow == 7) pTrackFilter->relEloss[iColumn-1] = num;
			else if (iRow == 8) pTrackFilter->displ[iColumn-1] = num;
			else if (iRow == 9) pTrackFilter->peakDispl[iColumn-1] = num;
			else if (iRow == 10) pTrackFilter->path[iColumn-1] = num;
			pGrid->SetItemText(iRow, iColumn, outputString);
		}
		if (num != prev) filterChanged = true;
	}
	pGrid->Invalidate();
	pGrid->UpdateWindow();
	return filterChanged;
}

void CMCNScriptDlg::OnViewCenterGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    CGridCellBase* pCell = m_viewCenterGrid.GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = m_viewCenterGrid.GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	int i = pItem->iRow - 1;
	char outputString[100];
	double num = CheckDouble(tmp, outputString, -1e300, 1e300);
	if (outputString[0] == '\0') {
		*pResult = -1;
		return;
	}
	double prev = *((double *)&viewCenter + pItem->iColumn);
	*((double *)&viewCenter + pItem->iColumn) = num;
	if (num != prev) changed = true;
	m_viewCenterGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
	m_viewCenterGrid.Invalidate();
	m_viewCenterGrid.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnCutawayPlanePointGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    CGridCellBase* pCell = m_cutawayPlanePointGrid.GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = m_cutawayPlanePointGrid.GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	int i = pItem->iRow - 1;
	char outputString[100];
	double num = CheckDouble(tmp, outputString, -1e300, 1e300);
	if (outputString[0] == '\0') {
		*pResult = -1;
		return;
	}
	double prev = *((double *)&cutawayPlanePoint + pItem->iColumn);
	*((double *)&cutawayPlanePoint + pItem->iColumn) = num;
	if (num != prev) changed = true;
	CutawayPlaneCoefs();
	m_cutawayPlanePointGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
	m_cutawayPlanePointGrid.Invalidate();
	m_cutawayPlanePointGrid.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnCutawayPlaneNormalGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
    CGridCellBase* pCell = m_cutawayPlaneNormalGrid.GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = m_cutawayPlaneNormalGrid.GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	int i = pItem->iRow - 1;
	char outputString[100];
	double num = CheckDouble(tmp, outputString, -1e300, 1e300);
	if (outputString[0] == '\0') {
		*pResult = -1;
		return;
	}
	VECTOR bak = cutawayPlaneNormal;
	double prev = *((double *)&cutawayPlaneNormal + pItem->iColumn);
	*((double *)&cutawayPlaneNormal + pItem->iColumn) = num;
	if (Norm(cutawayPlaneNormal) <= 0.) {
		cutawayPlaneNormal = bak;
		*pResult = -1;
		return;
	}
	if (num != prev) changed = true;
	CutawayPlaneCoefs();
	m_cutawayPlaneNormalGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
	m_cutawayPlaneNormalGrid.Invalidate();
	m_cutawayPlaneNormalGrid.UpdateWindow();
	*pResult = 0;
}

void CMCNScriptDlg::OnUseCutaway()
{
	if (m_cutawayPlaneCheck.GetCheck()) useCutawayPlane = TRUE;
	else useCutawayPlane = FALSE;
	UpdateCutawayGroup();
	changed = true;
}

void CMCNScriptDlg::CutawayPlaneCoefs()
{
	*((VECTOR *)cutawayPlaneCoefs) = cutawayPlaneNormal / Norm(cutawayPlaneNormal);
	cutawayPlaneCoefs[3] = *((VECTOR *)cutawayPlaneCoefs) * cutawayPlanePoint;
}

void CMCNScriptDlg::UpdateCutawayGroup()
{
    m_cutawayPlanePointGrid.SetSelectedRange(-1,-1,-1,-1);
	m_cutawayPlanePointGrid.SetFocusCell(-1,-1);
    m_cutawayPlaneNormalGrid.SetSelectedRange(-1,-1,-1,-1);
	m_cutawayPlaneNormalGrid.SetFocusCell(-1,-1);
	int i;
	if (useCutawayPlane) {
		m_cutawayPlaneCheck.SetCheck(1);
		m_cutawayPlanePointGrid.EnableWindow(TRUE);
		m_cutawayPlaneNormalGrid.EnableWindow(TRUE);
		for (i = 0; i < 3; i++) m_cutawayPlanePointGrid.SetItemBkColour(1, i, RGB(255, 255, 255));
		for (i = 0; i < 3; i++) m_cutawayPlaneNormalGrid.SetItemBkColour(1, i, RGB(255, 255, 255));
	}
	else {
		m_cutawayPlaneCheck.SetCheck(0);
		m_cutawayPlanePointGrid.EnableWindow(FALSE);
		m_cutawayPlaneNormalGrid.EnableWindow(FALSE);
		for (i = 0; i < 3; i++) m_cutawayPlanePointGrid.SetItemBkColour(1, i, RGB(224, 224, 224));
		for (i = 0; i < 3; i++) m_cutawayPlaneNormalGrid.SetItemBkColour(1, i, RGB(224, 224, 224));
	}
	m_cutawayPlanePointGrid.Invalidate();
	m_cutawayPlaneNormalGrid.Invalidate();
	m_cutawayPlanePointGrid.UpdateWindow();
	m_cutawayPlaneNormalGrid.UpdateWindow();
	CutawayPlaneCoefs();
}

void CMCNScriptDlg::OnRaytraceInGraphicsWindow()
{
	if (m_raytraceInGraphicsWindowCheck.GetCheck()) raytraceInGraphicsWindow = true;
	else raytraceInGraphicsWindow = false;
	changed = true;
}

void CMCNScriptDlg::OnShowWireframe()
{
	if (m_showWireframeCheck.GetCheck()) showWireframe = true;
	else showWireframe = false;
	changed = true;
}

void CMCNScriptDlg::OnRaytraceInLingeringWindow()
{
	if (m_raytraceInLingeringWindowCheck.GetCheck()) {
		raytraceInLingeringWindow = true;
		m_portNumEdit.EnableWindow();
	}
	else {
		raytraceInLingeringWindow = false;
		m_portNumEdit.EnableWindow(FALSE);
	}
	changed = true;
}

void CMCNScriptDlg::OnShowTracks()
{
	if (m_showTracksCheck.GetCheck()) {
		showTracks = true;
		m_nVerticesTotalMaxEdit.EnableWindow();
		m_nVerticesTrackMaxEdit.EnableWindow();
		m_showTrackSegmentsCheck.EnableWindow();
		m_extendEscapeTracksCheck.EnableWindow();
	}
	else {
		showTracks = false;
		m_nVerticesTotalMaxEdit.EnableWindow(FALSE);
		m_nVerticesTrackMaxEdit.EnableWindow(FALSE);
		m_showTrackSegmentsCheck.EnableWindow(FALSE);
		m_extendEscapeTracksCheck.EnableWindow(FALSE);
	}
	changed = true;
}

void CMCNScriptDlg::OnKeepTrackDataInMemory()
{
	if (m_keepTrackDataInMemoryCheck.GetCheck()) keepTrackDataInMemory = true;
	else {
		keepTrackDataInMemory = false;
		if (trackSets[curParmSet].nTracks) {
			int i;
			for (i = 0; i < trackSets[curParmSet].nTracks; i++) {
				delete trackSets[curParmSet].trackData[i].coord;
				delete trackSets[curParmSet].trackData[i].energy;
			}
			free(trackSets[curParmSet].trackData);
			trackSets[curParmSet].nTracks = 0;
		}
	}
	changed = true;
}

void CMCNScriptDlg::OnOverwritePrompt()
{
	if (m_overwritePromptCheck.GetCheck()) overwritePrompt = true;
	else overwritePrompt = false;
}

void CMCNScriptDlg::UpdateRegionColor(int i)
{
	int j;
	CGridCellCombo* pCell = (CGridCellCombo*)m_colorsGrid.GetCell(i + 1, 3);
	for (j = 0; j < NSTDCOLORS - 1 && (regions[i].color.r != stdColor[j].r || regions[i].color.g != stdColor[j].g || regions[i].color.b != stdColor[j].b); j++);
	pCell->SetCurSel(j);
	UpdateCellColor(i, j);
	m_colorsGrid.SetItemText(i + 1, 3, stdColorName[regions[i].color.index]);
}

void CMCNScriptDlg::UpdateTrackColor(int i)
{
	int j;
	CGridCellCombo* pCell = (CGridCellCombo*)pTrackColorsGrid->GetCell(i + 1, 4);
	for (j = 0; j < NSTDCOLORS && (trackColor[i].r != stdColor[j].r || trackColor[i].g != stdColor[j].g || trackColor[i].b != stdColor[j].b); j++);
	pCell->SetCurSel(j);
	UpdateCellColor_tracks(i, j);
	pTrackColorsGrid->SetItemText(i + 1, 4, stdColorName[trackColor[i].index]);
}

void CMCNScriptDlg::UpdateCellColor(int i, int j)
{
	if (j < NSTDCOLORS - 1) regions[i].color = stdColor[j];
	else regions[i].color.index = NSTDCOLORS;
	m_colorsGrid.SetItemBkColour(i + 1, 2, RGB(regions[i].color.r, regions[i].color.g, regions[i].color.b));
}

void CMCNScriptDlg::UpdateCellColor_tracks(int i, int j)
{
	if (j < NSTDCOLORS) trackColor[i] = stdColor[j];
	else trackColor[i].index = NSTDCOLORS;
	pTrackColorsGrid->SetItemBkColour(i + 1, 3, RGB(trackColor[i].r, trackColor[i].g, trackColor[i].b));
}

void CMCNScriptDlg::UpdateCellRGB(int i)
{
	char tmp[10];
	sprintf(tmp, "%d", regions[i].color.r); m_colorsGrid.SetItemText(i + 1, 4, tmp);
	sprintf(tmp, "%d", regions[i].color.g); m_colorsGrid.SetItemText(i + 1, 5, tmp);
	sprintf(tmp, "%d", regions[i].color.b); m_colorsGrid.SetItemText(i + 1, 6, tmp);
}

void CMCNScriptDlg::UpdateCellRGB_tracks(int i)
{
	char tmp[10];
	sprintf(tmp, "%d", trackColor[i].r); pTrackColorsGrid->SetItemText(i + 1, 5, tmp);
	sprintf(tmp, "%d", trackColor[i].g); pTrackColorsGrid->SetItemText(i + 1, 6, tmp);
	sprintf(tmp, "%d", trackColor[i].b); pTrackColorsGrid->SetItemText(i + 1, 7, tmp);
}

void CMCNScriptDlg::UpdateBeamAxisColor()
{
	int j;
	DWORD rgb = GetSysColor(COLOR_3DFACE);
	if (beamAxisColor.r == GetRValue(rgb) && beamAxisColor.g == GetGValue(rgb) && beamAxisColor.b == GetBValue(rgb)) j = NSTDCOLORS + 1;
	else {
		for (j = 0; j < NSTDCOLORS && (beamAxisColor.r != stdColor[j].r || beamAxisColor.g != stdColor[j].g || beamAxisColor.b != stdColor[j].b); j++);
	}
	m_beamAxisColor.SetCurSel(j);
	if (j < NSTDCOLORS) beamAxisColor = stdColor[j];
	else beamAxisColor.index = j;
	m_beamAxisColorPic.color = RGB(beamAxisColor.r, beamAxisColor.g, beamAxisColor.b);
	m_beamAxisColorPic.sel = j;
	m_beamAxisColorPic.Invalidate();
	m_beamAxisColorPic.UpdateWindow();
}

void CMCNScriptDlg::UpdateBackgroundColor()
{
	int j;
	for (j = 0; j < NSTDCOLORS && (backgroundColor.r != stdColor[j].r || backgroundColor.g != stdColor[j].g || backgroundColor.b != stdColor[j].b); j++);
	m_backgroundColor.SetCurSel(j);
	if (j < NSTDCOLORS) backgroundColor = stdColor[j];
	else backgroundColor.index = NSTDCOLORS;
	m_backgroundColorPic.color = RGB(backgroundColor.r, backgroundColor.g, backgroundColor.b);
	m_backgroundColorPic.sel = j;
	m_backgroundColorPic.Invalidate();
	m_backgroundColorPic.UpdateWindow();
}

void CMCNScriptDlg::UpdateShow(int i)
{
	int row = i + 1;
	BOOL isEditable = (regions[i].show != 0);
	int j;
	if (isEditable) {
		m_colorsGrid.SetItemBkColour(row, 1, RGB(255, 255, 255));
		m_colorsGrid.SetItemBkColour(row, 2, RGB(regions[i].color.r, regions[i].color.g, regions[i].color.b));
		for (j = 3; j < 9; j++) {
			m_colorsGrid.SetItemState(row, j, m_colorsGrid.GetItemState(row, j) & ~GVIS_READONLY);
			m_colorsGrid.SetItemBkColour(row, j, RGB(255, 255, 255));
		}
	}
	else {
		COLORREF lightGrey = RGB(224, 224, 224);
		m_colorsGrid.SetItemBkColour(row, 1, lightGrey);
		m_colorsGrid.SetItemBkColour(row, 2, lightGrey);
		for (j = 3; j < 9; j++) {
			m_colorsGrid.SetItemState(row, j, m_colorsGrid.GetItemState(row, j) | GVIS_READONLY);
			m_colorsGrid.SetItemBkColour(row, j, lightGrey);
		}
	}
}

void CMCNScriptDlg::UpdateShow_tracks(BOOL check, int i)
{
	int row = i + 1;
	int j;
	if (check) {
		pTrackColorsGrid->SetItemBkColour(row, 1, RGB(255, 255, 255));
		pTrackColorsGrid->SetItemBkColour(row, 3, RGB(trackColor[i].r, trackColor[i].g, trackColor[i].b));
		for (j = 4; j < 8; j++) {
			pTrackColorsGrid->SetItemState(row, j, pTrackColorsGrid->GetItemState(row, j) & ~GVIS_READONLY);
			pTrackColorsGrid->SetItemBkColour(row, j, RGB(255, 255, 255));
		}
	}
	else {
		COLORREF lightGrey = RGB(224, 224, 224);
		pTrackColorsGrid->SetItemBkColour(row, 1, lightGrey);
		pTrackColorsGrid->SetItemBkColour(row, 3, lightGrey);
		for (j = 4; j < 8; j++) {
			pTrackColorsGrid->SetItemState(row, j, m_colorsGrid.GetItemState(row, j) | GVIS_READONLY);
			pTrackColorsGrid->SetItemBkColour(row, j, lightGrey);
		}
	}
}

COLORREF CMCNScriptDlg::GetRegionColor(int i)
{
	return RGB(regions[i].color.r, regions[i].color.g, regions[i].color.b);
}

void CMCNScriptDlg::SetRegionColor(int i, COLORREF color)
{
	regions[i].color.r = GetRValue(color);
	regions[i].color.g = GetGValue(color);
	regions[i].color.b = GetBValue(color);
}

COLORREF CMCNScriptDlg::GetTrackColor(int i)
{
	return RGB(trackColor[i].r, trackColor[i].g, trackColor[i].b);
}

COLORREF CMCNScriptDlg::GetBeamAxisColor()
{
	return RGB(beamAxisColor.r, beamAxisColor.g, beamAxisColor.b);
}

COLORREF CMCNScriptDlg::GetBackgroundColor()
{
	return RGB(backgroundColor.r, backgroundColor.g, backgroundColor.b);
}

void CMCNScriptDlg::SetTrackColor(int i, COLORREF color)
{
	trackColor[i].r = GetRValue(color);
	trackColor[i].g = GetGValue(color);
	trackColor[i].b = GetBValue(color);
}

void CMCNScriptDlg::OnCancel()
{
	m_MGEDcmdLineCapt.SetFocus();
	if (changed) {
		int rc = MessageBox("Do you want to save the changes?", NULL, MB_YESNOCANCEL | MB_ICONQUESTION);
		if (rc == IDCANCEL) return;
		if (rc == IDYES && !OnSave()) return;
	}
	CDialogEx::OnCancel();
}

CNewParmSetDlg::CNewParmSetDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CNewParmSetDlg::IDD, pParent)
{
	parent = (CMCNScriptDlg *)pParent;
}

BOOL CNewParmSetDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	VERIFY(m_title.SubclassDlgItem(IDC_TITLE, this));
	VERIFY(m_insertAfter.SubclassDlgItem(IDC_INSERTAFTER, this));
	if (strspn(parent->parmSetName, " \t") != strlen(parent->parmSetName)) m_title.SetWindowText(parent->parmSetName);
	else m_title.SetWindowText("");
	char tmp[PARMSETNAMELEN+15];
	m_insertAfter.AddString("");
	int i;
	for (i = 0; i < parent->nViewParmSets; i++) {
		sprintf(tmp, " (%d)  %s", i + 1, parent->viewParmSets[i].parmSetName);
		m_insertAfter.AddString(tmp);
	}
	m_insertAfter.SetCurSel(parent->curParmSet + 1);
	m_title.SetFocus();
	m_title.SetSel(0, -1);
	return FALSE;  // return TRUE  unless you set the focus to a control
}

void CNewParmSetDlg::OnOK()
{
	char title_new[PARMSETNAMELEN+1];
	char tmp[100];
	int nBytes = m_title.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	strcpy(title_new, tmp + strspn(tmp, " \t"));
	parent->changed = true;
	strcpy(parent->parmSetName, title_new);
	parent->curParmSet = m_insertAfter.GetCurSel();
	EndDialog(IDOK);
}

CEditNameDlg::CEditNameDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CEditNameDlg::IDD, pParent)
{
	parent = (CMCNScriptDlg *)pParent;
}

BOOL CEditNameDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	VERIFY(m_title.SubclassDlgItem(IDC_TITLE, this));
	char tmp[100];
	sprintf(tmp, "Edit name of parameter set No. %d", parent->curParmSet + 1);
	SetWindowText(tmp);
	m_title.SetWindowText(parent->parmSetName);
	m_title.SetFocus();
	m_title.SetSel(0, -1);
	return FALSE;  // return TRUE  unless you set the focus to a control
}

void CEditNameDlg::OnOK()
{
	char title_new[PARMSETNAMELEN+1];
	char tmp[PARMSETNAMELEN+1];
	int nBytes = m_title.GetLine(0, tmp, PARMSETNAMELEN);
	tmp[nBytes] = '\0';
	strcpy(title_new, tmp + strspn(tmp, " \t"));
	if (strcmp(title_new, parent->parmSetName)) {
		parent->changed = true;
		strcpy(parent->parmSetName, title_new);
		EndDialog(IDOK);
	}
	else EndDialog(IDCANCEL);
}

void CMCNScriptDlg::OnOpenInputFile()
{
	CString docName('\0', 1000);
	int nBytes = m_inputFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnOpenOutputFile()
{
	CString docName('\0', 1000);
	int nBytes = m_outputFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnOpenLogFile()
{
	CString docName('\0', 1000);
	int nBytes = m_logFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnOpenTrackFile()
{
	CString docName('\0', 1000);
	int nBytes = m_trackFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnOpenRTscript()
{
	CString docName('\0', 1000);
	int nBytes = m_outputScriptName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnHelpMCNelectronUserManual()
{
	CString docName = (CString)exePath + "Docs\\MCNelectron - User Manual.pdf";
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"File \"" + docName + "\" could not be opened.", NULL, MB_OK | MB_ICONERROR);
}

void CMCNScriptDlg::OnReload()
{
	if (changed) {
		int rc = MessageBox("Do you want to discard all unsaved changes?", NULL, MB_YESNOCANCEL | MB_ICONQUESTION | MB_DEFBUTTON2);
		if (rc != IDYES) return;
	}
	FreeAll();
	OnOpen();
}

void CMCNScriptDlg::OnConsoleActivate()
{
	HWND consoleWndHandle = GetConsoleWindow();
	if (!consoleWndHandle || !IsWindow(consoleWndHandle)) return;
	CWnd *consoleWnd = CWnd::FromHandle(consoleWndHandle);
	consoleWnd->ShowWindow(SW_SHOWNORMAL);
	consoleWnd->SetForegroundWindow();
}

void CMCNScriptDlg::OnConsoleClose()
{
	HWND consoleWndHandle = GetConsoleWindow();
	if (!consoleWndHandle || !IsWindow(consoleWndHandle)) return;
	if (comp) {
		MessageBox("The console window cannot be closed while the input file is being processed.", NULL, MB_OK | MB_ICONINFORMATION);
		return;
	}
	FreeConsole();
	::ShowWindow(consoleWndHandle, SW_HIDE);
	::PostMessage(consoleWndHandle, WM_CLOSE, 0, 0);
}

BOOL CMCNScriptDlg::DestroyWindow()
{
	CMCNScriptApp *app = (CMCNScriptApp *)AfxGetApp();
	app->GetRecentFileList()->WriteList();
	FreeAll();
	delete surf;
	delete cell;
	delete unionData;
	delete materialID;
	delete materialName;
	return CDialogEx::DestroyWindow();
}

BEGIN_MESSAGE_MAP(CCmdLineDlg, CDialogEx)
	ON_BN_CLICKED(IDC_BROWSE_EXE, OnBrowseExe)
	ON_EN_KILLFOCUS(IDC_EXENAME, OnExeName)
	ON_EN_KILLFOCUS(IDC_NTHREADS, OnNthreads)
	ON_NOTIFY(UDN_DELTAPOS, IDC_NTHREADSSPIN, OnNthreadsSpin)
	ON_BN_CLICKED(IDC_BROWSE_INPUT, OnBrowseInput)
	ON_EN_KILLFOCUS(IDC_INPUTFILENAME, OnInputFileName)
	ON_EN_KILLFOCUS(IDC_SUFFIX, OnSuffix)
	ON_BN_CLICKED(IDC_SYSVARS, OnSystemVars)
	ON_BN_CLICKED(IDC_BROWSE_OUTPUT, OnBrowseOutput)
	ON_EN_KILLFOCUS(IDC_OUTPUTFILENAME, OnOutputFileName)
	ON_EN_KILLFOCUS(IDC_LOG_SUFFIX, OnLogSuffix)
	ON_BN_CLICKED(IDC_BROWSE_LOG, OnBrowseLog)
	ON_EN_KILLFOCUS(IDC_LOGFILENAME, OnLogFileName)
	ON_BN_CLICKED(IDC_BROWSE_ALT, OnBrowseAlt)
	ON_EN_KILLFOCUS(IDC_ALTCSINFOFN, OnAltCSinfoFileName)
	ON_BN_CLICKED(IDC_SAMEASINPUTFILE, OnSameAsInput)
	ON_BN_CLICKED(IDC_SAMEASMCNELECTRON, OnSameAsMCNelectron)
	ON_BN_CLICKED(IDC_SAMEASMCNSCRIPT, OnSameAsMCNScript)
	ON_BN_CLICKED(IDC_CUSTOM, OnCustom)
	ON_BN_CLICKED(IDC_SELECTFOLDER, OnSelectFolder)
	ON_BN_CLICKED(IDC_BROWSE_ENDF, OnBrowseENDF)
	ON_EN_KILLFOCUS(IDC_ENDFFOLDER, OnENDFfolder)
	ON_BN_CLICKED(IDC_OPENINPUTFILE, OnOpenInputFile)
	ON_BN_CLICKED(IDC_OPENOUTPUTFILE, OnOpenOutputFile)
	ON_BN_CLICKED(IDC_OPENLOGFILE, OnOpenLogFile)
	ON_BN_CLICKED(IDC_OPENALTCSINFOFILE, OnOpenAltCSinfoFile)
END_MESSAGE_MAP()

CCmdLineDlg::CCmdLineDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CCmdLineDlg::IDD, pParent)
{
	parent = (CMCNScriptDlg *)pParent;
	changed = false;
	strcpy(fn1, parent->fn1);
	cmdFolderOption = parent->cmdFolderOption;
	nThreads = parent->nThreads;
	checkSyntax_MCNelectron = parent->checkSyntax_MCNelectron;
	quitAfterChecking = parent->quitAfterChecking;
	MCNP_style_cmdLine = parent->MCNP_style_cmdLine;
	createLog = parent->createLog;
	strcpy(MCNelectronFolder, parent->MCNelectronFolder);
	strcpy(inputFileFolder, parent->inputFileFolder);
	strcpy(outputFileName, parent->outputFileName);
	strcpy(logFileName, parent->logFileName);
	strcpy(cmdFolder, parent->cmdFolder);
	strcpy(MCNelectronExeName, parent->MCNelectronExeName);
	strcpy(fn, parent->fn);
	strcpy(suffix, parent->suffix);
	strcpy(logSuffix, parent->logSuffix);
	strcpy(altCSinfoFileName, parent->altCSinfoFileName);
	strcpy(ENDFfolder, parent->ENDFfolder);
	strcpy(additionalArgs, parent->additionalArgs);
}

BOOL CCmdLineDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	VERIFY(m_sameAsInput.SubclassDlgItem(IDC_SAMEASINPUTFILE, this));
	VERIFY(m_sameAsMCNelectron.SubclassDlgItem(IDC_SAMEASMCNELECTRON, this));
	VERIFY(m_sameAsMCNScript.SubclassDlgItem(IDC_SAMEASMCNSCRIPT, this));
	VERIFY(m_checkSyntax_MCNelectron.SubclassDlgItem(IDC_CHECK_SYNTAX, this));
	VERIFY(m_quitAfterChecking.SubclassDlgItem(IDC_QUIT_AFTER_CHECKING, this));
	VERIFY(m_MCNP_style.SubclassDlgItem(IDC_MCNP_STYLE, this));
	VERIFY(m_createLog.SubclassDlgItem(IDC_CREATELOG, this));
	VERIFY(m_custom.SubclassDlgItem(IDC_CUSTOM, this));
	VERIFY(m_selectFolder.SubclassDlgItem(IDC_SELECTFOLDER, this));
	VERIFY(m_browseOutput.SubclassDlgItem(IDC_BROWSE_OUTPUT, this));
	VERIFY(m_browseLog.SubclassDlgItem(IDC_BROWSE_LOG, this));
	VERIFY(m_exeName.SubclassDlgItem(IDC_EXENAME, this));
	VERIFY(m_nThreads.SubclassDlgItem(IDC_NTHREADS, this));
	VERIFY(m_cmdFolder.SubclassDlgItem(IDC_CMDFOLDER, this));
	m_cmdFolder.disable = TRUE;
	VERIFY(m_inputFileName.SubclassDlgItem(IDC_INPUTFILENAME, this));
	VERIFY(m_suffix.SubclassDlgItem(IDC_SUFFIX, this));
	VERIFY(m_outputFileName.SubclassDlgItem(IDC_OUTPUTFILENAME, this));
	VERIFY(m_logSuffix.SubclassDlgItem(IDC_LOG_SUFFIX, this));
	VERIFY(m_logFileName.SubclassDlgItem(IDC_LOGFILENAME, this));
	VERIFY(m_altCSinfoFileName.SubclassDlgItem(IDC_ALTCSINFOFN, this));
	VERIFY(m_ENDFfolder.SubclassDlgItem(IDC_ENDFFOLDER, this));
	VERIFY(m_browseENDF.SubclassDlgItem(IDC_BROWSE_ENDF, this));
	VERIFY(m_additionalArgs.SubclassDlgItem(IDC_ADDITIONALARGS, this));
	m_exeName.SetWindowText(ShortenPath((CString)MCNelectronFolder + MCNelectronExeName, currentFolder));
	if (m_toolTip.Create(this)) {
		m_toolTip.AddTool(&m_nThreads, _T("If this number is -1, then the number of CPU threads will not be specified on the command line (unless it is included in the additional arguments)"));
		m_toolTip.AddTool(&m_ENDFfolder, _T("If this box is empty, then the ENDF folder will not be specified on the command line (unless it is included in the additional arguments). Enter \".\\\" to indicate the MCNScript installation folder."));
		m_toolTip.AddTool(&m_inputFileName, _T("The name of the MCNelectron input file"));
		m_toolTip.AddTool(&m_suffix, _T("The file name suffix cannot have spaces. The two-character string '%d', if present, will be replaced by the sequence number of the parameter set. Clear this box to enable customizing the MCNelectron output file name."));
		m_toolTip.AddTool(&m_logSuffix, _T("The file name suffix cannot have spaces. The two-character string '%d', if present, will be replaced by the sequence number of the parameter set. Clear this box to enable customizing the MCNelectron log file name."));
		m_toolTip.AddTool(&m_outputFileName, _T("The name of the MCNelectron output file"));
		m_toolTip.AddTool(&m_logFileName, _T("The name of the MCNelectron log file with all input directives (including the ones generated by executing the user code embedded in the MCNelectron input file)"));
		m_toolTip.AddTool(&m_checkSyntax_MCNelectron, _T("If this checkbox is checked or indeterminate, then the input directives specified in the MCNelectron input file will be checked for syntax and geometry errors. Otherwise, MCNelectron will quit after running the user programs embedded in the input file."));
		m_toolTip.AddTool(&m_quitAfterChecking, _T("If this checkbox is checked, MCNelectron will quit after checking the simulation setup for syntax and geometry errors."));
		m_toolTip.AddTool(&m_createLog, _T("If this checkbox is checked, MCNelectron will create the log file with the complete set of input directives (including the procedurally generated ones). The log file is functionally equivalent to the MCNelectron input file that was used to create it."));
		m_toolTip.AddTool(&m_MCNP_style, _T("If this checkbox is checked, then the MCNelectron command-line keywords \"in\" and \"out\" will be replaced with \"in=\" and \"out=\", respectively."));
		m_toolTip.AddTool(GetDlgItem(IDC_SYSVARS), _T("Click this button to modify the values of the MCNEcode system variables. The non-zero system variables will be specified on the MCNelectron command line. The system variables can be referenced in the user's programs embedded in the MCNelectron input file."));
		m_toolTip.SetDelayTime(TTDT_AUTOPOP, 20000);
		m_toolTip.Activate(TRUE);
	}
	char tmp[SUFFIXLEN+10];
	sprintf(tmp, "%d", nThreads);
	m_nThreads.SetWindowText(tmp);
	UpdateCmdFolderOptionGroup();
	if (checkSyntax_MCNelectron == 1) m_checkSyntax_MCNelectron.SetCheck(BST_CHECKED);
	else if (!checkSyntax_MCNelectron) m_checkSyntax_MCNelectron.SetCheck(BST_UNCHECKED);
	else m_checkSyntax_MCNelectron.SetCheck(BST_INDETERMINATE);
	if (quitAfterChecking) m_quitAfterChecking.SetCheck(BST_CHECKED);
	else m_quitAfterChecking.SetCheck(BST_UNCHECKED);
	if (MCNP_style_cmdLine) m_MCNP_style.SetCheck(BST_CHECKED);
	else m_MCNP_style.SetCheck(BST_UNCHECKED);
	if (strcmpi(ENDFfolder, currentFolder)) m_ENDFfolder.SetWindowText(ShortenPath(ENDFfolder, currentFolder));
	else m_ENDFfolder.SetWindowText(".\\");
	m_inputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn, currentFolder));
	m_suffix.SetWindowText(suffix);
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, parent->curParmSet + 1);
		m_outputFileName.disable = TRUE;
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		m_browseOutput.EnableWindow(FALSE);
	}
	else {
		m_outputFileName.disable = FALSE;
		m_outputFileName.SetWindowText(ShortenPath(outputFileName, currentFolder));
		m_browseOutput.EnableWindow();
	}
	if (createLog == 1) m_createLog.SetCheck(BST_CHECKED);
	else if (!createLog) m_createLog.SetCheck(BST_UNCHECKED);
	else m_createLog.SetCheck(BST_INDETERMINATE);
	m_logSuffix.SetWindowText(logSuffix);
	if (logSuffix[0] != '\0') {
		sprintf(tmp, logSuffix, parent->curParmSet + 1);
		m_logFileName.disable = TRUE;
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		m_browseLog.EnableWindow(FALSE);
	}
	else {
		m_logFileName.disable = FALSE;
		m_logFileName.SetWindowText(ShortenPath(logFileName, currentFolder));
		m_browseLog.EnableWindow();
	}
	m_altCSinfoFileName.SetWindowText(ShortenPath(altCSinfoFileName, currentFolder));
	m_additionalArgs.SetWindowText(additionalArgs);
	return TRUE;  // return TRUE  unless you set the focus to a control
}

int CALLBACK 
MyBrowseCallbackProc2(HWND hwnd,UINT uMsg, LPARAM, LPARAM lpData)
{
    if (uMsg == BFFM_INITIALIZED) {
        CCmdLineDlg* pThis= (CCmdLineDlg*)lpData; // set in BROWSEINFO
        if (pThis && pThis->GetSafeHwnd()) { // forgot to set it?
			char path[1000] = {'\0'};
			if (pThis->cmdFolder[0] != '\\' && !strchr(pThis->cmdFolder, ':')) strcpy(path, currentFolder);
			strcat(path, pThis->cmdFolder);
		 	::SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)path);
        }
    }
    return 0;
}

int CALLBACK 
MyBrowseCallbackProc3(HWND hwnd,UINT uMsg, LPARAM, LPARAM lpData)
{
    if (uMsg == BFFM_INITIALIZED) {
        CCmdLineDlg* pThis= (CCmdLineDlg*)lpData; // set in BROWSEINFO
        if (pThis && pThis->GetSafeHwnd()) { // forgot to set it?
			char path[1000] = {'\0'};
			if (pThis->ENDFfolder[0] != '\\' && !strchr(pThis->ENDFfolder, ':')) strcpy(path, currentFolder);
			strcat(path, pThis->ENDFfolder);
			CString tmp = ShortenPath(path, currentFolder);
		 	if (tmp.GetLength() && tmp != ".\\" && tmp != ".") ::SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)path);
			else ::SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)(currentFolder));
        }
    }
    return 0;
}

void CCmdLineDlg::OnSameAsInput()
{
	if (!cmdFolderOption) return;
	if (cmdFolderOption == 1) m_sameAsMCNelectron.SetCheck(0);
	else if (cmdFolderOption == 2) m_sameAsMCNScript.SetCheck(0);
	else m_custom.SetCheck(0);
	cmdFolderOption = 0;
	cmdFolder[0] = '\0';
	UpdateCmdFolderOptionGroup();
	changed = true;
}

void CCmdLineDlg::OnSameAsMCNelectron()
{
	if (cmdFolderOption == 1) return;
	if (!cmdFolderOption) m_sameAsInput.SetCheck(0);
	else if (cmdFolderOption == 2) m_sameAsMCNScript.SetCheck(0);
	else m_custom.SetCheck(0);
	cmdFolderOption = 1;
	cmdFolder[0] = '\0';
	UpdateCmdFolderOptionGroup();
	changed = true;
}

void CCmdLineDlg::OnSameAsMCNScript()
{
	if (cmdFolderOption == 2) return;
	if (!cmdFolderOption) m_sameAsInput.SetCheck(0);
	else if (cmdFolderOption == 1) m_sameAsMCNelectron.SetCheck(0);
	else m_custom.SetCheck(0);
	cmdFolderOption = 2;
	cmdFolder[0] = '\0';
	UpdateCmdFolderOptionGroup();
	changed = true;
}

void CCmdLineDlg::OnCustom()
{
	if (cmdFolderOption == 3) return;
	if (cmdFolderOption == 0) {
		strcpy(cmdFolder, inputFileFolder);
		m_sameAsInput.SetCheck(0);
	}
	else if (cmdFolderOption == 1) {
		strcpy(cmdFolder, MCNelectronFolder);
		m_sameAsMCNelectron.SetCheck(0);
	}
	else {
		strcpy(cmdFolder, currentFolder);
		m_sameAsMCNScript.SetCheck(0);
	}
	cmdFolderOption = 3;
	UpdateCmdFolderOptionGroup();
	changed = true;
}

void CCmdLineDlg::UpdateCmdFolderOptionGroup()
{
	if (!cmdFolderOption) {
		m_sameAsInput.SetCheck(1);
		m_cmdFolder.SetWindowText(ShortenPath(inputFileFolder, currentFolder));
	}
	else if (cmdFolderOption == 1) {
		m_sameAsMCNelectron.SetCheck(1);
		m_cmdFolder.SetWindowText(ShortenPath(MCNelectronFolder, currentFolder));
	}
	else if (cmdFolderOption == 2) {
		m_sameAsMCNScript.SetCheck(1);
		m_cmdFolder.SetWindowText("");
	}
	else {
		m_custom.SetCheck(1);
		m_cmdFolder.SetWindowText(ShortenPath(cmdFolder, currentFolder));
	}
	if (cmdFolderOption < 3) {
		m_selectFolder.EnableWindow(FALSE);
		m_cmdFolder.EnableWindow(FALSE);
	}
	else {
		m_selectFolder.EnableWindow();
		m_cmdFolder.EnableWindow();
	}
}

void CCmdLineDlg::OnNthreads()
{
	char tmp[100], outputString[100];
	int nBytes = m_nThreads.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num = parent->CheckInteger(tmp, outputString, -1, MAXTHREADS);
	if (outputString[0] != '\0' && num != nThreads) {
		changed = true;
		nThreads = num;
	}
	sprintf(tmp, "%d", nThreads); m_nThreads.SetWindowText(tmp);
}

void CCmdLineDlg::OnNthreadsSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	UpdateData(TRUE);
	GetDlgItem(IDC_STATIC1)->SetFocus();
	int increment = pNMUpDown->iDelta < 0 ? 1 : -1;
	int newValue = nThreads + increment;
	if (newValue > MAXTHREADS || newValue < -1) {
		*pResult = 1;
		return;
	}
	nThreads = newValue;
	char tmp[100];
	changed = true;
	sprintf(tmp, "%d", nThreads); m_nThreads.SetWindowText(tmp);
	*pResult = 0;
}

void CCmdLineDlg::OnSystemVars()
{
	if (!parent->systemVarsShown) {
		CRect systemVarsWindowRect, windowRect;
		GetWindowRect(&windowRect);
		parent->pSystemVarsDlg->GetWindowRect(&systemVarsWindowRect);
		int width = systemVarsWindowRect.right - systemVarsWindowRect.left;
		int height = systemVarsWindowRect.bottom - systemVarsWindowRect.top;
		int hor_shift = (windowRect.right - windowRect.left - width) / 2;
		int vert_shift = (windowRect.bottom - windowRect.top - height) / 2;
		systemVarsWindowRect.left = windowRect.left + hor_shift;
		systemVarsWindowRect.right = systemVarsWindowRect.left + width;
		systemVarsWindowRect.top = windowRect.top + vert_shift;
		systemVarsWindowRect.bottom = systemVarsWindowRect.top + height;
		parent->pSystemVarsDlg->SetWindowPos(&wndTop, systemVarsWindowRect.left, systemVarsWindowRect.top, systemVarsWindowRect.right - systemVarsWindowRect.left, systemVarsWindowRect.bottom - systemVarsWindowRect.top, SWP_SHOWWINDOW);
		parent->systemVarsShown = true;
	}
	else {
		parent->pSystemVarsDlg->ShowWindow(SW_SHOWNORMAL);
		parent->pSystemVarsDlg->SetForegroundWindow();
	}
}

void CCmdLineDlg::OnBrowseExe()
{
	char buffer[1000];
	parent->GetFileName(&m_exeName, "Locate a MCNelectron executable file", "Programs (*.exe)|*.exe|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!strcmp(ShortenPath(buffer, currentFolder), ShortenPath((CString)MCNelectronFolder + MCNelectronExeName, currentFolder))) return;
	changed = true;
	char drive[1000], dir[1000], fname[1000], ext[1000];
	_splitpath(buffer, drive, dir, fname, ext);
	strcpy(MCNelectronFolder, (CString)drive + dir);
	strcpy(MCNelectronExeName, (CString)fname + ext);
	if (cmdFolderOption == 1) m_cmdFolder.SetWindowText(ShortenPath(MCNelectronFolder, currentFolder));
}

void CCmdLineDlg::OnBrowseInput()
{
	char buffer[1000];
	parent->GetFileName(&m_inputFileName, "Select the MCNelectron input file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!strcmp(ShortenPath(buffer, currentFolder), ShortenPath((CString)inputFileFolder + fn, currentFolder))) return;
	changed = true;
	char drive[1000], dir[1000], ext[1000];
	_splitpath(buffer, drive, dir, fn1, ext);
	strcpy(inputFileFolder, (CString)drive + dir);
	strcpy(fn, (CString)fn1 + ext);
	if (!cmdFolderOption) m_cmdFolder.SetWindowText(ShortenPath(inputFileFolder, currentFolder));
	if (suffix[0] != '\0') m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + suffix + ".txt", currentFolder));
}

void CCmdLineDlg::OnBrowseOutput()
{
	char buffer[1000];
	parent->GetFileName(&m_outputFileName, "Select the MCNelectron output file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!strcmp(ShortenPath(buffer, currentFolder), ShortenPath(outputFileName, currentFolder))) return;
	strcpy(outputFileName, buffer);
	changed = true;
}

void CCmdLineDlg::OnBrowseLog()
{
	char buffer[1000];
	parent->GetFileName(&m_logFileName, "Select the MCNelectron log file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!strcmp(ShortenPath(buffer, currentFolder), ShortenPath(logFileName, currentFolder))) return;
	strcpy(logFileName, buffer);
	changed = true;
}

void CCmdLineDlg::OnBrowseAlt()
{
	char buffer[1000];
	parent->GetFileName(&m_altCSinfoFileName, "Select the MCNelectron alternative cross sections information file", "Text files (*.txt)|*.txt|All files (*.*)|*.*||", buffer);
	if (buffer[0] == '\0') return;
	if (!strcmp(ShortenPath(buffer, currentFolder), ShortenPath(altCSinfoFileName, currentFolder))) return;
	strcpy(altCSinfoFileName, buffer);
	changed = true;
}

void CCmdLineDlg::OnOpenInputFile()
{
	CString docName('\0', 1000);
	int nBytes = m_inputFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CCmdLineDlg::OnOpenOutputFile()
{
	CString docName('\0', 1000);
	int nBytes = m_outputFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CCmdLineDlg::OnOpenLogFile()
{
	CString docName('\0', 1000);
	int nBytes = m_logFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CCmdLineDlg::OnOpenAltCSinfoFile()
{
	CString docName('\0', 1000);
	int nBytes = m_altCSinfoFileName.GetWindowText(docName.GetBuffer(), 999); docName.ReleaseBuffer();
	if (docName.GetAt(nBytes) != '\0') docName.SetAt(nBytes, '\0');
	docName.TrimLeft();
	docName.TrimRight();
	if (docName.GetLength() == 0) return;
	if (docName.GetAt(0) != '\\' && docName.Find(':') < 0) docName = currentFolder + docName;
	int rc = (int)ShellExecute(NULL, "open", docName, NULL, NULL, SW_SHOWNORMAL);
	if (rc <= 32) MessageBox((CString)"An error occurred when attempting to open file \"" + docName + "\".", NULL, MB_OK | MB_ICONERROR);
}

void CCmdLineDlg::OnExeName()
{
	char exeName_old[FNLEN+1], exeName_new[FNLEN+1];
	strcpy(exeName_old, (CString)MCNelectronFolder + MCNelectronExeName);
	if (!parent->OnCommandLine(&m_exeName, exeName_old, exeName_new)) return;
	changed = true;
	char drive[1000], dir[1000], fname[1000], ext[1000];
	_splitpath(exeName_new, drive, dir, fname, ext);
	strcpy(MCNelectronFolder, (CString)drive + dir);
	strcpy(MCNelectronExeName, (CString)fname + ext);
	if (cmdFolderOption == 1) m_cmdFolder.SetWindowText(ShortenPath(MCNelectronFolder, currentFolder));
}

void CCmdLineDlg::OnInputFileName()
{
	char inputFileName_old[FNLEN+1], inputFileName_new[FNLEN+1];
	strcpy(inputFileName_old, (CString)inputFileFolder + fn);
	if (!parent->OnCommandLine(&m_inputFileName, inputFileName_old, inputFileName_new)) return;
	changed = true;
	char drive[1000], dir[1000], ext[1000];
	_splitpath(inputFileName_new, drive, dir, fn1, ext);
	strcpy(inputFileFolder, (CString)drive + dir);
	strcpy(fn, (CString)fn1 + ext);
	if (!cmdFolderOption) m_cmdFolder.SetWindowText(ShortenPath(inputFileFolder, currentFolder));
	if (suffix[0] != '\0') m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + suffix + ".txt", currentFolder));
}

void CCmdLineDlg::OnOutputFileName()
{
	if (m_outputFileName.disable) return;
	char outputFileName_old[FNLEN+1];
	strcpy(outputFileName_old, outputFileName);
	if (!parent->OnCommandLine(&m_outputFileName, outputFileName_old, outputFileName)) return;
	changed = true;
}

void CCmdLineDlg::OnLogFileName()
{
	if (m_logFileName.disable) return;
	char logFileName_old[FNLEN+1];
	strcpy(logFileName_old, logFileName);
	if (!parent->OnCommandLine(&m_logFileName, logFileName_old, logFileName)) return;
	changed = true;
}

void CCmdLineDlg::OnAltCSinfoFileName()
{
	char altCSinfoFileName_old[FNLEN+1];
	strcpy(altCSinfoFileName_old, altCSinfoFileName);
	if (!parent->OnCommandLine(&m_altCSinfoFileName, altCSinfoFileName_old, altCSinfoFileName)) return;
	changed = true;
}

void CCmdLineDlg::OnSelectFolder()
{
    BROWSEINFO rBI = {0};
    rBI.hwndOwner = GetSafeHwnd();
	rBI.ulFlags = BIF_RETURNONLYFSDIRS | BIF_BROWSEINCLUDEFILES | BIF_USENEWUI;
	rBI.lpszTitle = "Browse for the initial folder of the cmd.exe window";
	rBI.lpfn = MyBrowseCallbackProc2;
	rBI.lParam = (DWORD)this;   // used by static callback function
	LPITEMIDLIST pidl = SHBrowseForFolder(&rBI);
	if (!pidl) return;
	TCHAR szPath[MAX_PATH];
	BOOL fRet = SHGetPathFromIDList(pidl, szPath);
	FILE *f = fopen(szPath, "rb");
	if (f) {
		char drive[512], dir[512];
		_splitpath(szPath, drive, dir, NULL, NULL);
		sprintf(szPath, "%s%s", drive, dir);
	}
	else if (!strlen(szPath) || szPath[strlen(szPath)-1] != '\\') strcat(szPath, "\\");
	CString pathName = szPath;
	char cmdFolder_bak[1000];
	strcpy(cmdFolder_bak, cmdFolder);
	strcpy(cmdFolder, pathName.GetBuffer()); pathName.ReleaseBuffer();
	if (strcmp(ShortenPath(cmdFolder, currentFolder), ShortenPath(cmdFolder_bak, currentFolder))) {
		m_cmdFolder.SetWindowText(ShortenPath(cmdFolder, currentFolder));
		changed = true;
	}
}

void CCmdLineDlg::OnBrowseENDF()
{
    BROWSEINFO rBI = {0};
    rBI.hwndOwner = GetSafeHwnd();
	rBI.ulFlags = BIF_RETURNONLYFSDIRS | BIF_BROWSEINCLUDEFILES | BIF_USENEWUI;
	rBI.lpszTitle = "Browse for the ENDF data folder";
	rBI.lpfn = MyBrowseCallbackProc3;
	rBI.lParam = (DWORD)this;   // used by static callback function
	LPITEMIDLIST pidl = SHBrowseForFolder(&rBI);
	if (!pidl) return;
	TCHAR szPath[MAX_PATH];
	BOOL fRet = SHGetPathFromIDList(pidl, szPath);
	FILE *f = fopen(szPath, "rb");
	if (f) {
		char drive[512], dir[512];
		_splitpath(szPath, drive, dir, NULL, NULL);
		sprintf(szPath, "%s%s", drive, dir);
	}
	else if (!strlen(szPath) || szPath[strlen(szPath)-1] != '\\') strcat(szPath, "\\");
	CString pathName = szPath;
	char ENDFfolder_bak[1000];
	strcpy(ENDFfolder_bak, ENDFfolder);
	strcpy(ENDFfolder, pathName.GetBuffer()); pathName.ReleaseBuffer();
	if (strcmpi(ENDFfolder, currentFolder)) m_ENDFfolder.SetWindowText(ShortenPath(ENDFfolder, currentFolder));
	else m_ENDFfolder.SetWindowText(".\\");
	if (strspn(ENDFfolder_bak, " \t") == strlen(ENDFfolder_bak) || strcmp(ShortenPath(ENDFfolder, currentFolder), ShortenPath(ENDFfolder_bak, currentFolder))) changed = true;
}

void CCmdLineDlg::OnENDFfolder()
{
	char ENDFfolder_new[1000];
	int nBytes = m_ENDFfolder.GetLine(0, ENDFfolder_new, 999);
	ENDFfolder_new[nBytes] = '\0';
	if (strspn(ENDFfolder_new, " \t") == strlen(ENDFfolder_new)) {
		if (strspn(ENDFfolder, " \t") != strlen(ENDFfolder)) {
			ENDFfolder[0] = '\0';
			changed = true;
		}
		return;
	}
	else if (strspn(ENDFfolder, " \t") == strlen(ENDFfolder)) {
		strcpy(ENDFfolder, ENDFfolder_new);
		changed = true;
		return;
	}
	char ENDFfolder_old[FNLEN+1];
	strcpy(ENDFfolder_old, ENDFfolder);
	if (!parent->OnCommandLine(&m_ENDFfolder, ENDFfolder_old, ENDFfolder, true)) return;
	changed = true;
}

void CCmdLineDlg::OnSuffix()
{
	char suffix_old[SUFFIXLEN+1], suffix_new[SUFFIXLEN+1], tmp[SUFFIXLEN+10];
	strcpy(suffix_old, suffix);
	int nBytes = m_suffix.GetLine(0, suffix_new, SUFFIXLEN);
	suffix_new[nBytes] = '\0';
	char *ptr = strtok(suffix_new, " \t");
	if (!ptr) suffix_new[0] = '\0';
	else {
		strcpy(suffix_new, ptr);
		ptr = strchr(suffix_new, '%');
		if (ptr && (*(ptr + 1) != 'd' || *(ptr + 2) != '\0' && strchr(ptr + 2, '%'))) strcpy(suffix_new, suffix_old);
	}
	m_suffix.SetWindowText(suffix_new);
	if (!strcmp(suffix_new, suffix)) return;
	strcpy(suffix, suffix_new);
	changed = true;
	if (suffix[0] != '\0') {
		sprintf(tmp, suffix, parent->curParmSet + 1);
		m_outputFileName.disable = TRUE;
		m_browseOutput.EnableWindow(FALSE);
		m_outputFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		outputFileName[0] = '\0';
	}
	else {
		m_outputFileName.disable = FALSE;
		m_browseOutput.EnableWindow();
		int nBytes = m_outputFileName.GetWindowText(outputFileName, FNLEN);
		outputFileName[nBytes] = '\0';
		m_outputFileName.SetWindowText(ShortenPath(outputFileName, currentFolder));
	}
}

void CCmdLineDlg::OnLogSuffix()
{
	char logSuffix_old[SUFFIXLEN+1], logSuffix_new[SUFFIXLEN+1], tmp[SUFFIXLEN+10];
	strcpy(logSuffix_old, logSuffix);
	int nBytes = m_logSuffix.GetLine(0, logSuffix_new, SUFFIXLEN);
	logSuffix_new[nBytes] = '\0';
	char *ptr = strtok(logSuffix_new, " \t");
	if (!ptr) logSuffix_new[0] = '\0';
	else {
		strcpy(logSuffix_new, ptr);
		ptr = strchr(logSuffix_new, '%');
		if (ptr && (*(ptr + 1) != 'd' || *(ptr + 2) != '\0' && strchr(ptr + 2, '%'))) strcpy(logSuffix_new, logSuffix_old);
	}
	m_logSuffix.SetWindowText(logSuffix_new);
	if (!strcmp(logSuffix_new, logSuffix)) return;
	strcpy(logSuffix, logSuffix_new);
	changed = true;
	if (logSuffix[0] != '\0') {
		sprintf(tmp, logSuffix, parent->curParmSet + 1);
		m_logFileName.disable = TRUE;
		m_browseLog.EnableWindow(FALSE);
		m_logFileName.SetWindowText(ShortenPath((CString)inputFileFolder + fn1 + tmp + ".txt", currentFolder));
		logFileName[0] = '\0';
	}
	else {
		m_logFileName.disable = FALSE;
		m_browseLog.EnableWindow();
		int nBytes = m_logFileName.GetWindowText(logFileName, FNLEN);
		logFileName[nBytes] = '\0';
		m_logFileName.SetWindowText(ShortenPath(logFileName, currentFolder));
	}
}

BOOL CCmdLineDlg::PreTranslateMessage(MSG *pMsg) 
{
	m_toolTip.RelayEvent(pMsg);
	return CDialogEx::PreTranslateMessage(pMsg);
}

void CCmdLineDlg::OnOK()
{
	GetDlgItem(IDC_STATIC1)->SetFocus();
	char tmp1[2*FNLEN+1], tmp2[2*FNLEN+1];
	int nBytes = m_additionalArgs.GetWindowText(tmp1, 2*FNLEN);
	tmp1[nBytes] = '\0';
	tmp2[0] = '\0';
	char *ptr = strtok(tmp1, " \t\x0d\x0a");
	if (ptr) {
		strcpy(tmp2, ptr);
		ptr = strtok(NULL, " \t\x0d\x0a");
		while(ptr) {
			strcat(tmp2, " ");
			strcat(tmp2, ptr);
			ptr = strtok(NULL, " \t\x0d\x0a");
		}
	}
	if (strcmpi(tmp2, additionalArgs)) {
		strcpy(additionalArgs, tmp2);
		changed = true;
	}

	int ch = m_checkSyntax_MCNelectron.GetCheck();
	if (ch == BST_CHECKED) checkSyntax_MCNelectron = 1;
	else if (ch == BST_UNCHECKED) checkSyntax_MCNelectron = 0;
	else checkSyntax_MCNelectron = 2;
	if (checkSyntax_MCNelectron != parent->checkSyntax_MCNelectron) changed = true;

	quitAfterChecking = (m_quitAfterChecking.GetCheck() == BST_CHECKED);
	if (quitAfterChecking != parent->quitAfterChecking) changed = true;

	MCNP_style_cmdLine = (m_MCNP_style.GetCheck() == BST_CHECKED);
	if (MCNP_style_cmdLine != parent->MCNP_style_cmdLine) changed = true;

	ch = m_createLog.GetCheck();
	if (ch == BST_CHECKED) createLog = 1;
	else if (ch == BST_UNCHECKED) createLog = 0;
	else createLog = 2;
	if (createLog != parent->createLog) changed = true;

	if (!changed) {
		EndDialog(IDCANCEL);
		return;
	}
	parent->checkSyntax_MCNelectron = checkSyntax_MCNelectron;
	parent->quitAfterChecking = quitAfterChecking;
	parent->MCNP_style_cmdLine = MCNP_style_cmdLine;
	parent->createLog = createLog;
	strcpy(parent->fn1, fn1);
	parent->cmdFolderOption = cmdFolderOption;
	parent->nThreads = nThreads;
	strcpy(parent->MCNelectronFolder, MCNelectronFolder);
	strcpy(parent->inputFileFolder, inputFileFolder);
	strcpy(parent->outputFileName, outputFileName);
	strcpy(parent->logFileName, logFileName);
	strcpy(parent->cmdFolder, cmdFolder);
	strcpy(parent->MCNelectronExeName, MCNelectronExeName);
	strcpy(parent->fn, fn);
	strcpy(parent->suffix, suffix);
	strcpy(parent->logSuffix, logSuffix);
	strcpy(parent->altCSinfoFileName, altCSinfoFileName);
	strcpy(parent->ENDFfolder, ENDFfolder);
	strcpy(parent->additionalArgs, additionalArgs);
	EndDialog(IDOK);
}

VECTOR CMCNScriptDlg::ProjectionToPlane(VECTOR point, SURFACE *plane)
{
	VECTOR v = point - plane->init;
	double dist = v * plane->dir;
	return point - dist * plane->dir;
}

double CMCNScriptDlg::DistanceToSurface(VECTOR *a, SURFACE *s)
{
	VECTOR diff = *a - s->init;
	if (s->type == 2) { // sphere
		return Norm(diff) - s->R;
	}
	double distanceToPlane = s->dir * diff;
	if (s->type <= 1) { // plane
		return distanceToPlane;
	}
	if (s->type >= 4 && distanceToPlane < 0.) {	// cone, in the negative half-space relative to the plane normal to the symmetry axis and containing the tip
		double distanceToTip = Norm(diff);
		if (distanceToPlane / distanceToTip <= -s->sin) return distanceToTip; // > 0 when outside the cone
	}
	diff -= distanceToPlane * s->dir; 
	if (s->type == 3) { // cylinder
		return Norm(diff) - s->R;
	}
	// cone:
	return Norm(diff) * s->cos - distanceToPlane * s->sin; // < 0 when inside the cone
}

double CMCNScriptDlg::DistanceToSurface(double t, void *parms) {
	ROOTS_PARMS *p = (ROOTS_PARMS *)parms;
	VECTOR a = LocalToGlobal(p->curve, p->branchNo, t, p->distancePrecision);
	return DistanceToSurface(&a, p->surface);
}

void CMCNScriptDlg::LineParameterLimits(LINE *line, double distancePrecision) {
	if (line->type < 1) return; // line undefined (non-intersecting or coinciding surfaces)
	if (line->type > 11) return; // not implemented (intersection line not parameterized)
	if (line->type < 4) { // curve parameter is distance to the initial point (unlimited)
		line->tMin = maxDistance_ratio * distancePrecision;
		line->tMax = 0.;
	}
	else if (line->type < 8) { // curve parameter is an angle (limited)
		line->tMin = 0.;
		line->tMax = 2. * pi;
	}
	else if (line->type < 10) { // curve parameter is +/-sqrt(z')  (unlimited)
		line->tMin = sqrt(maxDistance_ratio * distancePrecision);
		line->tMax = 0.;
	}
	else if (line->type == 10) { // curve parameter is z' (limited)
		double radius_cyl = sqrt(line->coef[0]);
		double radius_sph = sqrt(line->coef[0] + line->coef[1]);
		double dist = line->coef[2];
		line->tMin = -radius_sph > dist - radius_cyl ? -radius_sph : dist - radius_cyl;
		line->tMax = radius_sph < dist + radius_cyl ? radius_sph : dist + radius_cyl;
	}
	else if (line->type == 11) { // curve parameter is z' (limited)
		double radius_cyl1 = sqrt(line->coef[0]);
		double radius_cyl2 = sqrt(line->coef[1]);
		double dist = line->coef[2];
		line->tMin = -radius_cyl1 > dist - radius_cyl2 ? -radius_cyl1 : dist - radius_cyl2;
		line->tMax = radius_cyl1 < dist + radius_cyl2 ? radius_cyl1 : dist + radius_cyl2;
	}
}

int CMCNScriptDlg::LinePositionRelativeToSurface(LINE *line, int branchNo, SURFACE *surf, double distancePrecision)
{	// line must not interset with the surface
	double tTest[3];
	int i, side = 0;
	if (line->tMin < line->tMax) { // limited
		double quarter = 0.25 * (line->tMax - line->tMin);
		for (i = 0; i < 3; i++) tTest[i] = line->tMin + (i + 1) * quarter;
	}
	else { // unlimited
		tTest[0] = 0.;
		if (line->type < 4) tTest[1] = 1. + 100. * distancePrecision;
		else if (line->type == 8 || line->type == 9) tTest[1] = sqrt(1. + 100. * distancePrecision);
		tTest[2] = -tTest[1];
	}
	double dist = 0.;
	for (i = 0; i < 3 && fabs(dist) <= distancePrecision; i++) {
		VECTOR a = LocalToGlobal(line, branchNo, tTest[i], distancePrecision);
		dist = DistanceToSurface(&a, surf);
	}
	if (dist > distancePrecision) side = 1;
	else if (dist < -distancePrecision) side = -1;
	return side;
}

VECTOR CMCNScriptDlg::LineTangentDir(LINE *line, int branch, double t)
{
	VECTOR a;
	if (line->type == 1 || line->type == 2) {
		return line->dir[branch];
	}
	else if (line->type == 3) {
		if (t >= 0.) return line->dir[1];
		else return line->dir[0];
	}
	if (line->type < 6) {
		a.z = 0.;
		a.y = line->coef[branch] * cos(t);
		a.x = -line->coef[branch] * sin(t);
	}
	else if (line->type == 6) {
		a.z = line->Z1 * cos(t);
		a.y = -line->Y1 * sin(t);
		a.x = a.z * line->tan;
	}
	else if (line->type == 7) {
		a.z = 0.5 * line->Z1 * cos(t);
		a.y = -line->Y1 * sin(t);
		a.x = a.z * line->tan;
	}
	else if (line->type == 8) {
		a.z = 2. * t;
		double aux = sqrt(line->coef[0] + line->coef[1] * t * t);
		a.y = aux + t * t * line->coef[1] / aux;
		a.x = a.z * line->tan;
	}
	else if (line->type == 9) {
		a.z = 2. * t;
		a.y = sqrt(line->coef[0]);
		a.x = a.z * line->tan;
	}
	else { // not implemented
//		printf("Error: Line tangent cannot be calculated for line type %d.\n", line->type);
		return zero;
	}
	a = Rotate(line, a);
	return a / Norm(a);
}

int CMCNScriptDlg::CheckCellGeometry0(unsigned short int *s, short int j, short int k, char *side, double distancePrecision, bool &notImplemented)
{
	short int intersectionType, coincidingSurfaces;
	LINE intersectionLine;
	intersectionType = Intersection(surf + s[j], surf + s[k], intersectionLine, distancePrecision);
	if (intersectionType) coincidingSurfaces = 0;
	else if ((surf + s[j])->type != 1 || (surf + s[j])->dir * (surf + s[k])->dir > 0.) coincidingSurfaces = 1;
	else coincidingSurfaces = -1;
	int aux = coincidingSurfaces * side[j] * side[k];
	if (aux < 0 || !aux && (intersectionType == -1 && side[j] < 0 && side[k] < 0 ||
		intersectionType == -2 && side[j] < 0 && side[k] > 0 ||
		intersectionType == -3 && side[j] > 0 && side[k] < 0 ||
		intersectionType == -4 && side[j] > 0 && side[k] > 0)) { // Regions j and k do not have common points
		return 0;
	}
	if (aux > 0 || !aux && (intersectionType == -1 && side[j] > 0 && side[k] < 0 ||
		intersectionType == -2 && side[j] > 0 && side[k] > 0 ||
		intersectionType == -3 && side[j] < 0 && side[k] < 0 ||
		intersectionType == -4 && side[j] < 0 && side[k] > 0)) { // Region k is inside region j
		return -1;
	}
	if (!aux && (intersectionType == -1 && side[j] < 0 && side[k] > 0 ||
		intersectionType == -2 && side[j] < 0 && side[k] < 0 ||
		intersectionType == -3 && side[j] > 0 && side[k] > 0 ||
		intersectionType == -4 && side[j] > 0 && side[k] < 0)) { // Region j is inside region k
		return -2;
	}
	if (intersectionType > 11) notImplemented = true;
	return 1; // No error (regions j and k intersect and neither of them is entirely inside the other one)
}

int CMCNScriptDlg::CheckCellGeometry(short int nS, unsigned short int *s, char *side, VECTOR &point, char &pointFound, double &distancePrecision)
{
	short int intersectionType, intersectionType_opt;
	LINE intersectionLine, intersectionLine_opt;
	int j, k, m, n;
	int j_opt = -1, k_opt, m_opt = -1, nBranches;
	pointFound = 0;
	int iPoint, sense;
	double intersectionPoints[NROOTS_MAX], intersectionPoint;
	int branch[NROOTS_MAX], branchNo, nAdditionalSurfaces_min = nS, additionalSurfaces_min[2*MAXSURFACES_CELL];
	VECTOR intersectionPoints0[NROOTS_MAX], intersectionPoint0;
	for (j = 0; j < nS; j++) {
		for (k = j + 1; k < nS; k++) {
			intersectionType = Intersection(surf + s[j], surf + s[k], intersectionLine, distancePrecision);
			if (intersectionType < 1 || intersectionType > 11) continue;
			nBranches = 1;
			if (intersectionType == 2 || intersectionType == 5) nBranches = 2;
			else if (intersectionType > 9) nBranches = 4;
			bool intersectionPointsFound = false;
			int nAdditionalSurfacesEdge[4] = {0, 0, 0, 0}, additionalSurfacesEdge[4][2*MAXSURFACES_CELL];
			for (m = 0; m < nS; m++) {
				if (m == j || m == k) continue;
				int nIntersectionPoints = LineSurfaceIntersection(&intersectionLine, surf + s[m], branch, distancePrecision, intersectionPoints, intersectionPoints0, sense);
				if (!nIntersectionPoints) {
					if (sense * side[m] < 0) break;
					for (n = 0; n < nBranches; n++) {
						int pos = LinePositionRelativeToSurface(&intersectionLine, n, surf + s[m], distancePrecision);
						if (pos * side[m] < 0) nAdditionalSurfacesEdge[n] = -1;
						else if (!pos && nAdditionalSurfacesEdge[n] >= 0) {
							additionalSurfacesEdge[n][nAdditionalSurfacesEdge[n]] = m;
							nAdditionalSurfacesEdge[n]++;
						}
					}
					continue;
				}
				intersectionPointsFound = true;
				for (iPoint = 0; iPoint < nIntersectionPoints; iPoint++) {
					int nAdditionalSurfaces = 0, additionalSurfaces[2*MAXSURFACES_CELL];
					for (n = 0; n < nS; n++) {
						if (n == j || n == k || n == m) continue;
						double dist = DistanceToSurface(intersectionPoints0 + iPoint, surf + s[n]);
						if (dist * side[n] < -distancePrecision) break;
						if (fabs(dist) <= distancePrecision) {
							additionalSurfaces[nAdditionalSurfaces] = n;
							nAdditionalSurfaces++;
						}
					}
					if (n == nS) { // a point on the cell surface (a vertex) has been found
						if (!nAdditionalSurfaces) {
							nAdditionalSurfaces_min = 0;
							pointFound = -1;
							intersectionPoint = intersectionPoints[iPoint];
							intersectionPoint0 = intersectionPoints0[iPoint];
							branchNo = branch[iPoint];
							goto FIND_INTERIOR_POINT;
						}
						if (nAdditionalSurfaces < nAdditionalSurfaces_min) {
							nAdditionalSurfaces_min = nAdditionalSurfaces;
							memcpy(additionalSurfaces_min, additionalSurfaces, nAdditionalSurfaces * sizeof(int));
							bool update_m_opt = false;
							if (j != j_opt || k != k_opt) {
								j_opt = j;
								k_opt = k;
								intersectionType_opt = intersectionType;
								intersectionLine_opt = intersectionLine;
								update_m_opt = true;
							}
							if (m != m_opt || update_m_opt) {
								m_opt = m;
								intersectionPoint = intersectionPoints[iPoint];
								intersectionPoint0 = intersectionPoints0[iPoint];
								branchNo = branch[iPoint];
							}
						}
					}
				}
			}
			if (m == nS && !intersectionPointsFound) { // a cell edge has been found
				for (n = 0; n < nBranches && nAdditionalSurfaces_min; n++) {
					if (nAdditionalSurfacesEdge[n] >= 0 && nAdditionalSurfacesEdge[n] < nAdditionalSurfaces_min) {
						nAdditionalSurfaces_min = nAdditionalSurfacesEdge[n];
						branchNo = n;
						if (nAdditionalSurfaces_min) {
							memcpy(additionalSurfaces_min, additionalSurfacesEdge[n], nAdditionalSurfacesEdge[n] * sizeof(int));
							if (j != j_opt || k != k_opt) {
								j_opt = j;
								k_opt = k;
								intersectionType_opt = intersectionType;
								intersectionLine_opt = intersectionLine;
							}
							m_opt = -1;
						}
					}
				}
				if (!nAdditionalSurfaces_min) goto FIND_INTERIOR_POINT;
			}
		}
	}
	if (nS == 2) return 1; // the cell has only two bounding surfaces, which do not intersect (the interior point is not needed in this case)
	if (j_opt < 0) return -1; // geometry error (cell volume is zero) or not implemented intersection types (intersectionType > 11)
	pointFound = m_opt >= 0 ? -1 : 0;
	j = j_opt;
	k = k_opt;
	m = m_opt;
	intersectionType = intersectionType_opt;
	intersectionLine = intersectionLine_opt;
FIND_INTERIOR_POINT:
	double t;
	int nTestSurf;
	int iTestSurf[7];
	int intersectingSurfaces[2*MAXSURFACES_CELL];
	if (pointFound) { // A point upon the surface of the cell (a vertex) has been found already
		nTestSurf = 3;
		iTestSurf[0] = m;
		iTestSurf[1] = k;
		iTestSurf[2] = j;
		point = intersectionPoint0;
		t = intersectionPoint;
		double dist = DistanceToSurface(&point, surf + s[m]);
		if (fabs(dist) > distancePrecision) {
			distancePrecision = fabs(dist);
//			printf("Warning: distance precision increased to %.15g.\n", distancePrecision);
		}
	}
	else { // No point upon the surface of the cell has been defined yet; only one edge is available
		nTestSurf = 2;
		iTestSurf[0] = j;
		iTestSurf[1] = k;
		if (intersectionType > 3 && intersectionType < 8 || intersectionType > 9) { // circular, elliptical or non-planar intersection
			t = 0.5 * (intersectionLine.tMin + intersectionLine.tMax);
			point = LocalToGlobal(&intersectionLine, branchNo, t, distancePrecision);
		}
		else if (intersectionType < 4) { // one or two parallel straight lines, or two rays with a common initial point
			t = 0.;
			point = intersectionLine.init[branchNo];
		}
		else { // parabola or hyperbola
			t = 0.;
			point = LocalToGlobal(&intersectionLine, 0, 0., distancePrecision);
		}
	}
	for (k = 0; k < nTestSurf; k++) intersectingSurfaces[k] = iTestSurf[k];
	for (j = 0; j < nAdditionalSurfaces_min; j++, k++) intersectingSurfaces[k] = additionalSurfaces_min[j];
	int nIntersectingSurfaces = k;
	bool err;
	double step;
	VECTOR normal[3], tangent[3], direction, normal0[2*MAXSURFACES_CELL];
	int count, countMax = 4;
	int k_twoRays = -1;
	if (!pointFound) { // A cell edge without vertices has been found. tangent[0,1] are the unit direction vectors of the surface tangents that are normal to the edge.
		for (k = 0; k < 2; k++) normal[k] = SurfaceNormal(surf + s[iTestSurf[k]], point, distancePrecision);
		tangent[2] = LineTangentDir(&intersectionLine, branchNo, t);
		if (!tangent[2].x && !tangent[2].y && !tangent[2].z) return -3;
		tangent[1] = tangent[2] & normal[0];
		tangent[0] = tangent[2] & normal[1];
		if (intersectionType == 3 && Norm(point - intersectionLine.init[0]) <= distancePrecision) k_twoRays = 2;
		direction = TestDirection(nTestSurf, &point, tangent, s, side, iTestSurf, k_twoRays, distancePrecision, &intersectionLine);
		if (direction.x == 0. && direction.y == 0. && direction.z == 0.) return -2;
	}
	else if (nIntersectingSurfaces == 3) { // A vertex formed of intersection of three surfaces has been found. tangent[0,1,2] are the unit direction vectors of the tangents of three edges that are not parallel to surfaces iTestSurf[0,1,2], respectively.
		for (k = 0; k < 3; k++) normal[k] = SurfaceNormal(surf + s[iTestSurf[k]], point, distancePrecision);
		tangent[0] = LineTangentDir(&intersectionLine, branchNo, t);
		if (!tangent[0].x && !tangent[0].y && !tangent[0].z) return -3;
//		tangent[0] = normal[1] & normal[2];
		tangent[1] = normal[2] & normal[0];
		tangent[2] = normal[0] & normal[1];
		if (intersectionType == 3 && Norm(point - intersectionLine.init[0]) <= distancePrecision) k_twoRays = 0;
		direction = TestDirection(nTestSurf, &point, tangent, s, side, iTestSurf, k_twoRays, distancePrecision, &intersectionLine);
		if (direction.x == 0. && direction.y == 0. && direction.z == 0.) return -2;
	}
	else { // A vertex formed of intersection of four or more surfaces has been found.
		for (k = 0; k < nIntersectingSurfaces; k++) normal0[k] = SurfaceNormal(surf + s[intersectingSurfaces[k]], point, distancePrecision);
		int ier, nEdges = 0;
		for (m = 0; m < nIntersectingSurfaces; m++) {
			for (n = m + 1; n < nIntersectingSurfaces; n++) {
				VECTOR tangent = CheckPairOfSurfaces(-1, m, n, -1, -1, point, nIntersectingSurfaces, intersectingSurfaces, s, side, normal0, distancePrecision, ier);
				if (ier) continue;
				for (k = 0; k < nEdges && Norm(edgeBuffer[k] & tangent) > anglePrecision; k++);
				if (k < nEdges) continue;
				edgeBuffer[nEdges] = tangent;
				nEdges++;
			}
		}
		if (nEdges < 3) return -2;
		for (direction = zero, m = 0; m < nEdges; m++) direction += edgeBuffer[m];
		direction /= Norm(direction);
	}
	count = 0;
	step = 0.5 * distancePrecision;
	bool prevErr = false;
	do {
		err = false;
		do { // Multiple checks are needed in order to ensure that the result is not affected by rounding errors.
			step *= 2.;
			if (step > maxDistance_ratio * distancePrecision) {
				err = true;
				count = countMax;
				break;
			}
			point += step * direction;
			for (k = 0; k < nIntersectingSurfaces; k++) {
				double dist = DistanceToSurface(&point, surf + s[intersectingSurfaces[k]]);
				if (fabs(dist) <= distancePrecision) break;
				if (dist * side[intersectingSurfaces[k]] < 0.) {
					err = true;
					break;
				}
			}
		} while(k < nTestSurf && !err);
		if (err == prevErr) count++;
		else {
			count = 0;
			prevErr = err;
		}
	} while(count < countMax);
	if (!err) {
		for (n = 0; n < nS; n++) {
			for (k = 0; k < nIntersectingSurfaces && intersectingSurfaces[k] != n; k++);
			if (k < nIntersectingSurfaces) continue;
			double dist = DistanceToSurface(&point, surf + s[n]);
			if (dist * side[n] <= distancePrecision) {
				break;
			}
		}
		if (n == nS) return (pointFound = 1);
	}
	if (j_opt < 0) return 0;
	else return -3;
}

int CMCNScriptDlg::CheckCellGeometry2(CELL *c, double &distancePrecision, bool &surfacesRemoved, bool *removeSurface)
{
	int j, k;
	surfacesRemoved = false;
	bool notImplemented = false;
	memset(removeSurface, 0, c->nS * sizeof(bool));
	for (j = 0; j < c->nS; j++) {
		if (removeSurface[j]) continue;
		for (k = j + 1; k < c->nS; k++) {
			if (removeSurface[k]) continue;
			int rc = CheckCellGeometry0(c->s, j, k, c->side, distancePrecision, notImplemented);
			if (!rc) return 0;
			if (rc == -1) {
				removeSurface[j] = true;
				surfacesRemoved = true;
				break;
			}
			else if (rc == -2) {
				removeSurface[k] = true;
				surfacesRemoved = true;
			}
		}
	}
	CELL c2, *pCell;
	if (surfacesRemoved) {
		c2 = *c;
		for (k = 0, j = 0; j < c2.nS; j++) {
			if (removeSurface[j]) continue;
			c2.s[k] = c2.s[j];
			c2.side[k] = c2.side[j];
			k++;
		}
		if (!k) return 0;
		c2.nS = k;
		pCell = &c2;
	}
	else pCell = c;
	if (pCell->nS > 1) {
		int rc = CheckCellGeometry(pCell->nS, pCell->s, pCell->side, pCell->point, pCell->pointFound, distancePrecision);
		if (surfacesRemoved) {
			c->point = c2.point;
			c->pointFound = c2.pointFound;
		}
		if (!rc || rc == -1 && !notImplemented || rc < -1) return 0;
	}
	return 1;
}

VECTOR CMCNScriptDlg::CheckPairOfSurfaces(int n, int &i, int &j, int exclude1, int exclude2, VECTOR point, int nIntersectingSurfaces, int *intersectingSurfaces, unsigned short *s, char *side, VECTOR *normal, double distancePrecision, int &ier)
{	// Checks if the intersection line of two bounding surfaces containing a vertex of a cell ("point") is an edge of the cell
	if (n >= 0) { // Determine the indices of the two surfaces (i and j, where j > i) from the sequence number of the pair (n):
		j = (int)(0.5 * (1. + sqrt(1. + 8. * n)) + 0.5);
		if ((j - 1) * j / 2 == n) i = 0;
		else if ((j + 1) * j / 2 == n) {
			j++;
			i = 0;
		}
		else i = n - (j - 1) * j / 2;
	}
	if (j == exclude1 || j == exclude2 || i == exclude1 || i == exclude2) {
		ier = 1;
		return zero;
	}
	VECTOR edgeTangent = normal[i] & normal[j]; // direction vector of the tangent of the intersection line at "point"
	double norm = Norm(edgeTangent);
	if (norm <= anglePrecision) {
		ier = 2;
		return zero;
	}
	edgeTangent /= norm;
	int k, l;
	for (l = 0; l < 2; l++, edgeTangent = -edgeTangent) {
		VECTOR testPoint = point;
		int count = 0, countMax = 4;
		double step = 0.5 * distancePrecision;
		int prevErr = 0;
		do { // Multiple checks are needed in order to ensure that the result is not affected by rounding errors.
			ier = 0;
			do { // Check if the intersection line of surfaces i and j is an edge of the cell:
				step *= 2.;
				if (step > maxDistance_ratio * distancePrecision) {
					ier = 3;
					prevErr = ier;
					count = countMax;
					break;
				}
				testPoint += step * edgeTangent;
				for (k = 0; k < nIntersectingSurfaces; k++) {
					if (k == i || k == j) continue;
					double dist = DistanceToSurface(&testPoint, surf + s[intersectingSurfaces[k]]);
					if (fabs(dist) <= distancePrecision) break;
					if (dist * side[intersectingSurfaces[k]] < 0.) {
						ier = 4;
						break;
					}
				}
			} while(k < nIntersectingSurfaces && ier);
			if (ier == prevErr) count++;
			else {
				count = 0;
				prevErr = ier;
			}
		} while(count < countMax);
		if (!ier) break;
	}
	return edgeTangent;
}

VECTOR CMCNScriptDlg::TestDirection(int nTestSurf, VECTOR *point, VECTOR *tangent, unsigned short *s, char *side, int *iTestSurf, int k_twoRays, double distancePrecision, LINE *intersectionLine)
{
	int k;
	VECTOR direction;
	int count, countMax = 4;
	double step;
	for (direction = zero, k = 0; k < nTestSurf; k++) {
		double norm = Norm(tangent[k]);
		if (norm <= anglePrecision) return zero;
		tangent[k] /= norm;
		VECTOR newPoint = *point;
		count = -1;
		step = 0.5 * distancePrecision;
		double dist, prevDist, change = 0., prevChange;
		do { // Multiple checks are needed in order to ensure that the result is not affected by rounding errors.
			step *= 2.;
			newPoint += tangent[k] * step;
			dist = DistanceToSurface(&newPoint, surf + s[iTestSurf[k]]);
			if (count == -1) count = 0;
			else {
				change = dist - prevDist;
				if (change * prevChange > 0.) count++;
				else count = 0;
			}
			prevDist = dist;
			prevChange = change;
		} while(count < countMax);
		if (side[iTestSurf[k]] * change < 0.) {
			if (k != k_twoRays) tangent[k] = -tangent[k];
			else if (Norm(tangent[k] - intersectionLine->dir[1]) < Norm(tangent[k] - intersectionLine->dir[0])) tangent[k] = intersectionLine->dir[0];
			else tangent[k] = intersectionLine->dir[1];
		}
		direction += tangent[k];
	}
	direction /= Norm(direction);
	return direction;
}

int CMCNScriptDlg::NonIntersectingSurfaces(SURFACE *s1, SURFACE *s2, double distancePrecision)
{
	VECTOR a = s2->init - s1->init;
	double dist = Norm(a);
	if (s1->type == s2->type) {
		if (s1->type == 1 && fabs(a * s1->dir) <= distancePrecision) return 0; // coinciding planes
		if (s1->type == 2 && dist <= distancePrecision && fabs(s1->R - s2->R) <= distancePrecision) return 0; // coinciding spheres
		bool parallelAxes = s1->type > 2 && Norm(s1->dir & s2->dir) <= anglePrecision;
		if (s1->type == 3 && parallelAxes && Norm(a & s1->dir) <= distancePrecision && fabs(s1->R - s2->R) <= distancePrecision) return 0; // coinciding cylinders
		if (s1->type == 4 && dist <= distancePrecision && parallelAxes && fabs(s1->sin - s2->sin) <= anglePrecision && s1->dir * s2->dir > 0.) return 0; // coinciding cones
	}
	VECTOR testPoint;
	double d[2];
	LINE p;
	SURFACE *S1, *S2;
	for (int i = 0; i < 2; i++) {
		if (!i) {
			S1 = s1;
			S2 = s2;
		}
		else {
			S1 = s2;
			S2 = s1;
		}
		if (S1->type == 1) {
			testPoint = S1->init;
			d[i] = DistanceToSurface(&testPoint, S2);
			if (fabs(d[i]) <= distancePrecision) {
				SURFACE sph;
				sph.type = 2;
				sph.init = S1->init;
				sph.R = 1. + 100. * distancePrecision;
				Intersection(S1, &sph, p, distancePrecision);
				testPoint = LocalToGlobal(&p, 0, 0., distancePrecision);
				d[i] = DistanceToSurface(&testPoint, S2);
				if (fabs(d[i]) <= distancePrecision) {
					testPoint = LocalToGlobal(&p, 0, 0.5 * pi, distancePrecision);
					d[i] = DistanceToSurface(&testPoint, S2);
				}
			}
		}
		else if (S1->type == 2) {
			testPoint = S1->init + S1->R * xDir;
			d[i] = DistanceToSurface(&testPoint, S2);
			if (fabs(d[i]) <= distancePrecision) {
				testPoint = S1->init + S1->R * yDir;
				d[i] = DistanceToSurface(&testPoint, S2);
				if (fabs(d[i]) <= distancePrecision) {
					testPoint = S1->init + S1->R * zDir;
					d[i] = DistanceToSurface(&testPoint, S2);
				}
			}
		}
		else if (S1->type == 3) {
			SURFACE pl;
			pl.type = 1;
			pl.init = S1->init;
			pl.dir = S1->dir;
			pl.D = -pl.dir * pl.init;
			Intersection(S1, &pl, p, distancePrecision);
			testPoint = LocalToGlobal(&p, 0, 0., distancePrecision);
			d[i] = DistanceToSurface(&testPoint, S2);
			if (fabs(d[i]) <= distancePrecision) {
				testPoint = LocalToGlobal(&p, 0, 0.5 * pi, distancePrecision);
				d[i] = DistanceToSurface(&testPoint, S2);
				if (fabs(d[i]) <= distancePrecision) {
					testPoint += (1. + 100. * distancePrecision) * S1->dir;
					d[i] = DistanceToSurface(&testPoint, S2);
				}
			}
		}
		else if (S1->type == 4) {
			testPoint = S1->init;
			d[i] = DistanceToSurface(&testPoint, S2);
			if (fabs(d[i]) <= distancePrecision) {
				SURFACE pl;
				pl.type = 1;
				pl.init = S1->init + (1. + 100. * distancePrecision) * S1->dir;
				pl.dir = S1->dir;
				pl.D = -pl.dir * pl.init;
				Intersection(S1, &pl, p, distancePrecision);
				testPoint = LocalToGlobal(&p, 0, 0., distancePrecision);
				d[i] = DistanceToSurface(&testPoint, S2);
				if (fabs(d[i]) <= distancePrecision) {
					testPoint = LocalToGlobal(&p, 0, 0.5 * pi, distancePrecision);
					d[i] = DistanceToSurface(&testPoint, S2);
				}
			}
		}
	}
	if (d[0] > 0. && d[1] > 0.) return 1;
	else if (d[0] < 0. && d[1] > 0.) return 2;
	else if (d[0] > 0. && d[1] < 0.) return 3;
	else return 4;
}

VECTOR CMCNScriptDlg::SurfaceNormal(SURFACE *s, VECTOR point, double distancePrecision) { // point must be on the surface
	if (s->type == 1) return s->dir; // plane
	VECTOR a = point - s->init;
	if (s->type == 2) return a / Norm(a); // sphere
	if (s->type == 3) { // cylinder
		a -= (a * s->dir) * s->dir;
		return a / Norm(a);
	}
	double d = Norm(a);
	if (d < distancePrecision) return -s->dir; // tip of a cone
	a -= (d / s->cos) * s->dir;
	return a / Norm(a); // surface of a cone
}

SURFACE *fit_s1, *fit_s2;
double fit_Q[3][3];
int fit_type;

int CMCNScriptDlg::Intersection(SURFACE *s1, SURFACE *s2, LINE &p, double distancePrecision)
{
	VECTOR a, u;
	double norm, dist;
	int i, j;
	if (s1->type == 1 && s2->type == 1) { // two planes
		a = s1->dir & s2->dir;
		norm = Norm(a);
		if (norm <= anglePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection (planes are parallel)
		p.type = 1;
		p.dir[0] = a / norm;
		if (fabs(a.z) >= fabs(a.x) && fabs(a.z) >= fabs(a.y)) {
			p.init[0].x = (s1->dir.y * s2->D - s2->dir.y * s1->D) / (s1->dir.x * s2->dir.y - s2->dir.x * s1->dir.y);
			p.init[0].y = (s1->dir.x * s2->D - s2->dir.x * s1->D) / (s1->dir.y * s2->dir.x - s2->dir.y * s1->dir.x);
			p.init[0].z = 0.;
		}
		else if (fabs(a.y) >= fabs(a.z) && fabs(a.y) >= fabs(a.x)) {
			p.init[0].x = (s1->dir.z * s2->D - s2->dir.z * s1->D) / (s1->dir.x * s2->dir.z - s2->dir.x * s1->dir.z);
			p.init[0].y = 0.;
			p.init[0].z = (s1->dir.x * s2->D - s2->dir.x * s1->D) / (s1->dir.z * s2->dir.x - s2->dir.z * s1->dir.x);
		}
		else {
			p.init[0].x = 0.;
			p.init[0].y = (s1->dir.z * s2->D - s2->dir.z * s1->D) / (s1->dir.y * s2->dir.z - s2->dir.y * s1->dir.z);
			p.init[0].z = (s1->dir.y * s2->D - s2->dir.y * s1->D) / (s1->dir.z * s2->dir.y - s2->dir.z * s1->dir.y);
		}
	}
	else if (s1->type == 1 && s2->type == 2 || s1->type == 2 && s2->type == 1) { // a plane and a sphere
		SURFACE *pl, *sph;
		if (s1->type == 1) {
			pl = s1;
			sph = s2;
		}
		else {
			pl = s2;
			sph = s1;
		}
		double distanceFromCenterToPlane = pl->dir * sph->init + pl->D;
		if (fabs(distanceFromCenterToPlane) >= sph->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		p.type = 4;
		p.dir[0] = pl->dir;
		p.init[0] = sph->init - distanceFromCenterToPlane * pl->dir;
		p.coef[0] = sqrt(sph->R * sph->R - distanceFromCenterToPlane * distanceFromCenterToPlane);
		RotationMatrix(p.Q, pl->dir);
	}
	else if (s1->type == 2 && s2->type == 2) { // two spheres
		SURFACE *sph1, *sph2;
		if (s1->R >= s2->R) {
			sph1 = s1;
			sph2 = s2;
		}
		else {
			sph1 = s2;
			sph2 = s1;
		}
		p.dir[0] = sph1->init - sph2->init;
		dist = Norm(p.dir[0]);
		if (dist <= distancePrecision || dist <= fabs(s1->R - s2->R) + distancePrecision || dist >= s1->R + s2->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		p.type = 4;
		p.dir[0] /= dist;
		double dist1 = (sph1->R * sph1->R - sph2->R * sph2->R + dist * dist) / (2. * dist); // always positive, because sph1->R >= sph2->R
		p.init[0] = sph1->init - p.dir[0] * dist1; // the center of the larger sphere is always in the positive half-space relative to the plane containing the intersection circle
		p.coef[0] = sqrt(sph1->R * sph1->R - dist1 * dist1);
		RotationMatrix(p.Q, p.dir[0]);
	}
	else if (s1->type == 1 && s2->type == 3 || s1->type == 3 && s2->type == 1) { // a plane and a cylinder
		SURFACE *pl, *cyl;
		if (s1->type == 1) {
			pl = s1;
			cyl = s2;
		}
		else {
			pl = s2;
			cyl = s1;
		}
		double scalarProduct = pl->dir * cyl->dir;
		if (fabs(scalarProduct) <= anglePrecision) { // parallel
			double distanceFromAxisToPlane = pl->dir * cyl->init + pl->D;
			if (fabs(distanceFromAxisToPlane) >= cyl->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			p.type = 2;
			dist = sqrt(cyl->R * cyl->R - distanceFromAxisToPlane * distanceFromAxisToPlane);
			a = pl->dir & cyl->dir;
			u = cyl->init - distanceFromAxisToPlane * pl->dir;
			p.init[0] = u + a * dist;
			p.init[1] = u - a * dist;
			p.dir[0] = cyl->dir;
			p.dir[1] = cyl->dir;
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		a = pl->dir & cyl->dir;
		if (Norm(a) <= anglePrecision) { // orthogonal; intersection is a circle
			p.type = 4;
			p.dir[0] = pl->dir;
			double distanceFromCenterToPlane = pl->dir * cyl->init + pl->D;
			p.init[0] = cyl->init - pl->dir * distanceFromCenterToPlane;
			p.coef[0] = cyl->R;
			RotationMatrix(p.Q, p.dir[0]);
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		p.type = 6;
		double distanceAlongTheLine = -(pl->dir * cyl->init + pl->D) / scalarProduct;
		p.init[0] = cyl->init + distanceAlongTheLine * cyl->dir;
		p.dir[0] = cyl->dir;
		p.tan = fabs(scalarProduct) / sqrt(1. - scalarProduct * scalarProduct);
		p.coef[0] = cyl->R * cyl->R;
		p.coef[1] = p.tan * p.tan;
		p.Y1 = cyl->R;
		p.Z1 = cyl->R / p.tan;
		RotationMatrix(p.Q, p.dir[0], pl->dir);
	}
	else if (s1->type == 1 && s2->type == 4 || s1->type == 4 && s2->type == 1) { // a plane and a one-sheet cone
		SURFACE *pl, *cone;
		if (s1->type == 1) {
			pl = s1;
			cone = s2;
		}
		else {
			pl = s2;
			cone = s1;
		}
		double scalarProduct = pl->dir * cone->dir;
		double distanceAlongTheLine;
		if (fabs(scalarProduct) > anglePrecision) distanceAlongTheLine = -(pl->dir * cone->init + pl->D) / scalarProduct;
		double distanceFromTipToPlane = pl->dir * cone->init + pl->D;
		if (fabs(scalarProduct) >= cone->sin - anglePrecision && (distanceAlongTheLine <= distancePrecision || fabs(distanceFromTipToPlane) <= distancePrecision)) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		p.dir[0]= cone->dir;
		a = pl->dir & cone->dir;
		if (Norm(a) <= anglePrecision) { // orthogonal; intersection is a circle
			p.type = 4;
			p.init[0] = cone->init + cone->dir * distanceAlongTheLine;
			p.coef[0] = distanceAlongTheLine * cone->tan;
			RotationMatrix(p.Q, p.dir[0]);
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		p.tan = fabs(scalarProduct) / sqrt(1. - scalarProduct * scalarProduct);
		RotationMatrix(p.Q, p.dir[0], pl->dir);
		if (fabs(distanceFromTipToPlane) <= distancePrecision) { // intersection is a pair of rays with a common initial point
			p.type = 3;
			a.x = p.tan;
			a.y = sqrt(cone->tan * cone->tan - p.tan * p.tan);
			a.z = 1.;
			p.init[0] = zero;
			for (i = 0; i < 2; i++) {
				p.dir[i] = LocalToGlobal(&p, 0, a);
				p.dir[i] = p.dir[i] / Norm(p.dir[i]);
				a.y = -a.y;
			}
			p.init[0] = cone->init;
			p.init[1] = cone->init;
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		if (p.tan > cone->tan + anglePrecision) { // intersection is an ellipse
			p.type = 7;
			p.X1 = -distanceAlongTheLine * p.tan * cone->tan / (p.tan + cone->tan);
			p.Y1 = -p.X1 * sqrt((p.tan + cone->tan) / (p.tan - cone->tan));
			p.Z1 = -2. * p.X1 / (p.tan - cone->tan);
			p.coef[0] = -2. * p.X1 * (p.tan + cone->tan);
			p.coef[1] = cone->tan * cone->tan - p.tan * p.tan;
			dist = distanceAlongTheLine * p.tan / (p.tan + cone->tan);
		}
		else if (p.tan >= cone->tan - anglePrecision) { // intersection is a parabola
			p.type = 9;
			dist = 0.5 * distanceAlongTheLine;
			p.X1 = -dist * cone->tan;
			p.Z1 = dist;
			p.coef[0] = -4. * p.X1 * cone->tan;
			p.coef[1] = 0.;
		}
		else { // intersection is a hyperbola
			p.type = 8;
			if (fabs(scalarProduct) <= anglePrecision) { // parallel
				p.X1 = -fabs(distanceFromTipToPlane);
				dist = fabs(distanceFromTipToPlane) / cone->tan;
				p.coef[0] = -2. * p.X1 * cone->tan;
				p.coef[1] = cone->tan * cone->tan;
			}
			else if (distanceAlongTheLine > 0.) {
				dist = distanceAlongTheLine * p.tan / (p.tan + cone->tan);
				p.X1 = -dist * cone->tan;
				p.coef[0] = -2. * p.X1 * (p.tan + cone->tan);
				p.coef[1] = cone->tan * cone->tan - p.tan * p.tan;
			}
			else {
				dist = -distanceAlongTheLine * p.tan / (-p.tan + cone->tan);
				p.X1 = dist * cone->tan;
				p.coef[0] = 2. * p.X1 * (-p.tan + cone->tan);
				p.coef[1] = cone->tan * cone->tan - p.tan * p.tan;
			}
			p.Z1 = dist;
		}
		p.init[0] = cone->init + dist * cone->dir;
	}
	else if (s1->type == 3 && s2->type == 3) { // two cylinders
		a = s1->dir & s2->dir;
		norm =  Norm(a);
		if (norm > anglePrecision) { // not parallel
			u = s1->init - s2->init;
			double distanceBetweenAxes = fabs(u * a) / norm;
			if (distanceBetweenAxes >= s1->R + s2->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			p.type = 11; // non-planar intersection of two cylinders (4 branches)
			p.coef[0] = s1->R * s1->R;
			p.coef[1] = s2->R * s2->R;
			p.coef[2] = distanceBetweenAxes;
			a = s1->dir & s2->dir;
			double sin_beta = Norm(a);
			double product1 = u * s1->dir;
			double product2 = u * s2->dir;
			double cos_beta = s1->dir * s2->dir;
			p.coef[3] = cos_beta;
			double t1 = -(product1 - product2 * cos_beta) / (sin_beta * sin_beta); // t1 is the cylinder 1 axis parameter corresponding to the minimum distance between the two axes
			double t2 = product2 + t1 * cos_beta; // t2 is the cylinder 2 axis parameter corresponding to the minimum distance between the two axes
			p.init[0] = s1->init + t1 * s1->dir;
			u = s2->init + t2 * s2->dir - p.init[0]; // vector directed from the axis of cylinder 1 to the axis of cylinder 2 along the line of closest approach
			p.dir[0] = u / Norm(u);
			RotationMatrix(p.Q, p.dir[0], s1->dir);
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		a = (s1->init - s2->init) & s1->dir;
		double distanceBetweenAxes = Norm(a); // because axes are parallel
		if (distanceBetweenAxes <= fabs(s1->R - s2->R) + distancePrecision || distanceBetweenAxes >= s1->R + s2->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		a /= distanceBetweenAxes; // unit normal vector of the plane parallel to both axes
		p.type = 2;
		SURFACE *cyl1, *cyl2;
		if (s1->R >= s2->R) {
			cyl1 = s1;
			cyl2 = s2;
		}
		else {
			cyl1 = s2;
			cyl2 = s1;
		}
		double scalarProduct = (cyl2->init - cyl1->init) * cyl1->dir; // projection to the axis of cylinder 1
		u = cyl2->init - (cyl1->init + cyl1->dir * scalarProduct); // vector directed from the axis of cylinder 1 to the axis of cylinder 2
		u /= Norm(u);
		double l1 = (distanceBetweenAxes * distanceBetweenAxes + cyl1->R * cyl1->R - cyl2->R * cyl2->R) / (2. * distanceBetweenAxes);
		double h = sqrt(cyl1->R * cyl1->R - l1 * l1);
		VECTOR base = cyl1->init + u * l1;
		p.init[0] = base + a * h;
		p.init[1] = base - a * h;
		p.dir[0] = cyl1->dir;
		p.dir[1] = cyl1->dir;
	}
	else if (s1->type == 2 && s2->type == 3 || s1->type == 3 && s2->type == 2) { // a sphere and a cylinder
		SURFACE *sph, *cyl;
		if (s1->type == 2) {
			sph = s1;
			cyl = s2;
		}
		else {
			sph = s2;
			cyl = s1;
		}
		u = cyl->init - sph->init;
		a = u & cyl->dir;
		double dist = Norm(a);
		if (dist <= cyl->R - sph->R + distancePrecision || dist >= cyl->R + sph->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		if (dist > distancePrecision) { // non-planar intersection of a sphere and a cylinder (4 branches)
			p.type = 10;
			p.coef[0] = cyl->R * cyl->R;
			p.coef[1] = sph->R * sph->R - p.coef[0];
			p.coef[2] = dist;
			p.init[0] = sph->init;
			double projection = u * cyl->dir;
			a = u - projection * cyl->dir;
			p.dir[0] = a / Norm(a);
			RotationMatrix(p.Q, p.dir[0], cyl->dir);
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		p.type = 5;
		p.dir[0] = cyl->dir;
		p.dir[1] = cyl->dir;
		p.coef[0] = cyl->R;
		p.coef[1] = cyl->R;
		double h = sqrt(sph->R * sph->R - cyl->R * cyl->R);
		p.init[0] = sph->init + cyl->dir * h;
		p.init[1] = sph->init - cyl->dir * h;
		RotationMatrix(p.Q, p.dir[0]);
	}
	else if (s1->type == 2 && s2->type == 4 || s1->type == 4 && s2->type == 2) { // a sphere and a one-sheet cone
		SURFACE *sph, *cone;
		if (s1->type == 2) {
			sph = s1;
			cone = s2;
		}
		else {
			sph = s2;
			cone = s1;
		}
		double distanceToCone = DistanceToSurface(&(sph->init), cone);
		if (fabs(distanceToCone) >= sph->R - distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
		a = sph->init - cone->init;
		double distanceToAxis = Norm(a & cone->dir);
		if (distanceToAxis > distancePrecision) { // non-planar intersection of a sphere and a cone (not implemented)
			p.type = 12; 
			return p.type;
		}
		p.type = 5;
		double distanceToTip = Norm(a);
		double beta = asin(distanceToTip * cone->sin / sph->R);
		double l1 = sph->R * cos(cone->angle + beta);
		p.init[0] = cone->init + (distanceToTip + l1) * cone->dir;
		p.coef[0] = sph->R * sin(cone->angle + beta);
		double l2 = sph->R * cos(beta - cone->angle);
		p.init[1] = cone->init + (distanceToTip - l2) * cone->dir;
		p.coef[1] = sph->R * sin(beta - cone->angle);
		if (p.coef[1] <= distancePrecision) p.type = 4;
		else if (p.coef[0] <= distancePrecision) {
			p.type = 4;
			p.init[0] = p.init[1];
			p.coef[0] = p.coef[1];
		}
		p.dir[0] = cone->dir;
		p.dir[1] = cone->dir;
		RotationMatrix(p.Q, p.dir[0]);
	}
	else if (s1->type == 3 && s2->type == 4 || s1->type == 4 && s2->type == 3) { // a cylinder and a one-sheet cone
		SURFACE *cyl, *cone;
		if (s1->type == 3) {
			cyl = s1;
			cone = s2;
		}
		else {
			cyl = s2;
			cone = s1;
		}
		u = cone->init - cyl->init;
		double distanceFromConeTipToCylinderAxis = Norm(u & cyl->dir);
		a = cone->dir & cyl->dir;
		double sin_beta = Norm(a);
		if (distanceFromConeTipToCylinderAxis <= distancePrecision && sin_beta <= anglePrecision) { // common symmetry axis (intersection is a circle)
			p.type = 4;
			p.coef[0] = cyl->R;
			dist = cyl->R / cone->tan;
			p.init[0] = cone->init + dist * cone->dir;
			p.dir[0] = cone->dir;
			RotationMatrix(p.Q, p.dir[0]);
		}
		else {
			if (distanceFromConeTipToCylinderAxis < cyl->R - distancePrecision) { // the cone tip is inside the cylinder: non-planar intersection of a cylinder and a cone (not implemented)
				p.type = 13;
				return p.type;
			}
			if (sin_beta < cone->sin - anglePrecision) { // the angle between projections of the two axes to the plane parallel to the two axes is less than the cone angle: non-planar intersection of a cylinder and a cone (not implemented)
				p.type = 13;
				return p.type;
			}
			double product1 = u * cone->dir;
			double product2 = u * cyl->dir;
			double cos_beta = cone->dir * cyl->dir;
			double t1 = -(product1 - product2 * cos_beta) / (sin_beta * sin_beta); // t1 is the cone axis parameter corresponding to the minimum distance between the two axes
			if (t1 + cyl->R / sin_beta <= distancePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection (the projections of the cone and the cylinder to the plane parallel to the two axes do not intersect)
			double t2 = product2 + t1 * cos_beta; // t2 is the cylinder axis parameter corresponding to the minimum distance between the two axes
			double distanceBetweenAxes = fabs(u * a) / sin_beta;
			// The line that is normal to both axes and crosses both of them contains the radii of circular cross sections with centers at the points corresponding to t1 and t2.
			// Consequently, if the sum of those radii is greater than distanceBetweenAxes, then the cone and the cylinder intersect:
			if (t1 > distancePrecision && cyl->R + t1 * cone->tan > distanceBetweenAxes + distancePrecision) { // non-planar intersection of a cylinder and a cone (not implemented)
				p.type = 13;
				return p.type;
			}
			double phi2 = 0.;
			RotationMatrix(fit_Q, cyl->dir);
			fit_s1 = cone;
			fit_s2 = cyl;
			fit_type = 1;
			double f = Iterate2(t2, phi2, distanceBetweenAxes, 0.5, distancePrecision, anglePrecision, 1e300, 1e300, distancePrecision, -distancePrecision);
			if (f < -distancePrecision) { // non-planar intersection of a cylinder and a cone (not implemented)
				p.type = 13;
			}
			else { // no intersection
				p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision);
			}
		}
	}
	else if (s1->type == 4 && s2->type == 4) { // two one-sheet cones
		double cos_beta = s1->dir * s2->dir;
		double beta = acos(cos_beta);
		a = s2->dir & s1->dir; // If axes of the two cones are not parallel, then vector "a" is normal to the plane parallel to axes of both cones
		double sin_beta = Norm(a);
		u = s2->init - s1->init;
		dist = Norm(u);
		if (dist <= distancePrecision) { // cone tips coincide (either no intersection, or intersection is a pair of rays with a common initial point)
			if (beta <= fabs(s1->angle - s2->angle) + anglePrecision || beta >= s1->angle + s2->angle - anglePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			p.type = 3;
			double b1 = (s2->cos / s1->cos - cos_beta) / sin_beta; // t1 = 1
			double h = sqrt(s1->tan * s1->tan - b1 * b1);
			a = s1->dir & a / sin_beta;
			u = s1->dir + a * b1;
			p.dir[0] = u + a * h;
			p.dir[0] /= Norm(p.dir[0]);
			p.dir[1] = u - a * h;
			p.dir[1] /= Norm(p.dir[1]);
			p.init[0] = s1->init;
			p.init[1] = s1->init;
			LineParameterLimits(&p, distancePrecision);
			return p.type;
		}
		SURFACE *cone1, *cone2;
		VECTOR a2 = u & s1->dir;
		double distanceToCone1Axis = Norm(a2);
		if (distanceToCone1Axis <= distancePrecision && sin_beta <= anglePrecision) { // common symmetry axis (either no intersection, or intersection is a circle)
			double t1 = u * s1->dir;
			if (t1 <= distancePrecision && cos_beta < 0.) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			if (t1 > distancePrecision) { // tip of cone 2 is inside cone 1
				cone1 = s1;
				cone2 = s2;
			}
			else { // tip of cone 1 is inside cone 2 (t1 is negative)
				cone1 = s2;
				cone2 = s1;
				t1 = -t1;
			}
			if (cos_beta > 0. && cone2->sin <= cone1->sin + anglePrecision) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			p.type = 4;
			double tan_alpha2 = cone2->tan;
			if (cos_beta < 0.) tan_alpha2 = -tan_alpha2;
			double t = t1 * tan_alpha2 / (tan_alpha2 - cone1->tan);
			p.coef[0] = t * cone1->tan;
			p.init[0] = cone1->init + t * cone1->dir;
			p.dir[0] = cone1->dir;
			RotationMatrix(p.Q, p.dir[0]);
		}
		else {
			double distanceToCone2 = DistanceToSurface(&(s1->init), s2);
			cone1 = NULL;
			if (distanceToCone2 < -distancePrecision) {
				cone1 = s1;
				cone2 = s2;
			}
			else {
				double distanceToCone1 = DistanceToSurface(&(s2->init), s1);
				if (distanceToCone1 < -distancePrecision) {
					cone1 = s2;
					cone2 = s1;
				}
			}
			if (cone1) { // tip of cone 1 is inside cone 2
				if (cone1->angle < cone2->angle - anglePrecision && beta < cone2->angle - cone1->angle) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
				else { // non-planar intersection of two cones (not implemented)
					p.type = 14;
					return p.type;
				}
			}
			if (beta < s1->angle + s2->angle) { // non-planar intersection of two cones (not implemented)
				p.type = 14;
				return p.type;
			}
			a /= sin_beta;
			VECTOR a1, u1, tmp;
			if (fabs(sin_beta) <= anglePrecision || fabs(u * a) / dist / sin_beta <= anglePrecision) { // projection plane contains both axes
				a1 = s1->dir;
				a2 = s2->dir;
				u1 = u / dist;
			}
			else {
				tmp = s1->dir & a;
				a1 = a & tmp;
				a1 = a1 / Norm(a1);
				tmp = s2->dir & a;
				a2 = a & tmp;
				a2 = a2 / Norm(a2); // unit vector of the projection of cone 2 axis
				tmp = u & a;
				u1 = a & tmp;
				u1 /= Norm(u1); // unit vector of the projection of the direction from cone 1 tip to cone 2 tip
			}
			double angle1 = acos(u1 * a1);
			tmp = u1 & a1;
			double angle2 = acos(u1 * a2);
			VECTOR tmp2 = u1 & a2;
			if (tmp * tmp2 < 0.) angle2 = -angle2;
			double alpha1[2], alpha2[2];
			alpha1[0] = angle1 - s1->angle;
			alpha1[1] = angle1 + s1->angle;
			alpha2[0] = angle2 - s2->angle;
			alpha2[1] = angle2 + s2->angle;
			for (i = 0; i < 2; i++) {
				for (j = 0; j < 2; j++) {
					if (alpha1[i] * alpha2[j] > anglePrecision && fabs(alpha1[i]) < fabs(alpha2[j])) break;
				}
				if (j < 2) break;
			}
			if (i == 2) return (p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision)); // no intersection
			double product1 = u * s1->dir;
			double product2 = u * s2->dir;
			double cos_beta = s1->dir * s2->dir;
			double t1 = (product1 - product2 * cos_beta) / (sin_beta * sin_beta); // t1 is the cone 1 axis parameter corresponding to the minimum distance between the two axes
			double t2 = -product2 + t1 * cos_beta; // t2 is the cone 2 axis parameter corresponding to the minimum distance between the two axes
			double distanceBetweenAxes = fabs(u * a);
			// The line that is normal to both axes and crosses both of them contains the radii of circular cross sections with centers at the points corresponding to t1 and t2.
			// Consequently, if the sum of those radii is greater than distanceBetweenAxes, then the cones intersect:
			if (t1 > distancePrecision && t2 > distancePrecision && t1 * s1->tan + t2 * s2->tan > distanceBetweenAxes + distancePrecision) { // non-planar intersection of two cones (not implemented)
				p.type = 14;
				return p.type;
			}
			double phi2 = 0.;
			RotationMatrix(fit_Q, s2->dir);
			fit_s1 = s1;
			fit_s2 = s2;
			fit_type = 2;
			double f = Iterate2(t2, phi2, distanceBetweenAxes, 0.5, distancePrecision, anglePrecision, 1e300, 1e300, distancePrecision, -distancePrecision);
			if (f < -distancePrecision) { // non-planar intersection of two cones (not implemented)
				p.type = 14;
			}
			else { // no intersection
				p.type = -NonIntersectingSurfaces(s1, s2, distancePrecision);
			}
		}
	}
	LineParameterLimits(&p, distancePrecision);
	return p.type;
}

bool CMCNScriptDlg::SurfacesCoincide(SURFACE *s1, SURFACE *s2, double distancePrecision)
{
	if (s1->type != s2->type) return false;
	if (s1->type == 1) { // two planes
		return (fabs(s1->dir * (s1->init - s2->init)) <= distancePrecision && Norm(s1->dir & s2->dir) <= anglePrecision);
	}
	else if (s1->type == 2) { // two spheres
		return (fabs(s1->R - s2->R) <= distancePrecision && Norm(s1->init - s2->init) <= distancePrecision);
	}
	else if (s1->type == 3) { // two cylinders
		return (fabs(s1->R - s2->R) <= distancePrecision && Norm(s1->dir & s2->dir) <= anglePrecision && Norm(s1->dir & (s1->init - s2->init)) <= distancePrecision);
	}
	else { // two one-sheet cones
		return (Norm(s2->init - s1->init) <= distancePrecision && fabs(s1->angle - s2->angle) <= anglePrecision &&
				Norm(s1->dir & s2->dir) <= anglePrecision && s1->dir * s2->dir > 0.);
	}
}

void CMCNScriptDlg::RotationMatrix(double Q[3][3], VECTOR a)
{	// rotation matrix for a circle
	if (fabs(a.y) > anglePrecision || fabs(a.z) > anglePrecision) { // not normal to the X axis
		if (fabs(a.x) > anglePrecision) { // not parallel to the X axis
			Q[0][0] = a.x - 1. / a.x;
			Q[1][0] = a.y;
			Q[2][0] = a.z;
			double norm = sqrt(Q[0][0] * Q[0][0] + Q[1][0] * Q[1][0] + Q[2][0] * Q[2][0]);
			Q[0][0] /= norm;
			Q[1][0] /= norm;
			Q[2][0] /= norm;
		}
		else { // parallel to the X axis
			Q[0][0] = 1.;
			Q[1][0] = 0.;
			Q[2][0] = 0.;
		}
	}
	else { // normal to the X axis
		Q[0][0] = 0.;
		Q[1][0] = (a.x > 0.) ? 1. : -1.;
		Q[2][0] = 0.;
	}
	Q[0][1] = a.y * Q[2][0] - a.z * Q[1][0];
	Q[1][1] = a.z * Q[0][0] - a.x * Q[2][0];
	Q[2][1] = a.x * Q[1][0] - a.y * Q[0][0];
	Q[0][2] = a.x;
	Q[1][2] = a.y;
	Q[2][2] = a.z;
}

void CMCNScriptDlg::RotationMatrix(double Q[3][3], VECTOR a, VECTOR b)
{
	// a is the unit vector of the symmetry axis of the cylinder or the cone, and b is the plane normal vector
	// In order to prevent calculation of the x' axis normal vector, it must be given in b, i.e., that vector must be normal to a.
	double sinTheta = a * b; // sine of the angle between the axis and the plane
	if (fabs(sinTheta) > anglePrecision) { // not parallel (i.e., b is not normal to a
		if (sinTheta < 0.) sinTheta = -sinTheta; // angle between the two vectors greater than 90 degrees
		else { // angle between the two vectors less than 90 degrees, so make it greater than 90 degrees (invert the normal vector of the plane)
			b.x = -b.x;
			b.y = -b.y;
			b.z = -b.z;
		}
		double cosTheta = sqrt(1. - sinTheta * sinTheta); // cosine of the angle between the axis and the plane
		Q[0][0] = (b.x + a.x * sinTheta) / cosTheta;
		Q[1][0] = (b.y + a.y * sinTheta) / cosTheta;
		Q[2][0] = (b.z + a.z * sinTheta) / cosTheta;
	}
	else { // parallel (i.e., b is normal to a
		Q[0][0] = b.x;
		Q[1][0] = b.y;
		Q[2][0] = b.z;
	}
	Q[0][1] = a.y * Q[2][0] - a.z * Q[1][0];
	Q[1][1] = a.z * Q[0][0] - a.x * Q[2][0];
	Q[2][1] = a.x * Q[1][0] - a.y * Q[0][0];
	Q[0][2] = a.x;
	Q[1][2] = a.y;
	Q[2][2] = a.z;
}

VECTOR CMCNScriptDlg::Rotate(LINE *p, VECTOR point)
{
	VECTOR out;
	out.x = p->Q[0][0] * point.x + p->Q[0][1] * point.y + p->Q[0][2] * point.z;
	out.y = p->Q[1][0] * point.x + p->Q[1][1] * point.y + p->Q[1][2] * point.z;
	out.z = p->Q[2][0] * point.x + p->Q[2][1] * point.y + p->Q[2][2] * point.z;
	return out;
}

VECTOR CMCNScriptDlg::Rotate2(LINE *p, VECTOR point)
{
	VECTOR out;
	out.x = p->Q[0][0] * point.x + p->Q[1][0] * point.y + p->Q[2][0] * point.z;
	out.y = p->Q[0][1] * point.x + p->Q[1][1] * point.y + p->Q[2][1] * point.z;
	out.z = p->Q[0][2] * point.x + p->Q[1][2] * point.y + p->Q[2][2] * point.z;
	return out;
}

void CMCNScriptDlg::MatrixProduct(double Q[3][3], double Q0[3][3])
{
	double Q_bak[3][3];
	memcpy(Q_bak[0], Q[0], 9 * sizeof(double));

	Q[0][0] = Q_bak[0][0] * Q0[0][0] + Q_bak[0][1] * Q0[1][0] + Q_bak[0][2] * Q0[2][0];
	Q[0][1] = Q_bak[0][0] * Q0[0][1] + Q_bak[0][1] * Q0[1][1] + Q_bak[0][2] * Q0[2][1];
	Q[0][2] = Q_bak[0][0] * Q0[0][2] + Q_bak[0][1] * Q0[1][2] + Q_bak[0][2] * Q0[2][2];

	Q[1][0] = Q_bak[1][0] * Q0[0][0] + Q_bak[1][1] * Q0[1][0] + Q_bak[1][2] * Q0[2][0];
	Q[1][1] = Q_bak[1][0] * Q0[0][1] + Q_bak[1][1] * Q0[1][1] + Q_bak[1][2] * Q0[2][1];
	Q[1][2] = Q_bak[1][0] * Q0[0][2] + Q_bak[1][1] * Q0[1][2] + Q_bak[1][2] * Q0[2][2];

	Q[2][0] = Q_bak[2][0] * Q0[0][0] + Q_bak[2][1] * Q0[1][0] + Q_bak[2][2] * Q0[2][0];
	Q[2][1] = Q_bak[2][0] * Q0[0][1] + Q_bak[2][1] * Q0[1][1] + Q_bak[2][2] * Q0[2][1];
	Q[2][2] = Q_bak[2][0] * Q0[0][2] + Q_bak[2][1] * Q0[1][2] + Q_bak[2][2] * Q0[2][2];

}

VECTOR CMCNScriptDlg::LocalToGlobal(LINE *p, int n, VECTOR point)
{
	return p->init[n] + Rotate(p, point);
}

VECTOR CMCNScriptDlg::LocalToGlobal(VECTOR init, double *Q, VECTOR point)
{
	VECTOR out;
	out.x = init.x + Q[0] * point.x + Q[1] * point.y + Q[2] * point.z;
	out.y = init.y + Q[3] * point.x + Q[4] * point.y + Q[5] * point.z;
	out.z = init.z + Q[6] * point.x + Q[7] * point.y + Q[8] * point.z;
	return out;
}

VECTOR CMCNScriptDlg::LocalToGlobal(LINE *p, int n, double t, double distancePrecision)
{
	VECTOR local;
	if (p->type == 1 || p->type == 2) {
		return p->init[n] + p->dir[n] * t;
	}
	else if (p->type == 3) {
		return p->init[0] + (t < 0. ? -p->dir[0] : p->dir[1]) * t;
	}
	else if (p->type < 6) { // parameter t is the azimuthal angle (0 <= t < 2*pi); z = 0
		local.z = 0.;
		local.y = p->coef[n] * sin(t);
		local.x = p->coef[n] * cos(t);
	}
	else if (p->type == 6) { // 0 <= t < 2*pi
		local.z = p->Z1 * sin(t);
		local.y = p->Y1 * cos(t);
		local.x = local.z * p->tan;
	}
	else if (p->type == 7) { // 0 <= t < 2*pi
		local.z = 0.5 * p->Z1 * (1. + sin(t));
		local.y = p->Y1 * cos(t);
		local.x = p->X1 + local.z * p->tan;
	}
	else if (p->type == 8) { // -infinity < t < +infinity (t < 0 corresponds to y' < 0)
		local.z = t * t;
		local.y = t * sqrt(p->coef[0] + p->coef[1] * local.z);
		local.x = p->X1 + local.z * p->tan;
	}
	else if (p->type == 9) { // -infinity < t < +infinity (t < 0 corresponds to y' < 0)
		local.z = t * t;
		local.y = t * sqrt(p->coef[0]);
		local.x = p->X1 + local.z * p->tan;
	}
	else if (p->type == 10) {
		local.z = t;
		double threshold = distancePrecision * (p->coef[1] > 0. ? p->coef[0] : p->coef[0] + p->coef[1]);
		double arg = p->coef[0] - (t - p->coef[2]) * (t - p->coef[2]);
		double y_abs = arg > threshold ? sqrt(arg) : 0.;
		if ((n / 2) * 2 == n) local.y = y_abs;
		else local.y = -y_abs;
		arg = p->coef[1] - p->coef[2] * (2. * t - p->coef[2]);
		double x_abs = arg > threshold ? sqrt(arg) : 0.;
		if (n < 2) local.x = x_abs;
		else local.x = -x_abs;
	}
	else if (p->type == 11) {
		local.z = t;
		double threshold = distancePrecision * (p->coef[0] < p->coef[1] ? p->coef[0] : p->coef[1]);
		double arg = p->coef[0] - t * t;
		double y_abs = arg > threshold ? sqrt(arg) : 0.;
		if (n < 2) local.y = y_abs;
		else local.y = -y_abs;
		arg = p->coef[1] - (t - p->coef[2]) * (t - p->coef[2]);
		double intercept_abs = arg > threshold ? sqrt(arg) : 0.;
		double intercept;
		if ((n / 2) * 2 == n) intercept = intercept_abs;
		else intercept = -intercept_abs;
		double sin_beta = sqrt(1. - p->coef[3] * p->coef[3]);
		local.x = (p->coef[3] * local.y - intercept) / sin_beta;
	}
	return LocalToGlobal(p, n, local);
}

int CMCNScriptDlg::LineCellIntersection(LINE *line, CELL *c, VECTOR &intersectionPoint, int &intersectionSurface, double distancePrecision, int excludeSurface1, int excludeSurface2)
{
	SURFACE *s[MAXSURFACES_CELL];
	char sense[MAXSURFACES_CELL];
	for (int l = 0; l < c->nS; l++) {
		s[l] = surf + c->s[l];
		sense[l] = c->side[l];
	}
	return LineCellIntersection(line, c->nS, s, sense, intersectionPoint, intersectionSurface, distancePrecision, excludeSurface1, excludeSurface2);
}

int CMCNScriptDlg::LineCellIntersection(LINE *line, short int nS, SURFACE **s, char *sense, VECTOR &intersectionPoint, int &intersectionSurface, double distancePrecision, int excludeSurface1, int excludeSurface2)
{
	int l, side, inside = 2; // 1 - a part of the line is inside (i.e., it intersects with one of the other bounding surfaces),
							 // 2 - entire line (or one branch of it) is inside (i.e., it does not intersect with any of the other bounding surfaces)
	int branch[NROOTS_MAX];
	int branch_inside[NROOTS_MAX];
	int nBranches = 1, nBranches_inside;
	if (line->type == 2 || line->type == 5) nBranches = 2;
	else if (line->type > 9) nBranches = 4;
	if (nBranches > 1) {
		for (l = 0; l < nBranches; l++) branch_inside[l] = 2;
		nBranches_inside = nBranches;
	}
	double intersectionPoints[NROOTS_MAX];
	VECTOR intersectionPoints0[NROOTS_MAX], test;
	double t, step0, step1, step, tMin, tMax;
	if (line->type < 4 || line->type == 8 || line->type == 9) { // line parameter is unlimited
		tMin = -line->tMin;
		tMax = line->tMin;
	}
	else { // line parameter is limited
		tMin = line->tMin;
		tMax = line->tMax;
	}
	if (line->type < 4 || line->type > 9) step0 = distancePrecision; // line parameter is distance
	else if (line->type < 8) step0 = anglePrecision; // line parameter is angle
	else step1 = sqrt(distancePrecision); // line parameter is square root of distance
	for (l = 0; l < nS; l++) {
		if (l == excludeSurface1 || l == excludeSurface2) continue;
		int nIntersectionPoints = LineSurfaceIntersection(line, s[l], branch, distancePrecision, intersectionPoints, intersectionPoints0, side);
		if (nIntersectionPoints) {
			if (nIntersectionPoints < 0) nIntersectionPoints = -nIntersectionPoints;
			int iPoint, iSurf;
			for (iPoint = 0; iPoint < nIntersectionPoints; iPoint++) {
				test = intersectionPoints0[iPoint];
				int iAttempt = 0;
				double dist, dist2;
				int iSurf_edge = -1;
RETEST:
				inside = 1; // if the line intersects with one of the bounding surfaces, it cannot be entirely inside the cell
				if (nBranches > 1 && branch_inside[branch[iPoint]] == 2) {
					branch_inside[branch[iPoint]] = 1;
					nBranches_inside--;
				}
				for (iSurf = 0; iSurf < nS && inside; iSurf++) {
					if (iSurf == excludeSurface1 || iSurf == excludeSurface2 || iSurf == l || iSurf == iSurf_edge) continue; // iAttempt = 1 or 2 corresponds to the test point shifted from s[l] along the tangent of the line forwards or backwards, respectively
					dist = sense[iSurf] * DistanceToSurface(&test, s[iSurf]);
					if (dist < -distancePrecision) inside = 0;
					else if (iSurf_edge < 0 && dist <= distancePrecision) iSurf_edge = iSurf;
				}
				if (inside && iSurf_edge < 0) { // the intersection point is on the surface (face) of the cell
					intersectionPoint = intersectionPoints0[iPoint];
					intersectionSurface = l;
					break;
				}
				if (iSurf_edge >= 0 && (!iAttempt && inside || iAttempt == 1 && !inside)) { // the intersection point is on the edge of the cell (i.e., if "line" also represents an edge, then it is a point of intersection of at least four faces)
RETEST2:
					t = intersectionPoints[iPoint];
					if (!iAttempt) {
						if (line->type == 8 || line->type == 9) {
							if (2. * fabs(t ) > 10. * distancePrecision && 4. * t * t > 10. * distancePrecision) step0 = distancePrecision / (2. * fabs(t));
							else step0 = step1;
						}
						step = step0;
					}
					else step = -step0;
					dist2 = 0.;
					do {
						step *= 2.;
						t += step;
						test = LocalToGlobal(line, branch[iPoint], t, distancePrecision);
						dist = sense[iSurf_edge] * DistanceToSurface(&test, s[iSurf_edge]);
						if (fabs(dist) > distancePrecision) dist2 = sense[l] * DistanceToSurface(&test, s[l]);
					} while((fabs(dist) <= distancePrecision || fabs(dist2) <= distancePrecision) && t >= tMin && t <= tMax);
					iAttempt++;
					if (dist > distancePrecision && dist2 > distancePrecision) goto RETEST;
					else if (iAttempt == 1) goto RETEST2;
					else inside = 0;
				}
				if (inside) { // a part of the line is inside the cell
					break;
				}
			}
			if (inside) { // a part of the line is inside the cell
				break;
			}
		}
		else if (!side && nBranches > 1) { // in this case, all branches can be grouped into (a) branches that are on both sides of surface s[l] and do not intersect with it, and (b) branches that lie on surface s[l]
			if (!nBranches_inside) continue;
			int n;
			for (n = 0; n < nBranches; n++) {
				if (branch_inside[n] != 2) continue;
				int pos = LinePositionRelativeToSurface(line, n, s[l], distancePrecision);
				if (pos * sense[l] < 0) { // the entire branch is outside the cell
					branch_inside[n] = 0;
					nBranches_inside--;
				}
			}
		}
		else if (side * sense[l] < 0) { // the entire line is outside the cell
			inside = 0;
			break;
		}
	}
	if (nBranches > 1 && inside == 2 && !nBranches_inside) return 0;
	return inside;
}

int CMCNScriptDlg::LineSurfaceIntersection(LINE *curve, SURFACE *surface, int *branch, double distancePrecision, double *param, VECTOR *intersectionPoints, int &side)
{	// "curve" is the intersection line of two surfaces.
	// "side" is defined only for the case of no intersection and indicates the side of the surface where the curve is (if the curve consists of 2 or more branches, which are on both sides of the surface, then side = 0).
	int i, j, k = 0, nBranches;
	int sideMin = 0, sideMax = 0;
	side = 0;
	if (curve->type < 1 || curve->type > 11) return curve->type; // no intersection or not implemented
//							else if (curve->type < 4 && curve->type > 1) { // for testing purposes
	else if (curve->type < 4) { // one or two straight lines (intersection points calculated analytically)
		nBranches = curve->type == 1 ? 1 : 2; // for the case curve->type = 3, the true number of branches is 1 (the ray selection is based on the sign of the line parameter)
		for (j = 0; j < nBranches; j++) {
			int k_prev = k;
			if (side > sideMax) sideMax = side;
			else if (side < sideMin) sideMin = side;
			if (surface->type == 1) { // plane
				double product1 = surface->dir * curve->dir[j];
				double product2 = surface->dir * curve->init[j];
				double dist = product2 + surface->D;
				if (fabs(product1) <= anglePrecision) {
					if (dist > distancePrecision) side = 1;
					else if (dist < -distancePrecision) side = -1;
					else side = 0;
					continue;
				}
				double t = -dist / product1;
				if (curve->type == 3 && t <= distancePrecision) {
					if (dist > distancePrecision) side = 1;
					else if (dist < -distancePrecision) side = -1;
					else {
						t = 1. + 100. * distancePrecision;
						dist = surface->dir * (curve->init[j] + curve->dir[j] * t) + surface->D;
						if (dist > distancePrecision) side = 1;
						else if (dist < -distancePrecision) side = -1;
						else side = 0;
					}
					continue;
				}
				side = 0;
				branch[k] = j;
				param[k] = t;
				intersectionPoints[k] = curve->init[j] + curve->dir[j] * t; k++;
			}
			else if (surface->type == 2) { // sphere
				VECTOR a = surface->init - curve->init[j];
				double product = a * curve->dir[j];
				double norm2 = a * a;
				double D = product * product - norm2 + surface->R * surface->R;
				side = 1;
				if (D <= 0.) continue;
				D = sqrt(D);
				if (D <= distancePrecision) continue;
				double t = product - D;
				if (curve->type == 3 && t <= distancePrecision) continue;
				side = 0;
				branch[k] = j;
				param[k] = t;
				intersectionPoints[k] = curve->init[j] + curve->dir[j] * t; k++;
				t = product + D;
				if (curve->type == 3 && t <= distancePrecision) continue;
				branch[k] = j;
				param[k] = t;
				intersectionPoints[k] = curve->init[j] + curve->dir[j] * t; k++;
			}
			else if (surface->type == 3) { // cylinder
				VECTOR a = curve->dir[j] & surface->dir;
				double sin_beta = Norm(a);
				VECTOR u = curve->init[j] - surface->init;
				double product1 = u * curve->dir[j];
				if (fabs(sin_beta) <= anglePrecision) {
					double dist = Norm(u - product1 * curve->dir[j]);
					if (dist > surface->R + distancePrecision) side = 1;
					else if (dist < surface->R - distancePrecision) side = -1;
					continue;
				}
				double dist = fabs(u * a) / sin_beta;
				side = 1;
				if (dist >= surface->R - distancePrecision) continue;
				double product2 = u * surface->dir;
				double cos_beta = curve->dir[j] * surface->dir;
				double t1 = -(product1 - product2 * cos_beta) / (sin_beta * sin_beta); // t1 is the line parameter corresponding to the minimum distance between the line and the axis of the cylinder
				double D = sqrt(surface->R * surface->R - dist * dist) / sin_beta;
				double t = t1 - D;
				if (curve->type == 3 && t <= distancePrecision) continue;
				side = 0;
				branch[k] = j;
				param[k] = t;
				intersectionPoints[k] = curve->init[j] + curve->dir[j] * t; k++;
				t = t1 + D;
				if (curve->type == 3 && t <= distancePrecision) continue;
				branch[k] = j;
				param[k] = t;
				intersectionPoints[k] = curve->init[j] + curve->dir[j] * t; k++;
			}
			else if (surface->type == 4) { // cone
				VECTOR d = curve->init[j] - surface->init;
				double vh = curve->dir[j] * surface->dir;
				double vd = curve->dir[j] * d;
				double dh = d * surface->dir;
				double t;
				if (Norm(d - vd * curve->dir[j]) <= distancePrecision) { // the line passes through the cone tip
					if (fabs(vh) < surface->cos - anglePrecision) { // the line is outside the cone
						side = 1;
						continue;
					}
					if (fabs(vh) <= surface->cos + anglePrecision) { // the line lies on the cone surface
						side = 0;
						continue;
					}
					// t is the line parameter corresponding to the tip of the cone
					if (fabs(curve->dir[j].x) >= fabs(curve->dir[j].y) && fabs(curve->dir[j].x) >= fabs(curve->dir[j].z)) t = (surface->init.x - curve->init[j].x) / curve->dir[j].x;
					if (fabs(curve->dir[j].y) >= fabs(curve->dir[j].x) && fabs(curve->dir[j].y) >= fabs(curve->dir[j].z)) t = (surface->init.y - curve->init[j].y) / curve->dir[j].y;
					else t = (surface->init.z - curve->init[j].z) / curve->dir[j].z;
					if (t <= distancePrecision && curve->type == 3) { // the ray is inside the cone
						side = -1;
						continue;
					}
					side = 0;
					branch[k] = j;
					param[k] = t;
					intersectionPoints[k] = surface->init; k++;
					continue;
				}
				double tan2 = 1. + surface->tan * surface->tan;
				double a = curve->dir[j] * curve->dir[j] - tan2 * vh * vh;
				double b = 2. * (vd - tan2 * vh * dh);
				double c = d * d - tan2 * dh * dh;
				double D = b * b - 4. * a * c;
				bool neg = D < 0.;
				D = sqrt(fabs(D)) / (2. * a);
				int side_ray;
				side = 1;
				if (curve->type == 3) {
					double dist = DistanceToSurface(curve->init + j, surface);
					VECTOR newPoint = curve->init[j] + (1. + 100. * distancePrecision) * curve->dir[j];
					if (fabs(dist) <= distancePrecision) dist = DistanceToSurface(&newPoint, surface);
					side_ray = dist > 0 ? 1 : -1;
				}
				if (fabs(D) > distancePrecision) {
					if (neg) {
						side = curve->type != 3 ? 1 : side_ray;
						continue;
					}
					double d = 4. * a * c / (b * b);
					bool TaylorSeries;
					if (fabs(d) <= pow(fabs(12.8 * b / c * distancePrecision), 1./3.)) {
						TaylorSeries = true;
						t = -2. * c / b * (0.5 + 0.125 * d + 0.0625 * d * d + 0.0390625 * d * d * d);
					}
					else {
						TaylorSeries = false;
						t = -b / (2. * a) - D;
					}
					if (curve->type == 3 && t <= distancePrecision) {
						side = side_ray;
						continue;
					}
					VECTOR test = curve->init[j] + curve->dir[j] * t;
					if ((test - surface->init) * surface->dir > distancePrecision) { // Check if the intersection point is in the correct half of the double cone
						side = 0;
						branch[k] = j;
						param[k] = t;
						intersectionPoints[k] = test; k++;
					}
					if (TaylorSeries) t = -b / a - t;
					else t = -b / (2. * a) + D;
					if (curve->type == 3 && t <= distancePrecision) {
						if (side) side = side_ray;
						continue;
					}
					test = curve->init[j] + curve->dir[j] * t;
					if ((test - surface->init) * surface->dir > distancePrecision) { // Check if the intersection point is in the correct half of the double cone
						side = 0;
						branch[k] = j;
						param[k] = t;
						intersectionPoints[k] = test; k++;
					}
				}
			}
			if (k - k_prev == 2) { // If the two intersection points are too close to each other, discard them
				double tAvg = 0.5 * (param[k-2] + param[k-1]);
				VECTOR testAvg = curve->init[j] + curve->dir[j] * tAvg;
				if (fabs(DistanceToSurface(&testAvg, surface)) <= distancePrecision) {
					k = k_prev;
					side = 1;
				}
			}
		}
		if (curve->type == 3) {
			for (i = 0; i < k; i++) {
				if (branch[i] == 0) param[0] = -param[0];
				else branch[i] = 0;
			}
			if (fabs(DistanceToSurface(curve->init, surface)) <= distancePrecision) { // In this case, at least one of the two rays would have been discarded as non-intersecting, even if they are on opposite sides of the surface. Correct this error by adding one more intersection point.
				VECTOR normal = SurfaceNormal(surface, curve->init[0], distancePrecision);
				double cos1 = normal * curve->dir[0];
				double cos2 = normal * curve->dir[1];
				if (surface->type == 4 && Norm(curve->init[0] - surface->init) <= distancePrecision) {
					cos1 += surface->cos;
					cos2 += surface->cos;
				}
				if (fabs(cos1) > anglePrecision && fabs(cos2) > anglePrecision && cos1 * cos2 < 0.) {
					side = 0;
					branch[k] = 0;
					param[k] = 0;
					intersectionPoints[k] = curve->init[0]; k++;
				}
			}
		}
	}
//					else if (curve->type < 6 && surface->type == -1) { // for testing purposes
	else if (curve->type < 6 && surface->type == 1) { // one or two circles, intersection with a plane (intersection points calculated analytically)
		SURFACE s_aux;
		LINE c_aux;
		VECTOR a;
		s_aux.dir = curve->dir[0];
		nBranches = curve->type == 4 ? 1 : 2;
		VECTOR x_prime = Init(curve->Q[0][0], curve->Q[1][0], curve->Q[2][0]);
		for (j = 0; j < nBranches; j++) {
			if (side > sideMax) sideMax = side;
			else if (side < sideMin) sideMin = side;
			s_aux.type = 1;
			s_aux.init = curve->init[j];
			s_aux.D = -s_aux.dir * s_aux.init;
			int intersectionType = Intersection(surface, &s_aux, c_aux, distancePrecision); // c_aux is the intersection line of the two planes
			if (!intersectionType) continue; // no intersection (planes coincide)
			if (intersectionType == -1 || intersectionType == -2) { // parallel planes (circle above the plane)
				side = 1;
				continue;
			}
			if (intersectionType == -3 || intersectionType == -4) { // parallel planes (circle below the plane)
				side = -1;
				continue;
			}
			a = c_aux.init[0] - curve->init[j]; // vector directed from the center of the circle (curve->init[j]) to the starting point of the intersection line of the two planes
			a -= (a * c_aux.dir[0]) * c_aux.dir[0]; // vector directed from the center of the circle (curve->init[j]) to the closest point of the intersection line of the two planes (the "symmetry point")
			double R2 = curve->coef[j] * curve->coef[j]; // squared radius of the circle
			double D = R2 - a * a;
			if (D <= distancePrecision * curve->coef[j]) { // no intersection
				if (a * surface->dir < 0.) side = 1;
				else side = -1;
				continue;
			}
			a += curve->init[j]; // the symmetry point
			D = sqrt(D); // distance from the symmetry point to the two points of intersection of the circle and the plane
			side = 0;
			intersectionPoints[k] = a + D * c_aux.dir[0];
			VECTOR b = intersectionPoints[k] - curve->init[j];
			param[k] = acos(x_prime * b / curve->coef[j]);
			if ((x_prime & b) * curve->dir[j] < 0.) param[k] = 2. * pi - param[k]; 
			branch[k] = j;
			k++;
			intersectionPoints[k] = a - D * c_aux.dir[0];
			b = intersectionPoints[k] - curve->init[j];
			param[k] = acos(x_prime * b / curve->coef[j]);
			if ((x_prime & b) * curve->dir[j] < 0.) param[k] = 2. * pi - param[k]; 
			branch[k] = j;
			k++;
		}
	}
//					else if (curve->type < 10 && surface->type == -1) { // for testing purposes
	else if (curve->type < 10 && surface->type == 1) { // ellipse, hyperbola or parabola, intersection with a plane (intersection points calculated analytically)
		// Calculate the parameters of the plane containing the curve:
		SURFACE curvePlane;
		curvePlane.type = 1;
		VECTOR curve_Xdir = Init(curve->Q[0][0], curve->Q[1][0], curve->Q[2][0]); curve_Xdir /= Norm(curve_Xdir);
		VECTOR curve_Zdir = Init(curve->Q[0][2], curve->Q[1][2], curve->Q[2][2]); curve_Zdir /= Norm(curve_Zdir);
		double co = 1. / sqrt(1 + curve->tan * curve->tan);
		double si = curve->tan * co;
		curvePlane.dir = co * curve_Xdir - si * curve_Zdir; curvePlane.dir /= Norm(curvePlane.dir);
		if (curve->type == 6) curvePlane.init = curve->init[0];
		else if (curve->type == 7) curvePlane.init = curve->init[0] + (-curve->X1 / curve->tan) * curve->dir[0];
		else curvePlane.init = LocalToGlobal(curve, 0, 0., distancePrecision);
		curvePlane.D = -curvePlane.dir * curvePlane.init;
		// Calculate the intersection line of the two planes:
		LINE intersectionLine;
		int intersectionType = Intersection(surface, &curvePlane, intersectionLine, distancePrecision);
		if (intersectionType == -1 || intersectionType == -2) { // parallel planes (curve above the plane)
			side = 1;
		}
		else if (intersectionType == -3 || intersectionType == -4) { // parallel planes (curve below the plane)
			side = -1;
		}
		if (intersectionType <= 0) return 0;
		// Parameters of the intersection line of the two planes in the local ("natural") coordinate system:
		VECTOR dir = Rotate2(curve, intersectionLine.dir[0]);
		VECTOR init = Rotate2(curve, intersectionLine.init[0] - curve->init[0]);
		if (curve->type < 8) { // ellipse
			// Ellipse parameters in the local ("natural") coordinate system:
			double a0 = (curve->type == 6 ? 0. : 0.5 * curve->Z1);
			double a = (curve->type == 6 ? curve->Z1 : 0.5 * curve->Z1);
			double b = curve->Y1;
			if (fabs(dir.y) <= anglePrecision) { // the intersection line is normal to the Y-prime axis
				if (fabs(init.y) >= b - distancePrecision) return 0;
				param[0] = acos(init.y / b);
				intersectionPoints[0] = LocalToGlobal(curve, 0, param[0], distancePrecision);
				branch[0] = 0;
				param[1] = 2. * pi - param[0];
				intersectionPoints[1] = LocalToGlobal(curve, 0, param[1], distancePrecision);
				branch[1] = 0;
				return 2;
			}
			double tg = dir.z / dir.y;
			double c = init.z - init.y * tg - a0;
			double d = b * tg;
			double e = a * a + d * d;
			double D = e - c * c;
			if (D <= 0.) {
				double dist = DistanceToSurface(&(curvePlane.init), surface);
				if (dist < -distancePrecision) side = -1;
				else if (dist > distancePrecision) side = 1;
				return 0;
			}
			D = sqrt(D);
			double cosine[2];
			cosine[0] = (-d * c - a * D) / e;
			cosine[1] = (-d * c + a * D) / e;
			if (fabs(cosine[0]) > 1. || fabs(cosine[1]) > 1. || fabs(cosine[0] - cosine[1]) <= anglePrecision) {
				double dist = DistanceToSurface(&(curvePlane.init), surface);
				if (dist < -distancePrecision) side = -1;
				else if (dist > distancePrecision) side = 1;
				return 0;
			}
			for (i = 0; i < 2; i++) {
				double angle1, angle2, test, test1, test2;
				angle1 = acos(cosine[i]);
				angle2 = 2. * pi - angle1;
				test = init.z - init.y * tg + d * cosine[i];
				test1 = a0 + a * sin(angle1);
				test2 = a0 + a * sin(angle2);
				if (fabs(test1 - test) <= fabs(test2 - test)) param[k] = angle1;
				else param[k] = angle2;
				k++;
			}
			if (fabs(param[1] - param[0]) <= anglePrecision) {
				double dist = DistanceToSurface(&(curvePlane.init), surface);
				if (dist < -distancePrecision) side = -1;
				else if (dist > distancePrecision) side = 1;
				return 0;
			}
			intersectionPoints[0] = LocalToGlobal(curve, 0, param[0], distancePrecision);
			intersectionPoints[1] = LocalToGlobal(curve, 0, param[1], distancePrecision);
			branch[0] = 0;
			branch[1] = 0;
			return 2;
		}
		else { // hyperbola or parabola
			double a = dir.z * dir.z * curve->coef[1] - dir.y * dir.y;
			double b = dir.z * curve->coef[0] + 2. * dir.z * curve->coef[1] * init.z - 2. * init.y * dir.y;
			double c = init.z * (curve->coef[0] + curve->coef[1] * init.z) - init.y * init.y;
			double D = b * b - 4. * a * c;
			if (D <= 0.) {
				double dist = DistanceToSurface(&(curvePlane.init), surface);
				VECTOR newPoint = LocalToGlobal(curve, 0, sqrt(1. + 100. * distancePrecision), distancePrecision);
				if (dist <= distancePrecision) dist = DistanceToSurface(&newPoint, surface);
				if (dist < -distancePrecision) side = -1;
				else if (dist > distancePrecision) side = 1;
				return 0;
			}
			D = sqrt(D) / (2. * a);
			int imax = 1;
			bool TaylorSeries;
			double t;
			if (fabs(a) <= anglePrecision * anglePrecision) imax = 0; // linear equation
			else if (fabs(D) <= distancePrecision) imax = -1; // no intersection
			else {
				double d = 4. * a * c / (b * b);
				if (fabs(d) <= pow(fabs(12.8 * b / c * distancePrecision), 1./3.)) {
					TaylorSeries = true;
					t = -2. * c / b * (0.5 + 0.125 * d + 0.0625 * d * d + 0.0390625 * d * d * d);
				}
				else TaylorSeries = false;
			}
			for (i = 0; i <= imax; i++) {
				double par;
				if (!imax) par = -c / b;
				else if (!i) {
					if (TaylorSeries) par = t;
					else par = -b / (2. * a) - D;
				}
				else if (TaylorSeries) par = -b / a - t;
				else par = -b / (2. * a) + D;
				double test1 = init.z + par * dir.z;
				if (test1 <= distancePrecision || curve->coef[0] + curve->coef[1] * test1 <= distancePrecision) continue;
				param[k] = sqrt(test1);
				double test2 = init.y + par * dir.y;
				double test3 = param[k] * sqrt(curve->coef[0] + curve->coef[1] * test1);
				if (fabs(-test3 - test2) < fabs(test3 - test2)) param[k] = -param[k];
				intersectionPoints[k] = LocalToGlobal(curve, 0, param[k], distancePrecision);
				branch[k] = 0;
				k++;
			}
			if (!k) {
				double dist = DistanceToSurface(&(curvePlane.init), surface);
				VECTOR newPoint = LocalToGlobal(curve, 0, sqrt(1. + 100. * distancePrecision), distancePrecision);
				if (dist <= distancePrecision) dist = DistanceToSurface(&newPoint, surface);
				if (dist < -distancePrecision) side = -1;
				else if (dist > distancePrecision) side = 1;
			}
			return k;
		}
	}
//					else if (curve->type < 6 && surface->type == -2) { // for testing purposes
	else if (curve->type < 6 && surface->type == 2) { // one or two circles, intersection with a sphere (intersection points calculated analytically)
		SURFACE s_aux;
		LINE c_aux;
		VECTOR a;
		s_aux.dir = curve->dir[0];
		nBranches = curve->type == 4 ? 1 : 2;
		VECTOR x_prime = Init(curve->Q[0][0], curve->Q[1][0], curve->Q[2][0]);
		for (j = 0; j < nBranches; j++) {
			if (side > sideMax) sideMax = side;
			else if (side < sideMin) sideMin = side;
			s_aux.type = 1;
			s_aux.init = curve->init[j];
			s_aux.D = -s_aux.dir * s_aux.init;
			int intersectionType = Intersection(surface, &s_aux, c_aux, distancePrecision); // c_aux is the intersection line of the sphere and the plane containing the circle
			if (!c_aux.type) { // no intersection
				side = 1;
				continue; 
			}
			a = c_aux.init[0] - curve->init[j]; // vector directed from the center of the original circle to the center of the auxiliary circle
			double D = Norm(a); // distance between the centers of the two circles
			if (D <= curve->coef[j] - c_aux.coef[0] + distancePrecision || D >= curve->coef[j] + c_aux.coef[0] - distancePrecision) {
				side = 1; // no intersection, the circle is outside the sphere (the intersection of the sphere with the plane of the circle is inside the circle, or distance between their centers is greater than the sum of their radii)
				continue;
			} else if (D <= c_aux.coef[0] - curve->coef[j] + distancePrecision) {
				side = -1;
				continue; // no intersection, the circle is inside the sphere (the circle is inside the intersection of the sphere with the plane of the circle)
			}
			a /= D; // unit vector
			D = 0.5 * (D - (c_aux.coef[0] * c_aux.coef[0] - curve->coef[j] * curve->coef[j]) / D); // distance from the center of the original circle to the "symmetry point", i.e., the point of intersection of the line joining the centers of the two circles and the line joining the two intersection points of the circle and the sphere
			VECTOR sym = curve->init[j] + D * a; // the symmetry point
			a = a & s_aux.dir; // unit vector of the line joining the two intersection points
			D = sqrt(curve->coef[j] * curve->coef[j] - D * D); // distance from the symmetry point to the two points of intersection of the circle and the sphere
			intersectionPoints[k] = sym + D * a;
			VECTOR b = intersectionPoints[k] - curve->init[j];
			param[k] = acos(x_prime * b / curve->coef[j]);
			if ((x_prime & b) * curve->dir[j] < 0.) param[k] = 2. * pi - param[k]; 
			branch[k] = j;
			k++;
			intersectionPoints[k] = sym - D * a;
			b = intersectionPoints[k] - curve->init[j];
			param[k] = acos(x_prime * b / curve->coef[j]);
			if ((x_prime & b) * curve->dir[j] < 0.) param[k] = 2. * pi - param[k]; 
			branch[k] = j;
			k++;
		}
	}
	else { // intersection points calculated numerically (using the simplex method) by minimizing the distance between the curve and the surface
		// When roots_curve is a parabola and roots_surface is a plane, incorrect results are possible after applying the simplex method.
		// However, normally this does not happen, because intersection points of a parabola and a plane are calculated analytically.
		ROOTS_PARMS parms;
		parms.curve = curve;
		parms.surface = surface;
		parms.distancePrecision = distancePrecision;
		nBranches = 1;
		if (curve->type > 9) nBranches = 4;
		else if (curve->type == 5 || curve->type == 2) nBranches = 2;
		double roots[NROOTS_MAX];
		for (parms.branchNo = 0; parms.branchNo < nBranches; parms.branchNo++) {
			if (side > sideMax) sideMax = side;
			else if (side < sideMin) sideMin = side;
			int nRoots = Roots(roots, &parms, distancePrecision, curve->tMin, curve->tMax, (curve->type > 3 && curve->type < 8), 0.1 * distancePrecision, distancePrecision, maxDistance_ratio * distancePrecision);
			if (!nRoots) {
				side = LinePositionRelativeToSurface(parms.curve, parms.branchNo, parms.surface, distancePrecision);
				continue;
			}
			bool maxNumberExceeded = (nRoots < 0);
			if (nRoots < 0) nRoots = -nRoots;
			if (curve->type < 10) {
				for (j = 0; j < nRoots; j++, k++) {
					if (k == NROOTS_MAX) return -NROOTS_MAX;
					intersectionPoints[k] = LocalToGlobal(curve, parms.branchNo, roots[j], distancePrecision);
					branch[k] = parms.branchNo;
					param[k] = roots[j];
				}
			}
			else {
				int k0 = k;
				for (j = 0; j < nRoots; j++) {
					VECTOR point = LocalToGlobal(curve, parms.branchNo, roots[j], distancePrecision);
					for (i = 0; i < k0 && Norm(point - intersectionPoints[i]) > distancePrecision; i++);
					if (i == k) {
						if (k == NROOTS_MAX) return -NROOTS_MAX;
						intersectionPoints[k] = point;
						branch[k] = parms.branchNo;
						param[k] = roots[j];
						k++;
					}
				}
			}
			if (maxNumberExceeded) return -NROOTS_MAX;
		}
	}
	if (side > sideMax) sideMax = side;
	if (side < sideMin) sideMin = side;
	side = sideMin + sideMax;
	return k;
}

double CMCNScriptDlg::FitFunc_ConeCylinder(double t2, double phi2) { // fit_type = 1: fit_s1 - cone, fit_s2 - cylinder; fit_type = 2: two cones
	if (fit_type < 0 || fit_type > 2) return 0.;
	VECTOR a = fit_s2->init + t2 * fit_s2->dir;
	if (fit_type == 2 && t2 < distancePrecision) return DistanceToSurface(&a, fit_s1);
	double R = fit_type == 1 ? fit_s2->R : t2 * fit_s2->tan;
	VECTOR c = Init(R * cos(phi2), R * sin(phi2), 0.);
	VECTOR b = LocalToGlobal(a, &(fit_Q[0][0]), c);
	return DistanceToSurface(&b, fit_s1);
}

double CMCNScriptDlg::Roots_func(double t, void *parms) {
	double t0;
	ROOTS_PARMS *p = (ROOTS_PARMS *)parms;
	if (p->limited) t0 = p->transform_factor * atan(t) + p->transform_term;
	else t0 = t;
	double value = DistanceToSurface(t0, parms);
	if (p->searchForMinimum) return fabs(value);
	else return -fabs(value);
}

int CMCNScriptDlg::Roots(double *roots, void *parms, double step0, double tMin, double tMax, bool periodic, double t_precision, double f_precision, double f_max) {
//	Finds up to NROOTS_MAX first-degree roots of a smooth function  (used for locating intersection points of a 3D curve and a surface)
	ROOTS_PARMS *p = (ROOTS_PARMS *)parms;
	double f[2], f1, f2;
	double mult = 10.;
	double t_limit;
	p->limited = (tMin < tMax);
	if (!p->limited) {
		t_limit = fabs(tMin) > fabs(tMax) ? fabs(tMin) : fabs(tMax);
		periodic = false;
	}
	else {
		t_limit = 1e200;
		f_max = 1e200;
	}
	int nRoots = 0;
	p->searchForMinimum = true;
	double root_increment, root_increment0, tMin_new, tMax_new;
	double lastExtremum[2], diff;
	if (p->limited) {
		double halfRange = 0.5 * (tMax - tMin);
		p->transform_factor = (tMax - tMin) / pi;
		f[0] = DistanceToSurface(tMin, parms);
		root_increment = 0.5 * step0;
		do {
			root_increment *= 2.;
			f1 = DistanceToSurface(tMax - root_increment, parms);
		} while(root_increment < halfRange && fabs(f1 - f[0]) < mult * f_precision);
		if (root_increment >= halfRange) return 0;
		root_increment = 0.5 * step0;
		do {
			root_increment *= 2.;
			f2 = DistanceToSurface(tMin + root_increment, parms);
		} while(root_increment < halfRange && fabs(f2 - f[0]) < mult * f_precision);
		if (root_increment >= halfRange) return 0;
		f[0] = fabs(f[0]);
		if (fabs(f1) > f[0] && fabs(f2) > f[0]) {
			if (f1 * f2 < 0.) { // a first-degree root
				roots[0] = tMin;
				nRoots = 1;
			}
			p->transform_term = tMin;
			double shift = -0.5 * (tMax - tMin);
			tMax_new = tMax + shift;
			tMin_new = tMin + shift;
			f[1] = f[0];
		}
		else {
			p->transform_term = 0.5 * (tMin + tMax);
			tMax_new = tMax;
			tMin_new = tMin;
		}
		lastExtremum[0] = 0.;
		lastExtremum[1] = 0.;
	}
	int i, ier;
	double t = 0., t0, u;
	if (!p->limited || p->transform_term != tMin) {
		f1 = Roots_func(0., parms);
		root_increment = 0.5 * t_precision;
		do {
			root_increment *= 2.;
			f2 = Roots_func(root_increment, parms);
		} while(root_increment < f_max && (fabs(f2 - f1) < mult * f_precision || fabs(root_increment - t) < 2. * t_precision));
		if (root_increment >= f_max) return 0;
		u = root_increment;
		f[0] = Iterate(t, parms, u, t_precision, t_limit, f_precision, -1e10 * f_max, roots_improvedPrecision, f_max, ier);
		if (ier > 0) return 0;
		f[1] = f[0];
		if (p->limited) t0 = p->transform_factor * atan(t) + p->transform_term;
		else t0 = t;
		root_increment = 0.5 * step0;
		while(t0 && fabs(root_increment / t0) < 1e-14) root_increment *= 2.;
		root_increment0 = root_increment;
		do {
			root_increment *= 2.;
			f1 = DistanceToSurface(t0 - root_increment, parms);
		} while(root_increment < f_max && fabs(t0 - root_increment) < t_limit && fabs(fabs(f1) - f[0]) < mult * f_precision);
		double root_increment1 = root_increment;
		root_increment = root_increment0;
		do {
			root_increment *= 2.;
			f2 = DistanceToSurface(t0 + root_increment, parms);
		} while(root_increment < f_max && fabs(t0 + root_increment) < t_limit && fabs(fabs(f2) - f[0]) < mult * f_precision);
		double root_increment2 = root_increment;
		if (f1 * f2 < 0.) { // a first-degree root
			roots[0] = t0;
			nRoots = 1;
		}
		lastExtremum[0] = t;
		lastExtremum[1] = t;
		if (!p->limited) {
			if (t0 >= t_limit) {
				t0 = t_limit - root_increment1;
				f[0] = fabs(DistanceToSurface(t0, parms));
				lastExtremum[0] = t0;
			}
			else if (t0 <= -t_limit) {
				t0 = -t_limit + root_increment2;
				f[1] = fabs(DistanceToSurface(t0, parms));
				lastExtremum[1] = t0;
			}
		}
	}
	bool skipBranch[2] = {false, false}, rootAtTheEdge = false;
	int nIterationsWithoutChange[2] = {0, 0};
	do {
		p->searchForMinimum = !p->searchForMinimum;
		for (i = 0; i < 2; i++) {
			if (skipBranch[i]) continue;
			root_increment = i ? 0.5 * t_precision : -0.5 * t_precision;
			while(lastExtremum[i] && fabs(root_increment / lastExtremum[i]) < 1e-14) root_increment *= 2.;
			do {
				root_increment *= 2.;
				u = lastExtremum[i] + root_increment;
				diff = Roots_func(u, parms) + f[i];
			} while(root_increment < f_max && fabs(u) < t_limit && (diff > -mult * f_precision || fabs(u - lastExtremum[i]) < 2. * t_precision));
			if (root_increment >= f_max || fabs(u) >= t_limit || diff > 0.) {
				skipBranch[i] = true;
				continue;
			}
			lastExtremum[i] += root_increment;
			u = lastExtremum[i] + root_increment;
			double old = lastExtremum[i];
			f[i] = Iterate(lastExtremum[i], parms, u, t_precision, t_limit, f_precision, -1e10 * f_max, roots_improvedPrecision, f_max, ier);
			if (old && lastExtremum[i] && fabs(lastExtremum[i] / old - 1.) < roots_minRelChange) {
				nIterationsWithoutChange[i]++;
				if (nIterationsWithoutChange[i] > roots_maxIterationsWithoutChange) {
					skipBranch[i] = true;
					continue;
				}
			}
			else nIterationsWithoutChange[i] = 0;
			if (p->limited) t0 = p->transform_factor * atan(lastExtremum[i]) + p->transform_term;
			else t0 = lastExtremum[i];
			if (ier) {
				skipBranch[i] = true;
				continue;
			}
			if (p->searchForMinimum) {
				root_increment = 0.5 * step0;
				while(t0 && fabs(root_increment / t0) < 1e-14) root_increment *= 2.;
				root_increment0 = root_increment;
				do {
					root_increment *= 2.;
					f1 = DistanceToSurface(t0 - root_increment, parms);
				} while(fabs(fabs(f1) - f[i]) < mult * f_precision);
				root_increment = root_increment0;
				do {
					root_increment *= 2.;
					f2 = DistanceToSurface(t0 + root_increment, parms);
				} while(fabs(fabs(f2) - f[i]) < mult * f_precision);
				if (p->limited && (!i && t0 - tMin_new < mult * step0 || i && tMax_new - t0 < mult * step0)) { // the boundary of the definition region reached
					t0 = !i ? tMin_new : tMax_new;
					skipBranch[i] = true;
				}
				if (f1 * f2 < 0. && (!skipBranch[i] || !rootAtTheEdge)) { // a new first-degree root
					if (skipBranch[i] && periodic) rootAtTheEdge = true;
					if (nRoots == NROOTS_MAX) return -NROOTS_MAX;
					roots[nRoots] = t0;
					nRoots++;
				}
			}
		}
	} while(!skipBranch[0] || !skipBranch[1]);
	return nRoots;
}

double CMCNScriptDlg::Iterate(double &t, void *parms, double &u, double t_precision, double t_max, double f_precision, double f_threshold, bool improvedPrecision, double f_max, int &ier) {
//	Minimization of a one-dimensional function using the simplex method (Nelder-Mead method)
	double t1 = t;
	double t2 = u;
	double f1 = Roots_func(t1, parms);
	double f2 = Roots_func(t2, parms);
	double t_r, f_r, t_e, f_e, t_c, f_c, bak;
	double alpha = 1., gamma = 1.5, eta = 0.5, sigma = 0.5;
	if (improvedPrecision) {
		alpha = 0.0689;
		gamma = 1.071;
	}
	int count0 = 0, count0_max = 100;
	if (f1 > f2 + f_precision) {
		bak = t1;
		t1 = t2;
		t2 = bak;
		bak = f1;
		f1 = f2;
		f2 = bak;
	}
	ier = 1;
	double f1_prev = f1;
	double f2_prev = f2;
	while(f1 >= f_threshold && fabs(t1 - t2) > t_precision && (!t2 || fabs(1. - t1 / t2) > 1e-14)) {
		if (count0 == count0_max) break;
		if (fabs(f1) > f_max) {
			ier = 2;
			break;
		}
		if (fabs(t1) > t_max) {
			ier = -1;
			break;
		}
		t_r = t1 + alpha * (t1 - t2);
		f_r = Roots_func(t_r, parms);
		if (f_r < f1) {
			t_e = t1 + gamma * (t1 - t2);
			f_e = Roots_func(t_e, parms);
			if (f_e < f_r) {
				t2 = t_e;
				f2 = f_e;
			}
			else {
				t2 = t_r;
				f2 = f_r;
			}
		}
		else {
			t_c = t2 + eta * (t1 - t2);
			f_c = Roots_func(t_c, parms);
			if (f_c < f2) {
				t2 = t_c;
				f2 = f_c;
			}
			else {
				t2 = t1 + sigma * (t2 - t1);
				f2 = Roots_func(t2, parms);
			}
		}
		if (f1 > f2 + f_precision) {
			bak = t1;
			t1 = t2;
			t2 = bak;
			bak = f1;
			f1 = f2;
			f2 = bak;
			count0 = 0;
		}
		if (f1 >= f2 - f_precision || fabs(f1 - f1_prev) <= f_precision && fabs(f2 - f2_prev) <= f_precision) count0++;
		else {
			count0 = 0;
			ier = 0;
		}
		f1_prev = f1;
		f2_prev = f2;
	}
	if (count0 < count0_max && ier == 1) ier = 0;
	t = t1;
	u = t2;
	return f1;
}

double CMCNScriptDlg::Iterate2(double &t, double &phi, double t_step0, double phi_step0, double t_precision, double phi_precision, double t_max, double phi_max, double f_precision, double f_threshold) {
//	Minimization of a two-dimensional function using the simplex method (Nelder-Mead method)
	double t1 = t;
	double phi1 = phi;
	double f1 = FitFunc_ConeCylinder(t1, phi1);

	double t2 = t + t_step0;
	double phi2 = phi + phi_step0;
	double f2 = FitFunc_ConeCylinder(t2, phi2);

	double t3_a = t;
	double phi3_a = phi + phi_step0;
	double t3_b = t + t_step0;
	double phi3_b = phi;
	double f3_a = FitFunc_ConeCylinder(t3_a, phi3_a);
	double f3_b = FitFunc_ConeCylinder(t3_b, phi3_b);
	double t3, phi3, f3;
	if (f3_a < f3_b) {
		t3 = t3_a;
		phi3 = phi3_a;
		f3 = f3_a;
	}
	else {
		t3 = t3_b;
		phi3 = phi3_b;
		f3 = f3_b;
	}

	double t_r, phi_r, f_r, t_e, phi_e, f_e, t_c, phi_c, f_c, bak;
	double alpha = 1., gamma = 2., eta = 0.5, sigma = 0.5;
//	double alpha = 1., gamma = 1.5, eta = 0.5, sigma = 0.5;

	int count0 = 0, count0_max = 100;
	if (f1 > f2 + f_precision && f1 > f3 + f_precision) {
		bak = t1;
		t1 = t3;
		t3 = bak;
		bak = phi1;
		phi1 = phi3;
		phi3 = bak;
		bak = f1;
		f1 = f3;
		f3 = bak;
	}
	else if (f2 > f1 + f_precision && f2 > f3 + f_precision) {
		bak = t2;
		t2 = t3;
		t3 = bak;
		bak = phi2;
		phi2 = phi3;
		phi3 = bak;
		bak = f2;
		f2 = f3;
		f3 = bak;
	}
	if (f1 > f2 + f_precision) {
		bak = t1;
		t1 = t2;
		t2 = bak;
		bak = phi1;
		phi1 = phi2;
		phi2 = bak;
		bak = f1;
		f1 = f2;
		f2 = bak;
	}
	double f1_prev = f1;
	double f2_prev = f2;
	double f3_prev = f3;
	while(f1 >= f_threshold && (fabs(t1 - t2) > t_precision || fabs(t2 - t3) > t_precision || fabs(phi1 - phi2) > phi_precision || fabs(phi2 - phi3) > phi_precision) && count0 < count0_max) {
		if (fabs(t1) >= t_max || fabs(phi1) >= phi_max) break;
		if (t3 && fabs(1. - t1 / t3) <= 1e-14 && phi3 && fabs(1. - phi1 / phi3) <= 1e-14) break;
		double t_avg = 0.5 * (t1 + t2);
		double phi_avg = 0.5 * (phi1 + phi2);
		t_r = t_avg + alpha * (t_avg - t3);
		phi_r = phi_avg + alpha * (phi_avg - phi3);
		f_r = FitFunc_ConeCylinder(t_r, phi_r);
		if (f1 <= f_r && f_r < f2) {
			t3 = t_r;
			phi3 = phi_r;
			f3 = f_r;
		}
		else if (f_r < f1) {
			t_e = t_avg + gamma * (t_avg - t3);
			phi_e = phi_avg + gamma * (phi_avg - phi3);
			f_e = FitFunc_ConeCylinder(t_e, phi_e);
			if (f_e < f_r) {
				t3 = t_e;
				phi3 = phi_e;
				f3 = f_e;
			}
			else {
				t3 = t_r;
				phi3 = phi_r;
				f3 = f_r;
			}
		}
		else {
			t_c = t_avg + eta * (t_avg - t3);
			phi_c = phi_avg + eta * (phi_avg - phi3);
			f_c = FitFunc_ConeCylinder(t_c, phi_c);
			if (f_c < f3) {
				t3 = t_c;
				phi3 = phi_c;
				f3 = f_c;
			}
			else {
				t2 = t1 + sigma * (t2 - t1);
				phi2 = phi1 + sigma * (phi2 - phi1);
				f2 = FitFunc_ConeCylinder(t2, phi2);
				t3 = t1 + sigma * (t3 - t1);
				phi3 = phi1 + sigma * (phi3 - phi1);
				f3 = FitFunc_ConeCylinder(t3, phi3);
			}
		}
		if (f1 > f2 + f_precision && f1 > f3 + f_precision) {
			bak = t1;
			t1 = t3;
			t3 = bak;
			bak = phi1;
			phi1 = phi3;
			phi3 = bak;
			bak = f1;
			f1 = f3;
			f3 = bak;
		}
		else if (f2 > f1 + f_precision && f2 > f3 + f_precision) {
			bak = t2;
			t2 = t3;
			t3 = bak;
			bak = phi2;
			phi2 = phi3;
			phi3 = bak;
			bak = f2;
			f2 = f3;
			f3 = bak;
		}
		if (f1 > f2 + f_precision) {
			bak = t1;
			t1 = t2;
			t2 = bak;
			bak = phi1;
			phi1 = phi2;
			phi2 = bak;
			bak = f1;
			f1 = f2;
			f2 = bak;
		}
		if (f1 >= f2 - f_precision || fabs(f1 - f1_prev) <= f_precision && fabs(f2 - f2_prev) <= f_precision && fabs(f3 - f3_prev) <= f_precision) count0++;
		else count0 = 0;
		f1_prev = f1;
		f2_prev = f2;
		f3_prev = f3;
	}
	t = t1;
	phi = phi1;
	return f1;
}

int CMCNScriptDlg::CheckSurfaceID0(char *line, char *&ptr, char &sense, short int &cellNum, short int &surfaceNum, short int &facetNum, bool allowZeroID) // cellNum should not be changed between calls to CheckSurfaceID0
{
	char *endPtr, sep[3] = {'.', '/', '*'}; // sep[2] can be used only after the identifier of a cell and means all bounding surfaces of that cell
	static int bracketDepth = 0; // 0, 1 or 2 (2 is allowed only with allowZeroID = true, i.e., in the definition of a cell-entry tally)
	static int separator[2]; // 0 - no separator, 1 - sep[0], 2 - sep[1] (2 is allowed only with allowZeroID = true, i.e., in the definition of a cell-entry tally)
	static int senseMult[3];
	static int objectNum[2];
	int surfaceID;
	int i, j, k, n;
	int decreaseBracketDepth = 0;
	double value_test;
	if (!ptr) {
		if (bracketDepth) {
			printf("Error: Missing closing %s.\n", bracketDepth == 1 ? "bracket" : "brackets"); return 0;
		}
		return 1;
	}
	if (!bracketDepth) {
		cellNum = -1;
		objectNum[0] = -1;
	}
	// If there is a left bracket, then it must be immediately after a separator.
	// There must be a space after a right bracket (except when it is followed by another right bracket).
NEXT_TOKEN:
	surfaceID = strtol(ptr, &endPtr, 10);
	if (*ptr == '-') {
		sense = -1;
		ptr++;
	}
	else sense = 1;
	if (bracketDepth) sense *= senseMult[bracketDepth-1];
	senseMult[bracketDepth] = sense;
	if (surfaceID < 0) surfaceID = -surfaceID;
	if (!allowZeroID && (*endPtr == sep[1] || *endPtr == sep[2])) {
		printf("Error: Separator '%c' found in the definition of a cell or a general macrobody.\n", *endPtr); return 0;
	}
	char *tempPtr = NULL;
	if (*endPtr != sep[0] && *endPtr != sep[1] && *endPtr != sep[2]) value_test = atof(ptr);
	else {
		char *tempPtr = new char[endPtr-ptr+1];
		if (endPtr != ptr) strncpy(tempPtr, ptr, endPtr - ptr);
		tempPtr[endPtr-ptr] = '\0';
		value_test = atof(tempPtr);
	}
	if (*endPtr == ']' && (bracketDepth && endPtr[1] == '\0' || bracketDepth == 2 && endPtr[1] == ']' && endPtr[2] == '\0')) {
		if (endPtr == ptr) {
			if (endPtr[1] == '\0') bracketDepth--;
			else bracketDepth -= 2;
			ptr = NextToken(line, NULL, " \t\n");
			if (tempPtr) delete tempPtr;
			return -1;
		}
		else if (value_test <= 0.5 || value_test >= 2147483647.5) {
			printf("Error: Invalid object identifier \"%s\" (it must be a positive integer number less than 2^31 = 2147483648.\n", tempPtr ? tempPtr : ptr);
			if (tempPtr) delete tempPtr;
			return 0;
		}
		decreaseBracketDepth = (endPtr[1] == '\0' ? 1 : 2);
	}
	else if ((*endPtr != '\0' && *endPtr != sep[0] && *endPtr != sep[2] || value_test <= 0.5) && (*endPtr != sep[1] || value_test <= -0.5) || value_test >= 2147483647.5) {
		printf("Error: Invalid object identifier \"%s\" (it must be a %s integer number less than 2^31 = 2147483648.\n",
				tempPtr ? tempPtr : ptr, (*endPtr == sep[1] ? "non-negative" : "positive"));
		if (tempPtr) delete tempPtr;
		return 0;
	}
	if (tempPtr) delete tempPtr;
	j = -2;
	if (!bracketDepth && (*endPtr == sep[2] && endPtr[1] == '\0' || *endPtr == sep[1] && endPtr[1] == sep[2] && endPtr[2] == '\0')) {
		for (j = 0; j < nCells && cell[j].ID != surfaceID; j++);
		if (j == nCells) {
			printf("Error: There is no cell with ID %d.\n", surfaceID); return 0;
		}
		cellNum = -j - 2;
		return 1;
	}
	else if (*endPtr == sep[0] || *endPtr == sep[1]) {
		if (bracketDepth == 2) {
			printf("Error: Separator '%c' found at bracket depth 2.\n", *endPtr); return 0;
		}
		if (bracketDepth == 1) {
			if (separator[0] == 1) {
				printf("Error: Separator '%c' found in the sequence number of a macrobody facet.\n", *endPtr); return 0;
			}
			if (*endPtr == sep[1]) {
				printf("Error: Separator '%c' found at bracket depth 1.\n", *endPtr); return 0;
			}
		}
		if (*endPtr == sep[0]) { // A facet of a previously-defined macrobody
			separator[bracketDepth] = 1;
			for (j = 0; j < nMacrobodies && macrobody[j].ID != surfaceID; j++);
			if (j == nMacrobodies) {
				printf("Error: There is no macrobody with ID %d.\n", surfaceID); return 0;
			}
		}
		else {
			separator[bracketDepth] = 2;
			if (surfaceID) {
				for (j = 0; j < nCells && cell[j].ID != surfaceID; j++);
				if (j == nCells) {
					printf("Error: There is no cell with ID %d.\n", surfaceID); return 0;
				}
			}
			else j = -1; // "0-..."
		}
		objectNum[bracketDepth] = j;
		ptr = endPtr + 1;
		if (*ptr == '[') {
			if (bracketDepth == 2) {
				printf("Error: Left bracket found at bracket depth 2.\n"); return 0;
			}
			j = -2;
			bracketDepth++;
			ptr++;
			if (*ptr == '\0') {
				ptr = NextToken(line, NULL, " \t\n");
				goto NEXT_TOKEN;
			}
			if (*ptr == '-') {
				sense = -sense;
				ptr++;
			}
			senseMult[bracketDepth] = sense;
		}
		else if (*ptr == '\0') {
			printf("Error: Separator '%c' is not followed either by an identifier or the left bracket.\n", *endPtr); return 0;
		}
	}

	if ((j > -2 || bracketDepth) && separator[0] == 2) cellNum = objectNum[0] + 1;
	else if (!bracketDepth) cellNum = -1;

	if (j > -2 && (*ptr == '-' || *ptr == '+')) { // j > -2 indicates that the surface ID is preceded by a separator
		printf("Error: Separator '%c' is followed by the sign '%c'.\n", *endPtr, *ptr); return 0;
	}

	surfaceID = strtol(ptr, &endPtr, 10);
	if (*endPtr == ']' && (bracketDepth && endPtr[1] == '\0' || bracketDepth == 2 && endPtr[1] == ']' && endPtr[2] == '\0')) {
		decreaseBracketDepth = (endPtr[1] == '\0' ? 1 : 2);
	}

	if (j > -2) {
		if (*endPtr == sep[1] || separator[bracketDepth] == 1 && *endPtr == sep[0]) {
			printf("Error: String \"%d%c%s\" contains an invalid combination of two separators.\n",
				(j > -1 ? (separator[bracketDepth] == 2 ? cell[j].ID : macrobody[j].ID) : 0), sep[separator[bracketDepth]-1], ptr);
			return 0;
		}
		else if (*endPtr != '\0' && *endPtr != ']' && *endPtr != sep[0]) {
			printf("Error: Invalid surface identifier \"%d%c%s\".\n",
				(j > -1 ? (separator[bracketDepth] == 2 ? cell[j].ID : macrobody[j].ID) : 0), sep[separator[bracketDepth]-1], ptr);
			return 0;
		}
	}
	else if (*endPtr != '\0' && *endPtr != ']' && *endPtr != sep[0]) {
		printf("Error: Invalid surface identifier \"%s\".\n", ptr); return 0;
	}

	bool allowZero = allowZeroID && j == -2 && !bracketDepth;
	if (!surfaceID) {
		if (bracketDepth) {
			printf("Error: Zero surface identifier found between brackets.\n"); return 0;
		}
		if (j > -2) {
			printf("Error: Zero surface identifier found after a separator.\n"); return 0;
		}
		if (!allowZeroID) {
			printf("Error: Zero surface identifier found.\n"); return 0;
		}
	}

	tempPtr = NULL;
	if (*endPtr != sep[0]) value_test = atof(ptr);
	else {
		tempPtr = new char[endPtr-ptr+1];
		if (endPtr != ptr) strncpy(tempPtr, ptr, endPtr - ptr);
		tempPtr[endPtr-ptr] = '\0';
		value_test = atof(tempPtr);
	}
	if (allowZero && value_test <= -0.5 || !allowZero && value_test <= 0.5 || value_test >= 2147483647.5) {
		printf("Error: Invalid surface identifier \"%s\" (it must be a %s integer number less than 2^31 = 2147483648.\n", tempPtr ? tempPtr : ptr, allowZero ? "non-negative" : "positive");
		if (tempPtr) delete tempPtr;
		return 0;
	}
	if (tempPtr) delete tempPtr;

	if (*endPtr != '\0' && *endPtr != ']' && *endPtr != sep[0] || !surfaceID && (*endPtr != '\0' && *endPtr != ']' || !allowZeroID)) {
		printf("Error: Invalid surface identifier \"%s\".\n", ptr); return 0;
	}

	if (!surfaceID) {
		cellNum = -1;
		surfaceNum = 0;
		facetNum = 0;
		bracketDepth -= decreaseBracketDepth;
		return 1;
	}

	MACROBODY *mb0 = NULL;
	if (j > -1 && separator[bracketDepth] != 1 || j < 0 && (!bracketDepth || separator[bracketDepth-1] != 1)) {
		for (i = 0; i < nSurfaces && (surf[i].mb_index >= 0 || surf[i].ID != surfaceID); i++);
		if (i == nSurfaces) {
			for (k = 0; k < nMacrobodies && macrobody[k].ID != surfaceID; k++);
		}
		if (i < nSurfaces) {
			if (*endPtr == sep[0]) {
				printf("Error: Surface %d is not a macrobody.\n", surfaceID); return 0;
			}
		}
		else {
			if (k == nMacrobodies) {
				printf("Error: Undefined surface identifier \"%d\".\n", surfaceID); return 0;
			}
			if (*endPtr == sep[0]) { // The second separator in the current token
				mb0 = macrobody + k;
				separator[bracketDepth] = 1;
				objectNum[bracketDepth] = k;
				ptr = endPtr + 1;
				if (*ptr == '[') {
					bracketDepth++;
					ptr++;
					if (*ptr == '\0') {
						ptr = NextToken(line, NULL, " \t\n");
						goto NEXT_TOKEN;
					}
					if (*ptr == '-') {
						sense = -sense;
						ptr++;
					}
					senseMult[bracketDepth] = sense;
				}
				else if (*ptr == '\0') {
					printf("Error: Separator '%c' is not followed by an identifier.\n", *endPtr); return 0;
				}
			}
		}
	}
	else if (j > -1) {
		k = j;
		mb0 = macrobody + k;
	}
	else if (objectNum[bracketDepth-1] > -1) {
		k = objectNum[bracketDepth-1];
		mb0 = macrobody + k;
	}

	if (mb0) { // A facet of a previously-defined macrobody
		n = strtol(ptr, &endPtr, 10);
		tempPtr = NULL;
		if (*endPtr != sep[0] && *endPtr != ']') value_test = atof(ptr);
		else {
			tempPtr = new char[endPtr-ptr+1];
			if (endPtr != ptr) strncpy(tempPtr, ptr, endPtr - ptr);
			tempPtr[endPtr-ptr] = '\0';
			value_test = atof(tempPtr);
		}
		if (value_test <= 0.5 || value_test >= 2147483647.5) {
			printf("Error: Invalid macrobody facet identifier \"%d.%s\" (it must be a positive integer number less than 2^31 = 2147483648.\n", mb0->ID, tempPtr ? tempPtr : ptr);
			if (tempPtr) delete tempPtr;
			return 0;
		}
		if (tempPtr) delete tempPtr;
		if (n <= 0) {
			printf("Error: Invalid macrobody facet identifier \"%d.%s\".\n", mb0->ID, ptr); return 0;
		}
		if (*endPtr == ']' && (bracketDepth && endPtr[1] == '\0' || bracketDepth == 2 && endPtr[1] == ']' && endPtr[2] == '\0')) {
			decreaseBracketDepth = (endPtr[1] == '\0' ? 1 : 2);
		}
		else if (*endPtr != '\0') {
			printf("Error: Invalid macrobody facet identifier \"%d.%s\".\n", mb0->ID, ptr); return 0;
		}
		if (n > mb0->nS) {
			printf("Error: Macrobody %d has less than %d facets.\n", mb0->ID, n); return 0;
		}
		surfaceNum = -k - 1; // 1-based macrobody sequence number with the minus sign
		facetNum = n - 1; // 0-based facet sequence number
	}
	else if (i == nSurfaces) { // A previously-defined macrobody
		surfaceNum = -k - 1; // 1-based macrobody sequence number with the minus sign
		facetNum = -1;
	}
	else { // A previously-defined "regular" surface
		surfaceNum = i; // 0-based surface sequence number
		facetNum = -1;
	}

	bracketDepth -= decreaseBracketDepth;
	return 1;
}

int CMCNScriptDlg::CheckSurfaceID(char *line, char *&ptr, bool isCell, int ID, unsigned short int &nS, unsigned short int *&s, char *&sense, short int *&mb, short int *&i_mb, int chunkSize)
{
	char senseMult;
	short int cellNum, surfaceNum, facetNum;
	int rc = CheckSurfaceID0(line, ptr, senseMult, cellNum, surfaceNum, facetNum);
	if (!rc) return 0;
	if (rc < 0) return rc;
	int i, j, k;
	char *name = isCell ? "cell" : "macrobody";
	if (facetNum >= 0) { // A facet of a previously-defined macrobody
		j = -surfaceNum - 1; // 0-based sequence number of the macrobody
		MACROBODY *mb0 = macrobody + j;
		if (nS == MAXSURFACES_CELL) {
			printf("Error: The number of facets of %s %d is greater than %d.\n", name, ID, MAXSURFACES_CELL); return 0;
		}
		if (nS / chunkSize * chunkSize == nS) {
			if (!isCell) {
				s = (unsigned short int *)realloc(s, (nS + chunkSize) * sizeof(unsigned short int));
				sense = (char *)realloc(sense, (nS + chunkSize) * sizeof(char));
			}
			mb = (short int *)realloc(mb, (nS + chunkSize) * sizeof(short int));
			i_mb = (short int *)realloc(i_mb, (nS + chunkSize) * sizeof(short int));
		}
		s[nS] = mb0->s[facetNum];
		if (mb0->type) sense[nS] = -senseMult * mb0->sense[facetNum];
		else sense[nS] = senseMult * mb0->sense[facetNum];
		mb[nS] = senseMult * (j + 1);
		i_mb[nS] = -(facetNum + 1);
		if (!isCell) surf[s[nS]].nMb++;
		nS++;
	}
	else if (surfaceNum < 0) { // A previously-defined macrobody
		j = -surfaceNum - 1; // 0-based sequence number of the macrobody
		MACROBODY *mb0 = macrobody + j;
		if (mb0->type && senseMult > 0) {
			printf("Error: The identifier of a standard macrobody has been specified in the definition of %s %d without the minus sign.\n", name, ID); return 0;
		}
		else if (!mb0->type && senseMult < 0) {
			printf("Error: The identifier of a general macrobody has been specified in the definition of %s %d with the minus sign.\n", name, ID); return 0;
		}
		int nS0 = nS;
		int nChunks0 = nS0 / chunkSize;
		if (nChunks0 * chunkSize != nS0) nChunks0++;
		nS += mb0->nS;
		if (nS > MAXSURFACES_CELL) {
			printf("Error: The number of facets of %s %d is greater than %d.\n", name, ID, MAXSURFACES_CELL); return 0;
		}
		int nChunks = nS / chunkSize;
		if (nChunks * chunkSize != nS) nChunks++;
		if (nChunks != nChunks0) {
			if (!isCell) {
				s = (unsigned short int *)realloc(s, nChunks * chunkSize * sizeof(unsigned short int));
				sense = (char *)realloc(sense, nChunks * chunkSize * sizeof(char));
			}
			mb = (short int *)realloc(mb, nChunks * chunkSize * sizeof(short int));
			i_mb = (short int *)realloc(i_mb, nChunks * chunkSize * sizeof(short int));
		}
		short int j1 = j + 1;
		for (i = nS0, k = 0; k < mb0->nS; k++, i++) {
			s[i] = mb0->s[k];
			sense[i] = mb0->sense[k];
			if (mb0->type) mb[i] = -j1;
			else mb[i] = j1;
			i_mb[i] = k + 1;
			if (!isCell) surf[s[i]].nMb++;
		}
	}
	else { // A previously-defined "regular" surface
		if (nS == MAXSURFACES_CELL) {
			printf("Error: The number of facets of %s %d is greater than %d.\n", name, ID, MAXSURFACES_CELL); return 0;
		}
		if (nS / chunkSize * chunkSize == nS) {
			if (!isCell) {
				s = (unsigned short int *)realloc(s, (nS + chunkSize) * sizeof(unsigned short int));
				sense = (char *)realloc(sense, (nS + chunkSize) * sizeof(char));
			}
			mb = (short int *)realloc(mb, (nS + chunkSize) * sizeof(short int));
			i_mb = (short int *)realloc(i_mb, (nS + chunkSize) * sizeof(short int));
		}
		s[nS] = surfaceNum;
		sense[nS] = senseMult;
		mb[nS] = 0;
		i_mb[nS] = 0;
		if (!isCell) surf[surfaceNum].nMb++;
		nS++;
	}
	return 1;
}

void CMCNScriptDlg::SurfaceIDstring(SURFACE *s, char sense, short int mb, short int mb_i, char string[100])
{
	char string0[100];
	if (s->mb_index < 0) { // The surface is not a part of a standard macrobody
		if (sense) sprintf(string0, "%c%d", (sense > 0 ? '+' : '-'), s->ID);
		else sprintf(string0, "%d", s->ID);
	}
	if (!mb) strcpy(string, string0); // A surface (not a macrobody) can be specified by a single number only when SURFACE::mb_index < 0
	else {
		int mbNum = mb > 0 ? mb - 1 : -mb - 1;
		int facetNum = mb_i > 0 ? mb_i : -mb_i;
		int mbType = macrobody[mbNum].type;
		int n;
		if (sense) n = sprintf(string, "%c%d.%d", (mbType && mb > 0 || !mbType && sense * macrobody[mbNum].sense[facetNum-1] > 0 ? '+' : '-'), macrobody[mbNum].ID, facetNum);
		else n = sprintf(string, "%d.%d", macrobody[mbNum].ID, facetNum);
		if (s->mb_index < 0) sprintf(string + n, " (%s)", string0); // The surface, although used in the specified macrobody, has been also defined separately
		else if (s->mb_index != mbNum) { // The surface, although used in the specified standard macrobody, has been originally defined as a part of another standard macrobody (and has not been defined separately)
			if (sense) sprintf(string + n, " (%c%d.%d)", (sense > 0 ? '+' : '-'), macrobody[s->mb_index].ID, s->ID); // A standard macrobody has negative sense with respect to the bounding surfaces that are created automatically when the macrobody is defined
			else sprintf(string + n, " (%d.%d)", macrobody[s->mb_index].ID, s->ID);
		}
	}
}

bool CMCNScriptDlg::ConvertRPPtoBOX(SURFACE *p, double *&arb_entries)
{
	int k, m, num[3];
	VECTOR dir[3];
	dir[0] = xDir;
	dir[1] = yDir;
	dir[2] = zDir;
	for (k = 0; k < 6; k += 2) {
		for (m = 0; m < 3 && Norm(p[k].dir & dir[m]) > anglePrecision; m++);
		if (m == 3) break;
		num[m] = k;
	}
	if (k == 6) { // All facets are orthogonal to the coordinate axes, so that there is no need to convert to "BOX"
		if (!arb_entries) arb_entries = new double[6];
		if (p[num[0]].init.x < p[num[0]+1].init.x) {
			arb_entries[0] = p[num[0]].init.x;
			arb_entries[1] = p[num[0]+1].init.x;
		}
		else {
			arb_entries[0] = p[num[0]+1].init.x;
			arb_entries[1] = p[num[0]].init.x;
		}
		if (p[num[1]].init.y < p[num[1]+1].init.y) {
			arb_entries[2] = p[num[1]].init.y;
			arb_entries[3] = p[num[1]+1].init.y;
		}
		else {
			arb_entries[2] = p[num[1]+1].init.y;
			arb_entries[3] = p[num[1]].init.y;
		}
		if (p[num[2]].init.z < p[num[2]+1].init.z) {
			arb_entries[4] = p[num[2]].init.z;
			arb_entries[5] = p[num[2]+1].init.z;
		}
		else {
			arb_entries[4] = p[num[2]+1].init.z;
			arb_entries[5] = p[num[2]].init.z;
		}
		return false;
	}
	VECTOR v[4];
	VECTOR diagonal = p[0].init - p[1].init;
	v[0] = p[1].init;
	v[1] = (diagonal * p[0].dir) * p[0].dir;
	v[2] = (diagonal * p[2].dir) * p[2].dir;
	v[3] = (diagonal * p[4].dir) * p[4].dir;
	if (arb_entries) delete arb_entries;
	arb_entries = new double[12];
	for (k = 0, m = 0; m < 4; m++, k+=3) memcpy(arb_entries + k, v + m, 3 * sizeof(double));
	return true;
}

